
.include "sys.inc"

	config __FICD	JTAGEN_OFF & ICS_PGD2
	config __FPOR	ALTI2C1_OFF & ALTI2C2_OFF & WDTWIN_WIN25
	config __FWDT	FWDTEN_OFF & WINDIS_OFF & WDTPRE_PR32 & WDTPOST_PS64
	config __FOSC	FCKSM_CSECMD & OSCIOFNC_OFF & POSCMD_XT & IOL1WAY_OFF
	config __FOSCSEL FNOSC_PRIPLL & IESO_OFF

.section .init,code

.global __resetPRI


__resetPRI:     
        .weak  __reset

.section .text
.global __DefaultInterrupt

__DefaultInterrupt:
	nop
	bra     $-2

__reset:
	mov      #__SP_init,w15    ; initialize w15
	mov      #__SPLIM_init,w0
	mov      w0,_SPLIM         ; initialize SPLIM

	mov	#0x200, w0
	mov	w0, DSRPAG

/* Interrupt nesting disabled */
	mov	#0x8000, w0
	mov	w0, INTCON1

/*   PMD1
bit 15 T5MD: Timer5 Module Disable bit
bit 14 T4MD: Timer4 Module Disable bit
bit 13 T3MD: Timer3 Module Disable bit
bit 12 T2MD: Timer2 Module Disable bit
bit 11 T1MD: Timer1 Module Disable bit
bit 7 I2C1MD: I2C1 Module Disable bit
bit 6 U2MD: UART2 Module Disable bit
bit 5 U1MD: UART1 Module Disable bit
bit 4 SPI2MD: SPI2 Module Disable bit
bit 3 SPI1MD: SPI1 Module Disable bit
bit 1 C1MD: ECAN1 Module Disable bit
bit 0 AD1MD: ADC1 Module Disable bit
*/
	mov	#0xf7b6, w0
	mov	w0, PMD1
/*  PMD2
bit 11 IC4MD: Input Capture 4 Module Disable bit
bit 10 IC3MD: Input Capture 3 Module Disable bit
bit 9 IC2MD: Input Capture 2 Module Disable bit
bit 8 IC1MD: Input Capture 1 Module Disable bit
bit 3 OC4MD: Output Compare 4 Module Disable bit
bit 2 OC3MD: Output Compare 3 Module Disable bit
bit 1 OC2MD: Output Compare 2 Module Disable bit
bit 0 OC1MD: Output Compare 1 Module Disable bit
*/
	mov	#0xffff, w0
	mov	w0, PMD2
/*  PMD3
bit 10 CMPMD: Comparator Module Disable bit
bit 7 CRCMD: CRC Module Disable bit
bit 1 I2C2MD: I2C2 Module Disable bit
*/
	mov	#0xfffd, w0
	mov	w0, PMD3
/*  PMD4
bit 3 REFOMD: Reference Clock Module Disable bit
bit 2 CTMUMD: CTMU Module Disable bit
*/
	mov	#0xffff, w0
	mov	w0, PMD4
/*  PMD7
bit 4 DMA0MD: DMAx Module Disable bit
bit 3 PTGMD: PTG Module Disable bit
*/
	mov	#0xffef, w0
	mov	w0, PMD7

/* ----------- Init Oscillator ------------------------------------------ */

	mov	#PLL_DIV, w0
	mov	w0, PLLFBD	;Setup PLL multiplier value

.if PLL_POST == 2
/*	CLK_DIV = 0x3040   default PLL_POST = 4 */
	mov	#0x3000, w0
	mov	w0, CLKDIV
.endif

OSC_Wait:
	btsc	OSCCON, #OSWEN		; Wait until oscillator switch finished
	bra	OSC_Wait

/* ----------- Init Port ------------------------------------------------ */
/* RA
0  iz.ANL0	1  iz.ANL1	2  --		3 --
4  iz.SDO1	5  --		6 --		7  ol.GK5
8  ioc.SDA	9  ih.SDI1	10 ol.GK4	11 ol.HT1_ON
12 ol.FAN2_ON	13  --		14  --		15  --
*/
	;            12   8   4   0
	;             |   |   |   |
	mov	#0b0000000000000000, w0
	mov	w0, PORTA
	mov	#0b1110001101111111, w0
	mov	w0, TRISA
	mov	#0b0000001000000000, w0
	mov	w0, CNPUA
	mov	#0b0000000000000000, w0
	mov	w0, CNPDA
	mov	#0b0000000100000000, W0
	mov	w0, ODCA

/* RB
0  iz.ANL2	1  iz.ANL3	2  iz.ANL4	3  ol.TEST
4  ooc.SCL	5  iz.PGD 	6  iz.PGC	7  oh.SO
8  ol.CCLK	9  ol.C2	10 ol.GK0	11 ol.GK1
12 ol.GK2	13 ol.GK3	14 ol.GK6	15 ol.HT2_ON
*/
	mov	#0b0000000010000000, w0
	mov	w0, PORTB
	mov	#0b0000000001100111, w0
	mov	w0, TRISB
	mov	#0b0000000000000000, w0
	mov	w0, CNPUB
	mov	#0b0000000000000000, w0
	mov	w0, CNPDB
	mov	#0b0000000000010000, w0
	mov	W0, ODCB

/* RC
0  il		1  il		2  il		3  iz.SCK1
4  oh.LED1	5  oh.LED2	6  ol.C1	7  ol.SEL
8  ol.XOR	9  ol.GV_ON	10 ih.SI	11 il
12 iz.OSC1	13 ol.C3	14 --		15 iz.OSC2
*/
	mov	#0b0000000000110000, w0
	mov	W0, PORTC
	mov	#0b1101110000001111, w0
	mov	W0, TRISC
	mov	#0b0000010000000000, w0
	mov	W0, CNPUC
	mov	#0b0000100000000111, w0
	mov	W0, CNPDC
;	mov	#0b0000 0000 0000 0000, W0
;	mov	W0, ODCC

/* RD
0  --		1  --		2  --		3  --
4  --		5  ol.TDRDC	6  ol.K01DB	7  --
8  il		9  --		10 --		11 --
12 --		13 --		14 --		15 --
*/
	mov	#0b0000000000000000, w0
	mov	W0, PORTD
	mov	#0b1111111110011111, w0
	mov	W0, TRISD
	mov	#0b0000000000000000, w0
	mov	W0, CNPUD
	mov	#0b0000000100000000, w0
	mov	W0, CNPDD
;	clr	ODCD

/* RE
0  --		1  --		2  --		3  --
4  --		5  --		6  --		7  --
8  --		9  --		10 --		11 --
12 oh.CSIO	13 ol.CSX	14 oh.A0	15 oh.CSLCD
*/
	mov	#0b1101000000000000, w0
	mov	W0, PORTE
	mov	#0b0000111111111111, w0
	mov	W0, TRISE
	mov	#0b0000000000000000, w0
	mov	W0, CNPUE
	mov	#0b0000000000000000, w0
	mov	W0, CNPDE
;	clr	ODCE

/* RF
0  ol.OFF_EN	1  ol.OP1ON	2 --	 	3  --
4  --		5  --		6  --		7  --
8  --		9  --		10 --		11 --
12 --		13 --		14 --		15 --
*/
	mov	#0b0000000000000000, w0
	mov	W0, PORTF
	mov	#0b1111111111111100, w0
	mov	W0, TRISF
	mov	#0b0000000000000000, w0
	mov	W0, CNPUF
	mov	#0b0000000000000000, w0
	mov	W0, CNPDF
;	clr	ODCF

/* RG
0  --		1  --		2  --		3  --
4  --		5  --		6 ol.FAN_PWM	7 ol.FAN_ON
8  ol.PWR_PA	9  ol.FAN1_ON	10 --		11 --
12 --		13 --		14 --		15 --
*/
	mov	#0b0000000000000000, w0
	mov	W0, PORTG
	mov	#0b1111110000111111, w0
	mov	W0, TRISG
	mov	#0b0000000000000000, w0
	mov	W0, CNPUG
	mov	#0b0000000000000000, w0
	mov	W0, CNPDG
;	clr	ODCG

/* ----------- Init UART Pin -------------------------------------------- */
.if USE_DEBUG_UART
/* Pin45(RC10/RPI58) --> U2RX    UART2 Receive */
	mov	#58, w0
	mov	w0, RPINR19
/* U2TX(3) --> Pin46(RB7/RP39) */
	mov	#0x0300, w0
	mov	w0, RPOR2
.else
	clr	RPOR2
	clr	RPINR19
.endif


/* ----------- Clr mem -------------------------------------------------- */
	mov	#__DATA_LENGTH, w1
	lsr	w1, #1, w1
	dec	w1, w1
	mov	#__DATA_BASE, w2
	mov	#0, w0
	repeat	w1
	mov	w0, [w2++]




	call	main		; call user's main()

	.pword 0xDA4000		; halt the simulator
	reset			; reset the processor
.end
