
.include "sys.inc"

	BRGVAL = (FCY/400000) - 2

.equ MAX_RETRY,		3
	EEPROM_MODE	= 0

.section .nbss, bss, near
i2c2_GesErrCount:	.space 2
i2c2_LastErr_id:	.space 2
i2c2_ErrLoop:		.space 2
i2c2_cmd:		.space 2
i2c2_id:		.space 2
i2c2_adr:		.space 2
i2c2_Flag:		.space 2
i2c2_Lock:		.space 2
i2c2_step:		.space 2
i2c2_Interrupt:		.space 2
i2c2_idx:		.space 2
i2c2_ErrStartCount:	.space 2
i2c2_DataSize:		.space 2
i2c2_Buf:		.space I2C_MAX_BUF

.global i2c2_GesErrCount
.global i2c2_cmd
.global i2c2_id
.global i2c2_adr
.global i2c2_Flag
.global i2c2_Interrupt
.global i2c2_Lock
.global i2c2_DataSize
.global i2c2_Buf

.global i2c2_step

.section .text
.global InitI2C2
.global __MI2C2Interrupt
.global Handle_I2C2

.if BRGVAL > 511
.error "BRGVAL > 511"
.endif

InitI2C2:
	mov	#0x1000, w0
	mov	w0, I2C2CON
	mov	#0x0000, w0
	mov	w0, I2C2ADD
	mov	w0, I2C2MSK

;	mov	#398, w0
	mov	#BRGVAL, w0		/* 60000kHz / 400kHz = 150 */
	mov	w0, I2C2BRG

	bset	I2C2CON, #I2CEN
	bset	IEC3, #MI2C2IE
	bclr	IFS3, #MI2C2IF

	setm	i2c2_Interrupt
	return

__MI2C2Interrupt:
	clr	i2c2_Interrupt
	bclr	IFS3, #MI2C2IF
	retfie


Handle_I2C2:
	btst	I2C2STAT, #BCL	/* Master Bus Collision Detect bit */
	bra	nz, h_Coll

	mov	i2c2_step, w0
	and	w0, #0x1f, w0
	bra	z, Step0

.IF EEPROM_MODE
	cp	w0, #20
.ELSE
	cp	w0, #19
.ENDIF
	bra	gt, h_OV
	add	w0, w0, w0
	mov	#i2c2_proc, w1
	mov	[w0+w1], w0
	goto	w0
h_OV:
	clr	i2c2_step
	return

h_Coll:
	inc	i2c2_GesErrCount
	bset	I2CErrorFlag, #I2C_Defect
	mov	#I2C_ERROR, w0
	mov	w0, i2c2_cmd
	bra	StepClr


Step0:	/* cmd test */
	mov	i2c2_cmd, w0
	cp	w0, #I2C_READ
	bra	z, 10f
	cp	w0, #I2C_WRITE
	bra	z, 10f
	return
10:
	inc	i2c2_step

Step1:	/* Start Condition */
;	bset	I2C2CON, #SEN

	mov	#0x9001, w0
	btsc	i2c2_Flag, #I2C_bADR10Bit
	bset	w0, #A10M
	mov	w0, I2C2CON
IncRet:
	inc	i2c2_step
NopRet:
	return

Step2:	/* Start Byte with device select id */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	mov	i2c2_id, w0
	mov	w0, I2C2TRN
	btst	i2c2_Flag, #I2C_bNoADR
	bra	z, IncRet
	mov	#5, w0		/* Step5_Force */
	mov	w0, i2c2_step
	return

Step3:	/* Send address byte 1, if ack is received. Else Retry */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	btst	I2C2STAT, #ACKSTAT
	bra	nz, S3_err
	clr	i2c2_ErrStartCount

	mov	i2c2_adr, w0
	btst	i2c2_Flag, #I2C_bADR10Bit
	bra	z, S3_7b
	lsr	w0, #8, w0
	mov	w0, I2C2TRN
	inc	i2c2_step
	return
S3_7b:
	mov	w0, I2C2TRN
	inc2	i2c2_step
	return
S3_err:
	mov	i2c2_ErrStartCount, w0
	cp	w0, #MAX_RETRY
	bra	ge, ErrAckNotRec
	mov	#18, w0		/* Ack Not received, Flag error and exit */
	mov	w0, i2c2_step
	return

Step4:	/* Send address byte 2, if ack is received. Else Flag error and exit */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	btst	I2C2STAT, #ACKSTAT
	bra	nz, ErrAckNotRec
	mov	i2c2_adr, w0
	mov	w0, I2C2TRN
	inc	i2c2_step
	return
ErrAckNotRec:
	inc	i2c2_GesErrCount
	mov	i2c2_id, w0
	mov	w0, i2c2_LastErr_id
	mov	#16, w0		/* Ack Not received, Flag error and exit */
	mov	w0, i2c2_step
	return

Step5:	/* Read or Write */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	btst	I2C2STAT, #ACKSTAT
	bra	nz, ErrAckNotRec
	clr	i2c2_idx
	mov	i2c2_cmd, w0
	cp	w0, #I2C_WRITE
	bra	z, Step6_Send
	mov	#8, w0		/* Read */
	mov	w0, i2c2_step
	bra	Step8

/*==================================*/
/* Write Data Phase                 */
/*==================================*/
Step6_Send:
	inc	i2c2_step
Step6:	/* Send data */
	mov	#i2c2_Buf, w0
	mov	i2c2_idx, w1
	inc	i2c2_idx
	mov.b	[w0+w1], w0
	mov	w0, I2C2TRN
	inc	i2c2_step
	return

Step7:	/* Look for end of data or no Ack */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	dec	i2c2_step	/* loop Send (Step6)*/
	mov	i2c2_DataSize, w0
	cp	i2c2_idx
	bra	lt, Step6

.IF EEPROM_MODE
	mov	#20, w0		/* Go to ACK polling to wait for write to complete */
	mov	w0, i2c2_step
	bra	Step20
.ELSE
	mov	#14, w0		/* Stop Sequence */
	mov	w0, i2c2_step
	bra	Step14
.ENDIF
/*==================================*/
/* Read Data Phase                  */
/*==================================*/
Step8:	/* Repeat Start */
	bset	I2C2CON, #RSEN
	inc	i2c2_step
	return
Step9:	/* Re-send control byte with W/R=R */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	mov	i2c2_id, w0
	bset	w0, #0
	mov	w0, I2C2TRN
	inc	i2c2_step
	return
Step10:	/* Check, if control byte went ok */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	btst	I2C2STAT, #ACKSTAT
	bra	nz, ErrAckNotRec
	inc	i2c2_step
;	bra	IncRet
Step11:	/* Receive Enable */
	bset	I2C2CON, #RCEN
	inc	i2c2_step
	return
Step12:	/* Receive data */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	mov	I2C2RCV, w2
	mov	#i2c2_Buf, w0
	mov	i2c2_idx, w1
	inc	i2c2_idx
	mov.b	w2, [w0+w1]

	mov	i2c2_DataSize, w0
	cp	i2c2_idx
	btsc	SR, #N
	bclr	I2C2CON, #ACKDT
	btss	SR, #N
	bset	I2C2CON, #ACKDT

	bset	I2C2CON, #ACKEN
	inc	i2c2_step
	return
Step13:	/* Loop data */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet

	mov	i2c2_DataSize, w0
	cp	i2c2_idx
	bra	ge, Step14_Stop

	bset	I2C2CON, #RCEN	/* Receive Enable */
	mov	#12, w0		/* Receive Loop */
	mov	w0, i2c2_step
	return
/*==================================*/
/* Stop Sequence                    */
/*==================================*/
Step14_Stop:
	inc	i2c2_step
Step14:	/*  */
	bset	I2C2CON, #PEN
	inc	i2c2_step
	return
Step15:	/*  */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	clr	i2c2_cmd
StepClrE:
	clr	i2c2_ErrLoop
StepClr:
	setm	i2c2_Interrupt
	clr	i2c2_step
	return
/*==================================*/
/* Set Error                         */
/*==================================*/
;Step16 == Step14:	/*  */
Step17:	/*  */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	cp0	i2c2_ErrLoop
	bra	z, s17_err0
	mov	#I2C_ERROR, w0
	mov	w0, i2c2_cmd
;	clr	i2c2_ErrLoop
	bra	StepClrE
s17_err0:
	inc	i2c2_ErrLoop
	bra	StepClr
/*==================================*/
/* Retry                             */
/*==================================*/
Step18:	/*  */
	inc	i2c2_ErrStartCount
	bra	Step14
Step19:	/*  */
	btsts	i2c2_Interrupt, #0
	bra	nz, NopRet
	clr	i2c2_step
	return
.IF EEPROM_MODE
/*==================================*/
/* ACK Polling                        */
/*==================================*/
Step20:	/*  */
	bset	I2C2CON, #PEN
10:
	btst	IFS3, #MI2C2IF
	bra	z, 10b
11:
	btst	IFS3, #MI2C2IF
	bra	nz, 11b
16:
	bset	I2C2CON, #RSEN
20:
	btst	IFS3, #MI2C2IF
	bra	z, 20b
21:
	btst	IFS3, #MI2C2IF
	bra	nz, 21b
	mov	i2c2_id, w1
	mov	w1, I2C2TRN
22:
	btst	I2C2STAT, #TRSTAT
	bra	nz, 22b
	btst	I2C2STAT, #ACKSTAT
	bra	nz, 16b

	bset	I2C2CON, #PEN
40:
	btst	IFS3, #MI2C2IF
	bra	z, 40b
41:
	btst	IFS3, #MI2C2IF
	bra	nz, 41b

;	clr	i2c2_Interrupt
	setm	i2c2_Interrupt
	mov	#14, w0
	mov	w0, i2c2_step
	return
.ENDIF

.section .const, psv

i2c2_proc:
	.dc	Step0,		Step1,		Step2,		Step3
	.dc	Step4,		Step5,		Step6,		Step7
	.dc	Step8,		Step9,		Step10,		Step11
	.dc	Step12,		Step13,		Step14,		Step15
	.dc	Step14,		Step17,		Step18,		Step19
.IF EEPROM_MODE
	.dc	Step20
.ENDIF

