/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "../main.inc"
.include "../math/libm.inc"

.section .nbss, bss, near

/* ---------- OC1 ---------- */
	USE_T2_OC1	= 0

initOC1_mk:		.space 2
	bOC2_OUT		= 0
.if USE_T2_OC1
	IS_T2_OC1_FQ		= 2
.else
	IS_OC1_FQ		= 2
.endif
	IS_T2_T3_OC1_FQ		= 4
	IS_OC1_OC2_FQ		= 5

CpuFreqStat:		.space 2
CpuPulsTime:		.space 4

oc_time:		.space 4
oc_high:		.space 4

;.global initOC1_mk
.global CpuFreqStat
.global CpuPulsTime

.section .text
.global OC1_PinEnabel
.global OC1_PinDisabel
.global OC1_SetFreq


OC1_PinEnabel:

;	CCLK_ON
	bset	CCLK_LAT, #CCLK_BIT
/* OC1 --> Pin48(RB8/RP40) */
	mov	RPOR3, w0
	mov	#0xFF00, w1
	and	w0, w1, w0

	mov	#0x0010, w1
	btsc	initOC1_mk, #bOC2_OUT
	mov	#0x0011, w1
	ior	w0, w1, w0
	mov	w0, RPOR3
	return

OC1_PinDisabel:
/* Default --> Pin48(RB8/RP40) */
	mov	#0xFF00, w0
	and	RPOR3
;	CCLK_OFF
	bclr	CCLK_LAT, #CCLK_BIT
	return

OC1_SetFreq:
	lnk	#F48_SIZE*3
	cp	w2, #1
	bra	gt, sf_oc_div
	mov	w14, w3
	rcall	u32_f48_w3
	bra	sf_oc_x
sf_oc_div:
	cp	w0, #50
	bra	gt, sf_oc_lt
	bset	GenLongTimeFlag, #bGLT_ON
sf_oc_lt:
	push	w2
	mov	w14, w3
	rcall	u16_f48_w3
	pop	w0
	add	w14, #F48_SIZE, w3
	rcall	u16_f48_w3
	mov	w14, w2
	mov	w14, w4
	rcall	f48_div3216
sf_oc_x:

	mov	#FC48_CPU_FREQ, w2
	mov	w14, w3
	add	w14, #F48_SIZE, w4
	rcall	f48_div

	rcall	f48_u32r

	mov	w0, oc_time
	mov	w1, oc_time+2
	lsr	w1, w1
	rrc	w0, w0
	addc	w0, #0, w0
	addc	w1, #0, w1
	mov	w0, oc_high
	mov	w1, oc_high+2
	ulnk

	bclr	CpuFreqStat, #bCPU_FQ_PULS

	btst	PA_OutFlag, #bPAOF_SQ_P
	bra	z, sf_cpu_sq

	mov	CpuPulsTime, w2
	mov	CpuPulsTime+2, w3
	ior	w2, w3, w4
	bra	z, sf_fq_puls

	cp	w2, w0
	cpb	w3, w1
	bra	ge, sf_cpu_sq

	mov	w2, oc_high
	mov	w3, oc_high+2

	bset	CpuFreqStat, #bCPU_FQ_PULS
	bra	sf_cpu_sq

sf_fq_puls:
;	SQUARE_CLR
	bclr	SQUARE_PORT, #SQUARE_BIT
	bra	sf_fq
sf_cpu_sq:
;	SQUARE_SET
	bset	SQUARE_PORT, #SQUARE_BIT
sf_fq:
	mov	oc_time, w0
	mov	oc_time+2, w1


	cp0	w1
.if USE_T2_OC1
	bra	z, sf_t2_oc1
.else
	bra	z, sf_oc1
.endif
	cp	w1, #1
	bra	z, sf_t2_t3_oc1

	btst	CpuFreqStat, #bCPU_FQ_PULS
	bra	z, sf_oc1_oc2
	mov	oc_high+2, w3
	cp0	w3
	bra	z, sf_t2_t3_oc1

sf_oc1_oc2:
	mov	initOC1_mk, w2
	mov	#IS_OC1_OC2_FQ, w3
	cp	w2, w3
	bra	z, sf_oc1_oc2_iok
	mov	w3, initOC1_mk

	rcall	oc1_clr_all
	bclr	PMD2, #OC1MD
	bclr	PMD2, #OC2MD
	rcall	sf_oc1_oc2_iok

;	mov	#0x1C00, w0
;	mov	w0, OC1CON1
;	mov	w0, OC2CON1

	mov	#0x111F, w0
	mov	w0, OC2CON2
	mov	#0x013F, w0
	mov	w0, OC1CON2

;	bset	OC1CON2, #OCTRIS

;	bset	OC2CON2, #OC32
;	bset	OC1CON2, #OC32

	mov	#0x1C06, w0
	mov	w0, OC2CON1
	mov	w0, OC1CON1
	return

sf_oc1_oc2_iok:
	mov	oc_high, w0
	mov	oc_high+2, w1
	sub	w0, #1, w0
	subb	w1, #0, w1
	mov	w0, OC1R
	mov	w1, OC2R

	mov	oc_time, w0
	mov	oc_time+2, w1
	sub	w0, #1, w0
	subb	w1, #0, w1
	mov	w0, OC1RS
	mov	w1, OC2RS

/*
	lsr	w1, w3
	rrc	w0, w2
	sub	w2, #1, w2
	subb	w3, #0, w3
	mov	w2, OC1R
	mov	w3, OC2R

	sub	w0, #1, w0
	subb	w1, #0, w1
	mov	w0, OC1RS
	mov	w1, OC2RS
*/
	clr	OC1TMR
	clr	OC2TMR
	clr	OC1TMR
	return

oc1_clr_all:
	clr	T2CON
	clr	T3CON
	clr	OC1CON1
	clr	OC1CON2
	clr	OC2CON1
	clr	OC2CON2

	clr	TMR2
	clr	TMR3
	clr	OC1TMR
	clr	OC2TMR

	bset	PMD1, #T2MD
	bset	PMD1, #T3MD
	bset	PMD2, #OC1MD
	bset	PMD2, #OC2MD
	return

.if USE_T2_OC1
sf_t2_oc1:
	mov	initOC1_mk, w2
	mov	#IS_T2_OC1_FQ, w3
	cp	w2, w3
	bra	z, sf_t2_oc1_iok
	mov	w3, initOC1_mk

	rcall	oc1_clr_all
	bclr	PMD1, #T2MD
	bclr	PMD2, #OC1MD
	rcall	sf_t2_oc1_iok

	mov	#0x0001, w0
	mov	w0, OC1R

	mov	#0x8000, w0
	mov	w0, T2CON
	mov	#0x1C0E, w0
	mov	w0, OC1CON1
	mov	#0x008C, w0	/* sync Timer 2 */
	mov	w0, OC1CON2

	return

sf_t2_oc1_iok:
	mov	oc_high, w0
	inc	w0, w0
	mov	w0, OC1RS
	mov	oc_time, w0
	dec	w0, w0
	mov	w0, PR2
	clr	TMR2
	return
.else
sf_oc1:
	mov	initOC1_mk, w2
	mov	#IS_OC1_FQ, w3
	cp	w2, w3
	bra	z, sf_oc1_iok
	mov	w3, initOC1_mk

	rcall	oc1_clr_all
	bclr	PMD2, #OC1MD
	rcall	sf_oc1_iok

	mov	#0x1C06, w0
	mov	w0, OC1CON1
	mov	#0x1001, w0	/* sync OC1 */
	mov	w0, OC1CON2

	return

sf_oc1_iok:
	mov	oc_high, w0
	dec	w0, w0
	mov	w0, OC1R
	mov	oc_time, w0
	dec	w0, w0
	mov	w0, OC1RS
	clr	OC1TMR
	return
.endif

sf_t2_t3_oc1:
	mov	initOC1_mk, w2
	mov	#IS_T2_T3_OC1_FQ, w3
	cp	w2, w3
	bra	z, sf_t2_t3_oc1_iok
	mov	w3, initOC1_mk
	rcall	oc1_clr_all
	bclr	PMD1, #T2MD
	bclr	PMD1, #T3MD
	bclr	PMD2, #OC1MD
	rcall	sf_t2_t3_oc1_iok

;	mov	#0x0001, w0	; min 0x0001
;	mov	w0, PR3
	mov	#0x0001, w0
	mov	w0, OC1R

	mov	#0x8008, w0
	mov	w0, T2CON
	mov	#0x1C0E, w0
	mov	w0, OC1CON1
	mov	#0x008C, w0	/* sync Timer 2 */
	mov	w0, OC1CON2

	return

sf_t2_t3_oc1_iok:
	mov	oc_high, w0
	inc	w0, w0
	btsc	SR, #Z
	mov	#0xffff, w0
	mov	w0, OC1RS

	mov	oc_time, w0
	mov	oc_time+2, w1
	sub	w0, #1, w0
	subb	w1, #0, w1
	bra	nz, sft23o1_ok
	clr	w0
	mov	#1, w1
sft23o1_ok:
	mov	w0, PR2
	mov	w1, PR3
	clr	TMR3HLD
	clr	TMR2

/*
	mov	oc_high, w0
	inc	w0, w0
	btsc	SR, #Z
	mov	#0xffff, w0
	mov	w0, OC1RS

	mov	oc_time, w0
	cp0	w0
	btss	SR, #Z
	dec	w0, w0
	mov	w0, PR2
	clr	TMR2
	clr	TMR3
*/
/*
	lsr	w1, w2
	rrc	w0, w2
	inc	w2, w2
	mov	w2, OC1RS

	cp0	w0
	btss	SR, #Z
	dec	w0, w0
	mov	w0, PR2
	clr	TMR3
*/
	return
