/*
 Copyright (C) 2015-2023 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/eingVx.inc"
.include "lib/Param.inc"
.include "lib/Analog.inc"
.include "math/libm.inc"
.if USE_DEBUG_UART_USER
.include "lib/uart_user.inc"
.endif


.section .nbss, bss, near
/* Timer Var */
BootOk:			.space 2
t1ms_mk:		.space 2
t1ms_10ms_step:		.space 2
t60s_mk:		.space 2

SysError:		.space 2
OV_FlagEg:		.space 2

MathFlags:		.space 2
PA_OutFlag:		.space 2

TestVal0:		.space 2
TestVal1:		.space 2
TestVal2:		.space 2
TestVal3:		.space 2
TestVal4:		.space 2
TestVal5:		.space 2
TestVal6:		.space 2
TestVal7:		.space 2

.if TEST_PLL_NPQR
TestPll_DIV1:		.space 2
TestPll_Stat:		.space 2
TestPll_N:		.space 2
TestPll_R:		.space 2
TestPll_Q:		.space 2
TestPll_P:		.space 2
TestPll_Send:		.space 2
.endif

Temp_OSC:		.space 2	/* 0.1C */
Temp_OP1_0025:		.space 2	/* 0.025C */
Temp_OP1:		.space 2	/* 0.1C */
Temp_PA:		.space 2	/* 0.1C */
Temp_PA_0025:		.space 2	/* 0.025C */
VCC_01V:		.space 2	/* 0.1V */

Temp_PA_pm1:		.space 2	/* +-0.1C */

;PaOut_mV_k:		.space 4	/* i1616  0.1mV */


;TempStatCange:		.space 2
;TempStatCange_param:	.space 2
;	bTSC_OSC		= 0
;	bTSC_PA			= 1

FanOn_T:		.space 2	/* in 10ms */
	FAN_MIN_TIME		= 30

FanPA_PWM_val:		.space 2
FanPA_IRR_X:		.space 2
FanPA_IRR_Y:		.space 2
FanPA_PWM_Voff:		.space 2

;LastGenTemp:		.space 2
SaveT_LastValue:	.space 2

; const
FC48_1000:		.space F48_SIZE	/* 1000.0 */
FC48_10:		.space F48_SIZE	/* 10.0 */
FC48_1:			.space F48_SIZE	/* 1.0 */
FC48_01:		.space F48_SIZE	/* 0.1 */
FC48_001:		.space F48_SIZE	/* 0.01 */
FC48_0001:		.space F48_SIZE	/* 0.001 */
FC48_00001:		.space F48_SIZE	/* 0.0001 */
FC48_000001:		.space F48_SIZE	/* 0.00001 */

FC48_CPU_FREQ:		.space F48_SIZE

; global var not use in subroutines !!!!
F48_0:			.space F48_SIZE
F48_1:			.space F48_SIZE
F48_2:			.space F48_SIZE
F48_3:			.space F48_SIZE
F48_4:			.space F48_SIZE
F48_5:			.space F48_SIZE
F48_6:			.space F48_SIZE
F48_7:			.space F48_SIZE
; global var only use in subroutines
F48_tmp_0:		.space F48_SIZE
F48_tmp_1:		.space F48_SIZE
F48_tmp_2:		.space F48_SIZE
F48_tmp_3:		.space F48_SIZE

.global BootOk
.global KeyWait

.global SysError
.global OV_FlagEg
.global MathFlags
.global PA_OutFlag
;.global TestFlags

.global TestVal0
.global TestVal1
.global TestVal2
.global TestVal3
.global TestVal4
.global TestVal5

.global Temp_OP1
.global Temp_OP1_0025
.global Temp_OSC
.global Temp_PA
.global Temp_PA_0025
.global VCC_01V
.global FanPA_PWM_Voff

.global PaOut_mV_k

.global FC48_1000
.global FC48_10
.global FC48_1
.global FC48_01
.global FC48_001
.global FC48_0001
.global FC48_00001
.global FC48_000001

.global FC48_CPU_FREQ

.global F48_0
.global F48_1
.global F48_2
.global F48_3
.global F48_4
.global F48_5
.global F48_6
.global F48_7

.global F48_tmp_0
.global F48_tmp_1
.global F48_tmp_2
.global F48_tmp_3

.section .text
.global main
.global TastUp
.global TastDown
.global ModSquareSet
.global ParamRead
.global LastFlag_Save
.global SaveMainLastValue32_8
.global SaveMainLastValue16_8
.global SetCpuFreq
;.global SetPaTemp_param
.global DoGenTemp

/*
.if USE_DEBUG_UART_USER
.global uuRecalcTemp
.endif
*/

main:
	bset	LATE, #13		/* LCD  + MCP23S18 Res H */
;	LED_RT_ON
	bclr	LED_RT_PORT, #LED_RT_BIT

	rcall	InitTimer1
	rcall	InitADC1
	rcall	InitSPI_LCD
	rcall	LCD_128x64_INIT
	rcall	MCP23S18_INIT		/* keyboard + LED */

	rcall	InitI2C2
	rcall	FM24CL048_INIT
	rcall	MCP3421_INIT_18B
	rcall	ADS112c04_INIT
	rcall	DAC7578_INIT
	rcall	MCP47FEB_INIT
	rcall	AD5667_INIT
	rcall	MCP4726_INIT
.if USE_PLL_CDCE913
	rcall	CDCE913_INIT
.endif

;	rcall	InitOC1
;	rcall	Init_OC2_PWM
	rcall	Init_OC3_PWM
;	rcall	Init_OC4_PWM

.if USE_DEBUG_UART
	rcall	InitUART_dbg
	rcall	rs232ald_Init
.if USE_DEBUG_UART_USER
	rcall	rs232ald_user_init
.endif
.endif


/* Wait 70ms VREFH */
10:
	mov	t_ms, w0
	cp	w0, #70
	bra	lt, 10b


	mov	#2, w0
	mov	w0, LCD_PProc

	rcall	LCD_CLR

	mov	t_ms, w0
	mov	w0, t1ms_mk

	mov	#250, w0
	mov	w0, tLCD_mk

	rcall	LCD_Refresh

;	LED_GN_ON

boot_loop:
.IF USE_DEBUG_UART
	rcall	rs232ald_RxD
.ENDIF
	cp0	MCP23S18_ms_count
	btsc	SR, #Z
	rcall	MCP23S18_Handle		/* keyboard + LED */

;	rcall	i2c_x_dev_handle

	rcall	LCD_Proc

	mov	t_ms, w0
	cp	t1ms_mk
	bra	eq, boot_Idle
	mov	w0, t1ms_mk

/* set ms time */
	cp0	MCP23S18_ms_count
	btss	SR, #Z
	dec	MCP23S18_ms_count

	dec	tLCD_mk
	bra	nz, boot_Idle
	mov	#250, w0
	mov	w0, tLCD_mk
	inc	LCD_Count

	mov	#1, w0
	mov	w0, LCD_Step
	bra	boot_end
boot_Idle:
	pwrsav	#IDLE_MODE
        bra     boot_loop

boot_end:
;	LED_GN_ON
	bclr	LED_GN_PORT, #LED_GN_BIT
	bset	OP1_ON_PORT, #OP1_ON_BIT


SetUp_loop:
.IF USE_DEBUG_UART
	rcall	rs232ald_RxD
.ENDIF
	cp0	MCP23S18_ms_count
	btsc	SR, #Z
	rcall	MCP23S18_Handle		/* keyboard + LED */

	rcall	i2c_x_dev_handle

	rcall	LCD_Proc

	mov	t_ms, w0
	cp	t1ms_mk
	bra	eq, SetUp_Idle
	mov	w0, t1ms_mk
/* set ms time */
	cp0	MCP23S18_ms_count
	btss	SR, #Z
	dec	MCP23S18_ms_count

	cp0	MCP3421_ms_count
	btss	SR, #Z
	dec	MCP3421_ms_count

	cp0	ADS112c04_ms_count
	btss	SR, #Z
	dec	ADS112c04_ms_count

	dec	tLCD_mk
	bra	nz, SetUp_Idle
	mov	#250, w0
	mov	w0, tLCD_mk
	inc	LCD_Count

	mov	#1, w0
	mov	w0, LCD_Step

	btst	LCD_Count, #2
	bra	z, SetUp_Idle
	
	mov	RDFlag, w0	/* Test Param read end */
	inc	w0, w0
	bra	z, SetUp_end
SetUp_Idle:
	pwrsav	#IDLE_MODE
        bra     SetUp_loop

SetUp_end:

	bset	BootOk, #bBootOk_SetUp
	clr	LCD_PProc


	bset	BootOk, #bBootOk_Init
;	LED_RT_OFF
	bset	LED_RT_PORT, #LED_RT_BIT



	mov	#0, w0
	mov	#AGRN0, w2
	rcall	DacSetValue

	mov	#0, w0
	mov	#AGRN1, w2
	rcall	DacSetValue

	mov	#0, w0
	mov	#AGRP0, w2
	rcall	DacSetValue

	mov	#0, w0
	mov	#AGRC, w2
	rcall	DacSetValue

	mov	#0, w0
	mov	#AGLIN0, w2
	rcall	DacSetValue

	mov	#0, w0
	mov	#AGLIN1, w2
	rcall	DacSetValue

	mov	#0, w0
	mov	#ALINPA, w2
	rcall	DacSetValue

	mov	#0, w0
	mov	#AFR0, w2
	rcall	DacSetValue

	mov	#0, w0
	mov	#AFR1, w2
	rcall	DacSetValue

	mov	#0, w0
	mov	#AOP1LC, w2
	rcall	DacSetValue

	mov	#0, w0
	mov	#OSYM, w2
	rcall	DacSetValue

	mov	#0x0000, w0
	mov	w0, FC48_1000
	mov	#0xfa00, w0
	mov	w0, FC48_1000+2
	mov	#0x4008, w0
	mov	w0, FC48_1000+4	; 1000.0

	mov	#0x0000, w0
	mov	w0, FC48_10
	mov	#0xa000, w0
	mov	w0, FC48_10+2
	mov	#0x4002, w0
	mov	w0, FC48_10+4	; 10.0

	mov	#0x0000, w0
	mov	w0, FC48_1
	mov	#0x8000, w0
	mov	w0, FC48_1+2
	mov	#0x3fff, w0
	mov	w0, FC48_1+4	; 1.0

	mov	#0xcccd, w0
	mov	w0, FC48_01
	mov	#0xcccc, w0
	mov	w0, FC48_01+2
	mov	#0x3ffb, w0
	mov	w0, FC48_01+4		; 0.1

	mov	#0x0a3d, w0
	mov	w0, FC48_001
	mov	#0xa3d7, w0
	mov	w0, FC48_001+2
	mov	#0x3ff8, w0
	mov	w0, FC48_001+4		; 0.01

	mov	#0x6e98, w0
	mov	w0, FC48_0001
	mov	#0x8312, w0
	mov	w0, FC48_0001+2
	mov	#0x3ff5, w0
	mov	w0, FC48_0001+4		; 0.001

	mov	#0x1759, w0
	mov	w0, FC48_00001
	mov	#0xd1b7, w0
	mov	w0, FC48_00001+2
	mov	#0x3ff1, w0
	mov	w0, FC48_00001+4	; 0.0001

	mov	#0xac47, w0
	mov	w0, FC48_000001
	mov	#0xa7c5, w0
	mov	w0, FC48_000001+2
	mov	#0x3fee, w0
	mov	w0, FC48_000001+4	; 0.00001

	mov	#FCY & 0xffff, w0
	mov	#FCY>>16, w1
	mov	#FC48_CPU_FREQ, w3
	rcall	u32_f48_w3

	mov.b	EEData0+oFR_LAST_FLAG, WREG
	btsc	w0, #bFR_LF_PAOF_SQ_P
	bset	PA_OutFlag, #bPAOF_SQ_P
	btsc	w0, #bFR_LF_PAOF_DC
	bset	PA_OutFlag, #bPAOF_DC
	btsc	w0, #bFR_LF_K60CM
	bset	K60CM_PORT, #K60CM_BIT

	rcall	SetUpImpedanz
	rcall	Init_Eing_main

	mov	EingCp_RiseFall+oEVX_VALUE, w0
	and	w0, #0x07, w0
	btss	SR, #Z
	bset	KFILT_PORT, #KFILT_BIT

	mov	#EEData0+oFR_AD5667_0_LK, w7
	rcall	DacLinKorr_ChkCrc
	mov	#EEData0+oFR_AD5667_1_LK, w7
	rcall	DacLinKorr_ChkCrc

; K2_ON
; GV_ON
; bset GV_PORT, #GV_BIT
; SQUARE_SET
; bset SQUARE_PORT, #SQUARE_BIT
; rcall	OC1_PinEnabel
.if 0
	bclr	PMD4, #3

	mov	RPOR0, w1
	mov	#0x00ff, w0
	and	w1, w0, w1
	mov	#0x3100, w0	/* REFCLKO */
	ior	w1, w0, w1
	mov	w1, RPOR0

	mov	#0x0023, w0	/* T2CKR */
	mov	w0, RPINR3

	bset	T2CON, #1

	mov	#0x1100, w0
	mov	w0, REFOCON
	bset	REFOCON, #15
.endif
;	OP_OFFS_ON
	bset	OP_OFFS_PORT, #OP_OFFS_BIT

;	rcall	SetParamGain
	rcall	SetUpGain
	rcall	SetUpOffs
	rcall	SetGenSym
	rcall	SetCpuFreq
;	rcall	FreqSetUp
	rcall	SetUpFreq
	rcall	SetRelAmpl
	rcall	SetUpFilter
	rcall	SetPuls

	rcall	ADS112c04_SetParam
	rcall	MCP3421_SetParam

	mov	#240, w0
	mov	w0, t60s_mk

	mov	#10, w0			; Start mit t0ms
	mov	w0, t1ms_10ms_step

main_loop:
; TEST_PIN0_TGL
.IF USE_DEBUG_UART
	rcall	rs232ald_RxD
.ENDIF
	btst	MathFlags, #bMF_FREQ
	bra	nz, mlm_Freq
	btst	MathFlags, #bMF_AMPL
	bra	nz, mlm_Ampl
	btst	MathFlags, #bMF_OFFSET
	bra	nz, mlm_Offset
	btst	MathFlags, #bMF_FILTER
	bra	nz, mlm_Filter

mlm_end:


	cp0	MCP23S18_ms_count
	btsc	SR, #Z
	rcall	MCP23S18_Handle		/* keyboard + LED */

	rcall	calc_CDCE913_Handle
	bset	t_proc_mk, #0

	rcall	i2c_x_dev_handle

	rcall	LCD_Proc

	mov	t_ms, w0
	cp	t1ms_mk
	bra	eq, GoIdle
	mov	w0, t1ms_mk

/* 1ms set ms time */
	cp0	MCP23S18_ms_count
	btss	SR, #Z
	dec	MCP23S18_ms_count

	cp0	MCP3421_ms_count
	btss	SR, #Z
	dec	MCP3421_ms_count

	cp0	ADS112c04_ms_count
	btss	SR, #Z
	dec	ADS112c04_ms_count

	rcall	GenLongTime

	btsc	TestFlags, #TF_DC_FREQ_ON
	rcall	SetOffsDcFq

.if USE_DEBUG_UART_USER
	cp0	rs232ald_wait
	btss	SR, #Z
	rcall	rs232ald_user_Handle
.endif

/* 10ms Timer */
	mov	t1ms_10ms_step, w0
	inc	w0, w0
	mov	#10, w1
	cpslt	w0, w1
	clr	w0
	mov	w0, t1ms_10ms_step

	add	w0, w0, w0
	mov	#t10ms_proc, w1
	mov	[w0+w1], w0
	call	w0


;thLCD:
/* LCD */
	dec	tLCD_mk
	bra	nz, GoIdle
	mov	#250, w0
	mov	w0, tLCD_mk
	inc	LCD_Count

	mov	#1, w0
	mov	w0, LCD_Step

	dec	t60s_mk
	bra	nz, GoIdle
	mov	#240, w0		; 4 x 60  ( 250ms )
	mov	w0, t60s_mk
	mov	EEData0+oFR_RUN_TIME, w0
	mov	EEData0+oFR_RUN_TIME+2, w1
	add	w0, #1, w0
	addc	w1, #0, w1
	mov	w0, EEData0+oFR_RUN_TIME
	mov	w1, EEData0+oFR_RUN_TIME+2
	bclr	WRFlag, #bFR_RUN_TIME
	bclr	ExMemFlag, #DataOk

;	bra	GoIdle
GoIdle:
.if 1
;	btsts	t_proc_mk, #0
 bra	main_loop
.else
;	btss	i2c2_Interrupt, #0
;	call	Handle_I2C2
	btsts	t_proc_mk, #0
	bra	z, main_loop

; TEST_PIN0_SET
	pwrsav	#IDLE_MODE
; TEST_PIN0_CLR


	btsts	t_proc_mk, #0
	bra	z, main_loop
        bra     GoIdle
.endif

mlm_Freq:
	bclr	MathFlags, #bMF_FREQ


;	bset	MathFlags, #bMF_FILTER

	rcall	MathFreq
	bra	mlm_end

mlm_Ampl:
	bclr	MathFlags, #bMF_AMPL

	rcall	MathAmpl
	rcall	SetAmplDacAndRelais
	rcall	SetRelAmpl
	bset	MathFlags, #bMF_OFFSET
	bset	MathFlags, #bMF_FILTER
	bra	mlm_end

mlm_Offset:
	bclr	MathFlags, #bMF_OFFSET
	rcall	MathPaOffs
	rcall	SetDacPaDC_Offset
	rcall	SetDacPaPow_Offset
	rcall	SetDacPaFine_Offset
	bra	mlm_end

mlm_Filter:
	bclr	MathFlags, #bMF_FILTER
	rcall	SetUpFilter
	bra	mlm_end

/* --------------- Key Up/Down --------------- */
TastUp:
;	clr	KeyWait
	return

TastDown:
	rcall	Key_proc
	bra	z, TastDown_Alg_Ret
	cp0	BootOk
	bra	z, TastDown_Alg_Ret


	mov	TastNr, w0
/* -------------- Key Inv ---------------------- */
	cp	w0, #KEY_Inv
	bra	nz, td_Square

	btst	TestFlags, #TF_DC_FREQ_ON
	bra	nz, TastDown_Alg_Ret

	btg	INV_PORT, #INV_BIT
	bra	TastDown_Alg_Ret

/* -------------- Key Square ---------------------- */
td_Square:
	cp	w0, #KEY_Square
	bra	nz, td_Offset

	btst	TestFlags, #TF_DC_FREQ_ON
	bra	nz, TastDown_Alg_Ret

;	btg	PA_OutFlag, #bPAOF_SQ_P

	mov	PA_OutFlag, w0
	and	#PAOF_SOPDC_MASK, w0
	inc	w0, w0
	cp	w0, #3
	btsc	SR, #Z
	clr	w0
	rcall	ModSquareSet
	bra	TastDown_Alg_Save

/* -------------- Key Offset ---------------------- */
td_Offset:
	cp	w0, #KEY_Offset
	bra	nz, TastDown_Alg_Ret
/*
	btst	TestVal5, #0
	bra	z, td_xx0
	bclr	TestVal5, #0
	rcall	SetGenV_Max
	bra	TastDown_Alg_Ret

td_xx0:
	bset	TestVal5, #0
	rcall	SetGenV_Min
*/
	bra	TastDown_Alg_Ret

TastDown_Alg_Save:
	rcall	LastFlag_Save

TastDown_Alg_Ret:
	return

ModSquareSet:
	mov	#PAOF_SOPDC_MASK, w1
	and	w0, w1, w0
	cp	w0, #3
	btsc	SR, #Z
	mov	#2, w0

	mov	PA_OutFlag, w2
	com	w1, w1
	and	w2, w1, w2
	ior	w0, w2, w0
	mov	w0, PA_OutFlag

	mov	#2, w0
	btst	PA_OutFlag, #bPAOF_SQ_P
	rcall	DPK02_Led_Z_NZ

	rcall	SetUpFreq
	bset	MathFlags, #bMF_AMPL
	return


/* ----------------------- ParamRead ------------------------------------ */
ParamRead:
	mov	#FM24CL048SIG, w0	/* OK */
	cp	FR_DataSystem
	bra	nz, FM24CL048_Format

	rcall	LCD_SetContrast
	return

LastFlag_Save:
	mov	PA_OutFlag, w1
	clr	w0
	btsc	w1, #bPAOF_SQ_P
	bset	w0, #bFR_LF_PAOF_SQ_P
	btsc	w1, #bPAOF_DC
	bset	w0, #bFR_LF_PAOF_DC

	btsc	K60CM_PORT, #K60CM_BIT
	bset	w0, #bFR_LF_K60CM

	mov	EingCp_RiseFall+oEVX_VALUE, w1
	and	w1, #0x07, w1
	sl	w1, #bFR_LF_NS, w1
	ior	w0, w1, w0

	mov.b	WREG, EEData0+oFR_LAST_FLAG
	bra	smlv_t

/* w0:w1=Value w2=x.byte w3=Fram.pointer */
SaveMainLastValue32_8:
	mov.b	w0, [w3++]
	swap	w0
	mov.b	w0, [w3++]
	mov.b	w1, [w3++]
	swap	w1
	mov.b	w1, [w3++]
	bra	smlv_w2

/* w0=Value w2=x.byte w3=Fram.pointer */
SaveMainLastValue16_8:
	mov.b	w0, [w3++]
	swap	w0
	mov.b	w0, [w3++]
smlv_w2:
	mov.b	w2, [w3++]
smlv_t:
	mov	#20, w0		/* 200ms */
	mov	w0, SaveT_LastValue
	return

/* ----------------------- System --------------------------------------- */
SetCpuFreq:
	btsc	TestFlags, #bTF_DAC
	return

	mov	Temp_OSC, w0		/* 0.1C */
	mov	#300, w1
	sub	w0, w1, w0		; Temp_OSC - 30.0C
	mov	EEData0+oFR_OSCT_PC, w1
	mul.ss	w0, w1, w0

	mov	#100, w2
	repeat	#17		/* div 32/16->16 */
	div.sd	w0, w2

	mov	EEData0+oFR_OSCT, w1
	add	w0, w1, w0
	mov	#AOSCT, w2
	bra	DacSetValue_R_Align

;SetPaTemp_param:
;	bset	TempStatCange_param, #bTSC_PA
;	return

DoGenTemp:
	btss	BootOk, #bBootOk_SetUp
	return

	btsc	AutoCalDAC, #bAC_ADS112_TEMP
	rcall	AutoCal
	cp0	AutoCalMod
	btss	SR, #Z
	return

	rcall	SetFilterTemp
	bra	AmplTempKorr

/* ------------------------ 10ms Timer Proc ----------------------------- */
t10ms_0:
;	clr	TempStatCange
;	mov	TempStatCange_param, w0
;	mov	w0, TempStatCange
;	clr	TempStatCange_param

/*
	3300mV / 0x10000 = 0.050354003906 mV/dig
	400  mV 0.050354003906 mV/dig = 7943.75757575758
	19.5 mV 0.050354003906 mV/dig = 387.258181820104 Dig/C
	400 / 19.5 = offset 20.5128205128205C

	0x10000 / 38.73 = 1692.30769229929 --> 0.1C
*/
/* temp OP1 */
	mov	Anl_Tmp236_0, w0
	mov	#6768, w1
	mul.uu	w0, w1, w0
	mov	#820, w0		/* offs 20.5C */
	sub	w1, w0, w1
	mov.b	EEData0+oFR_OT_OP1, WREG
	se	w0, w0
	sl	w0, #2, w0
	add	w1, w0, w0
	mov	w0, Temp_OP1_0025	/* 0.025C */

	asr	w0, w0
	asr	w0, w0
	addc	w0, #0, w0
	mov	w0, Temp_OP1		/* 0.1C */

/* temp osc */
	mov	Anl_Tmp236_2, w0
	mov	#1692, w1
	mul.uu	w0, w1, w0
	mov	#205, w0		/* offs 20.5C */
	sub	w1, w0, w1
	mov.b	EEData0+oFR_OT_OSC, WREG
	se	w0, w0
	add	w1, w0, w0
;	cp	Temp_OSC
;	btss	SR, #Z
;	bset	TempStatCange, #bTSC_OSC
	mov	w0, Temp_OSC		/* 0.1C */

/* temp PA */
	mov	Anl_Tmp236_1, w0
	mov	#6768, w1
	mul.uu	w0, w1, w0
	mov	#820, w0		/* offs 20.5C *4 */
	sub	w1, w0, w1
	mov.b	EEData0+oFR_OT_PA, WREG
	se	w0, w0
	sl	w0, #2, w0
	add	w1, w0, w0
	mov	w0, Temp_PA_0025	/* 0.025C */

	asr	w0, w0
	asr	w0, w0
	addc	w0, #0, w0
	mov	w0, Temp_PA		/* 0.1C */

	rcall	SetCpuFreq
	return

t10ms_1:
	mov	Anl_InDC, w0
	mov	#366, w1
	mul.uu	w0, w1, w0
	mov	w1, VCC_01V

	btsc	TestFlags, #bTF_POW
	return

	mov	#140, w0
	cp	w1, w0
	bra	gtu, tm1_pa_on
	mov	#130, w0
	cp	w1, w0
	bra	ltu, tm1_pa_off
	return

tm1_pa_off:
	clr	FR_WR_En
	bclr	PA_POW_PORT, #PA_POW_BIT
	return

tm1_pa_on:
	setm	FR_WR_En
	bset	PA_POW_PORT, #PA_POW_BIT
	return

t10ms_2:
	btsc	TestFlags, #bTF_DAC
	return

;--------------- Offset OP1 ----------------------------------
.if USE_DEBUG_UART_USER
	btst	uuCAL_DAC_Mod, #bOP1_OFFSET_KORR_OFF
	bra	nz, TcOP1_offs_off
.endif
	mov	Temp_OP1_0025, w0	/* 0.1C / 4 */
	mov	#1200, w1
	sub	w0, w1, w0		; Temp - 30.0C
	mov	EEData0+oFR_TC_OP1, w1
	mul.ss	w0, w1, w0
	mov	#250, w4
	mul.su	w1, w4, w2
	mul.uu	w0, w4, w0
	add	w1, w2, w1

	mov	#23600, w2	/* 1000 digit = 23.60mV */
	repeat	#17		/* div 32/16->32 */
	div.sd	w0, w2
	mov	EEData0+oFR_OFF_OP1, w2
	rcall	AddSat_s12b
	mov	#AOP1OFF, w2
	rcall	DacSetValue_R_Align
.if USE_DEBUG_UART_USER
TcOP1_offs_off:
.endif
;--------------- Offset ADC PA --------------------------------
	rcall	MCP3421_TempKorr

.if 0
;--------------- Offset PA in ----------------------------------
	mov	Temp_PA, w0		/* 0.1C */

	mov	#300, w1
	sub	w0, w1, w0		; Temp - 30.0C
	mov	EEData0+oFR_TC_PA_IN, w1
	mul.ss	w0, w1, w0
	mov	#1000, w4
	mul.su	w1, w4, w2
	mul.uu	w0, w4, w0
	add	w1, w2, w1

	mov	#7215, w2	/* 10000 digit = 72.15mV */
	repeat	#17		/* div 32/16->16 */
	div.sd	w0, w2

	mov	EEData0+oFR_OFF_PA_IN, w2
	rcall	AddSat_s12b
	mov	w0, PAin_0_Offs
	rcall	SetDacPa_Offset
.endif
;--------------- Offset PA out ----------------------------------
.if USE_DEBUG_UART_USER
	btst	uuCAL_DAC_Mod, #bPA_OFFSET_KORR_OFF
	bra	nz, TcPA_offs_off
.endif
	rcall	SetPaOffsTemp
.if USE_DEBUG_UART_USER
TcPA_offs_off:
.endif

;--------------- Offset OPT out ----------------------------------
	mov	Temp_PA_0025, w0		/* 0.025C */
	mov	#1200, w1		; 30.0C * 4
	sub	w0, w1, w0		; Temp - 30.0C
	mov	EEData0+oFR_TC_OPT, w1
	mul.ss	w0, w1, w0
	mov	#250, w4
	mul.su	w1, w4, w2
	mul.uu	w0, w4, w0
	add	w1, w2, w1

	mov	#9140, w2	/* 1000 digit = 9.14mV */
	repeat	#17		/* div 32/16->32 */
	div.sd	w0, w2

	mov	EEData0+oFR_OFF_OPT, w2
	rcall	AddSat_s12b

;	mov	EEData0+oFR_OFF_OPT, w0
	mov	#AOPTOFF, w2
	rcall	DacSetValue_R_Align
	return

/* ----------------------- t10ms 3 -------------------------------------- */
t10ms_3:
	btsc	TestFlags, #bTF_FAN
	return

	btst	TestFlags, #TF_FAN1_ON
	bra	z, tm3_nfant
	mov.b	EEData0+oFR_FAN_ON_V, WREG
	ze	w0, w0
	mov	#FAN_PWM, w2
	bra	DacSetValue_R_Align
tm3_nfant:
;--------------- PAN PA ----------------------------------

	mov.b	EEData0+oFR_FAN_ON_C, WREG
	se	w0, w0
	mov	#FAN_TEMP_ON_OFFS, w1
	add	w0, w1, w0
	sl	w0, #2, w0
	mov	Temp_PA_0025, w1		/* 0.0025C */
	sub	w1, w0, w2			/* w2 temp offs */

	btst	FAN1_PORT, #FAN1_BIT		; Test Fan1 on
	bra	z, tm3_fan1_is_off

	mov	#FAN_MIN_TIME, w0
	cp	FanOn_T
	bra	lt, tm3_inc_fot			; Test min time set

	btst	w2, #15				; Test Temp < 40C
	bra	nz, tm3_fan1_off

	mov.b	EEData0+oFR_FAN_ON_V, WREG
	ze	w0, w0				
	add	w2, w0, w0
tm3_fan_PWM_set:
	inc	w0, w0
	mov	#0xff00, w1
	and	w0, w1, w1
	btss	SR, #Z
	mov	#0x00FF, w0
	mov	w0, FanPA_PWM_val
	mov	#FAN_PWM, w2
	bra	DacSetValue_R_Align

tm3_inc_fot:
	inc	FanOn_T
	return
tm3_fan1_off:
	bclr	FAN1_PORT, #FAN1_BIT
	return

tm3_dec_fot:
	dec	FanOn_T
	return
tm3_fan1_on:
	bset	FAN1_PORT, #FAN1_BIT
	bclr	HT1_PORT, #HT1_BIT
	bclr	HT2_PORT, #HT2_BIT

	mov.b	EEData0+oFR_FAN_ON_V, WREG
	ze	w0, w0
	bra	tm3_fan_PWM_set

tm3_fan1_is_off:
	cp0	FanOn_T
	bra	nz, tm3_dec_fot

	btst	w2, #15				; Test Temp >= 40C
	bra	z, tm3_fan1_on


	mov.b	EEData0+oFR_HT_ON_C, WREG
	se	w0, w0
	mov	#HT_TEMP_ON_OFFS, w1
	add	w0, w1, w0
	mov	Temp_GEN_0025, w1		; 0.025C -> 0.1C
	asr	w1, #2, w1
	sub	w1, w0, w2			; w2 temp offs
	neg	w2, w2

	btst	w2, #15				; Test Gen Temp >= 30C
	bra	nz, tm3_fan1_PWM_off

.if 1
	mov	Temp_PA, w1			; 0.1C
	sub	w1, w0, w1			; w2 temp offs

	btsc	w1, #15				; Test PA Temp > 30C
	bset	HT2_PORT, #HT2_BIT
	btss	w1, #15				; Test PA Temp < 30C
	bclr	HT2_PORT, #HT2_BIT

	bset	HT1_PORT, #HT1_BIT
	sl	w2, #2, w0
	bra	tm3_fan_PWM_set

.else

	bset	HT1_PORT, #HT1_BIT
	sl	w2, #2, w0
	rcall	tm3_fan_PWM_set

	mov.b	EEData0+oFR_FAN_ON_C, WREG
	se	w0, w0
	mov	#FAN_TEMP_ON_OFFS, w1
	add	w0, w1, w0
	mov	#50, w1
	sub	w0, w1, w0			; PA Temp - 5.0C

	mov	Temp_PA, w1			; 0.1C
	sub	w1, w0, w2			; w2 temp offs
	btsc	w2, #15				; Test PA Temp >= 35C
	bset	HT2_PORT, #HT2_BIT
	btss	w2, #15				; Test PA Temp < 35C
	bclr	HT2_PORT, #HT2_BIT

	return
.endif

tm3_fan1_PWM_off:
	bclr	HT1_PORT, #HT1_BIT
	bclr	HT2_PORT, #HT2_BIT
	clr	w0
	mov	#FAN_PWM, w2
	bra	DacSetValue_R_Align

/* ----------------------- t10ms 4 -------------------------------------- */
t10ms_4:
;--------------- PAN PA Offset korr --------------
	btsc	TestFlags, #bTF_DAC
	return
	
	mov	FanPA_PWM_val, w0
	btss	FAN1_PORT, #FAN1_BIT
	clr	w0
	mov	#1500, w1		; 6 digit ca 0,8mV
	mul.uu	w0, w1, w0
	mov	#2, w0			; 2 digit ca 0,4mV min Offset
	btss	FAN1_PORT, #FAN1_BIT
	clr	w0
	add	w0, w1, w0

	mov	#FanPA_IRR_X, w2	; irr08u8 ca. 8sec
	mov	#0xFE00, w1
	add	w0, [w2], w3
	mov	w0, [w2]
	mul.uu	w1, [++w2], w0
	add	w1, w3, [w2]
	mov	FanPA_IRR_Y, w0
	lsr	w0, #7, w0
	lsr	w0, w0
	addc	w0, #0, w0

;	cp	FanPA_PWM_Voff
;	btss	SR, #Z
;	bset	TempStatCange_param, #bTSC_PA
	mov	w0, FanPA_PWM_Voff

	return
/* ----------------------- t10ms 5 -------------------------------------- */
t10ms_5:
	btsc	TestFlags, #bTF_DAC
	return
	rcall	SetOffsTemp
	return
t10ms_6:
	return
t10ms_7:
	return

/* ----------------------- t10ms 8 -------------------------------------- */
t10ms_8:
	clr	w0
	btsc	HT1_PORT, #HT1_BIT
	bset	w0, #0
	btsc	HT2_PORT, #HT2_BIT
	bset	w0, #0

	btss	PWM_FAN_PORT, #PWM_FAN_BIT
	bclr	w0, #0

	btsc	w0, #0
	bclr	LED_RT_PORT, #LED_RT_BIT
	btss	w0, #0
	bset	LED_RT_PORT, #LED_RT_BIT

	mov	#1, w0
	btst	INV_PORT, #INV_BIT
	rcall	DPK02_Led_Z_NZ
	return

t10ms_9:
	cp0	SaveT_LastValue
	btsc	SR, #Z
	return
	dec	SaveT_LastValue
	btss	SR, #Z
	return
	bclr	WRFlag, #bFR_LastValue
	bclr	ExMemFlag, #DataOk
	return

.section .const, psv
.global FC48_0000001
.global FC48_0025
.global FC48_100

.align 2

FC48_0000001:
	.word	0xbd06, 0x8637, 0x3feb		; 0.000001
FC48_0025:
	.word	0xcccd, 0xcccc, 0x3ff9		; 0.025
FC48_100:
	.word	0x0000, 0xc800, 0x4005		; 100.0

t10ms_proc:
	.dc	t10ms_0,	t10ms_1,	t10ms_2,	t10ms_3
	.dc	t10ms_4,	t10ms_5,	t10ms_6,	t10ms_7
	.dc	t10ms_8,	t10ms_9
