/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/eingVx.inc"
.include "lib/Analog.inc"
.include "lib/Param.inc"
.include "math/libm.inc"

	fPA_ADC_SUM		= F48_0
	fOP1_ADC_SUM		= F48_1
	pgm_Step		= AcTmpMem

/* ______________________________________________________________________ */
.section .text

.global InitAutoCalGain
.global LCD_AutoCalGain
.global K_Ret_AutoCalGain
.global AutoCalGain

InitAutoCalGain:

	bset	AutoCalDAC, #bAC_MCP3421
;	bset	TestFlags, #bTF_DAC

	rcall	pll_off
	rcall	SetGenPllSym

	rcall	OC1_PinDisabel
	bset	GenLongTimeFlag, #bGLT_ON

	bset	GV_PORT, #GV_BIT
	bset	SQUARE_PORT, #SQUARE_BIT

	mov	#0x0001, w0
	rcall	SetAmplRelais

	mov	#0x0800, w0
	mov	w0, DacGvp
	mov	w0, DacGvn
	rcall	SetAmplToDac
	bclr	INV_PORT, #INV_BIT

	mov	#AcF48_2, w2
	rcall	f48_1_w2

	mov	#6, w0
	mov	w0, ac_start_count

	return

/* ---------------------------------------------------------------------- */
LCD_AutoCalGain:
	mov	#Str_buffer, w3
	dec	w0, w0
	bra	z, Line1

	mov	#AcDispVals, w6
	dec	w0, w0
	bra	z, Line2
	dec	w0, w0
	bra	z, Line3
	dec	w0, w0
	bra	z, Line4
	dec	w0, w0
	bra	z, Line6
	bra	LCD_Paint

Line1:
	mov	AdcPa_1616, w0
	mov	AdcPa, w1
	
	mov	#1, w2
	mov	#4, w4
	rcall	i1616Dx6toppc
	mov	#0x81, w0
	mov.b	w0, [w3++]
	mov	#'m', w0
	mov.b	w0, [w3++]
	mov	#'V', w0
	mov.b	w0, [w3++]
	clr.b	[w3]
	mov	#72, w0
	mov	#10, w1
	bra	LCD_StrBufOut_cs
Line2:
	mov	#20, w2
	bra	LineV_8_1616

Line3:
	add	#8, w6
	mov	#29, w2
	bra	LineV_8_1616

Line4:
	add	#16, w6
	mov	#38, w2
	bra	LineV_8_1616

Line6:
	mov	#0, w0
	mov	#58, w1
	bra	StrbTemp_cs_step


LineV_8_1616:
	push	w2
	mov	[w6++], w0
	mov	[w6++], w1
	mov	#5, w2
	mov	#5, w4
	rcall	u1616Dx6toppc
	mov	#0x20, w0
	mov.b	w0, [w3++]
	mov	[w6++], w0
	mov	[w6], w1
	mov	#5, w2
	mov	#5, w4
	rcall	u1616Dx6toppc
	mov	#8, w0
	pop	w1
	bra	LCD_StrBufOut_cs

/* ---------------------------------------------------------------------- */
set_gain_mem:
	mov.d	[w4++], w0
	sub	w1, w5, w2
	mul.uu	w0, #10, w0
	mul.su	w2, #10, w2
	sl	w0, w0
	addc	w1, w2, [w6++]
	return

K_Ret_AutoCalGain:

	btst	ac_save_mod, #bACS_SAVE_DATA
	bra	z, Key_AutoCalClose

	mov	#AcDispVals+4, w4
	mov	#K02DB_GAIN_BASE, w5
	mov	#EEData0+oFR_GAIN_02DB, w6
	rcall	set_gain_mem

	mov	#K04DB_GAIN_BASE, w5
	rcall	set_gain_mem

	mov	#K08DB_GAIN_BASE, w5
	rcall	set_gain_mem

	mov	#K16DB_GAIN_BASE, w5
	rcall	set_gain_mem

	mov	#K32DB_GAIN_BASE, w5
	rcall	set_gain_mem
	
	mov	#K02DB_GAIN_BASE, w2
	mov	#9, w3
	rcall	FRam_Save
	bra	Key_AutoCalClose
/*
kr_e:
	bclr	TestFlags, #bTF_DAC
	mov	last_PA_OutFlag, w0
	mov	w0, PA_OutFlag
	rcall	SetUpGain
	rcall	SetUpFreq
	bra	Key_RetOk_Menu
*/
/* ---------------------------------------------------------------------- */
math_gain:
	push	w4
	lnk	#F48_SIZE*2

	mov	#AcF48_0, w2
	mov	#fPA_ADC_SUM, w3
	mov	w14, w4
	rcall	f48_sub

	mov	#AcF48_1, w2
	mov	#fOP1_ADC_SUM, w3
	add	w14, #F48_SIZE, w4
	rcall	f48_sub

	mov	#AcF48_2, w2
	mov	w4, w3
	rcall	f48_mul

	mov	w14, w2
	mov	#AcF48_0, w4
	rcall	f48_div

	mov	w4, w2
	mov	#FC48_DISP_MUL, w3
	mov	w14, w4
	rcall	f48_mul

	rcall	f48_u1616

	ulnk
	pop	w4
	mov.d	w0, [w4]
	return

/* w0=ac_step */
AutoCalGain:
	inc	ac_step
	and	#0x03, w0
	bra	nz, add_sum
	mov	#fAdcPA_mV, w2
	mov	#fPA_ADC_SUM, w4
	rcall	f48_mov

	mov	#fAdcOP1_mV, w2
	mov	#fOP1_ADC_SUM, w4
	rcall	f48_mov
	return

add_sum:
	mov	#fAdcPA_mV, w2
	mov	#fPA_ADC_SUM, w3
	mov	w3, w4
	rcall	f48_add

	mov	#fAdcOP1_mV, w2
	mov	#fOP1_ADC_SUM, w3
	mov	w3, w4
	rcall	f48_add

	mov	ac_step, w0
	and	#0x03, w0
	bra	z, next_acg
	return

next_acg:
	mov	pgm_Step, w0

	cp	w0, #2
	bra	geu, acg2db

	btst	w0, #0
	bra	z, acg_next_L

	mov	#AcDispVals, w4
	rcall	math_gain

	mov	#AcF48_0, w2
	mov	#AcF48_2, w4
	rcall	f48_mov

	mov	#0x0003, w0
	bra	acg_next_H

acg2db:
	btst	w0, #0
	bra	z, acg_next_L

	cp	w0, #4
	bra	geu, acg4db

	mov	#AcDispVals+4, w4
	rcall	math_gain
	mov	#0x0005, w0
	bra	acg_next_H

acg4db:
	cp	w0, #6
	bra	geu, acg8db

	mov	#AcDispVals+8, w4
	rcall	math_gain
	mov	#0x0009, w0
	bra	acg_next_H

acg8db:
	cp	w0, #8
	bra	geu, acg16db

	mov	#AcDispVals+12, w4
	rcall	math_gain
	mov	#0x0011, w0
	bra	acg_next_H

acg16db:
	cp	w0, #10
	bra	geu, acg32db

	mov	#AcDispVals+16, w4
	rcall	math_gain
	mov	#0x0021, w0
	bra	acg_next_H

acg32db:
	cp	w0, #12
	bra	geu, AutoCalScanEnd

	mov	#AcDispVals+20, w4
	rcall	math_gain
	bra	AutoCalScanEnd

acg_next_H:
	rcall	SetAmplRelais
	bclr	INV_PORT, #INV_BIT

	bra	acg_next

acg_next_L:
	mov	#fPA_ADC_SUM, w2
	mov	#AcF48_0, w4
	rcall	f48_mov
	mov	#fOP1_ADC_SUM, w2
	mov	#AcF48_1, w4
	rcall	f48_mov

	bset	INV_PORT, #INV_BIT
acg_next:
	mov	#6, w0
	mov	w0, ac_start_count
	clr	ac_flag
	inc	pgm_Step
	return

.section .const, psv
FC48_DISP_MUL:
	.word	0x0000, 0x9c40, 0x400c		; 10000.0

