/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/eingVx.inc"
.include "lib/Analog.inc"
.include "lib/Param.inc"
.include "math/libm.inc"

.section .nbss, bss, near

adc0:			.space 4
adcX:			.space 4
;adc0x1000:		.space 4
;adcp:			.space 4
;adcn:			.space 4
;last_PA_OutFlag:	.space 2
dac_id:			.space 2
dac_count:		.space 2
fDacDigitV:		.space F48_SIZE
fDacX:			.space F48_SIZE
	NR_SHIFT	= 0
	START_LOW	= 1

/* ______________________________________________________________________ */
.section .text

.global InitAutoCalLin_DAC
.global LCD_AutoCalLin_DAC
.global AutoCalLin_DAC
.global K_Ret_AutoCalLin_DAC

InitAutoCalLin_DAC:

	mov	#PA_POW, w0
	btsc	AutoDispMod, #bAC_OFFS_PA
	mov	#PA_DC, w0
	mov	w0, dac_id

	bclr	PA_OutFlag, #bPAOF_SQ_P
	bset	PA_OutFlag, #bPAOF_DC
	rcall	SetUpFreq

	clr	w0
	mov	#PA_DC, w2
	rcall	set_dac_val
	clr	w0
	mov	#PA_POW, w2
	rcall	set_dac_val

	mov	#7, w0
	mov	w0, dac_count

	bset	AutoCalDAC, #bAC_MCP3421
	bset	TestFlags, #bTF_DAC

	mov	#4, w0
	mov	w0, ac_start_count

	return

set_dac_val:
	btst	ac_save_mod, #bACS_SAVE_DATA
	bra	z, DacSetValue_LinKorr
	bra	DacSetValue

/* ---------------------------------------------------------------------- */
LCD_AutoCalLin_DAC:
	bra	LCD_Paint

/* ---------------------------------------------------------------------- */
K_Ret_AutoCalLin_DAC:
	btst	ac_save_mod, #bACS_SAVE_DATA
	bra	z, Key_AutoCalClose

	mov	#EEData0+oFR_AD5667_0_LK+1, w7
	btsc	AutoDispMod, #bAC_OFFS_PA
	mov	#EEData0+oFR_AD5667_1_LK+1, w7
	mov	#AcDispVals, w6
	mov	#15, w5
	mov	#0xFF00, w1
	bra	kr_s_loop

kr_s_next:
	mov.b	w0, [w7++]
	dec	w5, w5
	bra	z, kr_s_end
kr_s_loop:
	mov	[w6++], w0
	btst	w0, #15
	bra	nz, kr_s_neg
	and	w0, w1, w2
	btss	SR, #Z
	mov	#0x007F, w0
	bra	kr_s_next
kr_s_neg:
	and	w0, w1, w2
	cp	w1, w2
	btss	SR, #Z
	mov	#0xFF80, w0
	bra	kr_s_next

kr_s_end:
	mov	#EEData0+oFR_AD5667_0_LK, w7
	btsc	AutoDispMod, #bAC_OFFS_PA
	mov	#EEData0+oFR_AD5667_1_LK, w7
	rcall	DacLinKorr_SetCrc

	mov	#oFR_AD5667_0_LK, w2
	btsc	AutoDispMod, #bAC_OFFS_PA
	mov	#oFR_AD5667_1_LK, w2
	mov	#15, w3
	rcall	FRam_Save
	bra	Key_AutoCalClose
/*
kr_no_ds:
	bclr	TestFlags, #bTF_DAC
	mov	last_PA_OutFlag, w0
	mov	w0, PA_OutFlag
	rcall	SetUpGain
	rcall	SetUpOffs
	rcall	SetUpFreq
	bra	Key_RetOk_Menu
*/

/* ---------------------------------------------------------------------- */
/* w0=ac_step */
AutoCalLin_DAC:
	cp	w0, #4
	bra	geu, lac_scan

	btst	w0, #0
	bra	nz, lac_vp

	mov	AdcPa_1616, w0
	mov	AdcPa, w1
	mov	w0, adc0
	mov	w1, adc0+2

.if NR_SHIFT == 1
	mov	#0x1000, w0
.else
	mov	#0x5000, w0
.endif
	btsc	ac_step, #1
	neg	w0, w0
	bra	lac_set_vnext

lac_vp:
	btst	w0, #1
	bra	nz, lac_vn

	mov	AdcPa_1616, w0
	mov	AdcPa, w1
	mov	w0, adcX
	mov	w1, adcX+2
	clr	w0
	bra	lac_set_vnext

lac_vn:
	mov	AdcPa_1616, w0
	mov	AdcPa, w1
	mov	adcX, w2
	mov	adcX+2, w3
	sub	w2, w0, w0
	subb	w3, w1, w1

	lnk	#F48_SIZE*2

	mov	w14, w3
	rcall	i1616_f48_w3

.if NR_SHIFT == 1
	mov	#0x2000, w0
.else
	mov	#0xA000, w0
.endif
	add	w14, #F48_SIZE, w3
	rcall	u16_f48_w3

	mov	w14, w2
	mov	#fDacDigitV, w4
	rcall	f48_div

	ulnk
	clr	w0
	bra	lac_set_vnext

lac_scan:
	btst	w0, #0
	bra	nz, lac_scan_p

	mov	AdcPa_1616, w0
	mov	AdcPa, w1
	mov	w0, adc0
	mov	w1, adc0+2

	mov	dac_count, w0
.if START_LOW
	subr	w0, #8, w0
.endif
	sl	w0, #12, w0
	btsc	ac_step, #1
	neg	w0, w0
	push	w0
	mov	#fDacX, w3
	rcall	i16_f48_w3
	pop	w0
	bra	lac_set_vnext

lac_scan_p:
	btst	w0, #1
	bra	nz, lac_scan_vn

	mov	AdcPa_1616, w0
	mov	AdcPa, w1
	mov	adc0, w2
	mov	adc0+2, w3
	sub	w0, w2, w2
	subb	w1, w3, w3

	lnk	#F48_SIZE*2

	mov.d	w2, w0
	mov	w14, w3
	rcall	i1616_f48_w3


	mov	#fDacDigitV, w2
	mov	#fDacX, w3
	add	w14, #F48_SIZE, w4
	rcall	f48_mul

	mov	w14, w2
	mov	w4, w3
	rcall	f48_sub


	mov	w4, w2
	mov	#fDacDigitV, w3
	mov	#fDacX, w4
	rcall	f48_div3216
	mov	#1, w0
	rcall	f48_add_exp_w4
	rcall	f48_i16r

	mov	#AcDispVals, w4
	mov	dac_count, w2
.if START_LOW
	sub	w2, #1, w2
.else
	subr	w2, #7, w2
.endif
	sl	w2, w2
	add	w4, w2, w4
	nop
	mov	w0, [w4]

	ulnk

	clr	w0
	bra	lac_set_vnext

lac_scan_vn:
	mov	AdcPa_1616, w0
	mov	AdcPa, w1
	mov	adc0, w2
	mov	adc0+2, w3
	sub	w0, w2, w2
	subb	w1, w3, w3


	lnk	#F48_SIZE*2

	mov.d	w2, w0
	mov	w14, w3
	rcall	i1616_f48_w3


	mov	#fDacDigitV, w2
	mov	#fDacX, w3
	add	w14, #F48_SIZE, w4
	rcall	f48_mul

	mov	w14, w2
	mov	w4, w3
	rcall	f48_sub


	mov	w4, w2
	mov	#fDacDigitV, w3
	mov	#fDacX, w4
	rcall	f48_div3216

	mov	#1, w0
	rcall	f48_add_exp_w4
	rcall	f48_i16r

	mov	#AcDispVals, w4
	mov	dac_count, w2
.if START_LOW
	subr	w2, #8, w2
.endif
	add	w2, #7, w2
	sl	w2, w2
	add	w4, w2, w4
	nop
	mov	w0, [w4]

	ulnk

	dec	dac_count
	bra	z, lac_clr
	clr	w0

lac_set_vnext:
	mov	dac_id, w2
	rcall	set_dac_val
 mov	#4, w0
 mov	w0, ac_start_count
	inc	ac_step
	clr	ac_flag
	return

lac_clr:
	clr	w0
	mov	dac_id, w2
	rcall	set_dac_val
	bra	AutoCalScanEnd

;	clr	AutoCalMod
;	clr	AutoCalDAC
;	return


