/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/Analog.inc"
.include "lib/Param.inc"
.include "math/libm.inc"

.section .nbss, bss, near

AutoCalMod:		.space 2
AutoDispMod:		.space 2
AutoCalDAC:		.space 2
ac_flag:		.space 2
ac_step:		.space 2
ac_start_count:		.space 2
ac_save_mod:		.space 2

AcDispVals:		.space 48	; max 8 * F48_SIZE
AcF48_0:		.space F48_SIZE
AcF48_1:		.space F48_SIZE
AcF48_2:		.space F48_SIZE
AcTmpMem:		.space 16

; 48 + 6*3 + 16 = 82 / 2 = 41 - 1 = 40
	AC_MEM_CLR_COUNT		= 40

last_PA_OutFlag:	.space 2
LastIoMk:		.space 2

.global AutoCalMod
.global AutoDispMod
.global AutoCalDAC
.global ac_flag
.global ac_step
.global ac_start_count
.global ac_save_mod
.global AcDispVals
.global AcF48_0
.global AcF48_1
.global AcF48_2
.global AcTmpMem
.global last_PA_OutFlag
.global LastIoMk

/* ---------------------------------------------------------------------- */
.section .text

.global init_AutoCal
.global LCD_AutoCal
.global Key_AutoCal
.global Key_AutoCalClose
.global AutoCal
.global AutoCalScanEnd

init_AutoCal:
	mov	w4, AutoCalMod
	mov	w4, AutoDispMod
	clr	AutoCalDAC
	clr	ac_flag
	clr	ac_step

	mov	PA_OutFlag, w0
	mov	w0, last_PA_OutFlag

	clr	LastIoMk
	btsc	INV_PORT, #INV_BIT
	bset	LastIoMk, #bACIOF_INV

	mov	#2, w0
	mov	w0, ac_start_count
	mov	#AcDispVals, w0
	repeat	#AC_MEM_CLR_COUNT
	clr	[w0++]

	btst	w4, #bAC_16VALS_MCP3421
	bra	nz, InitAutoCalLin_DAC
	btst	w4, #bAC_GAIN
	bra	nz, InitAutoCalGain
	bra	InitAutoCalGenV


/* ______________________________________________________________________ */
LCD_AutoCal:
	mov	#AcDispVals, w6
	mov	#4, w7
	btsc	AutoDispMod, #bAC_16VALS
	mov	#8, w7

	dec	w0, w0
	bra	z, LCD_MenuCaption_cs
	dec	w0, w0
	bra	z, Line1

	mov	AutoDispMod, w3
	cp	w3, #AC_GEN_V
	bra	z, LCD_AutoCalGenV

	btst	AutoDispMod, #bAC_GAIN
	bra	nz, LCD_AutoCalGain

	dec	w0, w0
	bra	z, Line2
	add	w6, w7, w6
	dec	w0, w0
	bra	z, Line3
	add	w6, w7, w6
	dec	w0, w0
	bra	z, Line4
	add	w6, w7, w6
	dec	w0, w0
	bra	z, Line5
	dec	w0, w0
	bra	z, Line6

	btst	AutoDispMod, #bAC_16VALS_MCP3421
	bra	nz, LCD_AutoCalLin_DAC
	bra	LCD_AutoCalGenV

;	bra	LCD_Paint

Line1:
	mov	#Str_buffer, w3
	rcall	GetK_dbX
/*
	clr	w0
	btsc	K01DB_PORT, #K01DB_BIT
	bset	w0, #0
	btsc	K02DB_PORT, #K02DB_BIT
	bset	w0, #1
	btsc	K04DB_PORT, #K04DB_BIT
	bset	w0, #2
	btsc	K08DB_PORT, #K08DB_BIT
	bset	w0, #3
	btsc	K16DB_PORT, #K16DB_BIT
	bset	w0, #4
	btsc	K32DB_PORT, #K32DB_BIT
	bset	w0, #5
*/
	mov	#2, w2
	rcall	hex16toppc
	mov	#0x81, w0
	mov.b	w0, [w3++]

	btst	ac_flag, #bACF_OK
	bra	nz, l1_dend

	mov	ac_step, w0
	clr	w2
	rcall	uint16D3toppc
	bra	l1_lend
l1_dend:
	mov	#'E', w0
	mov.b	w0, [w3++]
	mov	#'n', w0
	mov.b	w0, [w3++]
	mov	#'d', w0
	mov.b	w0, [w3++]
	clr.b	[w3]
l1_lend:
	mov	#0, w0
	mov	#10, w1
	bra	LCD_StrBufOut_cs

Line2:
	mov	#Str_buffer, w3
	mov	#20, w2
	bra	LineV

Line3:
	mov	#Str_buffer, w3
	mov	#29, w2
	bra	LineV

Line4:
	mov	#Str_buffer, w3
	mov	#38, w2
	bra	LineV

Line5:
	mov	#Str_buffer, w3
	mov	#47, w2
	bra	LineV

Line6:
	mov	#0, w0
	mov	#58, w1
	bra	StrbTemp_cs_step

LineV:
	push	w2
	btst	AutoDispMod, #bAC_16VALS
	bra	nz, LineV_16

	mov	[w6++], w0
	mov	#1, w2
	rcall	int16toppc
	mov	#0x81, w0
	mov.b	w0, [w3++]
	mov	[w6], w0
	mov	#1, w2
	rcall	int16toppc
	mov	#0x81, w0
	mov.b	w0, [w3++]
	mov	#'m', w0
	mov.b	w0, [w3++]
	mov	#'V', w0
	mov.b	w0, [w3++]
	clr.b	[w3]
	mov	#0, w0
	pop	w1
	bra	LCD_StrBufOut_cs

LineV_16:
	mov	[w6++], w0
	mov	#0, w2
	rcall	int16toppc
	mov	#0x81, w0
	mov.b	w0, [w3++]
	mov	[w6++], w0
	mov	#0, w2
	rcall	int16toppc
	mov	#0x81, w0
	mov.b	w0, [w3++]
	mov	[w6++], w0
	mov	#0, w2
	rcall	int16toppc
	mov	#0x81, w0
	mov.b	w0, [w3++]
	mov	[w6], w0
	mov	#0, w2
	rcall	int16toppc
	clr.b	[w3]

	pop	w1
	mov	#0, w0
	bra	LCD_StrBufOut_cs
/* ______________________________________________________________________ */
Key_AutoCal:
	cp	w0, #KEY_RET
	bra	z, K_Ret
	bra	Key_RetOk

K_Ret:
	clr	AutoCalMod
	clr	AutoCalDAC
	btst	ac_flag, #bACF_OK
	bra	z, Key_AutoCalClose

	mov	AutoDispMod, w0
	btst	w0, #bAC_16VALS_MCP3421
	bra	nz, K_Ret_AutoCalLin_DAC
	btst	w0, #bAC_GAIN
	bra	nz, K_Ret_AutoCalGain
	bra	K_Ret_AutoCalGenV

Key_AutoCalClose:
	bclr	TestFlags, #bTF_DAC
	mov	last_PA_OutFlag, w0
	mov	w0, PA_OutFlag

	btsc	LastIoMk, #bACIOF_INV
	bset	INV_PORT, #INV_BIT
	btss	LastIoMk, #bACIOF_INV
	bclr	INV_PORT, #INV_BIT

	rcall	SetUpGain
	rcall	SetUpOffs
	rcall	SetUpFreq
	bra	Key_RetOk_Menu

/* ______________________________________________________________________ */
AutoCal:
	cp0	AutoCalMod
	btsc	SR, #Z
	return

	cp0	ac_start_count
	bra	nz, lac_ws

	mov	ac_step, w0
	btst	ac_flag, #bACF_SAMP0
	bra	nz, lac_m1
	bset	ac_flag, #bACF_SAMP0

	return

lac_ws:
	dec	ac_start_count
	return

lac_m1:
	mov	AutoDispMod, w1
	btst	w1, #bAC_16VALS_MCP3421
	bra	nz, AutoCalLin_DAC
	btst	w1, #bAC_GAIN
	bra	nz, AutoCalGain
	bra	AutoCalGenV

AutoCalScanEnd:
	bset	ac_flag, #bACF_OK
	clr	AutoCalMod
	clr	AutoCalDAC
	return
