/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/Analog.inc"
.include "lib/eingVx.inc"
.include "lib/Param.inc"
.include "lcd_flags.inc"

.section .nbss, bss, near
CursorIdx:		.space 2
MenuCount:		.space 2
MenuOff:		.space 2
pMenu:			.space 2
.equ MenuSize,		6
pMenuTxt:		.space 2

.equ LM_Size,		6
.equ LM_MAX_IDX,	4
.equ LM_pMenu,		0
.equ LM_CursorIdx,	2
.equ LM_MenuOff,	4
LastMenuIdx:		.space 2
LastMenu:		.space LM_Size * LM_MAX_IDX
pLCD_MenuCaption:	.space 2
pLCD_MenuData:		.space 2

.if USE_UART_LIST_PARA
;.equ oUCM_POINTER_0,		0
.equ oUCM_IDX,			2	/* lb = Menu idx   hb = Line idx */
.equ oUCM_MENU_IDX,		2
.equ oUCM_LINE_IDX,		3

.equ UCM_SIZE,			4

pucMenu:		.space UCM_SIZE * LM_MAX_IDX
pucIdx:			.space 2
.endif


.global pLCD_MenuCaption
.global pLCD_MenuData

.section .text
.global LCD_Menu
.global Key_Menu
.global LCD_MenuCaption
.global LCD_MenuCaption_cs

.if USE_UART_LIST_PARA
.global ucMenu0
.global ucMenuSend_NULL
.global ucMenuData
.global ucMenuOpen
.global ucMenuClose
.global ucMenuNext
.endif

.equ MAX_LINES,			5
.equ LINE_HEIGHT,		12
LCD_Menu:
	dec	w0, w0
	bra	nz, LineX

	mov	pMenu, w3
	cp0	w3
	bra	nz,10f
	mov	#ParaMenu, w3
	mov	w3, pMenu
10:
	clr	MenuCount
	inc	LCD_Step
	bra	Line0
LineX:
	dec	w0, w0
	bra	nz, Cursor
Line0:
	mov	pMenu, w2
	mov	MenuCount, w1
	mov	[w2++], w0
	cp	w1, w0
	bra	ge, L0_next
	cp	w1, #MAX_LINES
	bra	ge, L0_next

	mov	MenuOff, w0
	add	w0, w1, w0
	mul.uu	w0, #MenuSize, w0
;	add	w1, w0, w1

	mov	#Str_buffer, w3
	mov	[w0+w2], w0
	rcall	ppc_move


	mov	MenuCount, w0
	mul.uu	w0, #LINE_HEIGHT, w0
	inc	w0, w1
	mov	#0, w0
	mov	#Str_buffer, w2
	rcall	StrOut8x13
	inc	MenuCount
	return
L0_next:
	inc	LCD_Step
	bra	Cursor0

Cursor:
	dec	w0, w0
	bra	nz, LCD_Paint
Cursor0:
	mov	CursorIdx, w0
	mul.uu	w0, #LINE_HEIGHT, w0
	mov	w0, w1

	mov	#0, w0
	mov	#128, w2
	mov	#LINE_HEIGHT, w3
	rcall	InvRect

	inc	LCD_Step
	return

/* ______________________________________________________________________ */
PushMenu:	/* W0 = next pMenu */
	mov	LastMenuIdx, w2
	cp	w2, #LM_MAX_IDX-1
	bra	ge, pum_end

	mul.uu	w2, #LM_Size, w2
	mov	#LastMenu, w3
	add	w2, w3, w3
	mov	pMenu, w1
	mov	w0, pMenu
	mov	w1, [w3+LM_pMenu]

	add	w3, #LM_Size, w3
	inc	LastMenuIdx

	mov	[w3+LM_pMenu], w1
	mov	w0, [w3+LM_pMenu]

	cp0	w1
	bra	z, pum_c0
	cp	w0, w1
	bra	nz, pum_c0

	mov	[w3+LM_CursorIdx], w0
	mov	w0, CursorIdx
	mov	[w3+LM_MenuOff], w1
	mov	w1, MenuOff
	return

pum_c0:
	clr	w0
	mov	w0, [w3+LM_CursorIdx]
	mov	w0, [w3+LM_MenuOff]
	clr	CursorIdx
	clr	MenuOff
pum_end:
	return

PopMenu:	/* ret W0 = last pMenu */
	mov	LastMenuIdx, w0
	cp0	w0
	bra	z, pom_end

	dec	w0, w0
	mul.uu	w0, #LM_Size, w2
	mov	#LastMenu, w3
	add	w2, w3, w3
	mov	w0, LastMenuIdx		; stall

	mov	[w3+LM_MenuOff], w0
	mov	w0, MenuOff
	mov	[w3+LM_CursorIdx], w0
	mov	w0, CursorIdx
	mov	[w3+LM_pMenu], w0
	mov	w0, pMenu
pom_end:
	return

/* ______________________________________________________________________ */
Key_Menu:
	mov	pMenu, w3
	cp0	w3
	bra	z, K_Ret
	mov	[w3++], w2	/* MenuCountMax */

	cp	w0, #KEY_UP
	bra	z, K_CUp
	cp	w0, #KEY_DOWN
	bra	z, K_CDown
	cp	w0, #KEY_OPEN
	bra	z, K_COpen
	cp	w0, #KEY_SEL_TGL
	bra	z, K_COpen
	cp	w0, #KEY_RET
	bra	z, K_Ret
	return

K_CUp:
	cp0	CursorIdx
	bra	z, K_CUp_offs
	dec	CursorIdx
	bra	SaveMenuCursor
K_CUp_offs:
	cp0	MenuOff
	bra	z, Key_RetOk
	dec	MenuOff
	bra	SaveMenuCursor

K_CDown:
	dec	w2, w2
	mov	CursorIdx, w0
	cp	w0, w2
	bra	ge, Key_RetOk
	cp	w0, #MAX_LINES-1
	bra	z, K_CDown_offs
	inc	CursorIdx
	bra	SaveMenuCursor
K_CDown_offs:
	add	MenuOff, WREG
	cp	w0, w2
	bra	ge, Key_RetOk
	inc	MenuOff
SaveMenuCursor:
	mov	LastMenuIdx, w2
	cp	w2, #LM_MAX_IDX
	bra	ge, Key_RetRePaint

	mul.uu	w2, #LM_Size, w2
	mov	#LastMenu, w3
	add	w2, w3, w3
	mov	CursorIdx, w0
	mov	w0, [w3+LM_CursorIdx]
	mov	MenuOff, w1
	mov	w1, [w3+LM_MenuOff]
	bra	Key_RetRePaint

K_COpen:
	mov	MenuOff, w0
	add	CursorIdx, WREG
	mul.uu	w0, #MenuSize, w0
	add	w3, w0, w3
	nop			; stall
	mov	[w3], w0	/* Menu txt */
	mov	w0, pMenuTxt
	mov	[w3+4], w4	/* Ext Data */
	mov	w4, pLCD_MenuData

	mov	[w3+2], w0	/* Next Menu */
	cp0	w0
	bra	z, Key_RetOk
	call	w0
	bra	Key_RetRePaint

K_Ret:
	rcall	PopMenu
	cp0	w0
	bra	nz, Key_RetRePaint
	clr	LCD_PProc
	bra	Key_RetRePaint

/* ______________________________________________________________________ */
LCD_MenuCaption:
	mov	pLCD_MenuCaption, w0
	cp0	w0
	bra	z, LCD_Step_Inc
	mov	#Str_buffer, w3
	rcall	ppc_move

	mov	#0, w0
	mov	#0, w1
	bra	LCD_StrBufOut8x13

/* ______________________________________________________________________ */
LCD_MenuCaption_cs:
	mov	pLCD_MenuCaption, w2
	cp0	w2
	bra	z, LCD_Step_Inc
	mov	#0, w0
	mov	#0, w1
	bra	LCD_ConstStrBufOut_cs

/* ______________________________________________________________________ */
OpenAlgMenu:
	mov	[w3+4], w0
	bra	PushMenu	/* W0 = next pMenu */

/* ______________________________________________________________________ */
IDXOpen:
	mov	w0, LCD_PProc
	mov	pMenuTxt, w0
	mov	w0, pLCD_MenuCaption
	return

InfoOpen:
	mov	#MENU_IDX_Info, w0
	bra	IDXOpen

;TestBit_Open:
;	mov	#MENU_IDX_TF, w0
;	bra	IDXOpen

Flags_Open:
	cp0	w4
	btsc	SR, #Z
	return

	rcall	FlagsEdInit
	mov	#MENU_IDX_FLAGS, w0
	bra	IDXOpen

LcdDac_Open:
	cp0	w4
	btsc	SR, #Z
	return

	rcall	init_TestDac
	mov	#MENU_IDX_DAC, w0
	bra	IDXOpen

Param_Open:
	cp0	w4
	btsc	SR, #Z
	return
	rcall	init_EingPara
	mov	#MENU_IDX_EINGPARA, w0
	bra	IDXOpen


OpenSymMenu:
	rcall	init_SymPara
	btsc	SR, #OV
	return
	mov	#MENU_IDX_SYM, w0
	bra	IDXOpen

Filter_Open:
	cp0	w4
	btsc	SR, #Z
	return
	rcall	init_FilterPara
	btsc	SR, #OV
	return
	mov	#MENU_IDX_EINGFILER, w0
	bra	IDXOpen

FiltKx_Open:
	cp0	w4
	btsc	SR, #Z
	return
	rcall	init_FilterKxPara
	btsc	SR, #OV
	return
	mov	#MENU_IDX_EINGFILKX, w0
	bra	IDXOpen

AutoCal_Open:
	cp0	w4
	btsc	SR, #Z
	return
	rcall	init_AutoCal
	mov	#MENU_IDX_AUTOCAL, w0
	bra	IDXOpen

Impedanz_Open:
	rcall	init_Impedanz
	mov	#MENU_IDX_IMPEDANZ, w0
	bra	IDXOpen

/* ______________________________________________________________________ */
.if USE_UART_LIST_PARA
ucMenu0:
	mov	#pucMenu, w0			; clr pucMenu+pucIdx
	repeat	#UCM_SIZE * LM_MAX_IDX / 2
	clr	[w0++]

	mov	#ParaMenu, w0
	mov	w0, pucMenu
	clr	pucIdx
	return

/* w3=0 Send Str Null */
ucMenuSend_NULL:
	mov	#Str_buffer, w3
	mov	#tucNULL, w0
	bra	ppc_move

/* w0=pucIdx  result w6=UCM_Adr   SR Z nodata  */
ucmAdr:
	cp0	pucMenu
	bra	z, ucmadr0
	cp	w0, #LM_MAX_IDX
	bra	geu, ucmadr0
	mul.uu  w0, #UCM_SIZE, w0
	mov	#pucMenu, w6
	add	w6, w0, w6
	bclr	SR, #Z
	return
ucmadr0:
	clr	w6
	bset	SR, #Z
	return

/* result w1=UCM_IDX w2=UCM_LIDX w6=UCM_Adr w7=ParaHead   SR Z nodata  */
ucMenuAdr0:
	mov	pucIdx, w0
	rcall	ucmAdr
	bra	z, adr00
	add	w6, #2, w3
	mov	[w6], w7
	ze	[w3++], w1
	ze	[w3], w2
	bclr	SR, #Z
	return
adr00:
	bset	SR, #Z
	return

/* result w1=UCM_IDX w2=UCM_LIDX w6=UCM_Adr w7=ParaLine   SR Z nodata  */
ucMenuAdr:
	rcall	ucMenuAdr0
	bra	z, adr0
	mov	[w7], w3
	cp0	w3
	bra	z, adr0
	cp.b	w1, w3
	bra	geu, adr0
	inc2	w7, w7
	mul.uu	w1, #MenuSize, w0
	add	w7, w0, w7
	bclr	SR, #Z
	return
adr0:
	bset	SR, #Z
	return


ucMenuData:
	rcall	ucMenuAdr
	bra	z, ucMenuSend_NULL

	mov	[w7], w0
	cp0	w0
	bra	z, ucMenuSend_NULL
	mov	[w7+2], w3
	cp0	w2
	bra	nz, ucmnx_parm

	mov	#tSetting, w5
	cp	w0, w5
	bra	z, ucmnx_skip
	mov	#tTest, w5
	cp	w0, w5
	bra	z, ucmnx_skip
	mov	#tAutoCal, w5
	cp	w0, w5
	bra	z, ucmnx_skip


	mov	pucOpenAlgMenu, w5
	cp	w3, w5
; printf("w4=0x%04x w5=0x%04x\n", w4, w5);
	mov	#Str_buffer, w3
	bra	nz, ppc_move
	mov	#'/', w1
	mov.b	w1, [w3++]
	mov	[w7], w0
	bra	ppc_move

ucmnx_skip:
	mov	#Str_buffer, w3
	mov	#tucSKIP, w0
	rcall	ppc_move
	mov	#'/', w1
	mov.b	w1, [w3++]
	mov	[w7], w0
	bra	ppc_move


ucmnx_parm:
	inc	w2, w0
	mov.b	w0, [w6+oUCM_LINE_IDX]

	mov	#tab_ucPrg, w1
	mov	[w7+4], w4
ucmnx_loop:
	mov	[w1++], w5
	cp0	w5
	bra	z, ucMenuSend_NULL
	mov	[w1++], w0
	cp	w3, w5
	bra	nz, ucmnx_loop
	call	w0
	return

/*
	mov	[w7+4], w4
	mov	#Impedanz_Open, w5
	cp	w3, w5
	bra	z, get_val_Impedanz

	mov	#Flags_Open, w5
	cp	w3, w5
	bra	z, get_val_Flags

	mov	#Param_Open, w5
	cp	w3, w5
	bra	z, get_val_EingPara

	mov	#Filter_Open, w5
	cp	w3, w5
	bra	z, get_val_Filter

	mov	#FiltKx_Open, w5
	cp	w3, w5
	bra	z, get_val_FilterKx

	bra	ucMenuSend_NULL
*/

ucMenuOpen:
	rcall	ucMenuAdr
	bra	z, ucmo0
	mov	[w7+2], w4
	mov	pucOpenAlgMenu, w5
	cp	w4, w5
	bra	nz, ucmo_data

	mov	[w7+4], w4
	cp0	w4
	bra	z, ucmo0

	inc	pucIdx
	mov	pucIdx, w0
	rcall	ucmAdr
	bra	z, ucmoRet
	mov	w4, [w6]
	clr	w1
	mov	w1, [w6+oUCM_IDX]
	return

ucmo_data:
	mov	#1, w0
	mov.b	w0, [w6+oUCM_LINE_IDX]
	return

ucmo0:
	clr	w4
	inc	pucIdx
	mov	pucIdx, w0
	rcall	ucmAdr
	bra	z, ucmoRet
	mov	w4, [w6]
	clr	w1
	mov	w1, [w6+oUCM_IDX]
ucmoRet:
	return

ucMenuClose:
	rcall	ucMenuAdr
	bra	nz, ucmc_ak
	cp0	w6
	bra	nz, ucmc_dec
	return
ucmc_ak:
	cp0	w2
	bra	nz, ucmc_data
ucmc_dec:
	clr	w0
	mov	w0, [w6]
	mov	w0, [w6+oUCM_IDX]
	cp0	pucIdx
	btss	SR, #Z
	dec	pucIdx
	return

ucmc_data:
	clr	w0
	mov.b	w0, [w6+oUCM_LINE_IDX]
	return


ucMenuNext:
	rcall	ucMenuAdr0
	bra	z, ucmnx0
	mov	[w7], w3
	cp.b	w1, w3
	bra	geu, ucmnx0
	inc	w1, w1
	mov.b	w1, [w6+oUCM_MENU_IDX]
ucmnx0:
	return

.endif





/* ______________________________________________________________________ */
.section .const, psv

.if USE_UART_LIST_PARA
tucNULL:	.asciz	"NULL"
tucSKIP:	.asciz	"SKIP"
 .align 2
pucOpenAlgMenu:  .word OpenAlgMenu

tab_ucPrg:
	.word Param_Open, get_val_EingPara
	.word Flags_Open, get_val_Flags
	.word Filter_Open, get_val_Filter
	.word FiltKx_Open, get_val_FilterKx
	.word OpenSymMenu, get_val_Sym
	.word InfoOpen, get_val_Info
	.word 0
.endif

/*  ----------- Menu 0 -------------------------------------------------- */

tSetting:	.asciz	" 1 Setting"
tTest:		.asciz	" 2 Test"
tPaDC:		.asciz	" 3 PA DC DAC"
tGain:		.asciz	" 4 Gain"
tFilter:	.asciz	" 5 Filter"
tPuls:		.asciz	" 6 Puls korr"
tOffsParam:	.asciz	" 7 Offs Param"
tTempParam:	.asciz	" 8 \177C Param"
tAdcParam:	.asciz	" 9 ADC Param"
tAutoCal:	.asciz	"10 AutoCal"
tSystem:	.asciz	"11 System"
tGenSym:	.asciz	"12 Gen Sym"
tInfo:		.asciz	"13 Info"

 .align 2
ParaMenu:
	.word 13
	.word tSetting,		OpenAlgMenu,		tabSetting
	.word tTest,		OpenAlgMenu,		tabTest
	.word tPaDC,		OpenAlgMenu,		tabPaDCMenu
	.word tGain,		OpenAlgMenu,		tabGainMenu
	.word tFilter,		OpenAlgMenu,		tabFilterMenu
	.word tPuls,		OpenAlgMenu,		tabPulsMenu
	.word tOffsParam,	OpenAlgMenu,		tabOffsParam
	.word tTempParam,	OpenAlgMenu,		tabTempParam
	.word tAdcParam,	OpenAlgMenu,		tabAdcParam
	.word tAutoCal,		OpenAlgMenu,		tabAutoCal
	.word tSystem,		OpenAlgMenu,		tabSysMenu
	.word tGenSym,		OpenAlgMenu,		tabSymMenu
	.word tInfo,		InfoOpen,		0

/*  ----------- Setting ------------------------------------------------- */
tHighZ:		.asciz	" 1 Impedanz"
tTDR_DC:	.asciz	" 2 TDR DC"
tTestDisp:	.asciz	" 3 Test Disp"

;texHighZ:	.asciz	"0=50\200Ohm 1=highZ"

 .align 2
;PHighZ:	.word	bSF_HIGHZ | LCDF_TXT_CS | LCDF_ONCANGE, SettingFlags, SetHighZ, texHighZ
PTDR_DC: .word	bSF_TDR_DC | LCDF_ONCANGE, SettingFlags, SetTDR_DC
PTestDisp: .word bTDF_DISP, TestDispFlags

tabSetting:
	.word 3
;	.word tHighZ,	Flags_Open,	PHighZ
	.word tHighZ,	Impedanz_Open,	0
	.word tTDR_DC,	Flags_Open,	PTDR_DC
	.word tTestDisp,	Flags_Open,	PTestDisp

/*  ----------- System -------------------------------------------------- */
tOsct:		.asciz	" 1 Osc \177C"
tGent:		.asciz	" 2 Gen \177C"
tOp1t:		.asciz	" 3 Op1 \177C"
tPat:		.asciz	" 4 PA  \177C"
tOscFq:		.asciz	" 5 Osc Freq"
tOscFqPC:	.asciz	" 6 Osc dig/\177C"
tFanOnV:	.asciz	" 7 Fan on V"
tFanOnC:	.asciz	" 8 Fan on \177C"
tHtOnC:		.asciz	" 9 HT on \177C"
tLcdContrast:	.asciz	" 10 LCD Contrast"

 .align 2
POsct:		.word 0x2020, 0x1302, (TXT_TC << 12) | oFR_OT_OSC
PGent:		.word 0x2020, 0x1302, (TXT_TC << 12) | oFR_OT_GEN
POp1t:		.word 0x2020, 0x1302, (TXT_TC << 12) | oFR_OT_OP1
PPat:		.word 0x0020, 0x1302, (TXT_TC << 12) | oFR_OT_PA
POscFq:		.word 0x4021, 0x0400, (TXT_dig << 12) | oFR_OSCT, SetCpuFreq
POscFqPC:	.word 0x6021, 0x1300, oFR_OSCT_PC, SetCpuFreq
PFanOnV:	.word 0x0020, 0x0303, (TXT_dig << 12) | oFR_FAN_ON_V
PFanOnC:	.word 0x8124, 0x1B00, (TXT_TC << 12) | oFR_FAN_ON_C, FAN_TEMP_ON_OFFS, 0x4010
PHtOnC:		.word 0x8124, 0x1B00, (TXT_TC << 12) | oFR_HT_ON_C, HT_TEMP_ON_OFFS, 0x4010
PLcdContrast:	.word 0x3001, 0x8202,	oFR_CONTRAST,	LCD_SetContrast

tabSysMenu:
	.word 10
	.word tOsct,		Param_Open,	POsct
	.word tGent,		Param_Open,	PGent
	.word tOp1t,		Param_Open,	POp1t
	.word tPat,		Param_Open,	PPat
	.word tOscFq,		Param_Open,	POscFq
	.word tOscFqPC,		Param_Open,	POscFqPC
	.word tFanOnV,		Param_Open,	PFanOnV
	.word tFanOnC,		Param_Open,	PFanOnC
	.word tHtOnC,		Param_Open,	PHtOnC
	.word tLcdContrast,	Param_Open,	PLcdContrast

/*  ----------- Offst Param -------------------------------------------------- */
tOp1Offs:	.asciz	" 1 Offs OP1"
;tPaOffs:	.asciz	" 2 Offs PA"
tPaInOffs:	.asciz	" 3 Offs PA IN"
tPaOutOffs:	.asciz	" 4 Offs PA OUT"
tOpTOffs:	.asciz	" 5 Offs OPT"
tosym:		.asciz	" 6 PLL OSYM"
tosym115:	.asciz	" 7 PLL OSYM 115"
tpsym:		.asciz	" 8 Gen APSYM"
tgti0:		.asciz	" 9 Gen GTI0"
tgti1:		.asciz	"10 Gen GTI1"

 .align 2

POp1Offs:   .word 0x4111, 0x0400, (TXT_dig << 12) | oFR_OFF_OP1, SetParamPa, 0x4010  /* use timer to set */
;PPaOffs:    .word 0x4111, 0x0400, (TXT_dig << 12) | oFR_OFF_PA,  SetParamPa, 0x4010  /* use timer to set */
PPaInOffs:  .word 0x4111, 0x0400, (TXT_dig << 12) | oFR_OFF_PA_DC,  SetDacPaDC_Offset, 0x4010
PPaOutOffs: .word 0x4111, 0x0400, (TXT_dig << 12) | oFR_OFF_PA_POW, SetDacPaPow_Offset, 0x4010
POpTOffs:   .word 0x4010, 0x0400, (TXT_dig << 12) | oFR_OFF_OPT  /* use timer to set */

Posym:      .word 0x4111, 0x0400, (TXT_dig << 12) | oFR_OSYM, SetGenPllSym, 0x2431
Posym115:   .word 0x4111, 0x0400, (TXT_dig << 12) | oFR_OSYM_115MHZ, SetGenPllSym, 0x2431

;PPsym:	    .word 0x0113, 0x0400, (TXT_dig << 12) | oFR_APSYM,  DacVxOnChange, APSYM,  0x2431
;PGti0:	    .word 0x4113, 0x0400, (TXT_dig << 12) | oFR_AGTI0, DacVxOnChange, AGTI0, 0x2431
PPsym:	    .word 0x0111, 0x0400, (TXT_dig << 12) | oFR_APSYM,  SetGenPllSym,  0x2431
PGti0:	    .word 0x4111, 0x0400, (TXT_dig << 12) | oFR_AGTI0, SetGenPllSym, 0x2431
PGti1:	    .word 0x4113, 0x0400, (TXT_dig << 12) | oFR_AGTI1, DacVxOnChange, AGTI1, 0x2431

tabOffsParam:
	.word 9
	.word tOp1Offs,		Param_Open,	POp1Offs
;	.word tPaOffs,		Param_Open,	PPaOffs
	.word tPaInOffs,	Param_Open,	PPaInOffs
	.word tPaOutOffs,	Param_Open,	PPaOutOffs
	.word tOpTOffs,		Param_Open,	POpTOffs
	.word tosym,		Param_Open,	Posym
	.word tosym115,		Param_Open,	Posym115
	.word tpsym,		Param_Open,	PPsym
	.word tgti0,		Param_Open,	PGti0
	.word tgti1,		Param_Open,	PGti1


/*  ----------- Temp C Param ------------------------------------------- */
tOp1Temp:	.asciz	" 1 mV/\177C OP1"
tOptTemp:	.asciz	" 4 mV/\177C OPT"
tGenHpTemp:	.asciz	" 5 mV/\177C GHp"
tGenHnTemp:	.asciz	" 6 mV/\177C GHn"
tGenLpTemp:	.asciz	" 7 mV/\177C GLp"
tGenLnTemp:	.asciz	" 8 mV/\177C GLn"
tGenHp1dBT:	.asciz	" 9 mV/\177C GHp 1dB"
tGenHn1dBT:	.asciz	"10 mV/\177C GHn 1dB"
tGenLp1dBT:	.asciz	"11 mV/\177C GLp 1dB"
tGenLn1dBT:	.asciz	"12 mV/\177C GLn 1dB"

 .align 2
POp1Temp:	.word 0x4120, 0xA400, (TXT_mVpTC << 12) | oFR_TC_OP1, 0x4010
POptTemp:	.word 0x4120, 0xA400, (TXT_mVpTC << 12) | oFR_TC_OPT, 0x4010
PGenHpTemp:	.word 0x0121, 0x2303, (TXT_mVpTC << 12) | oFR_TC_GENV_HP, SetUpGain, 0x4010
PGenHnTemp:	.word 0x0121, 0x2303, (TXT_mVpTC << 12) | oFR_TC_GENV_HN, SetUpGain, 0x4010
PGenLpTemp:	.word 0x0121, 0x2303, (TXT_mVpTC << 12) | oFR_TC_GENV_LP, SetUpGain, 0x4010
PGenLnTemp:	.word 0x0121, 0x2303, (TXT_mVpTC << 12) | oFR_TC_GENV_LN, SetUpGain, 0x4010
PGenHp1dBT:	.word 0x0121, 0x2303, (TXT_mVpTC << 12) | oFR_TC_GENV_HP_1DB, SetUpGain, 0x4010
PGenHn1dBT:	.word 0x0121, 0x2303, (TXT_mVpTC << 12) | oFR_TC_GENV_HN_1DB, SetUpGain, 0x4010
PGenLp1dBT:	.word 0x0121, 0x2303, (TXT_mVpTC << 12) | oFR_TC_GENV_LP_1DB, SetUpGain, 0x4010
PGenLn1dBT:	.word 0x0121, 0x2303, (TXT_mVpTC << 12) | oFR_TC_GENV_LN_1DB, SetUpGain, 0x4010

tabTempParam:
	.word 10
	.word tOp1Temp,		Param_Open,	POp1Temp
	.word tOptTemp,		Param_Open,	POptTemp
	.word tGenHpTemp,	Param_Open,	PGenHpTemp
	.word tGenHnTemp,	Param_Open,	PGenHnTemp
	.word tGenLpTemp,	Param_Open,	PGenLpTemp
	.word tGenLnTemp,	Param_Open,	PGenLnTemp
	.word tGenHp1dBT,	Param_Open,	PGenHp1dBT
	.word tGenHn1dBT,	Param_Open,	PGenHn1dBT
	.word tGenLp1dBT,	Param_Open,	PGenLp1dBT
	.word tGenLn1dBT,	Param_Open,	PGenLn1dBT


/*  ----------- ADC Param ----------------------------------------------- */
.if ADC_PA_MCP3421_AUTORANGE
tPaAdcLOffs:	.asciz	" 1 PA ADL Offs"
tPaAdcLKorrP:	.asciz	" 2 PA ADL Korr+"
tPaAdcLKorrN:	.asciz	" 3 PA ADL Korr-"
tPaAdcLKTemp:	.asciz	" 4 PA ADL %/\177C"
.endif
tPaAdcHOffs:	.asciz	" 5 PA ADH Offs"
tPaAdcHKorrP:	.asciz	" 6 PA ADH Korr+"
tPaAdcHKorrN:	.asciz	" 7 PA ADH Korr-"
tPaAdcHKTemp:	.asciz	" 8 PA ADH %/\177C"

tPaAdcTemp:	.asciz	" 9 PA ADC mV/\177C"

tOP1AdcOffs:	.asciz	"10 OP1 ADC Offs"
tOP1AdcKorrP:	.asciz	"11 OP1 ADC Korr+"
tOP1AdcKorrN:	.asciz	"12 OP1 ADC Korr-"
tOP1AdcTemp:	.asciz	"13 OP1 ADC mV/\177C"
tOP1AdcKTemp:	.asciz	"14 OP1 ADC %/\177C"

 .align 2

.if ADC_PA_MCP3421_AUTORANGE
PPaAdcLOffs:  .word 0x0111, 0x2400, (TXT_mV << 12) | oFR_ADCL_PA_OFFS, MCP3421_SetParam, 0x0010
PPaAdcLKorrP: .word 0x0121, 0x3500, (TXT_perc << 12) | oFR_ADCL_PA_KORR_P, MCP3421_SetParam, 0x0010
PPaAdcLKorrN: .word 0x0121, 0xB402, (TXT_perc << 12) | oFR_ADCL_PA_KORR_N, MCP3421_SetParam, 0x0010
PPaAdcKTemp:  .word 0x0121, 0xC502, oFR_ADCL_PA_K_TC, MCP3421_SetParam, 0x0010
.endif

PPaAdcHOffs:  .word 0x0111, 0x2400, (TXT_mV << 12) | oFR_ADCH_PA_OFFS, MCP3421_SetParam, 0x0010
PPaAdcHKorrP: .word 0x0121, 0x3500, (TXT_perc << 12) | oFR_ADCH_PA_KORR_P, MCP3421_SetParam, 0x0010
PPaAdcHKorrN: .word 0x0121, 0xB402, (TXT_perc << 12) | oFR_ADCH_PA_KORR_N, MCP3421_SetParam, 0x0010
PPaAdcHKTemp: .word 0x0121, 0xC502, oFR_ADCH_PA_K_TC, MCP3421_SetParam, 0x0010

PPaAdcTemp:   .word 0x0121, 0x2302, (TXT_mVpTC << 12) | oFR_ADC_PA_TC, MCP3421_SetParam, 0x0010

POP1AdcOffs:  .word 0x4111, 0x0400, (TXT_dig << 12)  | oFR_ADC_OP1_OFFS, ADS112c04_SetParam, 0x4010
POP1AdcKorrP: .word 0x0121, 0x3500, (TXT_perc << 12) | oFR_ADC_OP1_KORR_P, ADS112c04_SetParam, 0x4010
POP1AdcKorrN: .word 0x0121, 0xB402, (TXT_perc << 12) | oFR_ADC_OP1_KORR_N, ADS112c04_SetParam, 0x4010
POP1AdcTemp:  .word 0x0121, 0x2302, (TXT_mVpTC << 12) | oFR_ADC_OP1_TC, ADS112c04_SetParam, 0x4010
POP1AdcKTemp: .word 0x0121, 0xC502, oFR_ADC_OP1_K_TC, ADS112c04_SetParam, 0x4010


tabAdcParam:
.if ADC_PA_MCP3421_AUTORANGE
	.word 14
	.word tPaAdcLOffs,	Param_Open,	PPaAdcLOffs
	.word tPaAdcLKorrP,	Param_Open,	PPaAdcLKorrP
	.word tPaAdcLKorrN,	Param_Open,	PPaAdcLKorrN
	.word tPaAdcLKTemp,	Param_Open,	PPaAdcKTemp
.else
	.word 10
.endif
	.word tPaAdcHOffs,	Param_Open,	PPaAdcHOffs
	.word tPaAdcHKorrP,	Param_Open,	PPaAdcHKorrP
	.word tPaAdcHKorrN,	Param_Open,	PPaAdcHKorrN
	.word tPaAdcHKTemp,	Param_Open,	PPaAdcHKTemp

	.word tPaAdcTemp,	Param_Open,	PPaAdcTemp

	.word tOP1AdcOffs,	Param_Open,	POP1AdcOffs
	.word tOP1AdcKorrP,	Param_Open,	POP1AdcKorrP
	.word tOP1AdcKorrN,	Param_Open,	POP1AdcKorrN
	.word tOP1AdcTemp,	Param_Open,	POP1AdcTemp
	.word tOP1AdcKTemp,	Param_Open,	POP1AdcKTemp

/*  ----------- PA DC Offs ---------------------------------------------- */
tPaOffs:	.asciz	" 1 PA Offs"
tPaTemp:	.asciz	" 2 PA mV/\177C"
tDacPowKTemp:	.asciz	" 3 DAC Pow %/\177C"
tDacDC_KTemp:	.asciz	" 4 DAC DC  %/\177C"
tDacPaPowVK:	.asciz	" 5 DAC Pow uV/d"
tDacPaDC_VK:	.asciz	" 6 DAC DC  uV/d"
tDacPaOffVK:	.asciz	" 7 DAC Off uV/d"
tPaOffsErrDC:	.asciz	" 8 PA Off.err DC"

 .align 2
PPaOffs:     .word 0x2111, 0xA302, (TXT_mV << 12) | oFR_OFF_PA,  SetParamPa, 0x0001  /* use timer to set */
PPaTemp:     .word 0x0121, 0x2402, (TXT_mVpTC << 12) | oFR_TC_PA_OUT, SetParamPa, 0x0001
PDacPowKTemp: .word 0x0121, 0x4502, oFR_DAC_POW_TC, SetParamOffsTemp, 0x0001
PDacDC_KTemp: .word 0x0121, 0x4502, oFR_DAC_DC_TC,  SetParamOffsTemp, 0x0001
PDacPaPowVK: .word 0x0125, 0xBD04, oFR_PA_POW_V_KORR, SetUpOffs, DAC_PA_POW_BASE, 0x0001
PDacPaDC_VK: .word 0x0125, 0xBD04, oFR_PA_DC_V_KORR, SetUpOffs, DAC_PA_POW_BASE, 0x0001
PDacPaOffVK: .word 0x5125, 0xB501, oFR_PA_OFFS_V_KORR, SetParamPa, DAC_PA_OFFSET_BASE, 0x0001
PPaOffsErrDC: .word 0x2111, 0xA302, (TXT_mV << 12) | oFR_OFF_PAEDC,  SetParamPa, 0x0001  /* use timer to set */

tabPaDCMenu:
	.word 8
	.word tPaOffs,		Param_Open,	PPaOffs
	.word tPaTemp,		Param_Open,	PPaTemp
	.word tDacPowKTemp,	Param_Open,	PDacPowKTemp
	.word tDacDC_KTemp,	Param_Open,	PDacDC_KTemp
	.word tDacPaPowVK,	Param_Open,	PDacPaPowVK
	.word tDacPaDC_VK,	Param_Open,	PDacPaDC_VK
	.word tDacPaOffVK,	Param_Open,	PDacPaOffVK
	.word tPaOffsErrDC,	Param_Open,	PPaOffsErrDC

/*  ----------- Gain ---------------------------------------------------- */
tGainPA:	.asciz	" 1 PA Gain"
tGainK1:	.asciz	" 2 Gain 2dB"
tGainK2:	.asciz	" 3 Gain 4dB"
tGainK3:	.asciz	" 4 Gain 8dB"
tGainK4:	.asciz	" 5 Gain 16dB"
tGainK5:	.asciz	" 6 Gain 32dB"
tGVHp:		.asciz	" 7 GenV Hp"
tGVHn:		.asciz	" 8 GenV Hn"
tGVLp:		.asciz	" 9 GenV Lp"
tGVLn:		.asciz	"10 GenV Ln"
tGVHp1dB:	.asciz	"11 GenV Hp 1dB"
tGVHn1dB:	.asciz	"12 GenV Hn 1dB"
tGVLp1dB:	.asciz	"13 GenV Lp 1dB"
tGVLn1dB:	.asciz	"14 GenV Ln 1dB"
tGainKFilt:	.asciz	"15 Gain K Filt"
tGainK60cm:	.asciz	"16 Gain K 60cm"

 .align 2
PGainPA: .word 0x0125, 0xCD04, oFR_GAIN_PA, SetUpGain, PA_GAIN_BASE, 0x2431
PGainK1: .word 0x4125, 0xDD04, oFR_GAIN_02DB, SetUpGain, K02DB_GAIN_BASE, 0x2431
PGainK2: .word 0x4125, 0xDD04, oFR_GAIN_04DB, SetUpGain, K04DB_GAIN_BASE, 0x2431
PGainK3: .word 0x5125, 0xDD04, oFR_GAIN_08DB, SetUpGain, K08DB_GAIN_BASE, 0x2431
PGainK4: .word 0x6125, 0xDD04, oFR_GAIN_16DB, SetUpGain, K16DB_GAIN_BASE, 0x2431
PGainK5: .word 0x8125, 0xDD04, oFR_GAIN_32DB, SetUpGain, K32DB_GAIN_BASE, 0x2431
PGVHp:    .word 0x4125, 0xAD04, (TXT_mV<<12)| oFR_GENV_HP, SetUpGain, GENV_MAX_BASE, 0x2431
PGVHn:    .word 0x4125, 0xAD04, (TXT_mV<<12)| oFR_GENV_HN, SetUpGain, -GENV_MAX_BASE, 0x2431
PGVLp:    .word 0x4125, 0xAD04, (TXT_mV<<12)| oFR_GENV_LP, SetUpGain, GENV_MIN_BASE, 0x2431
PGVLn:    .word 0x4125, 0xAD04, (TXT_mV<<12)| oFR_GENV_LN, SetUpGain, -GENV_MIN_BASE, 0x2431
PGVHp1dB: .word 0x4125, 0xAD04, (TXT_mV<<12)| oFR_GENV_HP_1DB, SetUpGain, GENV_MIN_BASE, 0x2431
PGVHn1dB: .word 0x4125, 0xAD04, (TXT_mV<<12)| oFR_GENV_HN_1DB, SetUpGain, -GENV_MIN_BASE, 0x2431
PGVLp1dB: .word 0x4125, 0xAD04, (TXT_mV<<12)| oFR_GENV_LP_1DB, SetUpGain, GENV_MIN_BASE_1DB, 0x2431
PGVLn1dB: .word 0x4125, 0xAD04, (TXT_mV<<12)| oFR_GENV_LN_1DB, SetUpGain, -GENV_MIN_BASE_1DB, 0x2431
PGainKFilt: .word 0x8125, 0xDD04, oFR_GAIN_KFILT, SetUpGain, 10000,  0x0001
PGainK60cm: .word 0x8125, 0xDD04, oFR_GAIN_K60CM, SetUpGain, 10000,  0x0001

tabGainMenu:
	.word 16
	.word tGainPA,		Param_Open,	PGainPA
	.word tGainK1,		Param_Open,	PGainK1
	.word tGainK2,		Param_Open,	PGainK2
	.word tGainK3,		Param_Open,	PGainK3
	.word tGainK4,		Param_Open,	PGainK4
	.word tGainK5,		Param_Open,	PGainK5
	.word tGVHp,		Param_Open,	PGVHp
	.word tGVHn,		Param_Open,	PGVHn
	.word tGVLp,		Param_Open,	PGVLp
	.word tGVLn,		Param_Open,	PGVLn
	.word tGVHp1dB,		Param_Open,	PGVHp1dB
	.word tGVHn1dB,		Param_Open,	PGVHn1dB
	.word tGVLp1dB,		Param_Open,	PGVLp1dB
	.word tGVLn1dB,		Param_Open,	PGVLn1dB
	.word tGainKFilt,	Param_Open,	PGainKFilt
	.word tGainK60cm,	Param_Open,	PGainK60cm

/*  ----------- Filter -------------------------------------------------- */
tFi10nsKx:	.asciz	" 1 Fi 1.0ns"
tFi12nsKx:	.asciz	" 2 Fi 1.2ns"
tFi15ns:	.asciz	" 3 Fi 1.5ns"
tFi18ns:	.asciz	" 4 Fi 1.8ns"
tFi20ns:	.asciz	" 5 Fi 2.0ns"
tFimax:		.asciz	" 6 Fi max"
tFiOp1lc:	.asciz	" 7 Fi OP1 LC"
tFiLongt:	.asciz	" 8 Fi Longtime"
tFiGenTC:	.asciz	" 9 Fi Gen d/\177C"

 .align 2
PFiOp1lc: .word 0x0001, 0x0303, oFR_OP1LC, SetUpFilter

tabFilterMenu:
	.word 9
	.word tFi10nsKx,	OpenAlgMenu,	tabFi10nsKx
	.word tFi12nsKx,	OpenAlgMenu,	tabFi12nsKx
	.word tFi15ns,		Filter_Open,	oFR_FILT_15ns
	.word tFi18ns,		Filter_Open,	oFR_FILT_18ns
	.word tFi20ns,		Filter_Open,	oFR_FILT_20ns
	.word tFimax,		Filter_Open,	oFR_FILT_max
	.word tFiOp1lc,		Param_Open,	PFiOp1lc
	.word tFiLongt,		OpenAlgMenu,	tabFiLongTime
	.word tFiGenTC,		OpenAlgMenu,	tabFiGenTC

/*  ----------- Filter LongTime ----------------------------------------- */
tGenPlt:	.asciz	" 1 Gen+ lt"
tGenPltT:	.asciz	" 3 Gen+ lt Time"
tGenNlt:	.asciz	" 3 Gen- lt"
tGenNltT:	.asciz	" 4 Gen- lt Time"

 .align 2
PGenPlt:	.word 0x1000, 0x0302, oFR_FILT_GENP_LT
PGenPltT:	.word 0x1000, 0x0303, oFR_FILT_GENP_LT_TIME
PGenNlt:	.word 0x1000, 0x0302, oFR_FILT_GENN_LT
PGenNltT:	.word 0x1000, 0x0303, oFR_FILT_GENN_LT_TIME

tabFiLongTime:
	.word 4
	.word tGenPlt,		Param_Open,	PGenPlt
	.word tGenPltT,		Param_Open,	PGenPltT
	.word tGenNlt,		Param_Open,	PGenNlt
	.word tGenNltT,		Param_Open,	PGenNltT

/*  ----------- Filter Kx ----------------------------------------------- */
tFi10ns:	.asciz	" 1 Fi 1.0ns"
tFi10nsK1:	.asciz	" 2 Fi 1.0ns K1"
tFi10nsK2:	.asciz	" 3 Fi 1.0ns K2"
tFi10nsK3:	.asciz	" 4 Fi 1.0ns K3"
tFi10nsK4:	.asciz	" 5 Fi 1.0ns K4"
tFi10nsK5:	.asciz	" 6 Fi 1.0ns K5"
tFi10nsK6:	.asciz	" 7 Fi 1.0ns K6"
tFi10nsK7:	.asciz	" 8 Fi 1.0ns K7"

 .align 2
tabFi10nsKx:
	.word 8
	.word tFi10ns,		Filter_Open,	oFR_FILT_10ns
	.word tFi10nsK1,	FiltKx_Open,	oFR_FILT_10ns_K1
	.word tFi10nsK2,	FiltKx_Open,	oFR_FILT_10ns_K2
	.word tFi10nsK3,	FiltKx_Open,	oFR_FILT_10ns_K3
	.word tFi10nsK4,	FiltKx_Open,	oFR_FILT_10ns_K4
	.word tFi10nsK5,	FiltKx_Open,	oFR_FILT_10ns_K5
	.word tFi10nsK6,	FiltKx_Open,	oFR_FILT_10ns_K6
	.word tFi10nsK7,	FiltKx_Open,	oFR_FILT_10ns_K7

tFi12ns:	.asciz	" 1 Fi 1.2ns"
tFi12nsK0:	.asciz	" 2 Fi 1.2ns K0"
tFi12nsK1:	.asciz	" 3 Fi 1.2ns K1"
tFi12nsK2:	.asciz	" 4 Fi 1.2ns K2"
tFi12nsK3:	.asciz	" 5 Fi 1.2ns K3"

 .align 2
tabFi12nsKx:
	.word 5
	.word tFi12ns,		Filter_Open,	oFR_FILT_12ns
	.word tFi12nsK0,	FiltKx_Open,	oFR_FILT_12ns_K0
	.word tFi12nsK1,	FiltKx_Open,	oFR_FILT_12ns_K1
	.word tFi12nsK2,	FiltKx_Open,	oFR_FILT_12ns_K2
	.word tFi12nsK3,	FiltKx_Open,	oFR_FILT_12ns_K3

/*  ----------- Filter Gen TempKorr ------------------------------------- */
tFiGenH0dB_TC:	.asciz	" 1 Fi H 0dB d/\177C"
tFiGenL0dB_TC:	.asciz	" 1 Fi L 0dB d/\177C"
tFiGenH1dB_TC:	.asciz	" 1 Fi H 1dB d/\177C"
tFiGenL1dB_TC:	.asciz	" 1 Fi L 1dB d/\177C"
 .align 2
PFiGenH0dB_TC:  .word 0x0121, 0x1302, oFR_GEN_H_0DB_TC, FilterTC_SetParam, 0x4010
PFiGenL0dB_TC:  .word 0x0121, 0x1302, oFR_GEN_L_0DB_TC, FilterTC_SetParam, 0x4010
PFiGenH1dB_TC:  .word 0x0121, 0x1302, oFR_GEN_H_1DB_TC, FilterTC_SetParam, 0x4010
PFiGenL1dB_TC:  .word 0x0121, 0x1302, oFR_GEN_L_1DB_TC, FilterTC_SetParam, 0x4010

tabFiGenTC:
	.word 4
	.word tFiGenH0dB_TC,	Param_Open,	PFiGenH0dB_TC
	.word tFiGenL0dB_TC,	Param_Open,	PFiGenL0dB_TC
	.word tFiGenH1dB_TC,	Param_Open,	PFiGenH1dB_TC
	.word tFiGenL1dB_TC,	Param_Open,	PFiGenL1dB_TC

/*  ----------- SymData ------------------------------------------------- */
tSym1M:		.asciz	" 1 Sym   1 MHz"
tSym2M:		.asciz	" 2 Sym   2 MHz"
tSym5M:		.asciz	" 3 Sym   5 MHz"
tSym10M:	.asciz	" 4 Sym  10 MHz"
tSym20M:	.asciz	" 5 Sym  20 MHz"
tSym25M:	.asciz	" 6 Sym  25 MHz"
tSym30M:	.asciz	" 7 Sym  30 MHz"
tSym40M:	.asciz	" 8 Sym  40 MHz"
tSym50M:	.asciz	" 9 Sym  50 MHz"
tSym60M:	.asciz	"10 Sym  60 MHz"
tSym75M:	.asciz	"11 Sym  75 MHz"
tSym88M:	.asciz	"12 Sym  88 MHz"
tSym100M:	.asciz	"13 Sym 100 MHz"
tSym110M:	.asciz	"14 Sym 110 MHz"
tSym120M:	.asciz	"15 Sym 120 MHz"
tSym135M:	.asciz	"16 Sym 135 MHz"
tSym150M:	.asciz	"17 Sym 150 MHz"
tSym160M:	.asciz	"18 Sym 160 MHz"
tSym170M:	.asciz	"19 Sym 170 MHz"
tSym190M:	.asciz	"20 Sym 190 MHz"
tSym200M:	.asciz	"21 Sym 200 MHz"
tSym210M:	.asciz	"22 Sym 210 MHz"
tSym220M:	.asciz	"23 Sym 220 MHz"
tSym230M:	.asciz	"24 Sym 230 MHz"

 .align 2
tabSymMenu:
	.word 24
	.word tSym1M,		OpenSymMenu,	0
	.word tSym2M,		OpenSymMenu,	1
	.word tSym5M,		OpenSymMenu,	2
	.word tSym10M,		OpenSymMenu,	3
	.word tSym20M,		OpenSymMenu,	4
	.word tSym25M,		OpenSymMenu,	5
	.word tSym30M,		OpenSymMenu,	6
	.word tSym40M,		OpenSymMenu,	7
	.word tSym50M,		OpenSymMenu,	8
	.word tSym60M,		OpenSymMenu,	9
	.word tSym75M,		OpenSymMenu,	10
	.word tSym88M,		OpenSymMenu,	11
	.word tSym100M,		OpenSymMenu,	12
	.word tSym110M,		OpenSymMenu,	13
	.word tSym120M,		OpenSymMenu,	14
	.word tSym135M,		OpenSymMenu,	15
	.word tSym150M,		OpenSymMenu,	16
	.word tSym160M,		OpenSymMenu,	17
	.word tSym170M,		OpenSymMenu,	18
	.word tSym190M,		OpenSymMenu,	19
	.word tSym200M,		OpenSymMenu,	20
	.word tSym210M,		OpenSymMenu,	21
	.word tSym220M,		OpenSymMenu,	22
	.word tSym230M,		OpenSymMenu,	23


/*  ----------- Puls ---------------------------------------------------- */
tPuls2ns:	.asciz	" 1 Puls   2ns"
tPuls7ns:	.asciz	" 2 Puls   7ns"
tPuls7ns1:	.asciz	" 3 Puls   7.1ns"
tPuls19ns9:	.asciz	" 4 Puls  19.9ns"
tPuls20ns:	.asciz	" 5 Puls  20ns"
tPuls99ns:	.asciz	" 6 Puls  99ns"
tPuls100ns:	.asciz	" 7 Puls 100ns"
tPuls319ns:	.asciz	" 8 Puls 319ns"
tPuls320ns:	.asciz	" 9 Puls 320ns"
tPuls999ns:	.asciz	"10 Puls 999ns"
tPuls1us:	.asciz	"11 Puls  1us"
tPuls3us99:	.asciz	"12 Puls  3.99us"
tPuls4us:	.asciz	"13 Puls  4us"
tPuls9us99:	.asciz	"14 Puls  9.99us"
tPuls10us:	.asciz	"15 Puls 10us"
tPuls62us5:	.asciz	"16 Puls 62.5us"

 .align 2
PPuls2ns:    .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_2NS, SetPuls
PPuls7ns:    .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_7NS, SetPuls
PPuls7ns1:   .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_7NS1, SetPuls
PPuls19ns9:  .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_19NS9, SetPuls
PPuls20ns:   .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_20NS, SetPuls
PPuls99ns:   .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_99NS, SetPuls
PPuls100ns:  .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_100NS, SetPuls
PPuls319ns:  .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_319NS, SetPuls
PPuls320ns:  .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_320NS, SetPuls
PPuls999ns:  .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_999NS, SetPuls
PPuls1us:    .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_1US, SetPuls
PPuls3us99:  .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_3US99, SetPuls
PPuls4us:    .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_4US, SetPuls
PPuls9us99:  .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_9US99, SetPuls
PPuls10us:   .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_10US, SetPuls
PPuls62us5:  .word 0x7021, 0x0300, (TXT_dig<<12)| oFR_PULS_62US5, SetPuls

tabPulsMenu:
	.word 16
	.word tPuls2ns,		Param_Open,	PPuls2ns
	.word tPuls7ns,		Param_Open,	PPuls7ns
	.word tPuls7ns1,	Param_Open,	PPuls7ns1
	.word tPuls19ns9,	Param_Open,	PPuls19ns9
	.word tPuls20ns,	Param_Open,	PPuls20ns
	.word tPuls99ns,	Param_Open,	PPuls99ns
	.word tPuls100ns,	Param_Open,	PPuls100ns
	.word tPuls319ns,	Param_Open,	PPuls319ns
	.word tPuls320ns,	Param_Open,	PPuls320ns
	.word tPuls999ns,	Param_Open,	PPuls999ns
	.word tPuls1us,		Param_Open,	PPuls1us
	.word tPuls3us99,	Param_Open,	PPuls3us99
	.word tPuls4us,		Param_Open,	PPuls4us
	.word tPuls9us99,	Param_Open,	PPuls9us99
	.word tPuls10us,	Param_Open,	PPuls10us
	.word tPuls62us5,	Param_Open,	PPuls62us5

/*  ----------- AutoCal ------------------------------------------------- */
tAcSave:	.asciz	" 1 AC data Save"
tAcGen:		.asciz	" 2 AutoCal Gen"
tAcOffsPA:	.asciz	" 3 PA Dc V"
tAcOffsPAV:	.asciz	" 4 PA Pow V"
tAcGain:	.asciz	" 5 PA Gain"

 .align 2
;PAcSave:	.word	ac_save_mod, bACS_SAVE_DATA, 0
PAcSave:	.word	bACS_SAVE_DATA, ac_save_mod

tabAutoCal:
	.word 5
	.word tAcGain	,	AutoCal_Open,	AC_GAIN
	.word tAcSave,		Flags_Open,	PAcSave
	.word tAcGen,		AutoCal_Open,	AC_GEN_V
	.word tAcOffsPA,	AutoCal_Open,	AC_OFFS_PA
	.word tAcOffsPAV,	AutoCal_Open,	AC_OFFS_PA_POW

/*  ----------- Test ---------------------------------------------------- */
;tTestDac:	.asciz	" 1 Test DAC On"
tTestMod:	.asciz	" 1 Test Mod"
tTestDacs:	.asciz	" 2 Test DACs"
tTestKatt:	.asciz	" 3 Test K ATT"
tDcGenOn:	.asciz	" 4 DC Gen On"
tDcFreqOn:	.asciz	" 5 DC Freq On"
tTestFan:	.asciz	" 6 Test Fan"
tTestPow:	.asciz	" 7 Test Pow"
.if ADC_PA_MCP3421_AUTORANGE
tTestPaDac:	.asciz	" 8 PA DAC H"
.endif

 /* --- Txt 2 --- */
txtTMod:	.asciz	"b2=Pow b1=Fan b0=Dac"
txtKatt:	.asciz	"K ATT bit 32dB .. 1dB"
txtFan:		.asciz	"b3=H2 b2=H1 b1=F2 b0=F1"
txtPow:		.asciz	"3=GenV 2=OP1 1=PaO 0=PaV"

 .align 2

;PTestDac:	.word	TestFlags, bTF_DAC, 0
PTestMod:	.word	LCDF_TXT_CS | LCDF_ONCANGE | LCDF_BITMOD | 0x20, TestFlags, OnChangeTestFlagMod, txtTMod
PTestKatt:	.word	LCDF_TXT_CS | LCDF_NOVALUE | LCDF_ONCANGE | LCDF_BITMOD | 0x50, OnChangeTestKatt, txtKatt
;PDcGenOn:	.word	TestFlags, TF_DC_GEN_ON, 3
PDcGenOn:	.word	LCDF_ONCANGE | TF_DC_GEN_ON, TestFlags, OnChangeTestFlagDc
PDcFreqOn:	.word	LCDF_ONCANGE | TF_DC_FREQ_ON, TestFlags, OnChangeTestFlagDcFq
PTestFan:	.word	LCDF_TXT_CS | LCDF_NOVALUE | LCDF_ONCANGE | LCDF_BITMOD | 0x30, OnChangeTestFlagFan, txtFan
PTestPow:	.word	LCDF_TXT_CS | LCDF_NOVALUE | LCDF_ONCANGE | LCDF_BITMOD | 0x30, OnChangeTestFlagPow, txtPow

.if ADC_PA_MCP3421_AUTORANGE
PTestPaDac:	.word	TF_PA_ADC_H_ON, TestFlags
.endif

tabTest:
.if ADC_PA_MCP3421_AUTORANGE
	.word 8
.else
	.word 7
.endif
;	.word tTestDac,		TestBit_Open,		PTestDac
	.word tTestMod,		Flags_Open,		PTestMod
	.word tTestDacs,	OpenAlgMenu,		tabDac
	.word tTestKatt,	Flags_Open,		PTestKatt
	.word tDcGenOn,		Flags_Open,		PDcGenOn
	.word tDcFreqOn,	Flags_Open,		PDcFreqOn
	.word tTestFan,		Flags_Open,		PTestFan
	.word tTestPow,		Flags_Open,		PTestPow
.if ADC_PA_MCP3421_AUTORANGE
	.word tTestPaDac,	Flags_Open,		PTestPaDac
.endif


/* ______________________________________________________________________ */
tdacRF:		.asciz	" 1 Gen R/F"
tdacLin:	.asciz	" 2 Gen Lin"
tdacAmpl:	.asciz	" 3 Gen Ampl/Filt"
tdacSym:	.asciz	" 4 Gen Sym"
tdacPuls:	.asciz	" 5 Gen Puls"
tdacPaOffs:	.asciz	" 6 Pa Offs"
tdacOpOffs:	.asciz	" 7 OPs Offs"
tdacFan:	.asciz	" 8 Fan PWM"
tdacGenVpp:	.asciz	" 9 Gen Vpp"
tdacPaDcV:	.asciz	"10 PA Dc V"
tdacPaPowV:	.asciz	"11 PA Pow V"
 .align 2

/*
word 0
	b0..b1	max.line-1
 */
PdacRF:		.word 0x00F3,	AGRP0,	AGRC,	AGRN0,	AGRN1
PdacLin:	.word 0x0003,	AGLIN0,	AGLIN1,	AOP1LC,	ALINPA
PdacAmpl:	.word 0x0003,	AGVP,	AGVN,	AFR0,	AFR1
PdacSym:	.word 0x0003,	OSYM,	APSYM,	AGTI0,	AGTI1
PdacPuls:	.word 0x0001,	VIMP,	AIMP,	0,	0
PdacPaOffs:	.word 0x0002,	PAVO,	PA_POW,	PA_DC,	0
PdacOpOffs:	.word 0x0001,	AOP1OFF,AOPTOFF,0,	0
PdacFan:	.word 0x0000,	FAN_PWM, 0,	0,	0
PdacGenVpp:	.word 0x0109,	0x1501, (TXT_mV<<12) | 0x0800, VxOcGenAmpl, 0x40CD
PdacPaDcV:	.word 0x010B,	0x1404, (TXT_mV<<12) | 0x0800, VxOcPaOffs, 0, 0xC100
PdacPaPowV:	.word 0x010B,	0x1404, (TXT_mV<<12) | 0x0800, VxOcPaOffs, 1, 0xC100

tabDac:
	.word 11
	.word tdacRF,		LcdDac_Open,	PdacRF
	.word tdacLin,		LcdDac_Open,	PdacLin
	.word tdacAmpl,		LcdDac_Open,	PdacAmpl
	.word tdacSym,		LcdDac_Open,	PdacSym
	.word tdacPuls,		LcdDac_Open,	PdacPuls
	.word tdacPaOffs,	LcdDac_Open,	PdacPaOffs
	.word tdacOpOffs,	LcdDac_Open,	PdacOpOffs
	.word tdacFan,		LcdDac_Open,	PdacFan
	.word tdacGenVpp,	Param_Open,	PdacGenVpp
	.word tdacPaDcV,	Param_Open,	PdacPaDcV
	.word tdacPaPowV,	Param_Open,	PdacPaPowV


/* ______________________________________________________________________ */
