/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/eingVx.inc"
.include "lib/Analog.inc"
.include "lib/Param.inc"

.if VX_FILTER_COUNT < VXF_10ns_K0_COUNT
.ERROR  "VX_FILTER_COUNT > VXF_10ns_K0_COUNT"
.endif

.section .nbss, bss, near

Last_CursLineIdx:	.space 4
Last_CursSelIdx:	.space 2
akt_filter_id:		.space 2

/* ______________________________________________________________________ */
.section .text

.global LCD_FiltKx
.global Key_FiltKx
.global init_FilterKxPara
.global SetupFilterKx

.if USE_UART_LIST_PARA
.global get_val_FilterKx
.endif

LCD_FiltKx:
	dec	w0, w0
	bra	z, LCD_MenuCaption_cs
	dec	w0, w0
	mov	w0, w1
	bra	z, Ldispl_val
	dec	w0, w0
	bra	z, Ldispl_val
	dec	w0, w0
	bra	z, Ldispl_val
	dec	w0, w0
	bra	z, Ldispl_val
	dec	w0, w0
	bra	z, Ldispl_val
	bra	LCD_Paint

Ldispl_val:
	mul.uu	w1, #EVX_SIZE, w0
	mov	#EingVx0, w5
	add	w5, w0, w5
	nop

	mov	[w5+oEVX_CURS], w0
	btst	w0, #bEVCURS_ON
	bra	z, LCD_Step_Inc

	mov	[w5+oEVX_DISP_X], w0
	sub	w0, #30, w0
	mov	[w5+oEVX_DISP_Y], w1
	mov	[w5+oEVX_P_TXT], w2
	rcall	StrOut_cs
	rcall	DispVxVal
	inc	LCD_Step
	return

/* ______________________________________________________________________ */
Key_FiltKx:
	mov	#EingVx0, w5
	rcall	EingVx_Key
	mov	#EingVx1, w5
	rcall	EingVx_Key
	mov	#EingVx2, w5
	rcall	EingVx_Key
	mov	#EingVx3, w5
	rcall	EingVx_Key
	mov	#EingVx4, w5
	rcall	EingVx_Key

	cp	w0, #KEY_RET
	bra	z, K_CRet
	return

K_CRet:
	mov	EingVx_CursSelIdx, w0
	mov	w0, Last_CursSelIdx

	mov	#Last_CursLineIdx, w6
	rcall	EingVx_Pop_CursLineIdx

	mov	#VXF_10ns_K0_COUNT, w2
	mov	EingVx0+oEVX_RAM_DATA_ADR, w0
	mov	#EEData0, w1
	add	w0, w1, w0
	mov	#Filter_fram_data, w1
lloop:
	mov.b	[w0++], w3
	cp.b	w3, [w1++]
	bra	nz, lsave
	dec	w2, w2
	bra	nz, lloop

	bra	Key_RetOk_Menu

lsave:
	mov	akt_filter_id, w2
	mov	#EVCURS_FRAM_MASK, w1
	and	w2, w1, w2

	mov	#VXF_10ns_K0_COUNT-1, w3
	call	FRam_Save
	bra	Key_RetOk_Menu

/* ______________________________________________________________________ */
/* param data pointer w4 */
init_FilterKxPara:
	mov	#Last_CursLineIdx, w6
	rcall	EingVx_Push_CursLineIdx

	mov	#EVCURS_FRAM_MASK, w0
	and	w4, w0, w2		; w9=Fram
	mov	#VXF_10ns_K0_COUNT-1, w3
	rcall	FRam_GetAdr
	btsc	SR, #OV
	return
	mov	#Filter_fram_data, w0
	repeat	w3
	mov.b	[w2++], [w0++]

	mov	akt_filter_id, w0
	cp	w0, w4
	bra	z, ifp_ok
	clr	Last_CursSelIdx
	mov	w4, akt_filter_id
ifp_ok:

	push.d	w8
	push.d	w10
	push	w12

	mov	#VXF_10ns_K0_COUNT-1, w0		/* max.line-1 */
	mov	Last_CursSelIdx, w1		/* akt.line */
	and	#EVCURS_SELNR_MASK, w1
	lsr	w1, #EVCURS_SELNR_SHL, w1
	rcall	EingVx_InitCursLineIdx

	mov	#0, w8			; w8=nr
	mov	#EVCURS_FRAM_MASK, w0
	and	w4, w0, w9		; w9=Fram
	mov	#30, w10		; w10=x
	mov	#10, w11		; w11=y

	mov	#14, w12		; GRN0
	rcall	init_line
	mov	#12, w12		; GRN1
	rcall	init_line
	mov	#24, w12		; GLIN  GLIN0+LINPA
	rcall	init_line
	mov	#2, w12			; GLIN1
	rcall	init_line
	mov	#5, w12			; FR1
	rcall	init_line


	pop	w12
	pop.d	w10
	pop.d	w8
	return

/* w8=nr w9=Fram w10=x w11=y w12=txt */
init_line:
	mov	w8, w0			/* line.idx */
	mul.uu	w0, #EVX_SIZE, w0
	mov	#EingVx0, w5
	add	w5, w0, w5

	mov	w8, w0			/* line.idx */
	mov	#0, w1			/* akt.X.pos */
	rcall	EingVx_SetCursSelIdx
	mov	#EVCURS_SELFKT_LEFT, w0
	rcall	EingVx_SetCursSelFkt

	mov	#Filter_fram_data, w2
	add	w2, w8, w2
	nop
	se	[w2], w0
	mov	#-128, w2		/* min. */
	mov	#127, w3		/* max. */
	mov	#0x0312, w4
	rcall	EingVx_SetValueMinMaxDflag

	mov	w9,  [w5+oEVX_RAM_DATA_ADR]
	mov	w10, [w5+oEVX_DISP_X]
	mov	w11, [w5+oEVX_DISP_Y]

	mov	w12, w0
	rcall	GetDAC_txt
	mov	w0, [w5+oEVX_P_TXT]

	mov	pgkx_on_change, w0
	rcall	EingVx_Init_OnCange

	inc	w8, w8
	inc	w9, w9
	add	#9, w11
	return

on_change_filter:
	mov	[w5+oEVX_RAM_DATA_ADR], w2
	mov	#0, w3
	rcall	FRam_SetValue
	rcall	SetUpFilter
	return

/* ______________________________________________________________________ */
/* w0=FilterDacIdx w5=DacDaten w7=DacOffset */
filtkx_add:
	mov	[w5+w0], w1
	se	[w7++], w2
	add	w1, w2, w1
	btsc	w1, #15
	clr	w1
	mov	#0xff00, w2
	and	w2, w1, w2
	btss	SR, #Z
	mov	#0x00ff, w1
	mov	w1, [w5+w0]
	return

/* w5=DacDaten */
SetupFilterKx:
	btsc	K32DB_PORT, #K32DB_BIT
	return
	btsc	K16DB_PORT, #K16DB_BIT
	return

	clr	w2
	btsc	K02DB_PORT, #K02DB_BIT
	bset	w2, #0
	btsc	K04DB_PORT, #K04DB_BIT
	bset	w2, #1
	btsc	K08DB_PORT, #K08DB_BIT
	bset	w2, #2

	mov	AktivFilerIdx, w1
	cp	w1, #4
	bra	gtu, sfkx_end

	sl	w1, #2, w1
	mov	#tab_pgkx, w6
	add	w6, w1, w6
	nop
	mov	[w6++], w7
	cp0	w7
	bra	z, sfkx_end
	mov	[w6], w0
	lsr	w0, #4, w1
	and	#0x0f, w0
	and	#0x0f, w1
	sub	w2, w1, w2
	btsc	w2, #15
	clr	w2
	cp	w2, w0
	bra	geu, sfkx_end

	mul.uu	w2, #VXF_10ns_K0_COUNT, w2
	add	w7, w2, w7


	mov	#2, w0		; GRN0
	rcall	filtkx_add


	mov	#4, w0		; GRN1
	rcall	filtkx_add


	mov	#6, w0		; GLIN0
	rcall	filtkx_add
	dec	w7, w7

	mov	#10, w0		; LINPA
	rcall	filtkx_add

	mov	#8, w0		; GLIN1
	rcall	filtkx_add

	mov	#14, w0		; FR1
	rcall	filtkx_add

sfkx_end:
	return

/* ______________________________________________________________________ */
.if USE_UART_LIST_PARA
/* w2=LineIdx w7=ParamLine w4=Ext Data */
get_val_FilterKx:
	cp0	w4
	bra	z, ucMenuSend_NULL

	cp	w2, #VXF_10ns_K0_COUNT
	bra	gtu, ucMenuSend_NULL

	dec	w2, w2
	add	w4, w2, w4
	mov	#EEData0, w0
	add	w4, w0, w4

	mov	#Str_buffer, w3
	mov	#0, w2
	se	[w4], w0
	bra	int16D3toppc

.endif  /* USE_UART_LIST_PARA */

/* ______________________________________________________________________ */
.section .const, psv
pgkx_on_change:	.word on_change_filter

tab_pgkx:
	.word	EEData0+oFR_FILT_10ns_K1, 0x0017
	.word	EEData0+oFR_FILT_12ns_K0, 0x0004
	.word	0x0000, 0x0000
	.word	0x0000, 0x0000
	.word	0x0000, 0x0000
