/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/eingVx.inc"
.include "lib/Param.inc"
.include "math/libm.inc"

.section .nbss, bss, near
EingImp_highZ:		.space EVX_SIZE
EingImpedanz:		.space EVX_SIZE

TmpCursLineIdx:		.space 4
Last_CursSelIdx:	.space 2
Impedanz_Step:		.space 2
Impedanz_Last:		.space 2
Impedanz_Flag:		.space 2

fImpedanz:		.space F48_SIZE

fImpGenMul:		.space F48_SIZE
fImpGenMul_1div:	.space F48_SIZE
fImpGenMulKorr:		.space F48_SIZE
fImpDmmMul:		.space F48_SIZE

;.global EingImpedanz
.global Impedanz_Flag
.global fImpedanz

.global fImpGenMul
.global fImpGenMul_1div
.global fImpGenMulKorr
.global fImpDmmMul

/* ---------------------------------------------------------------------- */
.section .text

.global LCD_Impedanz
.global StrImpedanz
.global Key_Impedanz
.global init_Impedanz
.global SetUpImpedanz
.global MathImpedanz
.global GetImpedanzVal_Div

.if USE_DEBUG_UART_USER
.global SetImpedanz_HZ
.endif

LCD_Impedanz:
	dec	w0, w0
	bra	z, LCD_MenuCaption_cs
	dec	w0, w0
	bra	z, Line_hz
	dec	w0, w0
	bra	z, Line_Imp
	dec	w0, w0
	bra	z, Line_PAadc
	bra	LCD_Paint

Line_hz:
	mov	#0, w0
	mov	EingImp_highZ+oEVX_DISP_Y, w1
	mov	#tImp_hz, w2
	rcall	StrOut8x13

	mov	#EingImp_highZ, w5
	rcall	DispVxVal

	inc	LCD_Step
	return

Line_Imp:
	mov	#94, w0
	mov	EingImpedanz+oEVX_DISP_Y, w1
	mov	#tImp_Ohm, w2
	btsc	Impedanz_Step, #1
	mov	#tImp_kOhm, w2
	rcall	StrOut8x13

	mov	#EingImpedanz, w5
	rcall	DispVxVal

	inc	LCD_Step
	return


Line_PAadc:
	mov	AdcPa_1616, w0
	mov	AdcPa, w1

	mov	#Str_buffer, w3
	mov	#1, w2
	mov	#5, w4
	rcall	i1616Dx6toppc

	mov	#'m', w0
	mov.b	w0, [w3++]
	mov	#'V', w0
	mov.b	w0, [w3++]
	clr.b	[w3]

	mov	#0, w0
	mov	#12, w1
	btss	TestDispFlags, #0
	mov	#58, w1
	bra	LCD_StrBufOut_cs


/* w3 Str_buffer */
StrImpedanz:
	mov	Impedanz_Flag, w0
	btst	w0, #4
	bra	nz, stri_hz
	and	#0x03, w0

	mov	#1, w2
	cp0	w0
	bra	z, stri_x
	cp	w0, #3
	bra	z, stri_x
	mov	#0, w2
	cp	w0, #1
	bra	z, stri_x
	mov	#2, w2
stri_x:
	mov	Impedanz_Last, w0
	rcall	uint16D3toppc
	mov	#'k', w0
	btsc	Impedanz_Flag, #1
	mov.b	w0, [w3++]
	mov	#0x81, w0
	mov.b	w0, [w3++]
	clr.b	[w3]
	return

stri_hz:
	mov	#'H', w0
	mov.b	w0, [w3++]
	mov	#'Z', w0
	mov.b	w0, [w3++]
	mov	#' ', w0
	mov.b	w0, [w3++]
	clr.b	[w3]
	return

/* ______________________________________________________________________ */
Key_Impedanz:
	mov	#EingImp_highZ, w5
	rcall	EingVx_Key
	bra	z, do_val_HZ

	mov	#EingImpedanz, w5
	rcall	EingVx_Key
	bra	z, do_val_Impedanz

	cp	w0, #KEY_RET
	bra	z, K_CRet
	return

K_CRet:
	mov	EingVx_CursSelIdx, w0
	mov	w0, Last_CursSelIdx

	mov	#TmpCursLineIdx, w6
	rcall	EingVx_Pop_CursLineIdx

	mov	EingImpedanz+oEVX_VALUE, w0
	mov	Impedanz_Step, w1
	mov	EingImp_highZ+oEVX_VALUE, w2
	btsc	w2, #0
	bset	w1, #4
	cp	Impedanz_Last
	bra	nz, imp_save
	mov	Impedanz_Flag, w2
	cp	w1, w2
	bra	z, Key_RetOk_Menu
imp_save:
	mov	w0, Impedanz_Last
	mov	w1, Impedanz_Flag

	mov	#oFR_IMPEDANZ, w2
	mov	#2, w3		; 3 byte
	rcall	FRam_SaveValue

	rcall	AmplSetMinMax
	rcall	OffsSetMinMax
	rcall	AmplOffsImpReCalcValue
	rcall	AmplOffsSave
	bset	MathFlags, #bMF_AMPL
	bset	MathFlags, #bMF_OFFSET
	bra	Key_RetOk_Menu

do_val_HZ:
	btst	[w5], #bEVSTAT_VAL_CHANGE
	bra	z, Key_RetOk
	rcall	MathImpedanz
	bra	Key_RetOk

do_val_Impedanz:
	rcall	etsr_Key
	bra	z, Key_RetOk
	bra	c, Key_RetOk

	rcall	MathImpedanz
	bra	Key_RetOk

/* ______________________________________________________________________ */
init_Impedanz:
	mov	#TmpCursLineIdx, w6
	rcall	EingVx_Push_CursLineIdx

;	rcall	LoadImpedanz


	mov	#1, w0			/* max.line-1 */
	mov	Last_CursSelIdx, w1	/* akt.line */
	and	#EVCURS_SELNR_MASK, w1
	lsr	w1, #EVCURS_SELNR_SHL, w1
	rcall	EingVx_InitCursLineIdx
/* EingImp_highZ */
	mov	#EingImp_highZ, w5
	mov	#0, w0			/* line.idx */
	mov	#0, w1			/* akt.X.pos */
	rcall	EingVx_SetCursSelIdx

	mov	#0, w0			/* w0:w1 Value */
	btsc	Impedanz_Flag, #4
	mov	#1, w0			/* w0:w1 Value */

	mov	#0, w1
	mov	#0, w2			/* min. */
	mov	#1, w3			/* max. */
	mov	#0x0101, w4		/* Num=1 bDVF_IS_UNSIGN */
	rcall	EingVx_SetValueMinMaxDflag

	mov	#-90, w0		/* X.pos */
	mov	#20, w1			/* Y.pos */
	mov	#0, w2			/* no txt */
	rcall	EingVx_SetDispXYTxt

/* EingImpedanz */
	mov	#EingImpedanz, w5
	mov	#1, w0			/* line.idx */
	mov	#0, w1			/* akt.X.pos */
	rcall	EingVx_SetCursSelIdx

	mov	Impedanz_Last, w0
	mov	Impedanz_Flag, w2
	and	w2, #0x03, w2
	mov	#tab_Impedanz_Owner, w6
	rcall	etsr_InitTabStep

	return

/* ______________________________________________________________________ */
LoadImpedanz:
	mov.b	EEData0+oFR_IMPEDANZ+2, WREG
	ze	w0, w1
	mov	#0x00EC, w2	; b0 b1 = seg    b4 = highZ on
	and	w2, w1, w2
	bra	nz, iimp_err

	mov	EEData0+oFR_IMPEDANZ, w0
	cp0	w0
	bra	nz, iimp_ok
iimp_err:
	mov	#500, w0
	mov	#0, w1
iimp_ok:
	mov	w0, Impedanz_Last
	mov	w1, Impedanz_Flag

	mov	w0, EingImpedanz+oEVX_VALUE
	and	w1, #0x03, w2
	mov	w2, Impedanz_Step

	mov	#0, w2
	btsc	w1, #4
	mov	#1, w2
	mov	w2, EingImp_highZ+oEVX_VALUE

	return

MathImpedanz:
	mov	EingImp_highZ+oEVX_VALUE, w0
	cp0	w0
	bra	z, mirx

	mov	#0x0000, w0
	mov	#0x8000, w1
	mov	#0x3fff, w2	; 1.0 == HighZ
	mov	w0, fImpGenMul
	mov	w1, fImpGenMul+2
	mov	w2, fImpGenMul+4
	mov	w0, fImpGenMul_1div
	mov	w1, fImpGenMul_1div+2
	mov	w2, fImpGenMul_1div+4
	mov	w0, fImpGenMulKorr
	mov	w1, fImpGenMulKorr+2
	mov	w2, fImpGenMulKorr+4
	mov	w0, fImpDmmMul
	mov	w1, fImpDmmMul+2
	mov	w2, fImpDmmMul+4
	rcall	PaAdcSetGlobMul
	return

mirx:
	mov	EingImpedanz+oEVX_VALUE, w0

	lnk	#F48_SIZE*2
	mov	#fImpedanz, w3
	rcall	u16_f48_w3

	mov	Impedanz_Step, w0
	and	w0, #0x03, w0

	mov	#FC48_01, w2
	cp0	w0
	bra	z, mimp_mul
	cp	w0, #1
	bra	z, mimp_1
	mov	#FC48_10, w2
	cp	w0, #2
	bra	z, mimp_mul

	add	w14, #F48_SIZE, w2
	mov	#0x0000, w0
	mov	w0, [w2]
	mov	#0xc800, w0
	mov	w0, [w2+2]
	mov	#0x4005, w0
	mov	w0, [w2+4]	; 100.0

mimp_mul:
	mov	#fImpedanz, w4
	rcall	f48_mul
mimp_1:

/* GenV Mul = (50.0 / fImpedanz) + 1.0 */
	mov	#FC48_IMP_50, w2
	mov	#fImpedanz, w3
	mov	w14, w4
	rcall	f48_div

	mov	w4, w2
	mov	#FC48_1, w3
	mov	#fImpGenMul, w4
	rcall	f48_add

	mov	#FC48_1, w2
	mov	w4, w3
	mov	#fImpGenMul_1div, w4
	rcall	f48_div

/* GenV Mul Korr = (ConImp / (50.0 + fImpedanz)) + 1.0 */
	mov	#FC48_IMP_50, w2
	mov	#fImpedanz, w3
	mov	w14, w4
	rcall	f48_add

	mov	#FC48_IMP_CON, w2
	btss	K60CM_PORT, #K60CM_BIT
	mov	#FC48_IMP_RG316_60cm, w2
	mov	w4, w3
	rcall	f48_div

	mov	#FC48_1, w2
	mov	#fImpGenMulKorr, w4
	rcall	f48_add

/* DMM Mul = 1.0 / (ConImp / fImpedanz + 1.0) */
	mov	#FC48_IMP_CON, w2
	btss	K60CM_PORT, #K60CM_BIT
	mov	#FC48_IMP_RG316_60cm, w2
	mov	#fImpedanz, w3
	mov	w14, w4
	rcall	f48_div

	mov	#FC48_1, w2
	mov	w4, w3
	rcall	f48_add

	mov	#fImpDmmMul, w4
	rcall	f48_div

	ulnk
	rcall	PaAdcSetGlobMul
	return



SetUpImpedanz:
	rcall	LoadImpedanz
	rcall	MathImpedanz
	return

.if USE_DEBUG_UART_USER
/* w0 b0 */
SetImpedanz_HZ:
	and	w0, #1, w1
	mov	w1, EingImp_highZ+oEVX_VALUE
	mov	Impedanz_Flag, w0

	btsc	w1, #0
	bset.b	w0, #bIMP_HIGHZ_ON
	btss	w1, #0
	bclr.b	w0, #bIMP_HIGHZ_ON

	mov	Impedanz_Flag, w1
	cp	w0, w1
	btsc	SR, #Z
	return

	mov	w0, Impedanz_Flag

	rcall	MathImpedanz
	rcall	AmplSetMinMax
	rcall	OffsSetMinMax
	rcall	AmplOffsImpReCalcValue
	bset	MathFlags, #bMF_AMPL
	bset	MathFlags, #bMF_OFFSET
	return
.endif

/* w0 / Z */
GetImpedanzVal_Div:
	lnk	#F48_SIZE

	mov	w14, w3
	rcall	i16_f48_w3

	mov	#fImpGenMul_1div, w2
	mov	w14, w4
	rcall	f48_mul

	rcall	f48_i16r

	ulnk
	return

/* ______________________________________________________________________ */
.section .const, psv
tImp_hz:	.asciz	"highZ"
tImp_Ohm:	.asciz	"Ohm"
tImp_kOhm:	.asciz	"kOhm"

 .align 2

tab_Impedanz_sel:
  .word 4
  .word 0x0000, 0x1301,   400,   999	/* s0   40.0 - 99.9 Ohm */
  .word 0x0000, 0x0301,   100,   999	/* s1   100 -  999  Ohm */
  .word 0x0000, 0x2301,   100,   999	/* s2   1.00 - 9.99 kOhm */
  .word 0x0000, 0x1301,   100,   999	/* s3   10.0 - 99.9 kOhm */

tab_Impedanz_Owner:
  .word tab_Impedanz_sel, Impedanz_Step, -90, 38

FC48_IMP_50:
	.word	0x0000, 0xc800, 0x4004		/* 50.0 Ohm */

FC48_IMP_CON:
	.word	0x6e98, 0x8312, 0x3ff7		/* 0.004 Ohm */
;	.word	0x0000, 0x0000, 0x0000		/* 0.000 Ohm */

FC48_IMP_RG316_60cm:
	.word	0x53f8, 0xa5e3, 0x3ffc		/* 0.162 Ohm */
