/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/eingVx.inc"
.include "lib/Param.inc"
.include "math/libm.inc"
.include "lib/rs232ald.inc"


.section .nbss, bss, near

StLineIdx:	.space 2
StLineStat:	.space 2
.equ STL_SYS_ERR,	0

EingCp_Freq:		.space EVX_SIZE
EingCp_Time:		.space EVX_SIZE
EingCp_Ampl:		.space EVX_SIZE
EingCp_Offs:		.space EVX_SIZE
EingCp_RiseFall:	.space EVX_SIZE
EingCp_RG316_cm:	.space EVX_SIZE
FqHz_Step:		.space 2
FqTime_Step:		.space 2
FqAmpl_Step:		.space 2
TestDispFlags:		.space 2

.global EingCp_Time
.global EingCp_Ampl
.global EingCp_Offs
.global EingCp_RiseFall
.global EingCp_RG316_cm
.global FqTime_Step
.global FqAmpl_Step
.global TestDispFlags

/* ---------------------------------------------------------------------- */
.section .text

.global PrintSysError
.global StatusLine

.global LCD_main

.global Key_main
.global AmplSetMinMax
.global OffsSetMinMax
.global AmplOffsImpReCalcValue
.global AmplOffsSave

.global Init_Eing_main

.if USE_DEBUG_UART_USER
.global Set_uuFreq
.global Set_uuAmpl
.global Set_uuOffs
.endif

;.global FreqSetUp
.global GetEingCp_Freq

LCD_main:
	dec	w0, w0
	bra	nz, LineTopMod

	cp0	SysError
	bra	nz, csye_end

	mov	I2CErrorFlag, w0
	ff1r	w0, w1
	cp0	w1
	bra	z, csye_end
	mov	w1, SysError
csye_end:

	clr	StLineStat
	cp0	SysError
	btss	SR, #Z
	bset	StLineStat, #STL_SYS_ERR
	btss	LCD_Count, #2
	bclr	StLineStat, #STL_SYS_ERR	/* error toggle */

	cp0	StLineStat
	bra	z, LCD_Step_Inc

	mov	#Str_buffer, w3
	rcall	PrintSysError

	clr.b	[w3]
	mov	#8, w0
	mov	#0, w1
	bra LCD_StrBufOut8x13

/* ______________________________________________________________________ */
PrintSysError:
	mov	SysError, w1
	and	w1, #0x0f, w1
	sl	w1, w1
	mov	#TAZ_SysError, w2
	nop
	mov	[w1+w2], w0
	bra	ppc_move

StatusLine:
	mov	#Str_buffer, w3
	rcall	StrImpedanz

	mov	PA_OutFlag, w1
	and	w1, #0x03, w1
	btsc	INV_PORT, #INV_BIT
	bset	w1, #2

	sl	w1, w1
	mov	#TAZ_StLineIdx, w2
	nop				; stall
	mov	[w1+w2], w0
	bra	ppc_move

/* ______________________________________________________________________ */
LineTopMod:
	cp0	StLineStat
	bra	nz, Line0
	dec	w0, w0
	bra	nz, Line0

/* no error print InRange */
	rcall	StatusLine

	mov	#0, w0
	mov	#0, w1
	bra	LCD_StrBufOut_cs

/* ______________________________________________________________________ */
Line0:
	dec	w0, w0
	bra	nz, Line1

	mov	#EingCp_RiseFall, w5
	mov	AktivFilerIdx, w1
	mov	EingCp_RiseFall+oEVX_VALUE, w0
	cp	w0, w1
	bra	z, rf_ok
	btst	LCD_Count, #0
	bra	z, rf_ok
	push	w0
	mov	w1, EingCp_RiseFall+oEVX_VALUE
	rcall	DispVxVal
	pop	w0
	mov	w0, EingCp_RiseFall+oEVX_VALUE
	bra	rf_v2
rf_ok:
	rcall	DispVxVal
rf_v2:
	mov	#EingCp_RG316_cm, w5
	rcall	DispVxVal
	
	inc	LCD_Step
	return

/* ______________________________________________________________________ */
Line1:
	dec	w0, w0
	bra	nz, Line2

	mov	#EingCp_Freq, w5
	rcall	DispVxVal
/*
	mov	AdcPa_1616, w0
	mov	AdcPa, w1
	btst	K60CM_PORT, #K60CM_BIT
	bra	nz, l1_nc
	btst	Impedanz_Flag, #bIMP_HIGHZ_ON
	bra	nz, l1_nc

	mov	PaAdc60cmMul, w2
	mov	PaAdc60cmMul+2, w3
	rcall	mul32su
	mov	w1, w0
	mov	w2, w1
l1_nc:
*/
	mov	#Str_buffer, w3
;	mov	PaOut_mV_k, w0
;	mov	PaOut_mV_k+2, w1
	mov	AdcPa_1616, w0
	mov	AdcPa, w1
	mov	#1, w2
	mov	#4, w4
	rcall	i1616Dx6toppc

	mov	#'m', w0
	mov.b	w0, [w3++]
	mov	#'V', w0
	mov.b	w0, [w3++]
	clr.b	[w3]

	mov	#0, w0
	mov	#12, w1
	btss	TestDispFlags, #0
	mov	#42, w1
	bra	LCD_StrBufOut_cs

/* ______________________________________________________________________ */
Line2:
	dec	w0, w0
	bra	nz, Line3

	mov	#EingCp_Time, w5
	rcall	DispVxVal

	btst	TestDispFlags, #0
	bra	z, LCD_Step_Inc

	mov	#Str_buffer, w3
.if 0
	mov	TestVal0, w0
	mov	#4, w2
	rcall	hex16toppc

	mov	#0x81, w0
	mov.b	w0, [w3++]

	mov	TestVal1, w0
	mov	#0, w2
	rcall	int16D2toppc

	mov	#0x81, w0
	mov.b	w0, [w3++]

	mov	TestVal4, w0
	mov	#4, w2
	rcall	hex16toppc

.else
	mov	GOV_01mV, w0
	mov	#1, w2
	rcall	int16toppc

	mov	#'m', w0
	mov.b	w0, [w3++]
	mov	#'V', w0
	mov.b	w0, [w3++]
/*
	mov	#' ', w0
	mov.b	w0, [w3++]
	mov	#' ', w0
	mov.b	w0, [w3++]
	mov	#' ', w0
	mov.b	w0, [w3++]

	mov	GenFreq+2, w0
	mov	#4, w2
	rcall	hex16toppc
	mov	#' ', w0
	mov.b	w0, [w3++]
	mov	GenFreq, w0
	mov	#4, w2
	rcall	hex16toppc
*/

	clr.b	[w3]
.endif
	mov	#0, w0
	mov	#22, w1
	bra	LCD_StrBufOut_cs
/* ______________________________________________________________________ */
Line3:
	dec	w0, w0
	bra	nz, Line4

	mov	#EingCp_Ampl, w5
	rcall	DispVxVal

	btst	TestDispFlags, #0
	bra	z, LCD_Step_Inc

	mov	#Str_buffer, w3
.if 0
	mov	TestVal2, w0
	mov	#4, w2
	rcall	hex16toppc

	mov	#0x81, w0
	mov.b	w0, [w3++]

	mov	TestVal3, w0
	mov	#0, w2
	rcall	int16D2toppc

	mov	#0x81, w0
	mov.b	w0, [w3++]

	mov	TestVal5, w0
	mov	#4, w2
	rcall	hex16toppc

.else
	mov	GRV_01mV, w0
	mov	#1, w2
	rcall	int16toppc

	mov	#'m', w0
	mov.b	w0, [w3++]
	mov	#'V', w0
	mov.b	w0, [w3++]
	clr.b	[w3]
.endif
	mov	#0, w0
	mov	#32, w1
	bra	LCD_StrBufOut_cs
/* ______________________________________________________________________ */
Line4:
	dec	w0, w0
	bra	nz, Line5

	btst	LCD_Count, #0
	bra	nz, az_offs

	btst	OV_FlagEg, #bOVF_OFFSET
	bra	nz, ov_offs
az_offs:
	mov	#EingCp_Offs, w5
	rcall	DispVxVal
ov_offs:

	btst	TestDispFlags, #0
	bra	z, LCD_Step_Inc

	mov	#Str_buffer, w3
	mov	OP1V_1616_001mV, w0
	mov	OP1V_1616_001mV+2, w1
	mov	#2, w2
	mov	#5, w4
	rcall	i1616Dx6toppc

	mov	#'m', w0
	mov.b	w0, [w3++]
	mov	#'V', w0
	mov.b	w0, [w3++]
	clr.b	[w3]

	mov	#0, w0
	mov	#42, w1
	bra	LCD_StrBufOut_cs

/* ______________________________________________________________________ */
Line5:
	dec	w0, w0
	bra	nz, Line6

	mov	#0, w0
	mov	#58, w1

	btst	TestDispFlags, #0
	bra	z, StrbTemp_cs_step

.if 0
	bra	StrbTemp_cs_step
.else
	rcall	StrbTemp_cs

	mov	#Str_buffer, w3
	mov	GainRelais, w0
	mov	#2, w2
	rcall	hex16toppc

	mov	#' ', w0
	mov.b	w0, [w3++]
/*
	clr	w0
	btsc	K01DB_PORT, #K01DB_BIT
	bset	w0, #0
	btsc	K02DB_PORT, #K02DB_BIT
	bset	w0, #1
	btsc	K04DB_PORT, #K04DB_BIT
	bset	w0, #2
	btsc	K08DB_PORT, #K08DB_BIT
	bset	w0, #3
	btsc	K16DB_PORT, #K16DB_BIT
	bset	w0, #4
	btsc	K32DB_PORT, #K32DB_BIT
	bset	w0, #5
*/
	rcall	GetK_dbX
	mov	#2, w2
	rcall	hex16toppc

	mov	#0, w0
	mov	#49, w1
	bra	LCD_StrBufOut_cs
.endif

	GR_HIGHT		= 20
	GR_CENTER		= 22

	GR_OFF_MAX		= 2
	GR_ZOOM			= (GR_OFF_MAX + 2)
	GR_ZOOM_HIGHT		= 8


Line6:
	dec	w0, w0
	bra	nz, LCD_Paint

	btst	TestDispFlags, #0
	bra	nz, LCD_Paint

	btst	PA_OutFlag, #bPAOF_DC
	bra	nz, LCD_Paint

	mov	uOffs, w0	; 0 ... +9000
	mov	#GR_HIGHT, w1
	mul.uu	w0, w1, w0
	mov	iAmpl, w2	; 0 ... +9000
	repeat	#17
	div.ud	w0, w2
	lsr	w2, w2		/* Round */
	sub	w1, w2, w1
	addc	w0, #0, w8		; w8 Gr offset

	mov	#GR_CENTER-(GR_HIGHT>>1), w1		; w1 Gr Y

	mov	#GR_HIGHT, w9		; w9 Gr high
	cp	w8, #GR_OFF_MAX+(GR_HIGHT>>1)
	bra	leu, grsq_le
	sub	w8, #GR_OFF_MAX+(GR_HIGHT>>1), w0

	mov	#GR_OFF_MAX, w3
	cpsgt	w3, w0
	mov	w3, w0
	btsc	iOffs, #15
	neg	w0, w0
	sub	w1, w0, w1

	cp	w8, #GR_ZOOM+(GR_HIGHT>>1)
	bra	leu, grsq_le
	sub	w8, #GR_ZOOM+(GR_HIGHT>>1), w0

	mov	#GR_HIGHT-GR_ZOOM_HIGHT, w3
	cpsgt	w3, w0
	mov	w3, w0
	sub	w9, w0, w9
	btst	iOffs, #15
	bra	z, grsq_le
	add	w1, w0, w1
grsq_le:

	btst	PA_OutFlag, #bPAOF_SQ_P
	bra	z, gr_square

	btst	INV_PORT, #INV_BIT
	bra	nz, gr_puls_inv
	add	w1, w9, w1
	neg	w9, w9
gr_puls_inv:
	mov	#2, w0
	mov	#17, w2
	rcall	PolLineH
	mov	w9, w2
	rcall	PolLineV
	mov	#8, w2
	rcall	PolLineH
	mov	w9, w2
	neg	w2, w2
	rcall	PolLineV
	mov	#17, w2
	rcall	PolLineH
	bra	gr_offset

gr_square:
;	mov	#20, w9

	mov	#2, w0
	mov	#6, w2
	rcall	PolLineH
	mov	w9, w2
	rcall	PolLineV
	mov	#10, w2
	rcall	PolLineH
	mov	w9, w2
	neg	w2, w2
	rcall	PolLineV
	mov	#10, w2
	rcall	PolLineH
	mov	w9, w2
	rcall	PolLineV
	mov	#10, w2
	rcall	PolLineH
	mov	w9, w2
	neg	w2, w2
	rcall	PolLineV
	mov	#6, w2
	rcall	PolLineH

gr_offset:
	mov	#GR_OFF_MAX+(GR_HIGHT>>1), w0
	cpsgt	w8, w0
	mov	w8, w0

	btsc	iOffs, #15
	neg	w0, w0

	mov	#GR_CENTER, w1
	add	w1, w0, w1

	mov	#0, w0
	mov	#48, w2
	call	PolDotLineH
	inc	LCD_Step
	return

/* ______________________________________________________________________ */
Key_main:
	cp	w0, #KEY_OPEN
	bra	z, K_COpen

	mov	#EingCp_Freq, w5
	rcall	EingVx_Key
	bra	z, do_val_freq

	mov	#EingCp_Time, w5
	rcall	EingVx_Key
	bra	z, do_val_Time

	mov	#EingCp_Ampl, w5
	rcall	EingVx_Key
	bra	z, do_val_Gain

	mov	#EingCp_Offs, w5
	rcall	EingVx_Key
	bra	z, do_val_Offs

	mov	#EingCp_RiseFall, w5
	rcall	EingVx_Key
	bra	z, do_RiseFall

	mov	#EingCp_RG316_cm, w5
	rcall	EingVx_Key
	bra	z, do_RG316_cm

	return

K_COpen:
	inc	LCD_PProc
	bra	Key_RetRePaint


;FreqSetUp:
;	rcall	GetEingCp_Freq
;	bra	SetFreq

do_val_freq:
	rcall	etsr_Key
	bra	z, Key_RetOk
	bra	c, Key_RetOk
;freq_para_save:
	mov	FqHz_Step, w2
	mov	#EEData0+oFR_LastFreq, w3
	rcall	SaveMainLastVx16

	rcall	GetEingCp_Freq
	mov	w0, GenFreq
	mov	w1, GenFreq+2
	mov	w2, GenFreq+4
	bset	MathFlags, #bMF_FREQ
;	rcall	SetFreq
	bra	Key_RetOk


GetEingCp_Freq:
	clr	w2		; no div
	mov	#EingCp_Freq, w5
	mov	[w5+oEVX_VALUE], w0
	clr	w1
	mov	FqHz_Step, w3
	cp0	w3
	bra	z, egc_div_10

	dec	w3, w3
	bra	z, egc_az
	dec	w3, w3
	bra	z, egc_az
	mov	#10, w1
	dec	w3, w3
	bra	z, egc_mul_1x
	mov	#100, w1
	dec	w3, w3
	bra	z, egc_mul_1x
	mov	#1000, w1
	dec	w3, w3
	bra	z, egc_mul_1x
	mov	#10000, w1

egc_mul_1x:
	mul.uu	w0, w1, w0
	bra	egc_az

egc_div_10:
	clr	w1
	mov	#10, w2
egc_az:
	return

do_val_Time:
	rcall	etsr_Key
	bra	z, Key_RetOk
	bra	c, Key_RetOk

	mov	FqTime_Step, w2
	mov	#EEData0+oFR_LastTime, w3
	rcall	SaveMainLastVx16

	rcall	SetPuls

	bra	Key_RetOk


do_val_Gain:
	rcall	etsr_Key
	bra	z, Key_RetOk
	bra	c, Key_RetOk


	btst	[w5], #bEVSTAT_UV_MIN
	bra	nz, dvg_uv
	btst	[w5], #bEVSTAT_OV_MAX
	bra	z, dvg
dvg_uv:
	push	w5
	rcall	AmplSetMinMax
	pop	w5
dvg:
	mov	FqAmpl_Step, w2

	mov	#EEData0+oFR_LastAmpl, w3
	rcall	SaveMainLastVx16

	bset	MathFlags, #bMF_AMPL
	bra	Key_RetOk

do_val_Offs:
	btst	[w5], #bEVSTAT_VAL_CHANGE
	bra	z, Key_RetOk

	clr	w2
	mov	#EEData0+oFR_LastOffs, w3
	rcall	SaveMainLastVx1616

	bset	MathFlags, #bMF_OFFSET
	bra	Key_RetOk

do_RiseFall:
	btst	[w5], #bEVSTAT_VAL_CHANGE
	bra	z, Key_RetOk
	rcall	SetUpFilter
	bset	MathFlags, #bMF_AMPL
	bra	kro_lf_save

do_RG316_cm:
	btst	[w5], #bEVSTAT_VAL_CHANGE
	bra	z, Key_RetOk

	mov	[w5+oEVX_VALUE], w0
;	cp0	w0
;	bra	z, rg316_0
;	bclr	K60CM_PORT, #K60CM_BIT
;	bra	kro_lf_save
;rg316_0:
;	bset	K60CM_PORT, #K60CM_BIT

	bset	K60CM_PORT, #K60CM_BIT
	btsc	w0, #0
	bclr	K60CM_PORT, #K60CM_BIT

	rcall	MathImpedanz
	bset	MathFlags, #bMF_AMPL
	bset	MathFlags, #bMF_OFFSET
kro_lf_save:
	rcall	LastFlag_Save
	bra	Key_RetOk

AmplSetMinMax:
	cp0	FqAmpl_Step
	bra	nz, as_max
	mov	#80, w0
	rcall	GetImpedanzVal_Div
	mov	w0, EingCp_Ampl+oEVX_DISP_MIN
	return
as_max:
	mov	#9000, w0
	rcall	GetImpedanzVal_Div
	mov	w0, EingCp_Ampl+oEVX_DISP_MAX
	return

OffsSetMinMax:
	mov	#9000, w0
	rcall	GetImpedanzVal_Div
	mov	w0, EingCp_Offs+oEVX_DISP_MAX
	neg	w0, w0
	mov	w0, EingCp_Offs+oEVX_DISP_MIN
	return

AmplOffsImpReCalcValue:
	lnk	#F48_SIZE

	mov	#fImpGenMul_1div, w2
	mov	#fAmpl, w3
	mov	w14, w4
	rcall	f48_mul

	rcall	f48_i16r
	mov	#1000, w2
	cp	w0, w2
	bra	geu, aui_ampl1V

	mov	#FC48_10, w2
	mov	w14, w3
	rcall	f48_mul
	rcall	f48_i16r
	cp0	FqAmpl_Step
	bra	z, aui_ampl_ok
	mov	#0, w2
	bra	aui_ampl_set

aui_ampl1V:
	cp0	FqAmpl_Step
	bra	nz, aui_ampl_ok
	mov	#1, w2
aui_ampl_set:
	mov	#EingCp_Ampl, w5
	mov	#tab_Ampl_Owner, w6
	rcall	etsr_InitTabStep
	rcall	AmplSetMinMax
	bra	aui_ampl_end

aui_ampl_ok:
	mov	#EingCp_Ampl, w5
	rcall	EingVx_SetValue
aui_ampl_end:
	mov	#fImpGenMul_1div, w2
	mov	#fOffs, w3
	mov	w14, w4
	rcall	f48_mul

	rcall	f48_i1616
	mov	#EingCp_Offs, w5
	rcall	EingVx_SetValue

	ulnk
/*
	mov	#EingCp_Ampl, w5
	mov	FqAmpl_Step, w2
	mov	#EEData0+oFR_LastAmpl, w3
	rcall	SaveMainLastVx16

	mov	#EingCp_Offs, w5
	clr	w2
	mov	#EEData0+oFR_LastOffs, w3
	rcall	SaveMainLastVx1616
*/
	return

AmplOffsSave:
	mov	#EingCp_Ampl, w5
	mov	FqAmpl_Step, w2
	mov	#EEData0+oFR_LastAmpl, w3
	rcall	SaveMainLastVx16

	mov	#EingCp_Offs, w5
	clr	w2
	mov	#EEData0+oFR_LastOffs, w3
	rcall	SaveMainLastVx1616

	return


/* w2=xdata.idx w3=Fram.pointer w5=Vx.pointer  */
SaveMainLastVx16:
	rcall	EingVx_GetAktXpos
	and	#0x07, w0
	sl	w0, #4, w1
	mov	[w5+oEVX_VALUE], w0
	and	#0x0F, w2
	ior	w2, w1, w2
	rcall	SaveMainLastValue16_8
	mov	EingVx_CursSelIdx, w0
	mov.b	WREG, EEData0+oFR_LastLineIdx
	return

/* w2=xdata.idx w3=Fram.pointer w5=Vx.pointer  */
SaveMainLastVx1616:
	rcall	EingVx_GetAktXpos
	and	#0x07, w0
	sl	w0, #4, w1
	and	#0x0F, w2
	ior	w2, w1, w2
	mov	[w5+oEVX_VALUE_1616], w0
	mov	[w5+oEVX_VALUE], w1
	rcall	SaveMainLastValue32_8
	mov	EingVx_CursSelIdx, w0
	mov.b	WREG, EEData0+oFR_LastLineIdx
	return


/* ______________________________________________________________________ */
Init_Eing_main:
	mov.b	EEData0+oFR_LastLineIdx, WREG
	and	#EVCURS_SELNR_MASK, w0
	lsr	w0, #EVCURS_SELNR_SHL, w1	/* akt.line */
	mov	#5, w0				/* max.line-1 */
;	mov	#0, w1				/* akt.line */
	rcall	EingVx_InitCursLineIdx

/* EingCp_Freq */
	mov	#EEData0+oFR_LastFreq, w3
	mov	#EingCp_Freq, w5
	mov	#0, w0			/* line.idx */
	rcall	init_LastValue16

	and	w2, #0x0F, w2
	cp0	w0
	bra	nz, lf_ok

	mov	#5000, w0
	mov	#1, w2
lf_ok:
	mov	#tab_Freq_Owner, w6
	rcall	etsr_InitTabStep

/* EingCp_Time */
	mov	#EEData0+oFR_LastTime, w3
	mov	#EingCp_Time, w5
	mov	#1, w0			/* line.idx */
	rcall	init_LastValue16

	and	w2, #0x0F, w2
	cp0	w0
	bra	nz, let_ok

	mov	#50, w0
	mov	#0, w2
let_ok:
	mov	#tab_Time_Owner, w6
	rcall	etsr_InitTabStep

/* EingCp_Ampl */
	mov	#EEData0+oFR_LastAmpl, w3
	mov	#EingCp_Ampl, w5
	mov	#2, w0			/* line.idx */
	rcall	init_LastValue16

	and	w2, #0x01, w2
	mov	#tab_Ampl_Owner, w6
	rcall	etsr_InitTabStep
	rcall	AmplSetMinMax

.if 0
	mov	#4, w2			/* min. */
	mov	#4500, w3		/* max. */
	mov	#0x0401, w4		/* Num=4 bDVF_IS_UNSIGN */
	rcall	EingVx_SetValueMinMaxDflag

	mov	#-102, w0		/* X.pos */
	mov	#37, w1			/* Y.pos */
	mov	#TXT_mV, w2		/* mV */
	rcall	EingVx_SetDispXYTxt
.endif

/* EingCp_Offs */
	mov	#9000, w0
	rcall	GetImpedanzVal_Div
	mov	w0, w8			/* max. */

	mov	#EEData0+oFR_LastOffs, w3
	mov	#EingCp_Offs, w5
	mov	#3, w0			/* line.idx */
	rcall	init_LastValue1616

;	mov	#4500, w3		/* max. */
	mov	w8, w3			/* max. */
	neg	w3, w2			/* min. */
	mov	#0x1404, w4		/* 1616 */
	rcall	EingVx_SetValueMinMaxDflag

	mov	#-102, w0		/* X.pos */
	mov	#46, w1			/* Y.pos */
	mov	#TXT_mV, w2		/* mV */
	rcall	EingVx_SetDispXYTxt

/* EingCp_RiseFall */
	mov	#EingCp_RiseFall, w5
	mov	#4, w0			/* line.idx */
	mov	#0, w1			/* akt.X.pos */
	rcall	EingVx_SetCursSelIdx

;	mov	#0, w0			/* w0 Value */
	mov.b	EEData0+oFR_LAST_FLAG, WREG
	lsr	w0, #bFR_LF_NS, w0
	and	#0x07, w0		/* w0 Value */
	mov	#0, w2			/* min. */
	mov	#5, w3			/* max. */
	mov	#0x0011, w4
	rcall	EingVx_SetValueMinMaxDflag

	mov	#80, w0			/* X.pos */
	mov	#1, w1			/* Y.pos */
	mov	w0, [w5+oEVX_DISP_X]
	mov	w1, [w5+oEVX_DISP_Y]
	mov	#tab_rf, w0
	mov	w0, [w5+oEVX_P_TXT]
	bset	[w5], #bEVSTAT_ONLY_TXT

/* EingCp_RG316_cm */
	mov	#EingCp_RG316_cm, w5
	mov	#5, w0			/* line.idx */
	mov	#0, w1			/* akt.X.pos */
	rcall	EingVx_SetCursSelIdx

	mov	#1, w0			/* w0 Value */
	btsc	K60CM_PORT, #K60CM_BIT
	mov	#0, w0			/* w0 Value */
	mov	#0, w2			/* min. */
	mov	#1, w3			/* max. */
	mov	#0x0011, w4
	rcall	EingVx_SetValueMinMaxDflag

	mov	#109, w0		/* X.pos */
	mov	#1, w1			/* Y.pos */
	mov	w0, [w5+oEVX_DISP_X]
	mov	w1, [w5+oEVX_DISP_Y]
	mov	#tab_rg316, w0
	mov	w0, [w5+oEVX_P_TXT]
	bset	[w5], #bEVSTAT_ONLY_TXT

	return

.if USE_DEBUG_UART_USER
/* -------------------------------------------------------
w0:w1	Start Value
w4	flag
result: 
  C	ERROR
--------------------------------------------------------- */
Set_uuFreq:
	clr	GenFreq+4	; fq / div

	btst	w4, #bFLAG_FLOAT32
	bra	z, fq_int

	lnk	#F48_SIZE*2
	mov	w14, w2
	rcall	f32_f48

	add	w14, #F48_SIZE, w3
	mov	#0xe354, w0
	mov	#0xc7e5, w1
	mov.d	w0, [w3]
	mov	#0x4005, w0
	mov	w0, [w3+4]	; 99.949

	rcall	f48_cp
	bra	gt, fq_gt_f100

	mov	#FC48_10, w3
	mov	w2, w4
	rcall	f48_mul
	rcall	f48_u32r
	ulnk
; mov w0, TestVal0
; mov w1, TestVal1
; clr TestVal2
	cp0	w0
	btsc	SR, #Z
	mov	#1, w0
	mov	#0, w2		/* s0    0.1 - 99.9 Hz  */
	bra	suu_set_10x

fq_gt_f100:
	mov	w14, w4
	rcall	f48_u32r
	ulnk
; mov w0, TestVal0
; mov w1, TestVal1
; setm TestVal2

	cp0	w1
	bra	nz, fq_gt_65535
	bra	fq_geu_100

fq_int:
	cp0	w1
	bra	nz, fq_gt_65535
	cp0	w0
	btsc	SR, #Z
	mov	#1, w0

	cp	w0, #100
	bra	geu, fq_geu_100
	mov	#0, w2		/* s0    0.1 - 99.9 Hz  */
	mul.uu	w0, #10, w0
	bra	suu_set_10x

fq_geu_100:
	mov	#1, w2		/* s1   100 -  999   Hz */
	mov	#999, w3
	cp	w0, w3
	bra	leu, suu_set_1x

	mov	#2, w2		/* s2  1000 -  9999   Hz */
	mov	#9999, w3
	cp	w0, w3
	bra	leu, suu_set_1x
	mov	#3, w2		/* s3  10,00 - 99,99 kHz */
	mov	#10, w4
	bra	suu_div

fq_gt_65535:
	mov	#3, w2		/* s3  10,00 - 99,99 kHz */
	mov	#10, w4
	mov	#99995 & 0xffff, w3
	cp	w0, w3
	cpb	w1, #99995 >> 16
	bra	n, suu_div

	mov	#4, w2		/* s4  100.0 - 999.9 kHz */
	mov	#100, w4
	mov	#999950 & 0xffff, w3
	cp	w0, w3
	cpb	w1, #999950 >> 16
	bra	n, suu_div

	mov	#5, w2		/* s5  1.000 - 9.999 MHz */
	mov	#1000, w4
	mov	#9999500 & 0xffff, w3
	cp	w0, w3
	cpb	w1, #9999500 >> 16
	bra	n, suu_div

	mov	#6, w2		/* s6  10,00 - 99,99 MHz */
	mov	#10000, w4
	mov	#99995000 & 0xffff, w3
	cp	w0, w3
	mov	#99995000 >> 16, w3
	cpb	w1, w3
	bra	n, suu_div

	mov	#7, w2		/* s7  100,00 - 230,00 MHz */
	mov	#230000000 & 0xffff, w3
	cp	w0, w3
	mov	#230000000 >> 16, w3
	cpb	w1, w3
	bra	n, suu_div
	mov	#230000000 & 0xffff, w0
	mov	#230000000 >> 16, w1
suu_div:
	mov	w0, GenFreq
	mov	w1, GenFreq+2

	repeat	#17		/* div 32/16->16 */
	div.ud	w0, w4
/* Round */
	lsr	w4, w4
	sub	w1, w4, w1
	addc	w0, #0, w0
suu_set_tab:
	mov	#EingCp_Freq, w5
	mov	#tab_Freq_Owner, w6
	rcall	etsr_InitTabStep
	bset	MathFlags, #bMF_FREQ
	return

suu_set_10x:
	mov	#10, w3
	mov	w3, GenFreq+4
suu_set_1x:
	mov	w0, GenFreq
	mov	w1, GenFreq+2
	bra	suu_set_tab

/* -------------------------------------------------------
w0:w1	Start Value
w4	flag
result: 
  C	ERROR
--------------------------------------------------------- */
Set_uuAmpl:
	lnk	#F48_SIZE*3
	mov	w14, w2
	rcall	f32_f48

	add	w14, #F48_SIZE, w3
	mov	#0xb776, w0
	mov	#0xfffc, w1
	mov.d	w0, [w3]
	mov	#0x3ffe, w0
	mov	w0, [w3+4]	; 0.9999499

	rcall	f48_cp
	bra	gt, ampl_gt_f1

	mov	#0, w0		/* s0   4.0 -  999.9 mV  */
	mov	w0, [w14+(F48_SIZE*2)]

	mov	#0x0000, w0
	mov	#0x9c40, w1
	mov.d	w0, [w3]
	mov	#0x400c, w0
	mov	w0, [w3+4]	; 10000.0
	bra	ampl_set
ampl_gt_f1:

	mov	#1, w0		/* s1   1000 - 9000  mV  */
	mov	w0, [w14+(F48_SIZE*2)]
	mov	#0x0000, w0
	mov	#0xfa00, w1
	mov.d	w0, [w3]
	mov	#0x4008, w0
	mov	w0, [w3+4]	; 1000.0
ampl_set:
	mov	w2, w4
	rcall	f48_mul
	rcall	f48_u32r
	mov	[w14+(F48_SIZE*2)], w2
	ulnk

	mov	#EingCp_Ampl, w5
	mov	#tab_Ampl_Owner, w6
	rcall	etsr_InitTabStep
	bset	MathFlags, #bMF_AMPL
	return

/* -------------------------------------------------------
w0:w1	Start Value
w4	flag
result: 
  C	ERROR
--------------------------------------------------------- */
Set_uuOffs:
	lnk	#F48_SIZE
	mov	w14, w2
	rcall	f32_f48
	mov	#FC48_1000, w3
	mov	w14, w4
	rcall	f48_mul
	rcall	f48_i1616
	ulnk
	mov	#EingCp_Offs, w5
	rcall	EingVx_SetValue
	bset	MathFlags, #bMF_OFFSET
	return
.endif



/* w0=line.idx w0=w3=Fram.pointer w5=Vx.pointer 
  result w0=Value w2=xdata */
init_LastValue16:
	mov.b	[w3++], w1
	swap	w1
	mov.b	[w3++], w1
	swap	w1
	push	w1
	ze	[w3], w1
	push	w1
	lsr	w1, #4, w1
	and	#0x07, w1		/* akt.X.pos */
	rcall	EingVx_SetCursSelIdx
	pop	w2
	pop	w0
	return

/* w0=line.idx w0=w3=Fram.pointer w5=Vx.pointer 
  result w0=Value w2=xdata */
init_LastValue1616:
	mov.b	[w3++], w1
	swap	w1
	mov.b	[w3++], w1
	swap	w1
	push	w1
	mov.b	[w3++], w1
	swap	w1
	mov.b	[w3++], w1
	swap	w1
	push	w1

	ze	[w3], w1
	push	w1
	lsr	w1, #4, w1
	and	#0x07, w1		/* akt.X.pos */
	rcall	EingVx_SetCursSelIdx
	pop	w2
	pop	w1
	pop	w0
	return

/* ______________________________________________________________________ */
.section .const, psv
tTC:		.asciz	"\177C"

/*  ----------- StatusLine ---------------------------------------------- */
tIR0:	.asciz	"SQ"
tIR1:	.asciz	"P"
tIR2:	.asciz	"DC"
tIR3:	.asciz	"--"
tIR4:	.asciz	"SQ INV"
tIR5:	.asciz	"P INV"
tIR6:	.asciz	"DC INV"
tIR7:	.asciz	"-- INV"

 .align 2
TAZ_StLineIdx:
  .word tIR0,	tIR1,	tIR2,	tIR3,	tIR4,	tIR5,	tIR6,	tIR7


tSE_ANY:	.asciz	"Sys ERROR ???"
tSE_I2C:	.asciz	"I2C Def."
tSE_FRAM:	.asciz	"F-RAM Def."
tSE_3:		.asciz	"MCP3421 Def."
tSE_4:		.asciz	"ADS112c04 Def."
tSE_5:		.asciz	"DAC7578 Def."
tSE_6:		.asciz	"MCP47FEB Def."
tSE_7:		.asciz	"AD5667 Def."
tSE_8:		.asciz	"MCP4726 Def."
tSE_9:		.asciz	"CDCE913 Def."
tSE_10:		.asciz	"10 Def."
tSE_11:		.asciz	"11 Def."

 .align 2

TAZ_SysError:
  .word tSE_ANY,	tSE_I2C,	tSE_FRAM,	tSE_3
  .word tSE_4,		tSE_5,		tSE_6	,	tSE_7
  .word tSE_8,		tSE_9,		tSE_10,		tSE_11
  .word tSE_ANY,	tSE_ANY,	tSE_ANY,	tSE_ANY

/*  ----------- rais / fall --------------------------------------------- */
trfmin:	.asciz	"min\201ns"
trf13n:	.asciz	"1.2ns"
trf15n:	.asciz	"1.5ns"
trf18n:	.asciz	"1.8ns"
trf20n:	.asciz	"2.0ns"
trfmax:	.asciz	"max\201ns"

 .align 2
tab_rf:
  .word trfmin, trf13n, trf15n, trf18n, trf20n, trfmax

/*  ----------- rais / fall --------------------------------------------- */
trg316_0:	.asciz	"0cm"
trg316_30:	.asciz	"60cm"

 .align 2
tab_rg316:
  .word trg316_0, trg316_30

/* tab_Freq_sel
    flag,  DISP_FLAG, min, max
    flag b0:3 txt
	 b4 no inc MUL/DIV
	 b5 dec no mul
 */

tab_Freq_sel:
.if USE_PLL_CDCE913
  .word 8
  .word 0x0004, 0x1301,     1,   999	/* s0    0.1 - 99.9 Hz  */
  .word 0x0024, 0x0301,   100,   999	/* s1   100 -  999   Hz */
  .word 0x0014, 0x0401,  1000,  9999	/* s2  1000 -  9999   Hz */
  .word 0x0005, 0x2401,  1000,  9999	/* s3  10,00 - 99,99 kHz */
  .word 0x0005, 0x1401,  1000,  9999	/* s4  100.0 - 999.9 kHz */
  .word 0x0006, 0x3401,  1000,  9999	/* s5  1.000 - 9.999 MHz */
  .word 0x0026, 0x2401,  1000,  9999	/* s6  10,00 - 99,99 MHz */
  .word 0x0056, 0x2501, 10000, 23000	/* s7  100,00 - 230,00 MHz */
.else
  .word 2
  .word 0x0004, 0x1301,    10,   999	/* s0  1.0 - 99.9 Hz  */
  .word 0x0004, 0x0401,   100,  8000	/* s1  100 -  9999   Hz */
.endif

tab_Freq_Owner:
  .word tab_Freq_sel, FqHz_Step, -102, 10

tab_Time_sel:
  .word 8
  .word 0x0008, 0x1301,    20,   199	/* s0  2.0 - 19.9    ns  */
  .word 0x0028, 0x0201,    20,    99	/* s1  20 - 99      ns */
  .word 0x0018, 0x0301,   100,   999	/* s2  100 - 999      ns */
  .word 0x0009, 0x2301,   100,   999	/* s3  1.00 - 9.99   us */
  .word 0x0009, 0x1301,   100,   999	/* s4  10.0 - 99.9   us */
  .word 0x0009, 0x0301,   100,   999	/* s5  100 - 999     us */
  .word 0x000A, 0x2301,   100,   999	/* s6  1.00 - 9.99   ms */
  .word 0x000A, 0x1401,   100,  5000	/* s7  10.0 - 99.9   ms */

tab_Time_Owner:
  .word tab_Time_sel, FqTime_Step, -102, 22


tab_Ampl_sel:
  .word 2
  .word 0x0003, 0x1401,    40,  9999	/* s0   4.0 -  999.9 mV  */
  .word 0x0003, 0x0401,  1000,  9000	/* s1   1000 - 9000  mV  */

tab_Ampl_Owner:
  .word tab_Ampl_sel, FqAmpl_Step, -102, 34
