/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "lcd_param.inc"
.include "lib/eingVx.inc"
.include "lib/Param.inc"

.section .nbss, bss, near
;EingVx1:		.space EVX_SIZE
EingVx1_CursLineIdx:	.space 4

LcdTeVal1:		.space 4
LcdTeVal2:		.space 4
.global LcdTeVal1
.global LcdTeVal2

DispValueId:		.space 2
DispValueFlag:		.space 2
	bVXDVF_ID4		= 0

	PARAVX_SIZE			= 12
	oPARAVX_STAT			= 0
	oPARAVX_OWNER			= 2
	oPARAVX_EG_TXT			= 4
	oPARAVX_ORG_VALUE		= 6	/* Fram Data 4byte */
	oPARAVX_OFFSET			= 10

ParaVx1_Owner:		.space PARAVX_SIZE

/*
struct EingVx1 {
 2B EVX_STAT
	 ....
 2B EVX_OWNER = EingVx1_Owner
}

*/


/* oPARAVX_STAT   -->  in sys.inc
  15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
 +---+-----------+---+-----------+-----------+---+---+---+---+---+
 |   |           |   |           |           |   |   |   |   |Off| PARAVX_STAT
 +---+-----------+---+-----------------------+---+---+---+---+---+
*/
/* --------PARAVX_STAT ---------------------------- */
	bPVXS_OFFSET		= 0
	bPVXS_OC		= 1


.section .text

.global init_EingPara
.global LCD_EingPara
.global Key_EingPara

.if USE_UART_LIST_PARA
.global get_val_EingPara
.endif

/* param data pointer w4 */
init_EingPara:
	push.d	w8
	push.d	w10

	mov	#EingVx1, w5
	mov	#0, w0
	mov	#0, w1
	rcall	EingVx_SetCursSelIdx

	mov	#ParaVx1_Owner, w8
	mov	w8, [w5+oEVX_OWNER]
/* clr PARAVX */
	mov	w8, w0
	repeat	#(PARAVX_SIZE >> 1) - 1
	clr	[w0++]

/* set txt 1 */
	mov	[w4], w0
	and	#LCDPA_EINGTXT_MASK, w0
	bra	z, iep_netxt
	lsr	w0, #LCDPA_EINGTXT_SHL, w0
	cp0	w0
	bra	z, iep_netxt
	mov	#tOffs, w1
	cp	w0, #1
	bra	z, iep_etxt
	mov	#tKorr, w1
	cp	w0, #2
	bra	z, iep_etxt
	mov	#tAbs, w1
iep_etxt:
	mov	w1, [w8+oPARAVX_EG_TXT]
iep_netxt:
/* set EVX_DISP_FLAG -> w9 */
	mov	[w4+2], w9
	btsc	w9, #bDVF_DP_MX1
	bset	[w5], #bEVSTAT_CX_M1
	bclr	w9, #bDVF_DP_MX1
	mov	w9, [w5+oEVX_DISP_FLAG]

/* set RAM_DATA_ADR */
	mov	[w4+4], w0
	mov	#0x07FF, w1
	and	w0, w1, w1
	mov	w1, [w5+oEVX_RAM_DATA_ADR]
	btsc	w0, #bLCDPA_IO_REL_RAM
	bset	[w5], #bEVSTAT_REL_RAM

	lsr	w0, #12, w0
	rcall	EingVx_SetTxt

/* set min/max -> w10, w11 */
	mov	[w4], w1
	lsr	w1, #12, w1
	btsc	w9, #bDVF_IS_BYTE
	and	#0x07, w1
	btsc	w9, #bDVF_IS_BYTE
	add	w1, #8, w1
	
	mov	#0x8000, w10
	btsc	w9, #bDVF_IS_UNSIGN
	mov	#0x0000, w10
	asr	w10, w1, w10		; w10 EVX_DISP_MIN

	mov	#0x7FFF, w11
	btsc	w9, #bDVF_IS_UNSIGN
	mov	#0xFFFF, w11
	lsr	w11, w1, w11		; w11 EVX_DISP_MAX

	add	w4, #6, w3
	clr	w0
	btsc	[w4], #bLCDPA_ONCANGE
	mov	[w3++], w0
	mov	w0, [w5+oEVX_ON_VAL_CHANGE]
	cp0	w0
	bra	z, no_oc
	mov	#0xFE00, w1
	and	w0, w1, w1
	bra	z, no_oc
	btsc	[w4], #bLCDPA_OC
	bset	[w8], #bPVXS_OC
no_oc:
	clr	w0
	btsc	[w4], #bLCDPA_OWNER
	mov	[w3++], w0
	mov	w0, [w8+oPARAVX_OWNER]

	btst	[w4], #bLCDPA_AZ_OFFSET
	bra	z, no_rd_az_offs
	mov	[w3++], w0
	mov	w0, [w8+oPARAVX_OFFSET]
	bset	[w8], #bPVXS_OFFSET
no_rd_az_offs:
/* DispExV Test value display */
	clr	DispValueFlag
	mov	[w4], w0
	mov	#LCDPA_DISEVX_MASK, w1
	and	w0, w1, w0
	bra	z, no_disevx
	lsr	w0, #LCDPA_DISEVX_SHL, w0
	cp	w0, #1
	bra	nz, no_disevx
	mov	[w3++], w0
	mov	w0, DispValueId
	bset	DispValueFlag, #bVXDVF_ID4
no_disevx:

	mov	[w4+4], w0		/* test txt (15) mV/C */
	mov	#LCDPA_IO_TXT_MASK, w1
	and	w0, w1, w0
	cp	w0, w1
	mov	#-100, w0
	btsc	SR, #Z
	mov	#-85, w0
	rcall	EingVx_SetDispX

	btst	[w8], #bPVXS_OFFSET
	bra	z, az_offs_end
;i1616_offs:
	btst	w9, #bDVF_DP_X10
	bra	z, no_x10_offs

	mov	#-3276, w0
	cpslt	w0, w10
	mov	w0, w10

	mov	#3276, w0
	cpsgt	w0, w11
	mov	w0, w11
no_x10_offs:
	btst	w9, #bDVF_IS_UNSIGN
	bra	nz, az_offs_u
	push	w4
	mov	#0x8001, w4		; add sign
	mov	w10, w0
	mov	[w8+oPARAVX_OFFSET], w2
	rcall	AddSub16_Sat
	mov	w0, w10
	mov	w11, w0
	mov	[w8+oPARAVX_OFFSET], w2
	rcall	AddSub16_Sat
	mov	w0, w11
	pop	w4
	bra	az_offs_end
az_offs_u:
	mov	[w8+oPARAVX_OFFSET], w0
	mov	w11, w1
	rcall	sub16u_sat
	mov	w0, w10
	mov	[w8+oPARAVX_OFFSET], w0
	mov	w11, w1
	rcall	add16u_sat
	mov	w0, w11

az_offs_end:
	mov	w10, [w5+oEVX_DISP_MIN]
	mov	w11, [w5+oEVX_DISP_MAX]

	add	w8, #oPARAVX_ORG_VALUE, w6
	rcall	EingVx_LoadValue

	btst	[w8], #bPVXS_OFFSET
	bra	z, add_offs_end
/* add Offset */
	btst	w9, #bDVF_IS_1616
	bra	nz, add_offs_v1616

	mov	w0, w1
	mov	[w8+oPARAVX_OFFSET], w0
	btss	w9, #bDVF_IS_UNSIGN
	rcall	add16s_sat
	btsc	w9, #bDVF_IS_UNSIGN
	rcall	add16us_sat
	bra	add_offs_end
add_offs_v1616:
	mov	w10, w6
	mov	w11, w7


	clr	w2
	mov	[w8+oPARAVX_OFFSET], w3
	push	w4
	mov	#0x8001, w4		; add sign
	rcall	AddSub1616_MinMax
	pop	w4
add_offs_end:

	rcall	EingVx_SetValue

	mov	#EingVx1_CursLineIdx, w6
	rcall	EingVx_Push_CursLineIdx
	mov	#0, w0			/* max.line-1 */
	mov	#0, w1			/* akt.line */
	rcall	EingVx_InitCursLineIdx

	btst	[w8], #bPVXS_OC
	bra	z, no_open_oc
	mov	#ON_CHANGE_OPEN, w3
	mov	[w8+oPARAVX_OWNER], w6
	rcall	EingVx_DoChange
no_open_oc:
	pop.d	w10
	pop.d	w8
	return

/* ______________________________________________________________________ */
LCD_EingPara:
	dec	w0, w0
	bra	z, LCD_MenuCaption
	dec	w0, w0
	bra	z, ParaStatus
	dec	w0, w0
	bra	z, Line1
	dec	w0, w0
	bra	z, Line2
	dec	w0, w0
	bra	z, Line3
	dec	w0, w0
	bra	z, Line4
	dec	w0, w0
	bra	z, Line5
	dec	w0, w0
	bra	z, Line6
	bra	LCD_Paint
ParaStatus:
	rcall	StatusLine
	mov	#0, w0
	mov	#14, w1
	bra	LCD_StrBufOut_cs

Line1:
	mov	#EingVx1, w5
	mov	#24, w0
	mov	w0, [w5+oEVX_DISP_Y]

	mov	[w5+oEVX_OWNER], w0
	cp0	w0
	bra	z, l1_netxt
	mov	[w0+oPARAVX_EG_TXT], w2
	cp0	w0
	bra	z, l1_netxt
	mov	#0, w0			; X pos
	mov	[w5+oEVX_DISP_Y], w1
	mov	[w5+oEVX_DISP_FLAG], w3
	rcall	StrOut_dvf
l1_netxt:
	rcall	DispVxVal

	inc	LCD_Step
	return

Line2:
	btst	DispValueFlag, #bVXDVF_ID4
	bra	z, LCD_Step_Inc
	mov	DispValueId, w0
	lsr	w0, #12, w0
	rcall	DispValId
	bra	z, LCD_Step_Inc

	mov	#0, w0
	mov	#42, w1
	bra	LCD_StrBufOut_cs

Line3:
	btst	DispValueFlag, #bVXDVF_ID4
	bra	z, LCD_Step_Inc
	mov	DispValueId, w0
	lsr	w0, #8, w0
	rcall	DispValId
	bra	z, LCD_Step_Inc

	mov	#66, w0
	mov	#42, w1
	bra	LCD_StrBufOut_cs

Line4:
	btst	DispValueFlag, #bVXDVF_ID4
	bra	z, LCD_Step_Inc
	mov	DispValueId, w0
	lsr	w0, #4, w0
	rcall	DispValId
	bra	z, LCD_Step_Inc

	mov	#0, w0
	mov	#50, w1
	bra	LCD_StrBufOut_cs
Line5:
	btst	DispValueFlag, #bVXDVF_ID4
	bra	z, LCD_Step_Inc
	mov	DispValueId, w0
	rcall	DispValId
	bra	z, LCD_Step_Inc

	mov	#66, w0
	mov	#50, w1
	bra	LCD_StrBufOut_cs

Line6:
	mov	#0, w0
	mov	#58, w1
	bra	StrbTemp_cs_step

/* ______________________________________________________________________ */
Key_EingPara:
	mov	#EingVx1, w5
	rcall	EingVx_Key
	bra	z, Val1_Cange

	cp	w0, #KEY_RET
	bra	z, K_CRet
	return

Val1_Cange:
	btst	[w5], #bEVSTAT_ONCHANGE
	bra	nz, Key_RetOk

	btst	[w5], #bEVSTAT_VAL_CHANGE
	bra	z, Key_RetOk

	rcall	korr_val
	rcall	EingVx_ValueToMem
	clr	w3
	rcall	EingVx_DoChange
	bra	Key_RetOk

K_CRet:
	mov	#EingVx1_CursLineIdx, w6
	rcall	EingVx_Pop_CursLineIdx

	rcall	korr_val
	rcall	EingVx_SaveValue

	mov	[w5+oEVX_OWNER], w0
	cp0	w0
	bra	z, Key_RetOk_Menu
	btst	[w0], #bPVXS_OC
	bra	z, Key_RetOk_Menu
	mov	#ON_CHANGE_CLOSE, w3
	mov	[w0+oPARAVX_OWNER], w6
	rcall	EingVx_DoChange

	bra	Key_RetOk_Menu

korr_val:
	mov [w5+oEVX_VALUE_1616], w0
	mov [w5+oEVX_VALUE], w1
	clr	w6
	clr	w7

	push	w8
	mov	[w5+oEVX_OWNER], w8
	cp0	w8
	bra	z, no_own
	btst	[w8], #bPVXS_OFFSET
	bra	z, no_vkorr
.if 1
	mov	[w8+oPARAVX_OFFSET], w3
	sub	w1, w3, w1
.else
	push	w4
	mov	#0x8003, w4		; sub sign
	mov	[w5+oEVX_DISP_FLAG], w2
	btsc	w2, #bDVF_IS_UNSIGN
	mov	#0x0003, w4		; sub unsign
	
	clr	w2
	mov	[w8+oPARAVX_OFFSET], w3

	rcall	AddSub1616_Sat
	pop	w4
.endif

no_vkorr:
	mov	[w8+oPARAVX_OWNER], w6
	add	w8, #oPARAVX_ORG_VALUE, w7
no_own:
	pop	w8
	return


/* ______________________________________________________________________ */
.if USE_UART_LIST_PARA
/* w2=LineIdx w7=ParamLine w4=Ext Data */
get_val_EingPara:
	cp	w2, #1
	bra	nz, ucMenuSend_NULL

	cp0	w4
	bra	z, ucMenuSend_NULL

	btst	[w4], #bLCDPA_AZ_OFFSET
	bra	z, gvep_dir

	mov	[w4+4], w2		; RAM_DATA_ADR
	mov	#EVCURS_FRAM_MASK, w1
	and	w2, w1, w2

	mov	[w4+2], w0		; DVF
	rcall	GetDVF_ParaSize_w3

	rcall	FRam_GetValue
	bra	ov, ucMenuSend_NULL

	clr	w6
	mov	[w4+2], w2		; DVF
	rcall	GetDVF_ConvertValue

	add	w4, #6, w3
	btsc	[w4], #bLCDPA_ONCANGE
	inc2	w3, w3
	btsc	[w4], #bLCDPA_OWNER
	inc2	w3, w3
	nop
	mov	[w3], w5

	mov	[w4+2], w2		; DVF
	btss	w2, #bDVF_IS_1616
	add	w0, w5, w0
	btsc	w2, #bDVF_IS_1616
	add	w1, w5, w1

	lnk	#4
	mov	w0, [w14]
	mov	w1, [w14+2]
	mov	w14, w0
	mov	[w4+2], w2
	mov	#Str_buffer, w3
	rcall	dvf_to_ppc
	ulnk
	return

gvep_dir:
	mov	[w4+4], w0		; RAM_DATA_ADR
	mov	#EVCURS_FRAM_MASK, w1
	and	w0, w1, w0
	mov	#EEData0, w1
	add	w0, w1, w0

	mov	[w4+2], w2
	mov	#Str_buffer, w3
	bra	dvf_to_ppc

.endif  /* USE_UART_LIST_PARA */

/* ______________________________________________________________________ */
.section .const, psv
tOffs:		.asciz	"Offs:"
tKorr:		.asciz	"Korr:"
tAbs:		.asciz	"Abs:"
