/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/Param.inc"
.include "lib/Analog.inc"
.include "lib/eingVx.inc"
.include "lib/CDCE913.inc"
.if USE_DEBUG_UART_USER
.include "lib/uart_user.inc"
.endif

.if MAX_SYM_DATA > 24
.ERROR  "MAX_SYM_DATA > 24"
.endif

.section .nbss, bss, near
Filter_CursLineIdx:	.space 4
OrigVal:		.space 2
SymFilterGTI0_offset:	.space 2

.global SymFilterGTI0_offset

/* ---------------------------------------------------------------------- */
.section .text

.global LCD_Sym
.global Key_Sym
.global init_SymPara
.global SetGenSym
.global SetGenPllSym

.if USE_UART_LIST_PARA
.global get_val_Sym
.endif

LCD_Sym:
	dec	w0, w0
	bra	z, LCD_MenuCaption
	mov	#EingVx0, w5
	dec	w0, w0
	bra	z, LineY
	mov	#EingVx1, w5
	dec	w0, w0
	bra	z, LineY
	dec	w0, w0
	bra	z, LineSt1
	dec	w0, w0
	bra	z, LineSt2
	dec	w0, w0
	bra	z, LineSt3
	dec	w0, w0
	bra	z, LineSt4
	bra	LCD_Paint

LineY:
	clr	w0
	mov	[w5+oEVX_DISP_Y], w1
	mov	[w5+oEVX_P_TXT], w2
	rcall	StrOut8x13
	rcall	DispVxVal

	inc	LCD_Step
	return

LineSt1:
	mov	#LCDD_GOV, w0
	rcall	DispValId
	bra	z, LCD_Step_Inc

	mov	#0, w0
	mov	#50, w1
	bra	LCD_StrBufOut_cs
LineSt2:
	mov	#LCDD_GRV, w0
	rcall	DispValId
	bra	z, LCD_Step_Inc

	mov	#0, w0
	mov	#58, w1
	bra	LCD_StrBufOut_cs

LineSt3:
	mov	#LCDD_OP1V, w0
	rcall	DispValId
	bra	z, LCD_Step_Inc

	mov	#66, w0
	mov	#50, w1
	bra	LCD_StrBufOut_cs

LineSt4:
	mov	#LCDD_PA, w0
	rcall	DispValId
	bra	z, LCD_Step_Inc

	mov	#66, w0
	mov	#58, w1
	bra	LCD_StrBufOut_cs

/* ______________________________________________________________________ */
Key_Sym:
	mov	#EingVx0, w5
	rcall	EingVx_Key
	bra	z, Val_Cange

	mov	#EingVx1, w5
	rcall	EingVx_Key
	bra	z, Val_Cange

	cp	w0, #KEY_Square
	bra	z, Key_RetOk

	cp	w0, #KEY_RET
	bra	z, K_CRet
	return

Val_Cange:
	btst	[w5], #bEVSTAT_ONCHANGE
	bra	nz, Key_RetOk
	btst	[w5], #bEVSTAT_VAL_CHANGE
	bra	z, Key_RetOk
	rcall	EingVx_ValueToMem

.if 1
	rcall	SetGenPllSym
.else
	mov	EingVx0+oEVX_VALUE, w0
	sl	w0, #2, w0
	mov	EEData0+oFR_APSYM, w2
	rcall	AddSat_s12b
	mov	#APSYM, w2
	rcall	DacSetValue_R_Align


	mov	EingVx1+oEVX_VALUE, w0
	mov	EEData0+oFR_AGTI0, w2
	rcall	AddSat_s12b
	mov	#AGTI0, w2
	rcall	DacSetValue_R_Align
.endif

	bra	Key_RetOk

K_CRet:
	mov	#Filter_CursLineIdx, w6
	rcall	EingVx_Pop_CursLineIdx

	mov	EingVx0+oEVX_RAM_DATA_ADR, w2
	mov	#EEData0, w0
	add	w2, w0, w3
	mov	OrigVal, w0
	cp	w0, [w3]
	bra	z, val_ok
	mov	#1, w3
	rcall	FRam_Save
val_ok:
	mov	last_PA_OutFlag, w0
	mov	w0, PA_OutFlag

	btsc	LastIoMk, #bACIOF_INV
	bset	INV_PORT, #INV_BIT
	btss	LastIoMk, #bACIOF_INV
	bclr	INV_PORT, #INV_BIT

	rcall	SetUpFreq
	bset	MathFlags, #bMF_AMPL

	bra	Key_RetOk_Menu

/* ______________________________________________________________________ */
/* param data pointer w4 */
init_SymPara:
	cp	w4, #MAX_SYM_DATA
	bra	geu, isp_ov
	sl	w4, w0
	mov	#tabFreqSym, w1
	add	w0, w1, w2
	nop
	mov	[w2], w0
	cp0	w0
	bra	z, isp_ov

	push	w4
	mov	#10000, w1
	mul.uu	w0, w1, w0

	mov	PA_OutFlag, w2
	mov	w2, last_PA_OutFlag

	clr	LastIoMk
	btsc	INV_PORT, #INV_BIT
	bset	LastIoMk, #bACIOF_INV

	clr	w2
	mov	w0, GenFreq
	mov	w1, GenFreq+2
	mov	w2, GenFreq+4
	rcall	MathFreq


	mov	#0x0013, w0		; ~ 500mV
	rcall	SetAmplRelais
	rcall	SetGenV_Max

	pop	w4

	mov	#Filter_CursLineIdx, w6
	rcall	EingVx_Push_CursLineIdx

	push.d	w8

	sl	w4, w0
	mov	#oFR_SYM_DATA, w1
	add	w0, w1, w9

	mov	#EEData0, w1
	add	w9, w1, w8

	mov	#1, w0			/* max.line-1 */
	mov	#0, w1			/* akt.line */
	rcall	EingVx_InitCursLineIdx

	mov	[w8], w0
	mov	w0, OrigVal

	mov	#EingVx0, w5
	mov	#0, w0			/* line.idx */
	mov	#0, w1			/* akt.X.pos */
	rcall	EingVx_SetCursSelIdx


	ze	[w8++], w0
	mov	#0, w2			/* min. */
	mov	#255, w3		/* max. */
	mov	#0x0303, w4		/* Num=3 bDVF_IS_UNSIGN */
	rcall	EingVx_SetValueMinMaxDflag

	mov	w9,  [w5+oEVX_RAM_DATA_ADR]
	mov	#58, w0			/* X.pos */
	mov	w0, [w5+oEVX_DISP_X]
	mov	#18, w0			/* Y.pos */
	mov	w0, [w5+oEVX_DISP_Y]

	mov	#APSYM, w0
	rcall	GetDAC_txt
	mov	w0, [w5+oEVX_P_TXT]


	mov	#EingVx1, w5
	mov	#1, w0			/* line.idx */
	mov	#0, w1			/* akt.X.pos */
	rcall	EingVx_SetCursSelIdx

	se	[w8], w0
	mov	#-128, w2		/* min. */
	mov	#127, w3		/* max. */
	mov	#0x0302, w4		/* Num=3 bDVF_IS_SIGN */
	rcall	EingVx_SetValueMinMaxDflag

	inc	w9, w9
	mov	w9,  [w5+oEVX_RAM_DATA_ADR]
	mov	#50, w0			/* X.pos */
	mov	w0, [w5+oEVX_DISP_X]
	mov	#34, w0			/* Y.pos */
	mov	w0, [w5+oEVX_DISP_Y]

	mov	#AGTI0, w0
	rcall	GetDAC_txt
	mov	w0, [w5+oEVX_P_TXT]

	pop.d	w8
	bclr	SR, #OV
	return

isp_ov:
	bset	SR, #OV
	return

/* ______________________________________________________________________ */
SetGenSym:
	mov	EEData0+oFR_APSYM, w0
	mov	#APSYM, w2
	rcall	DacSetValue_R_Align

	mov	EEData0+oFR_AGTI0, w0
	mov	#AGTI0, w2
	rcall	DacSetValue_R_Align

	mov	EEData0+oFR_AGTI1, w0
	mov	#AGTI1, w2
	rcall	DacSetValue_R_Align

	return


SetGenPllSym:
	btst	pll_stat, #bPLLST_POW_ON
	bra	nz, pll_pow_on
	mov	#-2048, w0
	mov	#OSYM, w2
	bra	DacSetValue_R_Align

pll_pow_on:
	mov	CDCE913_DIV1, w0
	cp	w0, #1
	bra	z, pll_115
	mov	EEData0+oFR_OSYM, w0
	bra	pll_OSYM
pll_115:
	mov	EEData0+oFR_OSYM_115MHZ, w0
pll_OSYM:
	mov	#OSYM, w2
	rcall	DacSetValue_R_Align

	push.d	w8
	push	w10

.if USE_DEBUG_UART_USER
	btst	uuCAL_DAC_Mod, #bGEN_SYM_OFF
	bra	nz, psy_le_500k
.endif

	mov	CDCE913_fout, w0
	mov	CDCE913_fout+2, w1
	mov	#10000, w2
	repeat	#17
	div.ud	w0, w2
/* Round */
	lsr	w2, w2
	sub	w1, w2, w1
	addc	w0, #0, w0

	mov	#50, w1
	cp	w0, w1
	bra	leu, psy_le_500k

	mov	#tabFreqSym, w8
	mov	#EEData0+oFR_SYM_DATA, w9
	mov	#MAX_SYM_DATA, w10
	clr	w4
	mov	#50, w6
lnext:
	mov	w4, w5
	mov	w6, w7
lloop:
	mov	[w8++], w6
	mov	[w9++], w4
	cp0	w4
	bra	z, lskip
	cp0	w6
	bra	z, lskip

	cp	w0, w6
	bra	z, lfind_0
	bra	ltu, lfind
	dec	w10, w10
	bra	nz, lnext
	bra	lfind
lskip:
	mov	w5, w4
	mov	w7, w6
	dec	w10, w10
	bra	nz, lloop
lfind_0:
	ze	w4, w8
	swap	w4
	se	w4, w9
	bra	lend

lfind:
	sub	w6, w7, w6
	sub	w0, w7, w7
	ze	w4, w0
	ze	w5, w8
	sub	w0, w8, w0
	mov	w7, w1
	mul.su	w0, w1, w0

	mov	w6, w2
	rcall	div_s32_u16_s16r
	add	w8, w0, w8

	swap	w4
	se	w4, w0
	swap	w5
	se	w5, w9
	sub	w0, w9, w0
	mov	w7, w1
	mul.su	w0, w1, w0

	mov	w6, w2
	rcall	div_s32_u16_s16r
	add	w9, w0, w9

lend:
	sl	w8, #2, w0
	mov	EEData0+oFR_APSYM, w2
	rcall	AddSat_s12b
	mov	#APSYM, w2
	rcall	DacSetValue_R_Align

	mov	EEData0+oFR_AGTI0, w2
	mov	SymFilterGTI0_offset, w0
	add	w2, w0, w2
	mov	w9, w0

	rcall	AddSat_s12b
	mov	#AGTI0, w2
	rcall	DacSetValue_R_Align

	pop	w10
	pop.d	w8

	return

psy_le_500k:
	clr	w8
	clr	w9
	bra	lend

/* ______________________________________________________________________ */
.if USE_UART_LIST_PARA
/* w2=LineIdx w7=ParamLine w4=Ext Data */
get_val_Sym:
	cp	w4, #MAX_SYM_DATA
	bra	z, ucMenuSend_NULL

	cp	w2, #2
	bra	gtu, ucMenuSend_NULL

	dec	w2, w2
	sl	w4, w4
	add	w4, w2, w4
	mov	#EEData0+oFR_SYM_DATA, w0
	add	w4, w0, w4

	btsc	w2, #0
	se	[w4], w0
	btss	w2, #0
	ze	[w4], w0
	mov	#Str_buffer, w3
	mov	#0, w2
	bra	int16D3toppc

.endif  /* USE_UART_LIST_PARA */


/* ______________________________________________________________________ */
.section .const, psv
tabFreqSym:
/*  Freq 10kHz */
 .word	100
 .word	200
 .word	500
 .word	1000
 .word	2000
 .word	2500
 .word	3000
 .word	4000

 .word	5000
 .word	6000
 .word	7500
 .word	8800
 .word	10000
 .word	11000
 .word	12000
 .word	13500

 .word	15000
 .word	16000
 .word	17000
 .word	19000
 .word	20000
 .word	21000
 .word	22000
 .word	23000
