/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
;.include "Analog.inc"
;.include "Param.inc"

/*  Use AD5667_A0 !!! */


.section .nbss, bss, near
AD5667_OutFlag:			.space 2
.equ bMK_INI_LDAC,		15

dac_channel:			.space 2
AD5667_Data:			.space 4

dac0_TempKorr:			.space 2
dac1_TempKorr:			.space 2
LastTemp:			.space 2

.global AD5667_OutFlag

.section .text
.global AD5667_INIT
.global AD5667_Handle
.global AD5667_i2c_run
.global AD5667_SetValue
.global AD5667_GetValue
.global AD5667_SetParam

AD5667_INIT:
	mov	#0x8000, w0
	mov	w0, AD5667_Data
	mov	w0, AD5667_Data+2
	mov	#0x0003 | (1<<bMK_INI_LDAC), w0
	mov	w0, AD5667_OutFlag
	return

AD5667_Handle:
	cp0	i2c2_Lock
	bra	nz, lend

	btst	I2CErrorFlag, #I2C_AD5667
	bra	nz, lerr

	btst	AD5667_OutFlag, #bMK_INI_LDAC
	bra	nz, IniLDAC

	mov	AD5667_OutFlag, w0
	ff1r	w0, w4
	cp0	w4
	bra	z, lend
	cp	w4, #2
	bra	gtu, lerr

	dec	w4, w4			/* channel */
	mov	w4, dac_channel
	mov	#1, w0
	sl	w0, w4, w0
	com	w0, w0
	and	AD5667_OutFlag		/* clr bit */

	sl	w4, w2
	mov	#AD5667_Data, w3
	add	w3, w2, w3

	and	w4, #0x01, w0
	ior	#0x18, w0
	mov	[w3], w1

Send_cd:
	mov	#i2c2_Buf, w2
	swap	w1
	mov.b	w0, [w2++]		/* Command + Reg */
	mov.b	w1, [w2++]		/* Data */
	swap	w1
	mov.b	w1, [w2++]

	mov	#3, w0
	mov	w0, i2c2_DataSize

	mov	#0x1E, w0
	mov	w0, i2c2_id

	mov	#I2C_WRITE, w0
	mov	w0, i2c2_cmd

	clr	i2c2_Flag
	bset	i2c2_Flag, #I2C_bNoADR

	mov	#I2C_AD5667, w0
	mov	w0, i2c2_Lock

lend:
	return

lerr:
	clr	AD5667_OutFlag
	return

IniLDAC:
	bclr	AD5667_OutFlag, #bMK_INI_LDAC
	mov	#0x30, w0		/* LDAC register setup */
	mov	#0x0003, w1		/* LDAC PIN 1=DISABLED */
;	mov	#0x0000, w1		/* LDAC PIN ENABLED */
	bra	Send_cd


AD5667_i2c_run:
	mov	i2c2_cmd, w0
	cp0	w0
	bra	nz, chk_err
	clr	i2c2_Lock
	return

chk_err:
	cp	w0, #I2C_ERROR
	bra	z, is_err
	return		/* Wait */

is_err:
	bset	I2CErrorFlag, #I2C_AD5667
	clr	i2c2_Lock
	return

/* w0=Value w2=DAC */
AD5667_SetValue:
	mov	#AD5667_Data, w3
	mov	#0x01, w1
	rcall	DAC_KorrSetValue
	btss	SR, #Z
	ior	AD5667_OutFlag
	return

/* w0=Value w2=DAC */
AD5667_GetValue:
	mov	#AD5667_Data, w3
	mov	#0x01, w1
	bra	DAC_KorrGetValue

/* w0 = 0.025C */
AD5667_TempKorr:
/*
	mov	Temp_OP1_0025, w0

	mov	#1200, w1
	sub	w0, w1, w0		; Temp - 30.0C
	bra	z, tk_clr

	mov LastTemp, w1
	sub	w1, w0, w1
	btsc	SR, #Z
	return
	btsc	w1, #15
	neg	w1, w1
	dec	w1, w1
	btsc	SR, #Z
	return
	mov	w0, LastTemp

	mov.b	EEData0+oFR_DAC_POW_TC, WREG
	se	w0, w0
	mov	LastTemp, w1
	mul.ss	w0, w1, w0		; 0.025C * 0.1 digit / C 
	mov	w0, dac0_TempKorr

	mov.b	EEData0+oFR_DAC_DC_TC, WREG
	se	w0, w0
	mov	LastTemp, w1
	mul.ss	w0, w1, w0		; 0.025C * 0.1 digit / C 
	mov	w0, dac1_TempKorr


	return

tk_clr:
	clr	LastTemp
	clr	AdcPa_TempOffs
	clr	AdcPa_TempKorr
*/
	return

AD5667_SetParam:
	clr	LastTemp
	return
