/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "Analog.inc"
.include "uart_user.inc"
.include "../math/libm.inc"

.if USE_DEBUG_UART_USER
.include "Param.inc"
.endif

.section .nbss, bss, near
.if USE_DEBUG_UART_USER
ADS112c04_ReadCount:		.space 2
.endif
ADS112c04_ms_count:		.space 2
InitFlag:			.space 2
Step:				.space 2

Anl_GOV_orig:			.space 2
Anl_GRV_orig:			.space 2
Anl_OP1V_orig:			.space 2
Anl_GenTemp_orig:		.space 2

GOV_01mV:			.space 2
GRV_01mV:			.space 2
OP1V_1616_001mV:		.space 4
Temp_GEN_0025:			.space 2

LastTemp:			.space 2

fAdcOP1_mV:			.space F48_SIZE
fAdcOP1_Mul_p:			.space F48_SIZE
fAdcOP1_Mul_n:			.space F48_SIZE
fAdcOP1_Offs:			.space F48_SIZE
fAdcOP1_Temp_KTC:		.space F48_SIZE
fAdcOP1_Korr_TC:		.space F48_SIZE
fAdcOP1_Mul_0_p:		.space F48_SIZE
fAdcOP1_Mul_0_n:		.space F48_SIZE
fAdcOP1_Offs_0:			.space F48_SIZE


.if USE_DEBUG_UART_USER
.global ADS112c04_ReadCount
.endif
.global ADS112c04_ms_count
.global Anl_GOV_orig
.global Anl_GRV_orig
.global Anl_OP1V_orig
.global Anl_GenTemp_orig

.global GOV_01mV
.global GRV_01mV
.global OP1V_1616_001mV
.global Temp_GEN_0025

.global fAdcOP1_mV
;.global fAdcOP1_Temp_KTC

.section .text
.global ADS112c04_INIT
.global ADS112c04_Handle
.global ADS112c04_i2c_run
;.global ADS112c04_TempKorr
.global ADS112c04_SetParam

ADS112c04_INIT:
	mov	#FC48_ADS112c04_OP1_DIGIT_MV, w2
	mov	#fAdcOP1_Mul_0_p, w4
	rcall	f48_mov
	mov	#fAdcOP1_Mul_0_n, w4
	rcall	f48_mov

	mov	#3, w0		/* b0=Set Config b1=Reset */
	mov	w0, InitFlag
	clr	Step
	bra	tk_clr

ADS112c04_Handle:
	cp0	i2c2_Lock
	bra	nz, lend

	btst	InitFlag, #0
	bra	nz, SetConfig

	btst	I2CErrorFlag, #I2C_ADS112c04
	bra	nz, SetTimeDefect

	mov	#0x80, w0
	mov	w0, i2c2_id

	mov	#0x10, w0	/* RDATA */
	mov	w0, i2c2_adr


	mov	#I2C_READ, w0
	mov	w0, i2c2_cmd

	clr	i2c2_Flag
;	bset	i2c2_Flag, #I2C_bNoADR

	mov	#I2C_ADS112c04, w0
	mov	w0, i2c2_Lock
	return


SetConfig:
	btst	I2CErrorFlag, #I2C_ADS112c04
	bra	nz, SetTimeDefect

	btst	InitFlag, #1
	bra	nz, SetReset

	mov	Step, w3
	and	#0x03, w3

	mov	#0x2640, w0	/* Reg0 0010 : AINP = AIN0, AINN = AIN3 G8 0.256 V */
	mov	#0x0044, w1	/* Reg1 0 GOV */
	cp	w3, #0
	bra	z, SendConf

	mov	#0x5640, w0	/* Reg0 0101 : AINP = AIN1, AINN = AIN3 G8 0.256 V */
	mov	#0x0044, w1	/* Reg1 0 GRV */
	cp	w3, #1
	bra	z, SendConf

	mov	#0x6640, w0	/* Reg0 0110 : AINP = AIN2, AINN = AIN3 G8 0.256 V */
	mov	#0x0044, w1	/* Reg1 0 OP1V */
	cp	w3, #2
	bra	z, SendConf

	mov	#0x2040, w0	/* Reg0 0010 : AINP = AIN0, AINN = AIN3 */
	mov	#0x0144, w1	/* Reg1 Temperature sensor */

SendConf:
	mov	w0, i2c2_Buf
	mov	w1, i2c2_Buf+2
	mov	#0x0808, w0	/* START/SYNC */
	mov	w0, i2c2_Buf+4

	mov	#5, w0
Send_size_w0:
	mov	w0, i2c2_DataSize

	mov	#0x80, w0
	mov	w0, i2c2_id

	mov	#I2C_WRITE, w0
	mov	w0, i2c2_cmd

	clr	i2c2_Flag
	bset	i2c2_Flag, #I2C_bNoADR

	mov	#I2C_ADS112c04, w0
	mov	w0, i2c2_Lock

	mov	#60, w0
	mov	w0, ADS112c04_ms_count
	return

SetReset:
	mov	#0x0606, w0	/* START/SYNC */
	mov	w0, i2c2_Buf

	mov	#1, w0
	bra	Send_size_w0


SetTimeDefect:
	mov	#10000, w0
	mov	w0, ADS112c04_ms_count
	clr	InitFlag
lend:
	return


ADS112c04_i2c_run:
	mov	i2c2_cmd, w0
	cp0	w0
	bra	nz, chk_err

	btst	InitFlag, #0
	bra	nz, EndConf

	mov	Step, w2
	and	#0x03, w2
	sl	w2, w0
	mov	#Anl_GOV_orig, w3
	add	w3, w0, w3

	mov	i2c2_Buf, w0
	swap	w0
	mov	w0, [w3]

	cp	w2, #0
	bra	z, rd_ad0
	cp	w2, #1
	bra	z, rd_ad1
	cp	w2, #2
	bra	z, rd_ad2

	cp	w2, #3
	bra	nz, rd_ad0
;Read Temp C
.if USE_DEBUG_UART_USER
	inc	ADS112c04_ReadCount
.endif
	mov	#20480, w1	/* Value -> 0.025C */
	mul.su	w0, w1, w0
	mov.b	EEData0+oFR_OT_GEN, WREG
	se	w0, w0
	sl	w0, #2, w0
	add	w1, w0, w1
	mov	w1, Temp_GEN_0025
	rcall	ADS112c04_TempKorr
	rcall	DoGenTemp
	bra	rd_e

rd_ad0:
	mov	#33320, w1	/* Value -> 0.1mV */
	mul.su	w0, w1, w0
	mov	w1, GOV_01mV
	btsc	AutoCalDAC, #bAC_ADS112_GOV
	rcall	AutoCal
	bra	rd_e

rd_ad1:
	mov	#33320, w1	/* Value -> 0.1mV */
	mul.su	w0, w1, w0
	mov	w1, GRV_01mV
	btsc	AutoCalDAC, #bAC_ADS112_GRV
	rcall	AutoCal
	bra	rd_e

rd_ad2:
/*
R1=68k+10k=78k   R2=20k  -> 78/20+1 = 4.9
ref=2048mV / 8G = +-256mV => 256*2/65536 = 0.0078125mV / digit
0.0078125 * 4.9 = 0.03828125mV / digit
0.03828125 / 0.05 = 0.765625 * 65536 = 50176
*/

	mov	w0, Anl_OP1V_orig

	mov	#F48_tmp_0, w3
	rcall	i16_f48_w3

	mov	#fAdcOP1_Mul_p, w2
	btsc	Anl_OP1V_orig, #15
	mov	#fAdcOP1_Mul_n, w2
	mov	#fAdcOP1_mV, w4
	rcall	f48_mul

	mov	#fAdcOP1_Offs, w2
	mov	w4, w3
	rcall	f48_add

	mov	#FC48_10, w2
	mov	#F48_tmp_0, w4
	rcall	f48_mul
	rcall	f48_i1616

	mov	w0, OP1V_1616_001mV
	mov	w1, OP1V_1616_001mV+2

	btsc	AutoCalDAC, #bAC_ADS112_OP1V
	rcall	AutoCal
;	bra	rd_e

rd_e:
	bset	InitFlag, #0
	inc	Step
	clr	i2c2_Lock
	return

EndConf:
	btst	InitFlag, #1
	bra	nz, EndReset

	bclr	InitFlag, #0
	clr	i2c2_Lock
	return

EndReset:
	bclr	InitFlag, #1
	clr	i2c2_Lock
;	mov	#10, w0
;	mov	w0, ADS112c04_ms_count
	return


chk_err:
	cp	w0, #I2C_ERROR
	bra	z, is_err
	return		/* Wait */

is_err:
	bset	I2CErrorFlag, #I2C_ADS112c04
	clr	i2c2_Lock
	mov	#10000, w0
	mov	w0, ADS112c04_ms_count
	return

/* w0 = 0.025C */
ADS112c04_TempKorr:
.if USE_DEBUG_UART_USER
	btst	uuCAL_DAC_Mod, #bADS112C04_KORR_OFF
	bra	nz, tk_clr
.endif

	mov	Temp_GEN_0025, w0
	mov	#1200, w1
	sub	w0, w1, w0		; Temp - 30.0C
	bra	z, tk_clr

	mov	LastTemp, w1
	sub	w1, w0, w1
	btsc	SR, #Z
	return
	btsc	w1, #15
	neg	w1, w1
	dec	w1, w1
	btsc	SR, #Z
	return
	mov	w0, LastTemp

	mov	#F48_tmp_0, w3
	rcall	i16_f48_w3
	mov	#FC48_0025, w2
	mov	w3, w4
	rcall	f48_mul

	mov	#fAdcOP1_Temp_KTC, w2
	mov	#F48_tmp_1, w4
	rcall	f48_mul

	mov	#FC48_1, w2
	mov	#F48_tmp_1, w3
	rcall	f48_add

	mov	#fAdcOP1_Mul_0_p, w2
	mov	#fAdcOP1_Mul_p, w4
	rcall	f48_mul

	mov	#fAdcOP1_Mul_0_n, w2
	mov	#fAdcOP1_Mul_n, w4
	rcall	f48_mul

	mov	#fAdcOP1_Korr_TC, w2
	mov	#F48_tmp_0, w3
	mov	#fAdcOP1_Offs, w4
	rcall	f48_mul

	mov	#fAdcOP1_Offs_0, w2
	mov	w4, w3
	rcall	f48_add

	return

tk_clr:
	mov	#fAdcOP1_Mul_0_p, w2
	mov	#fAdcOP1_Mul_p, w4
	rcall	f48_mov
	mov	#fAdcOP1_Mul_0_n, w2
	mov	#fAdcOP1_Mul_n, w4
	rcall	f48_mov
	mov	#fAdcOP1_Offs_0, w2
	mov	#fAdcOP1_Offs, w4
	rcall	f48_mov
	clr	LastTemp
	return

ADS112c04_SetParam:
	mov	EEData0+oFR_ADC_OP1_KORR_P, w0		; 0.001%
	mov	#fAdcOP1_Mul_0_p, w3
	rcall	i16_f48_w3
	mov	#FC48_000001, w2
	mov	w3, w4
	rcall	f48_mul

	mov.b	EEData0+oFR_ADC_OP1_KORR_N, WREG	; 0.001%
	se	w0, w0
	mov	#fAdcOP1_Mul_0_n, w3
	rcall	i16_f48_w3
	mov	#FC48_000001, w2
	mov	w3, w4
	rcall	f48_mul
	mov	#fAdcOP1_Mul_0_p, w2
	rcall	f48_add

	mov	#FC48_1, w2
	mov	#fAdcOP1_Mul_0_p, w3
	mov	#fAdcOP1_Mul_0_p, w4
	rcall	f48_add
	mov	#FC48_ADS112c04_OP1_DIGIT_MV, w2
	rcall	f48_mul
	
	mov	#FC48_1, w2
	mov	#fAdcOP1_Mul_0_n, w3
	mov	#fAdcOP1_Mul_0_n, w4
	rcall	f48_add
	mov	#FC48_ADS112c04_OP1_DIGIT_MV, w2
	rcall	f48_mul

	mov	EEData0+oFR_ADC_OP1_OFFS, w0
	mov	#fAdcOP1_Offs_0, w3
	rcall	i16_f48_w3
	mov	#fAdcOP1_Mul_0_p, w2
	mov	#fAdcOP1_Offs_0, w4
	rcall	f48_mul

	mov.b	EEData0+oFR_ADC_OP1_K_TC, WREG		; 0.0001 %/C
	se	w0, w0
	mov	#F48_tmp_0, w3
	rcall	i16_f48_w3
	mov	#FC48_0000001, w2
	mov	#fAdcOP1_Temp_KTC, w4
	rcall	f48_mul


	mov.b	EEData0+oFR_ADC_OP1_TC, WREG		; 0.01mV/C
	se	w0, w0
	mov	#fAdcOP1_Korr_TC, w3
	rcall	i16_f48_w3
	mov	#FC48_001, w2
	mov	w3, w4
	rcall	f48_mul

	bra tk_clr

.section .const, psv
.align 2

.global FC48_ADS112c04_OP1_DIGIT_MV

/*
R1=68k+10k=78k   R2=20k  -> 78/20+1 = 4.9
ref=2048mV / 8G = +-256mV => 256*2/65536 = 0.0078125mV / digit
0.0078125 * 4.9 = 0.03828125mV / digit
0.03828125 / 0.05 = 0.765625
*/
FC48_ADS112c04_OP1_DIGIT_MV:
	.word	0xcccd, 0x9ccc, 0x3ffa		; 0.03828125
;	.word	0x21a7, 0x9cec, 0x3ffa		; 0.03831113

;FC48_ADS112c04_OP1_MUL:
;	.word	0x0000, 0xc400, 0x3ffe		; 0.765625

/*
0.0001 %/C / 100 * 0.025C = 0.025E-6
*/

;FC48_ADS112c04_OP1_KTC:
;	.word	0x94d6, 0xd6bf, 0x3fe5		; 0.025E-6
