/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "libald.inc"

.section .text

.global AddSub1616_MinMax
.global AddSub1616_Sat

/* -------------------- AddSub1616_MinMax ------------------------
Input:  w0:w1=Value	w2:w3= Value Add/Sub
	w6=min.		w7=max
	w4=flag
		b0 0=only cp  1=add/sub
		b1 0=add  1=sub
		b15=sign
		
Output: w0=Value			ERROR (min|max): SR C set
	w4=flag
		b8 0=UV  1=OV

used; w0:w1:w4				ca. 42 Clk
------------------------------------------------------------------- */

AddSub1616_MinMax:
	push.d	w8
	and	w1, w4, w8	/* set sign */
	asr	w8, #15, w8	/* copy sign */

	btst	w4, #bADDSUB_MM_ON
	bra	z, as_1616_only_cp

	and	w3, w4, w9	/* set sign */
	asr	w9, #15, w9	/* copy sign */
	btst	w4, #bADDSUB_MM_SUB
	bra	nz, as_1616_sub
	add	w0, w2, w0
	addc	w1, w3, w1
	addc	w8, w9, w8
	bra	as_1616_only_cp
as_1616_sub:
	sub	w0, w2, w0
	subb	w1, w3, w1
	subb	w8, w9, w8

as_1616_only_cp:
	and	w6, w4, w9	/* set sign */
	asr	w9, #15, w9	/* copy sign */
	cp	w0, #0
	cpb	w1, w6
	cpb	w8, w9
	bra	lt, as_1616_UV

	and	w7, w4, w9	/* set sign */
	asr	w9, #15, w9	/* copy sign */
	cp	w0, #0
	cpb	w1, w7
	cpb	w8, w9
	bra	gt, as_1616_OV

	bclr	SR, #C
	pop.d	w8
	return

as_1616_UV:
	clr	w0
	mov	w6, w1
	bset	SR, #C
	pop.d	w8
	return
as_1616_OV:
	clr	w0
	mov	w7, w1
	bset	w4, #bADDSUB_MM_OV
	bset	SR, #C
	pop.d	w8
	return


AddSub1616_Sat:
	mov	#0xFFFF, w6
	btsc	w4, #15
	mov	#0x8000, w6

	mov	#0x0000, w7
	btsc	w4, #15
	mov	#0x7FFF, w7
	bra	AddSub1616_MinMax

