/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "libald.inc"

.section .text

.global AddSub16_MinMax
.global AddSub16_Sat

/* -------------------- AddSub16_MinMax ------------------------
Input:  w0=Value	w2= Value Add/Sub
	w6=min.		w7=max
	w4=flag
		b0 0=only cp  1=add/sub
		b1 0=add  1=sub
		b15=sign
		
Output: w0=Value			ERROR (min|max): SR C set
	w4=flag
		b8 0=UV  1=OV

used; w0:w1:w2:w3:w4			ca. 33 Clk
------------------------------------------------------------------- */

AddSub16_MinMax:
	and	w0, w4, w1	/* set sign */
	asr	w1, #15, w1	/* copy sign */

	btst	w4, #bADDSUB_MM_ON
	bra	z, as_16_only_cp

.if 1
	and	w2, w4, w3	/* set sign */
	asr	w3, #15, w3	/* copy sign */

	btsc	w4, #bADDSUB_MM_SUB
	subr	w2, #0, w2
	btsc	w4, #bADDSUB_MM_SUB
	subbr	w3, #0, w3
	add	w0, w2, w0
	addc	w1, w3, w1
.else
	and	w2, w4, w3	/* set sign */
	asr	w3, #15, w3	/* copy sign */

	btst	w4, #1
	bra	nz, as_16_sub
	add	w0, w2, w0
	addc	w1, w3, w1
	bra	as_16_only_cp
as_16_sub:
	sub	w0, w2, w0
	subb	w1, w3, w1
.endif
as_16_only_cp:
	and	w6, w4, w3	/* set sign */
	asr	w3, #15, w3	/* copy sign */
	cp	w0, w6
	cpb	w1, w3
	bra	lt, as_16_UV

	and	w7, w4, w3	/* set sign */
	asr	w3, #15, w3	/* copy sign */
	cp	w0, w7
	cpb	w1, w3
	bra	gt, as_16_OV
	bclr	SR, #C
	return

as_16_UV:
	bset	SR, #C
	mov	w6, w0
	return
as_16_OV:
	bset	w4, #bADDSUB_MM_OV
	bset	SR, #C
	mov	w7, w0
	return

AddSub16_Sat:
	mov	#0x0000, w6
	btsc	w4, #15
	mov	#0x8000, w6

	mov	#0xFFFF, w7
	btsc	w4, #15
	mov	#0x7FFF, w7
	bra	AddSub16_MinMax
