/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"

.if USE_PLL_CDCE913

.include "CDCE913.inc"

	OSC_FREQ		= 8000000
;	CDCE913_OUT_FREQ_MAX	= 230000000

;	CDCE913_VCO_FREQ_MIN	= 80000000	/* 80MHz */
	CDCE913_VCO_PWR_1	= 125000000	/* 125MHz */
	CDCE913_VCO_PWR_2	= 150000000	/* 150MHz */
	CDCE913_VCO_PWR_3	= 175000000	/* 175MHz */

.equ bTABP_N_AUTO,		0
.equ bTABP_INC_DEC,		1

.section .nbss, bss, near
pll_stat:		.space 2
;.equ bPLLST_POW_ON,		0
.equ PLLST_PLL_OFF,		1
.equ PLLST_PLL_RESTART,		2
/* b8..b9 = VCO1_0_RANGE */

CDCE913_setup:		.space 2
.equ bSUP_RD_00_7,		0
.equ bSUP_WR_01_1,		1
.equ bSUP_WR_DIV_1,		2
.equ bSUP_WR_14_1,		3
.equ bSUP_WR_16_2,		4


.equ bSUP_WR_NRQP_0,		8
;.equ bSUP_WR_NRQP_1,	9

CDCE913_data:		.space 32

CDCE913_fout:		.space 4

CDCE913_NRQP_0:		.space 4
;CDCE913_NRQP_1:		.space 4

CDCE913_DIV1:		.space 2	/* b15 = 1 use xtal */
CDCE913_N:		.space 2
CDCE913_R:		.space 2
CDCE913_Q:		.space 2
CDCE913_P:		.space 2

CDCE913_CalcStat:	.space 2
.equ bCS_OK,		0
.equ bCS_ERR0,		1
.equ bCS_DIV_MOD2,	2
.equ bCS_CALC_RUN,	4

CDCE913_rt:		.space 4
CDCE913_fpll:		.space 4


CDCE913_CalcStep:	.space 2
.equ CS_CALC_START,	0
.equ CS_CALC_D,		1
.equ CS_CALC_Q,		2
.equ CS_CALC_Q2,	3
.equ CS_CALC_D2,	4
.equ CS_CALC_D2Q,	5


last_DIV1:		.space 2

;TempMaxDec:		.space 2

fNQP:			.space 4

cp_DIV1:		.space 2
cp_N:			.space 2
cp_R:			.space 2
cp_Q:			.space 2
cp_P:			.space 2
cp_diff:		.space 4
cp_diff_sign:		.space 4

QP2_4x:			.space 2
Nmin_Rmul:		.space 2
P_max_diff:		.space 2
R_Max:			.space 2
kN_Max:			.space 2
DIV2:			.space 2
RunStat:		.space 2
.equ bRS_OV,		0
.equ bRS_KN_ADD_R,	1

.global pll_stat
.global CDCE913_setup
.global CDCE913_data
.global CDCE913_fout

.global CDCE913_DIV1
.global CDCE913_NRQP_0
;.global CDCE913_NRQP_1
.global CDCE913_N
.global CDCE913_R
.global CDCE913_Q
.global CDCE913_P
.global CDCE913_f_vco

.if TEST_PLL_NPQR
TestPll_DIV1:		.space 2
TestPll_Stat:		.space 2
TestPll_N:		.space 2
TestPll_R:		.space 2
TestPll_Q:		.space 2
TestPll_P:		.space 2
TestPll_Send:		.space 2
.global TestPll_DIV1
.global TestPll_Stat
.global TestPll_N
.global TestPll_P
.global TestPll_Q
.global TestPll_R
.global TestPll_Send
.endif


.section .text
.global CDCE913_INIT
.global CDCE913_Handle
.global CDCE913_i2c_run

.global pll_on
.global pll_off
.global calc_CDCE913_8MHz_fout
.global calc_CDCE913_Handle

CDCE913_INIT:
	mov	#0x001F, w0
	mov	w0, CDCE913_setup

	mov	#1000, w0
	mov	w0, CDCE913_DIV1

	mov	#0x4000, w0
	mov	w0, CDCE913_NRQP_0
;	mov	w0, CDCE913_NRQP_1

	mov	#0x0802, w0
	mov	w0, CDCE913_NRQP_0+2
;	mov	w0, CDCE913_NRQP_1+2

	return

CDCE913_Handle:
	cp0	i2c2_Lock
	bra	nz, lend
	btsc	I2CErrorFlag, #I2C_CDCE913
	return
	cp0	CDCE913_setup
	bra	nz, chk_setup
lend:
	return

CDCE913_i2c_run:
	mov	i2c2_cmd, w0
	cp0	w0
	bra	nz, chk_err

	cp0	CDCE913_setup
	bra	nz, run_setup
	clr	i2c2_Lock
	return
chk_err:
	cp	w0, #I2C_ERROR
	bra	z, is_err
	return		/* Wait */

is_err:
	bset	I2CErrorFlag, #I2C_CDCE913
	clr	i2c2_Lock
	return

chk_setup:
	btsc	CDCE913_setup, #bSUP_RD_00_7
	bra	nz, rd_00_7_data
	btsc	CDCE913_setup, #bSUP_WR_01_1
	bra	nz, wr_01_1_data
	btsc	CDCE913_setup, #bSUP_WR_DIV_1
	bra	nz, wr_div_1
	btsc	CDCE913_setup, #bSUP_WR_14_1
	bra	nz, wr_14_1_data
	btsc	CDCE913_setup, #bSUP_WR_16_2
	bra	nz, wr_16_2_data
	btsc	CDCE913_setup, #bSUP_WR_NRQP_0
	bra	nz, wr_NRQP_0
;	btsc	CDCE913_setup, #bSUP_WR_NRQP_1
;	bra	nz, wr_NRQP_1
	clr	CDCE913_setup
	return

rd_00_7_data:
	mov	#I2C_CDCE913, w0
	mov	w0, i2c2_Lock

	mov	#0xca, w0
	mov	w0, i2c2_id
	mov	#0x18, w0
	mov	w0, i2c2_adr
	clr	i2c2_Flag

	mov	#I2C_READ, w0
	mov	w0, i2c2_cmd
	mov	#8, w0
	mov	w0, i2c2_DataSize
	return

wr_01_1_data:
	mov	#0x19, w0
	btsc	pll_stat, #bPLLST_POW_ON
	bclr	w0, #4
	mov	w0, i2c2_Buf
wr_byte_01_1:
	mov	#0x81, w0
wr_byte_xx_1:
	mov	w0, i2c2_adr
	mov	#1, w0
	mov	w0, i2c2_DataSize
wr_start:
	mov	#I2C_CDCE913, w0
	mov	w0, i2c2_Lock

	mov	#0xca, w0
	mov	w0, i2c2_id
	clr	i2c2_Flag
	mov	#I2C_WRITE, w0
	mov	w0, i2c2_cmd
	return

wr_div_1:
	mov	CDCE913_DIV1, w0
	mov	#0x83FF, w1
	and	w0, w1, w0
	mov	#0x3c00, w1
	ior	w0, w1, w0
	btg	w0, #15
	swap	w0
	mov	w0, CDCE913_data+2


	mov	#0x02, w0
	mov	#CDCE913_data+2, w2
;wr_2Byte_x:
	mov	w0, i2c2_adr
	mov	#i2c2_Buf, w3
	mov	#2, w0
	mov.b	w0, [w3++]		/* Byte Count */
	mov.b	[w2++], [w3++]		/* Data */
	mov.b	[w2++], [w3++]
	mov	#3, w0
	mov	w0, i2c2_DataSize
	bra	wr_start


wr_14_1_data:
	mov	#0xE5, w0
	btss	pll_stat, #PLLST_PLL_OFF
	bclr	w0, #7
	mov	w0, i2c2_Buf
	mov	#0x94, w0
	bra	wr_byte_xx_1

wr_16_2_data:
	mov	#0x16, w0
	mov	w0, i2c2_adr
	mov	#i2c2_Buf, w3
	mov	#2, w0
	mov.b	w0, [w3++]		/* Byte Count */
	clr.b	[w3++]			/* Data */
	clr.b	[w3++]
	mov	#3, w0
	mov	w0, i2c2_DataSize
	bra	wr_start

wr_NRQP_0:
	mov	CDCE913_N, w0
	mov	#0x0FFF, w1
	and	w0, w1, w0
	btsc	SR, #Z
	mov	#1, w0
	sl	w0, #4, w0

	mov	CDCE913_R, w1
	mov	#0x01FF, w3
	and	w1, w3, w1
	lsr	w1, #5, w2
	ior	w0, w2, w0
	sl	w1, #11, w1

	mov	CDCE913_Q, w2
	mov	#0x003F, w3
	and	w2, w3, w2
	mov	#0x0030, w3
	and	w2, w3, w3
	btsc	SR, #Z
	mov	#0x0010, w2
	sl	w2, #5, w2
	ior	w1, w2, w1

	mov	CDCE913_P, w2
	and	w2, #0x07, w2
	sl	w2, #2, w2
	ior	w1, w2, w1

 mov	pll_stat, w2
 lsr	w2, #8, w2
 and	w2, #0x03, w2
 ior	w1, w2, w1


	swap	w0
	mov	w0, CDCE913_NRQP_0
	swap	w1
	mov	w1, CDCE913_NRQP_0+2

	mov	#0x18, w0
	mov	#CDCE913_NRQP_0, w2
wr_NRQP_x:
	mov	w0, i2c2_adr
	mov	#i2c2_Buf, w3
	mov	#4, w0
	mov.b	w0, [w3++]		/* Byte Count */
	mov.b	[w2++], [w3++]		/* Data */
	mov.b	[w2++], [w3++]
	mov.b	[w2++], [w3++]
	mov.b	[w2++], [w3++]
	mov	#5, w0
	mov	w0, i2c2_DataSize
	bra	wr_start

;wr_NRQP_1:
;	mov	#0x1C, w0
;	mov	#CDCE913_NRQP_1, w2
;	bra	wr_NRQP_x


run_setup:
	btsc	CDCE913_setup, #bSUP_RD_00_7
	bra	nz, run_00_7_data
	btsc	CDCE913_setup, #bSUP_WR_01_1
	bra	nz, run_wr_01_1_data
	btsc	CDCE913_setup, #bSUP_WR_DIV_1
	bra	nz, run_wr_div_1
	btsc	CDCE913_setup, #bSUP_WR_14_1
	bra	nz, run_wr_14_1_data
	btsc	CDCE913_setup, #bSUP_WR_16_2
	bra	nz, run_wr_16_2_data
	btsc	CDCE913_setup, #bSUP_WR_NRQP_0
	bra	nz, run_NRQP_0
;	btsc	CDCE913_setup, #bSUP_WR_NRQP_1
;	bra	nz, run_NRQP_1
	clr	CDCE913_setup
	clr	i2c2_Lock
	return

run_00_7_data:
	bclr	CDCE913_setup, #bSUP_RD_00_7
	mov	#CDCE913_data, w0
	mov	#i2c2_Buf+1, w1
	mov	#7, w2
	repeat	w2
	mov.b	[w1++], [w0++]
	clr	i2c2_Lock
	return

run_wr_01_1_data:
;	bclr	pll_stat, #bPLLST_POW_ON
	bclr	CDCE913_setup, #bSUP_WR_01_1
	clr	i2c2_Lock
	return

run_wr_div_1:
	bclr	CDCE913_setup, #bSUP_WR_DIV_1
	clr	i2c2_Lock
	return

run_wr_14_1_data:
	bclr	CDCE913_setup, #bSUP_WR_14_1
	clr	i2c2_Lock
	return

run_wr_16_2_data:
	bclr	CDCE913_setup, #bSUP_WR_16_2
	clr	i2c2_Lock
	return

run_NRQP_0:
	btst	pll_stat, #PLLST_PLL_RESTART
	bra	z, NRQP_0_nor
	bclr	pll_stat, #PLLST_PLL_RESTART
	bclr	pll_stat, #PLLST_PLL_OFF 
	bset	CDCE913_setup, #bSUP_WR_14_1
NRQP_0_nor:
	bclr	CDCE913_setup, #bSUP_WR_NRQP_0
	clr	i2c2_Lock
	return

;run_NRQP_1:
;	bclr	CDCE913_setup, #bSUP_WR_NRQP_1
;	clr	i2c2_Lock
;	return


pll_on:
	btsc	pll_stat, #bPLLST_POW_ON
	return
	bset	pll_stat, #bPLLST_POW_ON
	bset	CDCE913_setup, #bSUP_WR_01_1
;	rcall	OC3_PinEnabel
	bra	OC1_PinDisabel

pll_off:
	btss	pll_stat, #bPLLST_POW_ON
	return
	bclr	pll_stat, #bPLLST_POW_ON
	bset	CDCE913_setup, #bSUP_WR_01_1
;	rcall	OC3_PinDisabel
	bra	OC1_PinEnabel

/* --------------------- calc NRQP -------------------------------------- */
Add_PllFreq_NR:
	mov	R_Max, w0
	cp	CDCE913_R
	bra	z, cPF_ov

	inc	CDCE913_R
	mov	CDCE913_R, w4

/* Np = (fNQP * R) + R */
	mov	fNQP, w0
	mov	fNQP+2, w1	/* 16.16 */
	mul.uu	w4, w1, w2	/* w3:w2    */
	mul.uu	w4, w0, w0	/*    w1:w0 */
	sl	w0, w0
	addc	w1, w2, w1
	add	w1, w4, w1	/* Np */

	mov	CDCE913_P, w3
	lsr	w1, w3, w0	/* N = Np >> P */

	mov	kN_Max, w1
	lsr	w4, w3, w2	/* R >> P */
;	cp	w3, #3		/* test P < 3 */
;	btsc	SR, #C
	btsc	RunStat, #bRS_KN_ADD_R
	add	w2, w1, w1	/* N max.= 1024(2048) + (R >> 4(3)) */
	cp	w0, w1		/* test N > Nmax */
	bra	gtu, cPF_A_max
cPF_nx:
	mov	Nmin_Rmul, w2	/* Nmin_Rmul = 17 * 16 >> P  */
	mulw.uu	w4, w2, w2
	lsr	w2, #4, w2	/* N_min = (Nmin_Rmul * R) / 16  */
	cp	w0, w2		/* test N < Nmin */
	bra	ltu, cPF_A_ov
; cp	w0, w4
; bra	leu, cPF_skip
;	cpsgt	w0, w2
;	mov	w2, w0
	mov	w0, CDCE913_N

/* xmul = R * 128 / ((N << P) - R) */
	sl	w0, w3, w2	/* (N << P) */
	sub	w2, w4, w2	/* (N << P) - R */

	mul.uu	w6, #0, w6	/* clr w7:w6 */
	sl	w4, #7, w1	/* R << 7 */
	repeat	#17		/* div 32/16->48 */
	div.u	w1, w2
	exch	w0, w7
	repeat	#17
	div.ud	w0, w2		
	exch	w0, w6
	repeat	#17
	div.ud	w0, w2		/* xmul w7:w6:w0 */

/* fx = xmul * OSC_FREQ */
	mov	#OSC_FREQ >> 7, w4  /* 8MHz = 0x007A1200 >> 7 =  0xF424 */
/* mul 32*16=48 */
	mul.uu	w6, w4, w2	/*    w3:w2     */
	mul.uu	w0, w4, w0	/*       w1:w0  */
	mul.uu	w7, w4, w6	/* w7:w6       */
	add	w2, w1, w5
	addc	w6, w3, w0
	addc	w7, #0, w1	/* fx = w1:w0:w5  */
/* (OSC_FREQ + fx) */
	mov	#OSC_FREQ & 0xffff, w4	/* OSC_FREQ + fx */
	add	w0, w4, w0
;	mov	#OSC_FREQ >> 16, w4
;	addc	w1, w4, w1
	addc	#OSC_FREQ >> 16, w1
/* fpll_16 = QP2_4x * (OSC_FREQ + fx) */
	mov	QP2_4x, w4	/* Q << (4-P) */
	mul.uu	w4, w1, w2	/*  w3:w2       */
	mul.uu	w4, w0, w0	/*     w1:w0    */
	mul.uu	w4, w5, w4	/*        w5:w4 */
	add	w0, w5, w0
	addc	w1, w2, w1	/* +   w1:w0    */
/* fpll = fpll_16 >> 4 */
	mov	#4096, w4
	mul.uu	w1, w4, w2
	mul.uu	w0, w4, w0
	sl	w0, w0		/* Round pll +-0.5Hz */
	addc	w2, w1, w0
	addc	w3, #0, w1

/* getPllDiffAbs     result diff_abs w0:w1  diff w2:w3  */
;	mov	CDCE913_fpll, w2
;	sub	w0, w2, w0
	mov	CDCE913_fpll+2, w3
	subr	CDCE913_fpll, WREG
	subb	w1, w3, w1
	mov.d	w0, w2		/* pll diff +-0.5Hz */
	bra	nn, gPDA_pos
	subr	w0, #0, w0
	subbr	w1, #0, w1
gPDA_pos:
 /* test diff 0 */
	cp	w0, #2
	cpb	w1, #0
	bra	n, CO_opt_00	/* diff 0 (pll +-1.5Hz) */
/* test last_diff > diff */
	cp	cp_diff
	mov	cp_diff+2, w4
	cpb	w4, w1
	bra	gtu, CO_opt
	ior	w0, w1, w4
	return

CO_opt_00:
	mul.uu	w0, #0, w0	/* clr w1:w0 */
CO_opt:
	mov	#CDCE913_DIV1, w4
	mov	#cp_DIV1, w5
	repeat	#4
	mov	[w4++], [w5++]
	mov	w0, cp_diff
	mov	w1, cp_diff+2
	mov	w2, cp_diff_sign
	mov	w3, cp_diff_sign+2
	ior	w0, w1, w4
	return

cPF_A_max:
	cp	w4, #1
	bra	nz, cPF_A_ov
	mov	w1, w0		/* Nmax -> N */
	bra	cPF_nx
cPF_ov:
	bclr	SR, #Z
cPF_A_ov:
	bset	RunStat, #bRS_OV
	return
;cPF_skip:
;	bclr	SR, #Z
;	return

calc_Q_Px_rt:
	mov	CDCE913_DIV1, w0
	mov	CDCE913_fout, w1
	mov	CDCE913_fout+2, w2
	mulw.uu	w0, w2, w2
	mul.uu	w0, w1, w0
	add	w1, w2, w1
	mov	w0, CDCE913_fpll
	mov	w1, CDCE913_fpll+2

	mov	CDCE913_P, w2

	cp	w2, #1
	bra	z, calc_Q_P1_rt
	cp	w2, #2
	bra	z, calc_Q_P2_rt
	cp	w2, #3
	bra	z, calc_Q_P3_rt
	cp	w2, #4
	bra	z, calc_Q_P4_rt
;calc_Q_P0_rt:
	mov	#240, w3	/* R_Max */
	mov	#4095, w4	/* kN_Max */
	bra	calc_Q

calc_Q_P1_rt:
	mov	#480, w3
	mov	#4095, w4
	bra	calc_Q

calc_Q_P2_rt:
	mov	#511, w3
	mov	#4095, w4
	bra	calc_Q

calc_Q_P3_rt:
	mov	#511, w3
	mov	#2048, w4
	bset	RunStat, #bRS_KN_ADD_R
	bra	calc_Q

calc_Q_P4_rt:
	mov	#511, w3
	mov	#1024, w4
	bset	RunStat, #bRS_KN_ADD_R
;	bra	calc_Q

calc_Q: /* w0,w1=fpll w2=P */
	mov	w3, R_Max
	mov	w4, kN_Max

	mov	#272, w3		/* Nmin_Rmul = 17 * 16 >> P */
	lsr	w3, w2, w3
	mov	w3, Nmin_Rmul

;	mov	w2, CDCE913_P
	add	w2, #9, w3
	subr	w2, #7, w4
; Q = d->fpll / d->fosc;
	sl	w1, w3, w2
	lsr	w1, w4, w1
	lsr	w0, w4, w0
	ior	w0, w2, w0
	mov	#OSC_FREQ >> 7, w2
	repeat	#17
	div.ud	w0, w2
	mov	w0, CDCE913_Q
;	bra	calc_rt
calc_rt:
	clr	CDCE913_R
;	clr	RunStat
	bclr	RunStat, #bRS_OV

	mov	CDCE913_P, w2
	add	w2, #9, w3
	subr	w2, #7, w4

; rt = d->fpll - (d->fosc * Q);
	mov	CDCE913_Q, w0
	subr	w2, #4, w2
	sl	w0, w2, w2
	mov	w2, QP2_4x

	mov	#OSC_FREQ >> 7, w2
	mul.uu	w0, w2, w0
	lsr	w0, w3, w2
	sl	w1, w4, w1
	sl	w0, w4, w0
	ior	w1, w2, w1
	mov	CDCE913_fpll, w2
	sub	w2, w0, w0
	mov	CDCE913_fpll+2, w2
	subb	w2, w1, w1
; if (!rt) return true;
	mov	w0, CDCE913_rt
	mov	w1, CDCE913_rt+2
	ior	w0, w1, w2	/* w1 = 0 --> w2=w0 */
	btsc	SR, #Z
	return

/* -------------- calc fNQP ------------------------------------- */
;	    kN = OSC_FREQ / diff / fP2;
;	    fN = kN / fP2 * fR * fQ + (fR / fP2);

;	mov	w0, w2		/* ior	w0, w1, w2 -> w1 = 0 --> w2=w0*/
	mov	CDCE913_P, w5
	subr	w5, #7, w5
	ff1l	w1, w3
	cp0	w3
	bra	z, rt_w3_0
	dec	w3, w3
	subr	w3, #16, w4
	sub	w5, w4, w5
	sl	w1, w3, w2
	lsr	w1, w4, w1
	lsr	w0, w4, w0
	ior	w0, w2, w2
rt_w3_0:
	mov	#OSC_FREQ >> 7, w0	/* 8MHz = 0x007A1200 >> 7 =  0xF424 */
	clr	w1

	clr	w3
	repeat	#17		/* div 32/16->32 */
	div.u	w0, w2
	exch	w0, w3
	repeat	#17
	div.ud	w0, w2		/* result w0,w3 */
/* Round */
	lsr	w2, w2
	sub	w1, w2, w1
	addc	w0, #0, w0
	addc	w3, #0, w3	/* result w0,w3 */


	cp0	w5
	bra	z, rt_w5_0
	bra	n, rt_w5_neg
	subr	w5, #16, w4

	lsr	w0, w4, w1
	sl	w0, w5, w0
	sl	w3, w5, w3
	ior	w3, w1, w3
	bra	rt_w5_0
rt_w5_neg:
	subr	w5, #0, w5
	subr	w5, #16, w4
	sl	w3, w4, w1
	lsr	w3, w5, w3
	lsr	w0, w5, w0
	ior	w0, w1, w0
rt_w5_0:
;	    fN = kN / fP2 * fQ * fR + (fR / fP2);
;	    fNQP = kN / fP2 * fQ;
	mov	CDCE913_Q, w2
	mul.uu	w0, w2, w0	/* mul 32*16=48 */
	mul.uu	w2, w3, w2
	add	w1, w2, w1
	mov	w0, fNQP
	mov	w1, fNQP+2	/* fNQP = kN / fP2 * fQ; */
	bclr	SR, #Z
	return



/* CDCE913_fpll -> N,R,Q,P */
calc_CDCE913_8MHz_fout:
; mov	t_free_run, w0
; mov	w0, TestVal0



	setm	cp_diff
	mov	#0x00ff, w0
	mov	w0, cp_diff+2


	mov	#4, w2
	mov	w2, CDCE913_N
	mov	w2, CDCE913_P
	clr	CDCE913_R
	mov	#16, w2
	mov	w2, CDCE913_Q

	mov	CDCE913_DIV1, w0
	mov	w0, last_DIV1

	mov	#1<<bCS_CALC_RUN, w0
	mov	w0, CDCE913_CalcStat
	clr	DIV2
	clr	CDCE913_CalcStep
	clr	RunStat
	clr	w3	/* DIV2 */

	cp0	CDCE913_fout+2
	bra	nz, c913_f32B

	mov	CDCE913_fout, w0
	mov	#8500, w1
	cp	w0, w1
	bra	gtu, c913_8500

	mov	#1000, w2
	mov	#1001, w3
	mov	#4, w4
	bra	c913_set_DP
c913_8500:
	mov	CDCE913_fout, w0
	mov	#10000, w1
	cp	w0, w1
	bra	gtu, c913_10000

	mov	#1000, w2
	mov	#1018, w3
	mov	#4, w4
	bra	c913_set_DP

c913_10000:
	mov	CDCE913_fout, w0
	mov	#16000, w1
	cp	w0, w1
	bra	gtu, c913_16000

	mov	#1000, w2
	mov	#1008, w3
	mov	#4, w4
	bra	c913_set_DP

c913_16000:
	mov	CDCE913_fout, w0
	mov	#32000, w1
	cp	w0, w1
	bra	gtu, c913_32000

	mov	#1000, w2
	mov	#1008, w3
	mov	#3, w4
	bra	c913_set_DP

c913_32000:
	mov	CDCE913_fout, w0
	mov	#64000, w1
	cp	w0, w1
	bra	gtu, c913_64000

	mov	#1000, w2
	mov	#1004, w3
	mov	#2, w4
	bra	c913_set_DP

c913_f32B:
	mov	CDCE913_fout, w0
	mov	CDCE913_fout+2, w1
	mov	#100000 & 0xffff, w2
	cp	w0, w2
	cpb	w1, #100000 >> 16
	bra	nn, c913_100000
c913_64000:
	mov	#640, w2
	mov	#800, w3
	mov	#2, w4
	bra	c913_set_DP

c913_100000:
	mov	#125000 & 0xffff, w2
	cp	w0, w2
	cpb	w1, #125000 >> 16
	bra	nn, c913_125000

	mov	#1000, w2
	mov	#2, w4
	bra	c913_set_DP

c913_125000:
	mov	#250000 & 0xffff, w2
	cp	w0, w2
	cpb	w1, #250000 >> 16
	bra	nn, c913_250000

	mov	#500, w2
;	mov	#500, w3
	mov	#2, w4
	bra	c913_set_DP

c913_250000:
	mov	#500000 & 0xffff, w2
	cp	w0, w2
	cpb	w1, #500000 >> 16
	bra	nn, c913_500000

	mov	#128, w2
	mov	#250, w3
	mov	#2, w4
	bra	c913_set_DP

c913_500000:
	mov	#1000000 & 0xffff, w2
	cp	w0, w2
	cpb	w1, #1000000 >> 16
	bra	nn, c913_1000000

	mov	#64, w2
	mov	#65, w3
	mov	#2, w4
	bra	c913_set_DP

c913_1000000:
	mov	#2000000 & 0xffff, w2
	cp	w0, w2
	cpb	w1, #2000000 >> 16
	bra	nn, c913_2000000

	mov	#32, w2
	mov	#2, w4
	bra	c913_set_DP

c913_2000000:
	mov	#4000000 & 0xffff, w2
	cp	w0, w2
	cpb	w1, #4000000 >> 16
	bra	nn, c913_4000000

	mov	#16, w2
	mov	#2, w4
	bra	c913_set_DP

c913_4000000:
	mov	#8000000 & 0xffff, w2
	cp	w0, w2
	cpb	w1, #8000000 >> 16
	bra	nn, c913_8000000

	mov	#8, w2
;	mov	#15, w3
	mov	#16, w3
	mov	#2, w4
	bra	c913_set_DP

c913_8000000:
	mov	#10000000 & 0xffff, w2
	cp	w0, w2
	cpb	w1, #10000000 >> 16
	bra	nn, c913_10000000

	mov	#4, w2
	mov	#8, w3
	mov	#2, w4
	bra	c913_set_DP

c913_10000000:
	mov	#16000000 & 0xffff, w2
	cp	w0, w2
	cpb	w1, #16000000 >> 16
	bra	nn, c913_16000000

	mov	#4, w2
;	mov	#8, w3
	mov	#2, w4
	bra	c913_set_DP

c913_16000000:
	mov	#32000000 & 0xffff, w2
	cp	w0, w2
	mov	#32000000 >> 16, w2
	cpb	w1, w2
	bra	nn, c913_32000000

	mov	#2, w2
	mov	#2, w4
	bra	c913_set_DP

c913_32000000:
	mov	#64000000 & 0xffff, w2
	cp	w0, w2
	mov	#64000000 >> 16, w2
	cpb	w1, w2
	bra	nn, c913_64000000

	mov	#2, w2
	mov	#2, w4
	bra	c913_set_DP

c913_64000000:
	mov	#115000000 & 0xffff, w2
	cp	w0, w2
	mov	#115000000 >> 16, w2
	cpb	w1, w2
	bra	nn, c913_115000000

	mov	#2, w2
;	mov	#1, w3
	mov	#1, w4
	bra	c913_set_DP
.if 1
c913_115000000:
	mov	#128000000 & 0xffff, w2
	cp	w0, w2
	mov	#128000000 >> 16, w2
	cpb	w1, w2
	bra	nn, c913_128000000
	mov	#1, w2
	mov	#2, w4
	bra	c913_set_DP

c913_128000000:
	mov	#230000001 & 0xffff, w2
	cp	w0, w2
	mov	#230000001 >> 16, w2
	cpb	w1, w2
	bra	nn, c913_err
	mov	#1, w2
	mov	#1, w4
.else
c913_115000000:
	mov	#230000000 & 0xffff, w2
	cp	w0, w2
	mov	#230000000 >> 16, w2
	cpb	w1, w2
	bra	nn, c913_err
	mov	#1, w2
	mov	#1, w4
.endif
c913_set_DP:
	mov	w2, CDCE913_DIV1
	mov	w3, DIV2
	mov	w4, CDCE913_P
	return

c913_err:
	clr	CDCE913_CalcStat
	return

calc_CDCE913_Handle:
	btss	CDCE913_CalcStat, #bCS_CALC_RUN
	return

	mov	CDCE913_CalcStep, w0

	cp	w0, #CS_CALC_D
	bra	z, c913_run_loop
	cp	w0, #CS_CALC_Q
	bra	z, c913_run_Q_loop
	cp	w0, #CS_CALC_Q2
	bra	z, c913_run_Q2_loop
	cp	w0, #CS_CALC_D2
	bra	z, c913_run_D2_loop
	cp	w0, #CS_CALC_D2Q
	bra	z, c913_run_D2Q_loop
	cp	w0, #CS_CALC_START
	bra	z, c913_start
	clr	CDCE913_CalcStat
	return

c913_start:
	rcall	calc_Q_Px_rt
	bra	z, c913_Err0

	mov	#CS_CALC_D, w0
	mov	w0, CDCE913_CalcStep
c913_run_loop:
	btss	t_proc_mk, #0
	return
	rcall	Add_PllFreq_NR
	bra	z, c913_Err0
	btst	RunStat, #bRS_OV
	bra	z, c913_run_loop
;c913_chk_Q:
	mov	CDCE913_Q, w0	/* test Q > 16 */
	cp	w0, #16
	bra	leu, c913_chk_D2

	dec	CDCE913_Q
	rcall	calc_rt
	bra	z, c913_Err0

	mov	#CS_CALC_Q, w0
	mov	w0, CDCE913_CalcStep
c913_run_Q_loop:
	btss	t_proc_mk, #0
	return
	rcall	Add_PllFreq_NR
	bra	z, c913_Err0
	btst	RunStat, #bRS_OV
	bra	z, c913_run_Q_loop

;c913_chk_Q2:
	cp0	DIV2		/* test DIV2 */
	bra	nz, c913_chk_D2
	cp	w0, #32
	bra	leu, c913_chk_D2

	dec	CDCE913_Q
	rcall	calc_rt
	bra	z, c913_Err0

	mov	#CS_CALC_Q2, w0
	mov	w0, CDCE913_CalcStep
c913_run_Q2_loop:
	btss	t_proc_mk, #0
	return
	rcall	Add_PllFreq_NR
	bra	z, c913_Err0
	btst	RunStat, #bRS_OV
	bra	z, c913_run_Q2_loop
	inc	CDCE913_CalcStep

c913_chk_D2:
	cp0	DIV2		/* test DIV2 */
	bra	z, c913_Opti
	mov	DIV2, w0
	mov	w0, CDCE913_DIV1

	rcall	calc_Q_Px_rt
	bra	z, c913_Err0

	mov	#CS_CALC_D2, w0
	mov	w0, CDCE913_CalcStep
c913_run_D2_loop:
	btss	t_proc_mk, #0
	return
	rcall	Add_PllFreq_NR
	bra	z, c913_Err0
	btst	RunStat, #bRS_OV
	bra	z, c913_run_D2_loop

;c913_chk_D2Q:
	mov	CDCE913_Q, w0	/* test Q > 16 */
	cp	w0, #16
	bra	leu, c913_Opti

	dec	CDCE913_Q
	rcall	calc_rt
	bra	z, c913_Err0

	mov	#CS_CALC_D2Q, w0
	mov	w0, CDCE913_CalcStep
c913_run_D2Q_loop:
	btss	t_proc_mk, #0
	return
	rcall	Add_PllFreq_NR
	bra	z, c913_Err0
	btst	RunStat, #bRS_OV
	bra	z, c913_run_D2Q_loop

c913_Opti:
	mov	#cp_DIV1, w5
	mov	#CDCE913_DIV1, w4
	repeat	#4
	mov	[w5++], [w4++]
	mov	#(1<<bCS_OK), w0
	bra	c913_Ok

c913_Err0:
	mov	#(1<<bCS_ERR0)|(1<<bCS_OK), w0
c913_Ok:
	mov	w0, CDCE913_CalcStat

	mov	CDCE913_DIV1, w0
	ff1r	w0, w1
	sub	w1, #2, w1
	bra	n, no_mod2
	lsr	w0, w1, w2
	cp	w2, #2
	btsc	SR, #Z
	bset	CDCE913_CalcStat, #bCS_DIV_MOD2
no_mod2:
	cp	last_DIV1
	btss	SR, #Z
	bset	CDCE913_setup, #bSUP_WR_DIV_1
	cp	w0, #1
	bra	nz, no_Pll_restart
	cp0	CDCE913_R
	bra	nz, no_Pll_restart
	bset	pll_stat, #PLLST_PLL_RESTART
	bset	pll_stat, #PLLST_PLL_OFF 
	bset	CDCE913_setup, #bSUP_WR_14_1

no_Pll_restart:
	bset	CDCE913_setup, #bSUP_WR_NRQP_0

/* set VCO1_0_RANGE */
	clr	w2
	mov	CDCE913_fpll+2, w1
	mov	#CDCE913_VCO_PWR_1 >> 16, w0
	cp	w0, w1
	bra	gtu, l_set_vco_pwr
	mov	#0x0100, w2
	mov	#CDCE913_VCO_PWR_2 >> 16, w0
	cp	w0, w1
	bra	gtu, l_set_vco_pwr
	mov	#0x0200, w2
	mov	#CDCE913_VCO_PWR_3 >> 16, w0
	cp	w0, w1
	bra	gtu, l_set_vco_pwr
	mov	#0x0300, w2
l_set_vco_pwr:
	mov	pll_stat, w0
	mov	#0xfcff, w1
	and	w0, w1, w0
	ior	w0, w2, w0
	mov	w0, pll_stat

.if TEST_PLL_NPQR
	mov	CDCE913_DIV1, w0
	mov	w0, TestPll_DIV1
	mov	pll_stat, w0
	mov	w0, TestPll_Stat
	mov	CDCE913_N, w0
	mov	w0, TestPll_N
	mov	CDCE913_R, w0
	mov	w0, TestPll_R
	mov	CDCE913_Q, w0
	mov	w0, TestPll_Q
	mov	CDCE913_P, w0
	mov	w0, TestPll_P
.endif

;	rcall	SetSym
	return

.endif /* USE_PLL_CDCE913 */
