/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"

.section .nbss, bss, near
;DAC7578_A0_ms_count:		.space 2
DAC7578_OutFlag:		.space 2
dac_channel_16:			.space 2
DAC7578_Data:			.space 32


;.global DAC7578_A0_ms_count
.global DAC7578_OutFlag
.global DAC7578_Data

.section .text
.global DAC7578_INIT
.global DAC7578_Handle
.global DAC7578_i2c_run
.global DAC7578_SetValue
.global DAC7578_GetValue

DAC7578_INIT:
	mov	#DAC7578_Data, w3
	mov	#0x8000, w0
	repeat	#15
	mov	w0, [w3++]
	mov	#0xffff, w0
	mov	w0, DAC7578_OutFlag
	return

DAC7578_Handle:
	cp0	i2c2_Lock
	bra	nz, lend

	btst	I2CErrorFlag, #I2C_DAC7578
	bra	nz, lerr


	mov	DAC7578_OutFlag, w0
	ff1r	w0, w1
	cp0	w1
	bra	z, lend
	dec	w1, w1			/* channel */
	mov	w1, dac_channel_16
	mov	#1, w0
	sl	w0, w1, w0
	com	w0, w0
	and	DAC7578_OutFlag		/* clr bit */

	sl	w1, w2
	mov	#DAC7578_Data, w3
	add	w3, w2, w3
	mov	#i2c2_Buf, w2


	and	w1, #0x07, w0
	add	#0x30, w0
	mov.b	w0, [w2++]		/* Command */

	mov	[w3], w0
	swap	w0
	mov.b	w0, [w2++]		/* Data */
	swap	w0
	mov.b	w0, [w2++]

	mov	#3, w0
	mov	w0, i2c2_DataSize

	mov	#0x90, w0
	btsc	w1, #3
	mov	#0x94, w0
	mov	w0, i2c2_id

	mov	#I2C_WRITE, w0
	mov	w0, i2c2_cmd

	clr	i2c2_Flag
	bset	i2c2_Flag, #I2C_bNoADR

	mov	#I2C_DAC7578, w0
	mov	w0, i2c2_Lock

lend:
	return

lerr:
	clr	DAC7578_OutFlag
	return

DAC7578_i2c_run:
	mov	i2c2_cmd, w0
	cp0	w0
	bra	nz, chk_err
	clr	i2c2_Lock
	return

chk_err:
	cp	w0, #I2C_ERROR
	bra	z, is_err
	return		/* Wait */

is_err:
	bset	I2CErrorFlag, #I2C_DAC7578
	clr	i2c2_Lock
	return

/* w0=Value w2=DAC */
DAC7578_SetValue:
.if 1
	mov	#DAC7578_Data, w3
	mov	#0x0f, w1
	rcall	DAC_KorrSetValue
	btss	SR, #Z
	ior	DAC7578_OutFlag
	return
.else
	mov	#DAC7578_Data, w3

	and	#0x0f, w2
	add	w3, w2, w3
	add	w3, w2, w3
	mov	0xfff0, w1
	and	w0, w1, w0
	cp	w0, [w3]
	btsc	SR, #Z
	return

	mov	w0, [w3]

	mov	#1, w0
	sl	w0, w2, w0
	ior	DAC7578_OutFlag
	return
.endif

/* w0=Value w2=DAC */
DAC7578_GetValue:
.if 1
	mov	#DAC7578_Data, w3
	mov	#0x0f, w1
	bra	DAC_KorrGetValue
.else
	mov	#DAC7578_Data, w3

	and	#0x0f, w2
	add	w3, w2, w3
	add	w3, w2, w3
	nop
	mov	[w3], w0
	return
.endif
