/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc";
.include "Analog.inc"
.include "Param.inc"
.if USE_DEBUG_UART_USER
.include "uart_user.inc"
.endif

.include "../math/libm.inc"

/* ---------------------------------------------------------------------- */
.section .text

.global DacLinKorr
.global DacLinKorr_GetCrc
.global DacLinKorr_ChkCrc
.global DacLinKorr_SetCrc

.global DacSetValue_LinKorr

/* -------------------- DacLinKorr --------------------------
   Only s16 !!!
Input:  w0=Value  w2=DacIdx   w7=KorrTab 16 Signed Byte
Output: w0=Value  w2=DacIdx
used; w0:w1:w3:w4:w7

 Test Spannung Korrektur
	max. 0x7000		Digit 4x			min. 0x9000
        0     7   6   5   4   3   2   1  0   f   e   d   c   b   a   9
 .byte	crc,  9,  5,  5, 23, 23, 19, 14, 0, -7, -6, -5,-10, 10, 12, 10
        0    +1                ......                              +15
------------------------------------------------------------------- */

DacLinKorr:
	cp0	w7
	btsc	SR, #Z
	return
	ze	[w7++], w3
	cp0	w3
	btsc	SR, #Z
	return
	push	w0
	push	w2

	btg	w0, #15
	lsr	w0, #12, w1
	cp0	w1
	bra	z, dlk_min
	cp	w1, #0x0f
	bra	z, dlk_max
	subr	w1, #14, w1
	add	w7, w1, w7
	mov	#0x0FFF, w3
dlk_calc:
	se	[w7++], w2
	se	[w7], w1
	sub	w2, w1, w2
	and	w0, w3, w3
dlk_calc_w:
	mul.su	w2, w3, w2
	asr	w3, #11, w4
	sl	w3, #5, w3
	lsr	w2, #11, w2
	ior	w2, w3, w2
	asr	w4, w4
	rrc	w2, w2
	addc	w2, #0, w2

	add	w1, w2, w1
	asr	w1, w1
;	addc	w1, #0, w1
	bra	dlk_add

dlk_max:
.if 1
	mov	#0x1FFF, w3
	bra	dlk_calc
.else
	se	[w7], w1
	asr	w1, w1
	bra	dlk_add
.endif

dlk_min:
.if 1
	add	w7, #13, w7
	mov	#0x0FFF, w3
	com	w0, w1
	and	w1, w3, w3

	se	[w7++], w2
	se	[w7], w1
	sub	w1, w2, w2
	bra	dlk_calc_w
.else
	mov.b	[w7+14], w1
	se	w1, w1
	asr	w1, w1
.endif
dlk_add:
	pop	w2
	pop	w0
; btss w2, #0
; mov w0, TestVal0
; btss w2, #0
; mov w1, TestVal1
; btsc w2, #0
; mov w0, TestVal2
; btsc w2, #0
; mov w1, TestVal3
	neg	w1, w1
	bra	add16s_sat

/* -------------------- DacLinKorr_GetCrc -------------------------
Input:  w7=KorrTab 16 Signed Byte
Output: w0=CRC7  --> w0=0x00 no data   b0=Flag		SR=Z no data
used; w0:w1
------------------------------------------------------------------- */
DacLinKorr_GetCrc:
	cp0	w7
	btsc	SR, #Z
	return

	clr	w0
	add	w7, #1, w1
	repeat	#14
	ior.b	w0, [w1++], w0
	cp0	w0
	btsc	SR, #Z
	return

	clr	w0
	add	w7, #1, w1
	repeat	#14
	add.b	w0, [w1++], w0
	and	#0xFE, w0
	btsc	SR, #Z
	bset	w0, #4
	cp0	w0
	return

DacLinKorr_ChkCrc:
	cp0	w7
	btsc	SR, #Z
	return
	rcall	DacLinKorr_GetCrc
	bra	z, dlkc_0
	ze	[w7], w1
	and	#0xFE, w1
	cp	w0, w1
	bra	nz, dlkc_0
	return

DacLinKorr_SetCrc:
	cp0	w7
	btsc	SR, #Z
	return
	rcall	DacLinKorr_GetCrc
	bra	z, dlkc_0
	mov.b	w0, [w7]
	return

dlkc_0:
	clr.b	[w7]
	return

/* w0=Value w2=DAC idx */
DacSetValue_LinKorr:
.if USE_DEBUG_UART_USER
	btst	uuCAL_DAC_Mod, #bDAC_LIN_KORR_OFF
	bra	nz, DacSetValue
.endif
	mov	#PA_POW, w1
	cp	w1, w2
	bra	z, AD5667_0_lk
	mov	#PA_DC, w1
	cp	w1, w2
	bra	z, AD5667_1_lk
	bra	DacSetValue

AD5667_0_lk:
;	push	w2
	mov	#EEData0+oFR_AD5667_0_LK, w7
	rcall	DacLinKorr
; mov w0, TestVal4
;	pop	w2
	bra	DacSetValue

AD5667_1_lk:
;	push	w2
	mov	#EEData0+oFR_AD5667_1_LK, w7
	rcall	DacLinKorr
; mov w0, TestVal5
;	pop	w2
	bra	DacSetValue
