/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "Analog.inc"

.section .nbss, bss, near

conv_DAC7578_V:			.space 32

.global conv_DAC7578_V

/* ---------------------------------------------------------------------- */
.section .text

.global DacSetValue
.global DacGetValue

.global DAC_KorrSetValue
.global DAC_KorrGetValue

.global DacVxOnChange
;.global DacSetParamValue
.global DacSetValue_R_Align
.global DacGetValue_R_Align

/* w0=Value w2=DAC idx */
DacSetValue:
	ze	w2, w1

	cp	w1, #LAST_DAC7578
	bra	leu, conv_DAC7578_SetValue

	cp	w1, #LAST_MCP47EFB22
	bra	leu, MCP47FEB_SetValue

	cp	w1, #LAST_MCP4726
	bra	leu, MCP4726_SetValue

	cp	w1, #LAST_AD5667
	bra	leu, AD5667_SetValue

	cp	w1, #LAST_OC_PWM
	bra	leu, PWM_SetValue
	return

/* w2=DAC idx  result w0 */
DacGetValue:
	ze	w2, w1
	cp	w1, #LAST_DAC7578
	bra	leu, conv_DAC7578_GetValue

	cp	w1, #LAST_MCP47EFB22
	bra	leu, MCP47FEB_GetValue

	cp	w1, #LAST_MCP4726
	bra	leu, MCP4726_GetValue

	cp	w1, #LAST_AD5667
	bra	leu, AD5667_GetValue

	cp	w1, #LAST_OC_PWM
	bra	leu, PWM_GetValue

	clr	w0
	return

/* -------------------- DAC_KorrSetValue --------------------------
Input:  w0=Value w1=mask(max.Dac) w2=DacIdx w3=DacDataPointer
Output: w0=mask(OutFlag) w3=DacDataPointer	SR Z Data OK
used; w0:w1:w2:w3
------------------------------------------------------------------- */

DAC_KorrSetValue:
	btsc	w2, #bDAC_INV
	com	w0, w0

	cp0	w2
	bra	z, ksv_e

	push	w4
	mov	#DAC_XB_MASK, w4
	and	w2, w4, w4
	lsr	w4, #8, w4		/* bit shift */
	btss	w2, #bDAC_R_ALIGN
	lsr	w0, w4, w0		/* clr bits */
	sl	w0, w4, w0
	btsc	w2, #bDAC_SIGN
	btg	w0, #15
	btsc	w2, #bDAC_R_ALIGN
	lsr	w0, w4, w0

	ze	w2, w4
	and	w4, w1, w4	; w4 dac nr
	add	w3, w4, w3
	add	w3, w4, w3	; Set DAC data pointer

	mov	#1, w1
	sl	w1, w4, w1	; dac send or mask

	cp	w0, [w3]
	btss	SR, #Z
	mov	w0, [w3]
	mov	w1, w0
	pop	w4
ksv_e:
	return

/* -------------------- DAC_KorrGetValue --------------------------
Input:  w1=mask(max.Dac) w2=DacIdx w3=DacDataPointer
Output: w0=Value w3=DacDataPointer
used; w0:w1:w2:w3
------------------------------------------------------------------- */

DAC_KorrGetValue:
	ze	w2, w0
	and	w0, w1, w0
	sl	w0, w0
	add	w3, w0, w3	; Set DAC data pointer

	mov	#DAC_XB_MASK, w1
	and	w2, w1, w1
	lsr	w1, #8, w1		/* bit shift */
	mov	[w3], w0
	btsc	w2, #bDAC_R_ALIGN
	sl	w0, w1, w0
	btsc	w2, #bDAC_SIGN
	btg	w0, #15

	btsc	w2, #bDAC_INV
	com	w0, w0

	btst	w2, #bDAC_R_ALIGN
	bra	z, kgv_nra
	btsc	w2, #bDAC_SIGN
	asr	w0, w1, w0
	btss	w2, #bDAC_SIGN
	lsr	w0, w1, w0
kgv_nra:
	return

/* -------------------- DacVxOnChange ------------------------------
OnCange(w0:w1=Value, w3=stat, w6=Owner (Dac.id))
------------------------------------------------------------------- */
DacVxOnChange:
	cp0	w3	; Test Value change
	btss	SR, #Z
	return
	mov	w6, w2

;DacSetParamValue:
DacSetValue_R_Align:
	btst	w2, #bDAC_R_ALIGN
	bra	nz, DacSetValue
	mov	#DAC_XB_MASK, w3
	and	w2, w3, w3
	bra	z, DacSetValue		/* 0 == 16 Bit */
	lsr	w3, #DAC_XB_SIFT, w3	/* bit shift */
	sl	w0, w3, w0
	bra	DacSetValue

DacGetValue_R_Align:
	btst	w2, #bDAC_R_ALIGN
	bra	nz, DacGetValue
	mov	#DAC_XB_MASK, w3
	and	w2, w3, w3
	bra	z, DacGetValue		/* 0 == 16 Bit */
	push	w2
	rcall	DacGetValue
	pop	w2
	mov	#DAC_XB_MASK, w3
	and	w2, w3, w3
	lsr	w3, #DAC_XB_SIFT, w3	/* bit shift */

	btss	w2, #bDAC_SIGN
	lsr	w0, w3, w0
	btsc	w2, #bDAC_SIGN
	asr	w0, w3, w0
	return


conv_DAC7578_SetValue:
	push	w5
	mov	#tab_DAC7578_conv, w5
	and	w2, #0x0F, w3
	nop				; stall
	mov.b	[w5+w3], w1
	ze	w1, w1
	bra	z, nokorr

	mov	#conv_DAC7578_V, w5
	sl	w3, w3
	nop				; stall
	mov	w0, [w5+w3]

	rcall	GetCapValue
	ze	w2, w2
	mov	#DAC_12B, w1
	ior	w2, w1, w2
nokorr:
	pop	w5
	bra	DAC7578_SetValue

conv_DAC7578_GetValue:
	mov	#tab_DAC7578_conv, w0
	and	w2, #0x0F, w3
	nop				; stall
	mov.b	[w0+w3], w1
	ze	w1, w1
	bra	z, DAC7578_GetValue
	sl	w3, w3
	mov	#conv_DAC7578_V, w1
	nop				; stall
	mov	[w1+w3], w0
	return

/* ---------------------------------------------------------------------- */
.section .const, psv

tab_DAC7578_conv:
	.byte	CAP_BB174,	CAP_BB174,	CAP_BB174,	CAP_BB857
	.byte	CAP_BB174,	CAP_BB857,	CAP_BB857,	CAP_BB857
	.byte	0,		0,		0,		0
	.byte	CAP_BB181,	0,		CAP_BB181,	0
