/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
;.include "Analog.inc"
;.include "../main.inc"

;.equ Led_Z,		8
;.equ Led_INV,		7
;.equ Led_AC,		9

.section .nbss, bss, near
OutData:		.space 2
InData:			.space 2
MCP23S18_ms_count:	.space 2
ms_rep:			.space 2
ms_lock:		.space 2
io_stat:		.space 2
.equ IOST_ERR,				0
.equ IOST_REP,				1
.equ IOST_SENDKEY,			2
TaCount:		.space 2
TaDataAll:		.space 2
TaData:			.space 8
TastNr:			.space 2
TastStat:		.space 2

LedData:		.space 2

.global MCP23S18_ms_count
.global TaCount
.global TastNr
.global TastStat
.global LedData


.section .text
.global MCP23S18_INIT
.global MCP23S18_Handle
.global SetTastRepeat
.global SendKey

.global DPK02_Led_On
.global DPK02_Led_Off
.global DPK02_Led_Z_NZ


.equ bTaCol0,	0
.equ bTaCol1,	1
.equ bTaCol2,	2
.equ bTaCol3,	6

.equ bTaRow0,	3
.equ bTaRow1,	4
.equ bTaRow2,	5
;.equ bTaRow3,	13

/*
PA0=LED3	PA1=LED1	PA2=LED4	PA3=LED5
PA4=exp3	PA5=exp4	PA6=exp5	PA7=exp6
PB0=Col0	PB1=Col1	PB2=Col2	PB3=Row0
PB4=Row1	PB5=Row2	PB6=Col3	PB7=LED2
*/

/*         TastNr
14		10	6	2
13		 9	5	1
12		 8	4	0
*/

MCP23S18_INIT:
	setm	LedData

	SPI_LCD_CE_MCP23S18_OFF
	mov	#0x4000, w0	;IODIR
	rcall	SendWord_SPI_LCD_Wait
	mov	#0xF047, w0	;IODIR data 0=Output 1=Input
	rcall	SendWord_SPI_LCD_Wait
	SPI_LCD_CE_MCP23S18_ON

	mov	#0xFFC7, w0
	mov	w0, OutData
	nop
	nop


	SPI_LCD_CE_MCP23S18_OFF
	mov	#0x400c, w0	;GPPU - GPIO PULL-UP RESISTOR REGISTER
	rcall	SendWord_SPI_LCD_Wait
	mov	#0xF047, w0	;GPPU data 1=Pull-up enabled.
	rcall	SendWord_SPI_LCD_Wait
	SPI_LCD_CE_MCP23S18_ON

	mov	#5, w0
	mov	w0, MCP23S18_ms_count


	SPI_LCD_CE_MCP23S18_OFF
	mov	#0x4012, w0	;GPIO - GENERAL PURPOSE I/O PORT REGISTER
	rcall	SendWord_SPI_LCD_Wait
	mov	OutData, w0	;GPPU data
	rcall	SendWord_SPI_LCD_Wait
	SPI_LCD_CE_MCP23S18_ON

	return

MCP23S18_Handle:
	SPI_LCD_CE_BTSS
	return

	SPI_LCD_CE_MCP23S18_OFF
	mov	#0x4112, w0	;GPIO - GENERAL PURPOSE I/O PORT REGISTER
	SENDWORD_SPI_LCD

	mov	#5, w0		/* 5ms */
	mov	w0, MCP23S18_ms_count

sh_w0:
	SPI_LCD_TestTxD_End
	bra	z, sh_w0

	mov	#0xffff, w0	;NOP
	SENDWORD_SPI_LCD
sh_w1:
	SPI_LCD_TestTxD_End
	bra	z, sh_w1

	SPI_LCD_CE_MCP23S18_ON
/* SPI 2 RD Data Ende */

	mov	SPI1BUF, w0
	mov	w0, InData
; mov	w0, TestVal2

	btst	io_stat, #IOST_SENDKEY
	bra	nz, send_data

;	mov	#0x0238, w2
	mov	#(1<<bTaCol0)|(1<<bTaCol1)|(1<<bTaCol2)|(1<<bTaCol3), w2
	and	w0, w2, w0

	cp0	TaCount
	bra	nz, TaRun

	cp0	ms_lock
	bra	nz, is_lock

	cp	w0, w2			/* TaCount 0 */
	bra	z, send_data
	mov	w0, TaDataAll
	bclr	io_stat, #IOST_ERR
	bra	set_next_step

is_lock:
	dec	ms_lock
	bra	send_data

TaRun:
	mov	TaCount, w1
	dec	w1, w1
	bra	z, Step1
	dec	w1, w1
	bra	z, Step2
	dec	w1, w1
	bra	z, Step3
	dec	w1, w1
	bra	z, Step4
;	dec	w1, w1
;	bra	z, Step5
	dec	w1, w1
	bra	z, Step6
	dec	w1, w1
	bra	z, Step7
	dec	w1, w1
	bra	z, Step8

	cp	w0, w2			/* test tast is down */
	bra	nz, teRepeat

	rcall	TastUp
	clr	TaCount
	clr	io_stat
	mov	#20, w0			/* n * 5ms */
	mov	w0, ms_lock
	bra	send_data

teRepeat:
	cp0	ms_rep
	bra	z, send_data
	dec	ms_rep
	bra	nz, send_data
	clr	TaCount			/* Start Repeat */
	bra	send_data


Step1:
	cp	TaDataAll
	bra	z, set_next_step
set_error:
	bset	io_stat, #IOST_ERR
	bra	set_next_step
Step2:
	cp	TaDataAll
	bra	nz,set_error
	bset	OutData, #bTaRow1
	bset	OutData, #bTaRow2
;	bset	OutData, #bTaRow3
	bra	set_next_step
Step3:
	mov	w0, TaData		/* TaCount 3 */
	bset	OutData, #bTaRow0
	bclr	OutData, #bTaRow1
	bra	set_next_step
Step4:
	mov	w0, TaData+2		/* TaCount 4 */
	bset	OutData, #bTaRow1
	bclr	OutData, #bTaRow2
	bra	set_next_step
;Step5:
;	mov	w0, TaData+4		/* TaCount 5 */
;	bset	OutData, #bTaRow2
;	bclr	OutData, #bTaRow3
;	bra	set_next_step
Step6:
	mov	w0, TaData+4		/* TaCount 6 */
	mov	#~((1<<bTaRow0)|(1<<bTaRow1)|(1<<bTaRow2)), w0
	and	OutData
	bra	set_next_step
Step7:
	cp	TaDataAll
	bra	nz,set_error
	clr	w3			/* Test only 1 Taste */
	mov	TaData, w0
	cpseq	w0, w2
	inc	w3, w3
	mov	TaData+2, w0
	cpseq	w0, w2
	inc	w3, w3
	mov	TaData+4, w0
	cpseq	w0, w2
	inc	w3, w3
;	mov	TaData+6, w0
;	cpseq	w0, w2
;	inc	w3, w3
	cp	w3, #1
	bra	z, set_next_step
	bra	set_error
Step8:
	cp	TaDataAll
	bra	nz, set_error
	btst	io_stat, #IOST_ERR
	bra	nz, set_next_step
	clr	w3
	cp	TaData
	bra	z, s8col
	inc	w3, w3
	cp	TaData+2
	bra	z, s8col
	inc	w3, w3
	cp	TaData+4
	bra	z, s8col
	inc	w3, w3
	cp	TaData+6
	bra	nz, set_error
s8col:
	btst	w0, #bTaCol0
	bra	z, s8stat
	add	#4, w3
	btst	w0, #bTaCol1
	bra	z, s8stat
	add	#4, w3
	btst	w0, #bTaCol2
	bra	z, s8stat
	add	#4, w3
	btst	w0, #bTaCol3
	bra	nz, set_error
s8stat:
	mov	w3, w0
	clr	w1
	btss	InData, #8
	inc	w1, w1
	btst	io_stat, #IOST_REP	/* test rep akt */
	bra	z, s8ok
	cp	TastNr
	bra	nz, set_error
	mov	TastStat, w2
	cp	w1, w2
	bra	nz, set_error
s8ok:
	mov	w0, TastNr
	mov	w1, TastStat
; mov	w0, TestVal0
; mov	w1, TestVal1
	rcall	TastDown

set_next_step:
	inc	TaCount
send_data:
	SPI_LCD_CE_MCP23S18_OFF
	mov	#0x4012, w0	;GPIO - GENERAL PURPOSE I/O PORT REGISTER

	SENDWORD_SPI_LCD
sh_w2:
	SPI_LCD_TestTxD_End
	bra	z, sh_w2

	mov	LedData, w1
	mov	#0xF07F, w2
	ior	w1, w2, w1
	mov	OutData, w0	;GPPU data
	and	w0, w1, w0

	SENDWORD_SPI_LCD
	rcall	Chk_SysError
sh_w3:
	SPI_LCD_TestTxD_End
	bra	z, sh_w3

	SPI_LCD_CE_MCP23S18_ON
	return

SetTastRepeat:
	btsc	io_stat, #IOST_SENDKEY
	return

	mov	#11, w0			/* (20-9) * 5ms = 100ms */
	btst	io_stat, #IOST_REP
	bra	nz, 10f
	mov	#60, w0			/* 60 * 5ms = 300ms */
	bset	io_stat, #IOST_REP
10:
	mov	w0, ms_rep
	return

SendKey:
	bset	io_stat, #IOST_SENDKEY
	mov	w0, TastNr
	rcall	TastDown
	rcall	TastUp

	clr	ms_rep
	clr	TaCount
	mov	#~((1<<bTaRow0)|(1<<bTaRow1)|(1<<bTaRow2)), w0
	and	OutData
	clr	io_stat
	return


/* ------------------------ Set Led ------------------------------------- */

.equ LED1,		8
.equ LED2,		7
.equ LED3,		9
.equ LED4,		10
.equ LED5,		11

DPK02_Led_On:
	cp0	w0
	bra	z, Led1_on
	dec	w0, w0
	bra	z, Led2_on
	dec	w0, w0
	bra	z, Led3_on
	dec	w0, w0
	bra	z, Led4_on
	dec	w0, w0
	bra	z, Led5_on
	return

Led1_on:
	bclr	LedData, #LED1
	return
Led2_on:
	bclr	LedData, #LED2
	return
Led3_on:
	bclr	LedData, #LED3
	return
Led4_on:
	bclr	LedData, #LED4
	return
Led5_on:
	bclr	LedData, #LED5
	return

DPK02_Led_Z_NZ:
	bra	nz, DPK02_Led_On
DPK02_Led_Off:
	cp0	w0
	bra	z, Led1_off
	dec	w0, w0
	bra	z, Led2_off
	dec	w0, w0
	bra	z, Led3_off
	dec	w0, w0
	bra	z, Led4_off
	dec	w0, w0
	bra	z, Led5_off
	return

Led1_off:
	bset	LedData, #LED1
	return
Led2_off:
	bset	LedData, #LED2
	return
Led3_off:
	bset	LedData, #LED3
	return
Led4_off:
	bset	LedData, #LED4
	return
Led5_off:
	bset	LedData, #LED5
	return


/* ------------------------ Test SysError ------------------------------- */
Chk_SysError:
.if 0
	cp0	SysError
	bra	nz, csye_end

;	cp0	I2CErrorFlag
;	bra	z, csye_end
	clr	w0
/*
	btsc	MCP3913_Stat, #MCP3913ST_DEFEKT
	mov	#SYSERR_MCP3913, w0

	btsc	I2CErrorFlag, #MCP47FEB_Defect
	mov	#SYSERR_MCP47FEB, w0

	btsc	I2CErrorFlag, #AD5667_0_Defect
	mov	#SYSERR_AD5667_0, w0

;	btsc	I2CErrorFlag, #Temp2SensDefect
;	mov	#SYSERR_TEMP2SENSOR, w0

	btsc	I2CErrorFlag, #Temp1SensDefect
	mov	#SYSERR_TEMP1SENSOR, w0

	btsc	I2CErrorFlag, #FRamDefect
	mov	#SYSERR_FRAM, w0
*/
	btsc	I2CErrorFlag, #I2C_Defect
	mov	#SYSERR_I2C, w0

	btsc	I2CErrorFlag, #MCP3421A0_Defect
	mov	#SYSERR_MCP3421A0, w0

	btsc	I2CErrorFlag, #ADS112c04_Defect
	mov	#SYSERR_ADS112c04, w0

	cp0	w0
	bra	z, csye_end
csye_err:
	mov	w0, SysError
csye_end:
.endif
	return
