/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "Param.inc"

.section .nbss, bss, near
pData:		.space 2
ExMemFlag:	.space 2
.if FR_MAX_DATA_SEG > 16
RDFlag:		.space 4
WRFlag:		.space 4
.else
RDFlag:		.space 2
WRFlag:		.space 2
.endif
FR_WR_En:	.space 2
EEData0:	.space FR_DATA_SEG_SIZE * FR_MAX_DATA_SEG

.if FR_MAX_DATA_SEG > 32
.ERROR  "FR_MAX_DATA_SEG > 32"
.endif

.global ExMemFlag
.global RDFlag
.global WRFlag
.global FR_WR_En
.global EEData0

.section .text
.global FM24CL048_INIT
.global FM24CL048_Handle
.global FM24CL048_i2c_run
.global FM24CL048_Format
.global FRam_SaveSeg
.global FRam_GetVal
.global FRam_SetVal
.global FRam_Save
.global FRam_SaveVal

.global FRam_GetAdr
.global FRam_GetValue
.global FRam_SetValue
.global FRam_SaveValue

FM24CL048_INIT:
.if FR_MAX_DATA_SEG > 16
	mov	#0xFFFF, w0
	mov	w0, WRFlag
	mov	w0, WRFlag+2
	clr	w0
	mov	w0, RDFlag
	mov	#(0xFFFF << (FR_MAX_DATA_SEG-16)) & 0xFFFF, w0
	mov	w0, RDFlag+2
.else
	mov	#0xFFFF, w0
	mov	w0, WRFlag
	mov	#(0xFFFF << FR_MAX_DATA_SEG) & 0xFFFF, w0
	mov	w0, RDFlag
.endif
	return

FM24CL048_Handle:
	btsc	I2CErrorFlag, #I2C_FM24CL048
	return
	cp0	i2c2_Lock
	btss	SR, #Z
	return
	clr	pData
	mov	#RDFlag, w3
	mov	#FR_MAX_DATA_SEG, w4
lloop_rd:
	inc	[w3++], w0
	bra	nz, te_rd		/* test all bit 1 */
	sub	w4, #16, w4
	bra	gtu, lloop_rd

	cp0	FR_WR_En
	bra	z, lend

	mov	#WRFlag, w3
	mov	#FR_MAX_DATA_SEG, w4
lloop_wr:
	inc	[w3++], w0
	bra	nz, te_wr		/* test all bit 1 */
	sub	w4, #16, w4
	bra	gtu, lloop_wr
	bset	ExMemFlag, #DataOk
	return

te_rd:
	rcall	fram_calc_io
	bra	c, lend

	mov	w0, pData
;	bclr	ExMemFlag, #DataWR
	mov	#I2C_READ, w1
	bra	set_i2c
 
te_wr:
	rcall	fram_calc_io
	bra	c, lend

	mov	#i2c2_Buf, w1
	mov	#(FR_DATA_SEG_SIZE / 2) - 1, w2
	repeat	w2
	mov	[w0++], [w1++]
	mov	#I2C_WRITE, w1
set_i2c:
	clr	i2c2_Flag

	mov	#0xA0, w0
	lsr	w3, #7, w2
	and	w2, #0x0e, w2
	ior	w0, w2, w0
	ze	w3, w3

	mov	w0, i2c2_id
	mov	w3, i2c2_adr
	mov	w1, i2c2_cmd
	mov	#FR_DATA_SEG_SIZE, w0
	mov	w0, i2c2_DataSize

	mov	#I2C_FM24CL048, w0
	mov	w0, i2c2_Lock
lend:
	return

fram_calc_io:
	mov	[--w3], w0
	com	w0, w0
	ff1r	w0, w2

	cp	w2, w4
	bra	gtu, rd_err

	mov	#FR_MAX_DATA_SEG, w0
	sub	w0, w4, w4

	dec	w2, w2
	mov	#0x0001, w0		/* mask */
	sl	w0, w2, w0
	ior	w0, [w3], [w3]

	add	w2, w4, w2
	sl	w2, #4, w3	/* x16 */
	mov	#EEData0, w0
	add	w0, w3, w0
	bclr	SR, #C
	return
rd_err:
	setm	[w3]
	bset	SR, #C
	return


FM24CL048_i2c_run:
	mov	i2c2_cmd, w0
	cp0	w0
	bra	nz, chk_err


;	btst	ExMemFlag, #DataWR
;	bra	nz, rend

	mov	pData, w0
	cp0	w0
	bra	z, rend
	mov	#i2c2_Buf, w1
	mov	#(FR_DATA_SEG_SIZE / 2) - 1, w2
	repeat	w2
	mov	[w1++], [w0++]

	btst	ExMemFlag, #ParamOk
	bra	nz, rend

	mov	#RDFlag, w3
	mov	#FR_MAX_DATA_SEG, w4
lrun_rd:
	inc	[w3++], w0
	bra	nz, rend		/* test all bit 1 */
	sub	w4, #16, w4
	bra	gtu, lrun_rd

	bset	ExMemFlag, #ParamOk
	rcall	ParamRead
rend:
	clr	i2c2_Lock
	return

chk_err:
	cp	w0, #I2C_ERROR
	bra	z, is_err
	return		/* Wait */
is_err:
	bset	I2CErrorFlag, #I2C_FM24CL048
	clr	i2c2_Lock
	return

FM24CL048_Format:
.if 1
	mov	#((FR_DATA_SEG_SIZE * FR_MAX_DATA_SEG) / 2) - 1, w1
	mov	#EEData0, w2
	repeat	w1
	clr	[w2++]

	mov	#FM24CL048SIG, w0	/* OK */
	mov	w0, EEData0

.if FR_MAX_DATA_SEG > 16
	clr	WRFlag
	mov	#(0xFFFF << (FR_MAX_DATA_SEG-16)) & 0xFFFF, w0
	mov	w0, WRFlag+2
.else
	mov	#(0xFFFF << (FR_MAX_DATA_SEG-16)) & 0xFFFF, w0
	mov	w0, WRFlag
.endif
	bclr	ExMemFlag, #DataOk
	bclr	ExMemFlag, #ParamOk
	return

.else
	mov	#FM24CL048SIG, w0	/* OK */
	mov	w0, EEData0

	mov	#0xfffe, w0
	mov	w0, WRFlag
	bclr	ExMemFlag, #DataOk
	bclr	ExMemFlag, #ParamOk
	return
.endif

/* w0=FramSeg */
FRam_SaveSeg:
	cp	w0, #FR_MAX_DATA_SEG
	bra	geu, fss_ret
	push	w8
	lsr	w0, #4, w1
	sl	w1, w1
	mov	#WRFlag, w8
	add	w8, w1, w8
	and	#0x0f, w0
	mov	#0x0001, w1
	sl	w1, w0, w0
	com	w0, w0
	and	w0, [w8], [w8]
	pop	w8
	bclr	ExMemFlag, #DataOk
	return
fss_ret:
	return

/* -------------------- FRam_GetAdr -------------------------------
Input:  w2=FramAdr
	w3=size
Output: w2=MemAdr			ERROR SR set OV
used: w0:w2
------------------------------------------------------------------- */
FRam_GetAdr:
	mov	#(FR_DATA_SEG_SIZE * FR_MAX_DATA_SEG), w0
	cp	w2, w0
	bra	geu, fraga_ov

	mov	#EEData0, w0
	add	w2, w0, w2
	bclr	SR, #OV
	return
fraga_ov:
	bset	SR, #OV
	return

/* -------------------- FRam_GetValue -----------------------------
Input:	w2=FramAdr
	w3=size 0..3 (1-4Byte)
Output: w0:w1=Value			ERROR SR set OV
used:	w0:w1:w2
------------------------------------------------------------------- */
FRam_GetValue:
	cp	w3, #3
	bra	gtu, fragv_e
	rcall	FRam_GetAdr
	bra	ov, fragv_e

	clr	w1
	ze	[w2++], w0
	cp0	w3
	bra	z, fragv_e
	swap	w0
	mov.b	[w2++], w0
	swap	w0
	cp	w3, #1
	bra	z, fragv_e
	
	ze	[w2++], w1
	cp	w3, #2
	bra	z, fragv_e
	swap	w1
	mov.b	[w2++], w1
	swap	w1
fragv_e:
	return

/* -------------------- FRam_SetValue -----------------------------
Input:	w0:w1=Value	w2=FramAdr
	w3=size 0..3 (1-4Byte)
Output: 				ERROR SR set OV
used:	w0:w1:w2
------------------------------------------------------------------- */
FRam_SetValue:
	cp	w3, #3
	bra	gtu, frasv_ov
	push	w0
	rcall	FRam_GetAdr
	pop	w0
	bra	ov, frasv_ov

	mov.b	w0, [w2++]
	cp0	w3
	bra	z, frasv_ok
	swap	w0
	mov.b	w0, [w2++]
	cp	w3, #1
	bra	z, frasv_ok

	mov.b	w1, [w2++]
	cp	w3, #2
	bra	z, frasv_ok
	swap	w1
	mov.b	w1, [w2++]
	
frasv_ok:
	bclr	SR, #OV
frasv_ov:
	return

/* -------------------- FRam_Save ---------------------------------
Input:	w2=FramAdr	w3=size 0..15 (1-16 Byte)
Output: 				ERROR SR set OV
used:	w0:w1:w2
------------------------------------------------------------------- */
FRam_Save:
	cp	w3, #15
	bra	gtu, frsave_ov

	lsr	w2, #4, w0
	call	FRam_SaveSeg
	add	w2, w3, w2
	lsr	w2, #4, w0
	call	FRam_SaveSeg
frsave_ov:
	return

/* -------------------- FRam_SaveValue ----------------------------
Input:	w0:w1=Value	w2=FramAdr
	w3=size 0..3 (1-4Byte)
Output: 				ERROR SR set OV
used:	w0:w1:w2
------------------------------------------------------------------- */
FRam_SaveValue:
	cp	w3, #3
	bra	gtu, frsavev_ov

	push	w2
	call	FRam_SetValue
	pop	w2
	bra	ov, frsavev_ov

	lsr	w2, #4, w0
	call	FRam_SaveSeg
	add	w2, w3, w2
	lsr	w2, #4, w0
	call	FRam_SaveSeg
frsavev_ov:
	return
