/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "LCD_128x64.inc"
.include "Param.inc"

.section .nbss, bss, near
LCD_Status:	.space 2
LCD_Page:	.space 2
LCD_Col:	.space 2
LCD_SendWord:	.space 2


.global Str_buffer
.global LCD_Status
.global LCD_SendWord

.section .ybss, bss, ymemory

Str_buffer:	.space STR_MAX_BUF
Video_buffer:	.space LCD_MAX_MEM

.global Video_buffer


.section .text
.global LCD_128x64_INIT
.global LCD_SetContrast
.global LCD_CLR
.global LCD_Refresh
.global LCD_Handle

;.global ClrRect
.global SetRect
.global PolLineH
.global PolLineV
.global PolDotLineH
.global InvRect

.global Char8B_Out
.global Char8B_OutK4
.global StrOut8x13
.global StrLen8x13
.global StrOut_cs

.global StrLen_dvf
.global StrOut_dvf

;.global BixFan
;.global BixVConst


LCD_128x64_INIT:
	SPI_LCD_A0_OFF
	SPI_LCD_CE_OFF

	mov	#0x40a1, w0	;Display start line 0 ;ADC reverse
	rcall	SendWord_SPI_LCD_Wait

	mov	#0xc0a6, w0	;Normal COM0~COM63 ;Display normal
	rcall	SendWord_SPI_LCD_Wait

	mov	#0xa22f, w0	;Set bias 1/9 (Duty 1/65) ;Booster, Regulator and Follower on
	rcall	SendWord_SPI_LCD_Wait

	mov	#0xf800, w0	;Set internal Booster to 4x
	rcall	SendWord_SPI_LCD_Wait

	mov	#0x2781, w0	;Contrast set
	rcall	SendWord_SPI_LCD_Wait

	mov	#0x10ac, w0	;Contrast set ;No indicator
	rcall	SendWord_SPI_LCD_Wait

	mov	#0x00af, w0	;No indicator ;Display on
	rcall	SendWord_SPI_LCD_Wait

;	SPI2_A0_ON
	SPI_LCD_CE_ON
	clr	LCD_Status
	return

LCD_SetContrast:
	mov.b	EEData0+oFR_CONTRAST, WREG
	add	w0, #0x10, w0
	and	w0, #0x1f, w0
	mov	#0x8100, w1
	ior	w0, w1, w0
	mov	w0, LCD_SendWord
	return

LCD_CLR:
	mov	#0x0000, w0
	mov	#Video_buffer, w1
	repeat	#(LCD_MAX_MEM / 2) - 1
	mov	w0, [w1++]
	return

LCD_Refresh:
	btsc	LCD_Status, #bLCD_RUN
	return
	bset	LCD_Status, #bLCD_RUN
	bset	LCD_Status, #bLCD_SETADR
	bset	LCD_Status, #bLCD_SETADR2
	clr	LCD_Page
	return

LCD_Handle:
	btss	LCD_Status, #bLCD_RUN
	return
/*
 on p33EPxxxxx not work ????
	SPI2_TestTxD_FULL
	bra	nz, lcdh_end
*/
	SPI_LCD_TestTxD_End
	bra	z, lcdh_end

	btst	LCD_Status, #bLCD_SETADR
	bra	z, lcdh_SendData

	mov	LCD_Page, w0
	cp	w0, #8
	bra	ltu, lcdh_TeExData
	bclr	LCD_Status, #bLCD_RUN
	SPI_LCD_CE_ON
	return

lcdh_TeExData:		/* LCD_SetContrast */
	mov	LCD_SendWord, w0
	cp0	w0
	bra	z, lcdh_SetAdrPage

	SPI_LCD_A0_OFF
	SPI_LCD_CE_OFF
	SENDWORD_SPI_LCD
	clr	LCD_SendWord
	return

lcdh_SetAdrPage:  ;SetAdr Page
	btst	LCD_Status, #bLCD_SETADR2
	bra	z, lcdh_SetAdrCol0

	SPI_LCD_A0_OFF
	SPI_LCD_CE_OFF
	mov	LCD_Page, w0
	mov	#0xe3b0, w1
	ior	w0, w1, w0
	SENDWORD_SPI_LCD
	bclr	LCD_Status, #bLCD_SETADR2
	return

lcdh_SetAdrCol0:  ;SetAdr Col 0
	mov	#0x0010, w0
	SENDWORD_SPI_LCD
	clr	LCD_Col
	bclr	LCD_Status, #bLCD_SETADR
	return

lcdh_SendData:  ;Send Data
	SPI_LCD_A0_ON

	mov	#Video_buffer, w1
	mov	LCD_Page, w0
	sl	w0, #7, w0
	add	w1, w0, w1

	mov	LCD_Col, w2
	lsr	w2, #3, w0
	sl	w0, #1, w0
	add	w1, w0, w1

	and	w2, #0x07, w2
	sl	w2, w2
	inc	w2, w3
	mov	#7, w5
	mov	#15, w4
;0
	btst.c	[w1], w2
	bsw.c	w0, w4
	btst.c	[w1], w3
	bsw.c	w0, w5
	add	w1, #16, w1
;1
	lsr	w0, w0
	btst.c	[w1], w2
	bsw.c	w0, w4
	btst.c	[w1], w3
	bsw.c	w0, w5
	add	w1, #16, w1
;2
	lsr	w0, w0
	btst.c	[w1], w2
	bsw.c	w0, w4
	btst.c	[w1], w3
	bsw.c	w0, w5
	add	w1, #16, w1
;3
	lsr	w0, w0
	btst.c	[w1], w2
	bsw.c	w0, w4
	btst.c	[w1], w3
	bsw.c	w0, w5
	add	w1, #16, w1
;4
	lsr	w0, w0
	btst.c	[w1], w2
	bsw.c	w0, w4
	btst.c	[w1], w3
	bsw.c	w0, w5
	add	w1, #16, w1
;5
	lsr	w0, w0
	btst.c	[w1], w2
	bsw.c	w0, w4
	btst.c	[w1], w3
	bsw.c	w0, w5
	add	w1, #16, w1
;6
	lsr	w0, w0
	btst.c	[w1], w2
	bsw.c	w0, w4
	btst.c	[w1], w3
	bsw.c	w0, w5
	add	w1, #16, w1
;7
	lsr	w0, w0
	btst.c	[w1], w2
	bsw.c	w0, w4
	btst.c	[w1], w3
	bsw.c	w0, w5

	SENDWORD_SPI_LCD

	inc	LCD_Col
	btst	LCD_Col, #6	/* Test gt 63 */
	bra	z, lcdh_end
	inc	LCD_Page
	bset	LCD_Status, #bLCD_SETADR
	bset	LCD_Status, #bLCD_SETADR2
lcdh_end:
	return

/* ----------------------------------------------------------------------
W0=x W2=w W1=Video_buffer
------------------------------------------------------------------------- */
Line_Set:
	mov	#0xffff, w4
	cp	w2, #15
	bra	gt, li_b1
	sl	w4, w2, w4
	com	w4, w4
li_b1:
	sl	w4, w0, w4
	ior	w4, [w1], [w1++]
	subr	w0, #16, w4
	sub	w2, w4, w2
	bra	le, li_end

	cp	w2, #16
	bra	le, li_el

	lsr	w2, #4, w4
	dec	w4, w4
	repeat	w4
	setm	[w1++]
	and	#0x0F, w2
	bra	z, li_end
li_el:
	subr	w2, #16, w2
	mov	#0xFFFF, w4
	lsr	w4, w2, w4
	ior	w4, [w1], [w1]
li_end:
	return

.if 0
/* ----------------------------------------------------------------------
W0=x W1=y W2=w W3=h
------------------------------------------------------------------------- */
SetRect:
	cp	w0, #LCD_MAX_X
	bra	c, setr_ret
	cp	w1, #LCD_MAX_Y
	bra	c, setr_ret

	cp0	w2
	bra	le, setr_ret

	cp0	w3
	bra	le, setr_ret

	push.d	w4

	mov	#LCD_MAX_X, w5
	add	w0, w2, w4
	cpslt	w4, w5
	sub	w5, w0, w2

	mov	#LCD_MAX_Y, w5
	add	w1, w3, w4
	cpslt	w4, w5
	sub	w5, w1, w3

	mulw.uu	w1, #LCD_LINE_BYTE, w4
	mov	#Video_buffer, w5
	add	w4, w5, w1
	lsr	w0, #4, w4
	add	w1, w4, w1
	add	w1, w4, w1
	and	w0, #0x0F, w0

	dec	w3, w3
	do	w3, setr_do
	push	w1
	push	w2
	rcall	Line_Set
	pop	w2
	pop	w1
setr_do: add	#LCD_LINE_BYTE, w1
	pop.d	w4
setr_ret:
	return
.endif

/* ----------------------------------------------------------------------
W0=x W1=y W2=w W3=h
------------------------------------------------------------------------- */
PolLineH:
	cp0	w2
	bra	z, plh_ret
	cp	w1, #LCD_MAX_Y
	bra	c, plh_ret

	push.d	w0
	push	w2

	btst	w2, #15
	bra	z, plh_p
	add	w0, w2, w0
	neg	w2, w2
	inc	w0, w0
plh_p:
	cp	w0, #LCD_MAX_X
	bra	c, plh_pw2

	push	w4
	mov	#LCD_MAX_X, w3
	add	w0, w2, w4
	cpslt	w4, w3
	sub	w3, w0, w2

	mulw.uu	w1, #LCD_LINE_BYTE, w4
	mov	#Video_buffer, w3
	add	w4, w3, w1
	lsr	w0, #4, w4
	add	w1, w4, w1
	add	w1, w4, w1
	and	w0, #0x0F, w0

	rcall	Line_Set
	pop	w4
plh_pw2:
	pop	w2
	pop.d	w0
	add	w0, w2, w0
plh_ret:
	return

/* ----------------------------------------------------------------------
W0=x W1=y W2=lg
------------------------------------------------------------------------- */
PolLineV:
	cp0	w2
	bra	z, plv_ret
	cp	w0, #LCD_MAX_X
	bra	c, plv_ret

	push.d	w0
	push	w2

	btst	w2, #15
	bra	z, plv_p
	add	w1, w2, w1
	neg	w2, w2
	inc	w1, w1
plv_p:
	cp	w1, #LCD_MAX_Y
	bra	c, plv_pw2

	push	w4
	mov	#LCD_MAX_Y, w3
	add	w1, w2, w4
	cpslt	w4, w3
	sub	w3, w1, w2

	mulw.uu	w1, #LCD_LINE_BYTE, w4
	mov	#Video_buffer, w3
	add	w4, w3, w1
	lsr	w0, #4, w4
	add	w1, w4, w1
	add	w1, w4, w1
	and	w0, #0x0F, w0

	mov	#0x01, w4	/* x=2 w=3 xxx111xx */
	sl	w4, w0, w4

	dec	w2, w2
	do	w2, plv_do
	ior	w4, [w1], [w1]
plv_do:	add	#LCD_LINE_BYTE, w1
	pop	w4
plv_pw2:
	pop	w2
	pop.d	w0
	add	w1, w2, w1
plv_ret:
	return

/* ---------------------- DotLine_Set ----------------------------------- */
DotLine_Set:
/*
W0=x W2=w W1=Video_buffer w5=dot
*/
	mov	#0xffff, w4
	cp	w2, #15
	bra	gt, dli_b1
	sl	w4, w2, w4
	com	w4, w4
dli_b1:
	sl	w4, w0, w4
	and	w4, w5, w4
	ior	w4, [w1], [w1++]
	subr	w0, #16, w4
	sub	w2, w4, w2
	bra	le, dli_end

	cp	w2, #16
	bra	le, dli_el

	lsr	w2, #4, w4
	dec	w4, w4
	repeat	w4
	mov	w5, [w1++]
	and	#0x0F, w2
	bra	z, dli_end
dli_el:
	subr	w2, #16, w2
	mov	#0xFFFF, w4
	lsr	w4, w2, w4
	and	w4, w5, w4
	ior	w4, [w1], [w1]
dli_end:
	return

/* ----------------------------------------------------------------------
W0=x W1=y W2=lg
------------------------------------------------------------------------- */
PolDotLineH:
	cp0	w2
	bra	z, pdlh_ret
	cp	w1, #LCD_MAX_Y
	bra	c, pdlh_ret

	push.d	w0
	push	w2

	btst	w2, #15
	bra	z, pdlh_p
	add	w0, w2, w0
	neg	w2, w2
	inc	w0, w0
pdlh_p:
	cp	w0, #LCD_MAX_X
	bra	c, pdlh_pw2

	push.d	w4
	mov	#LCD_MAX_X, w3
	add	w0, w2, w4
	cpslt	w4, w3
	sub	w3, w0, w2

	mov	#0x5555, w5
	btsc	w1, #0
	rlnc	w5, w5

	mulw.uu	w1, #LCD_LINE_BYTE, w4
	mov	#Video_buffer, w3
	add	w4, w3, w1
	lsr	w0, #4, w4
	add	w1, w4, w1
	add	w1, w4, w1
	and	w0, #0x0F, w0

	rcall	DotLine_Set
	pop.d	w4
pdlh_pw2:
	pop	w2
	pop.d	w0
	add	w0, w2, w0
pdlh_ret:
	return

.if 0
/* ----------------------------------------------------------------------
W0=x W2=w W1=Video_buffer
------------------------------------------------------------------------- */
Line_Inv:
	and	w0, #0x07, w6
	bra	z, linv_ba

	subr	w6, #8, w5

	cp	w5, w2
	bra	lt, linv_b1

	mov	#0xff, w4	/* x=2 w=3 xxx111xx */
	subr	w2, #8, w5
	lsr	w4, w5, w4
	sl	w4, w6, w4
	xor.b	w4, [w1], [w1]
	return
linv_b1:
	mov	#0xff00, w4
	lsr	w4, w5, w4
	xor.b	w4, [w1], [w1++]
	sub	w2, w5, w2
;	bra	z, linv_end
linv_ba:
	cp	w2, #8
	bra	le, linv_el

	lsr	w2, #3, w4
	dec	w4, w4
	mov	w1, w5		/* [w1], [w1++]  --> stall */
	repeat	w4
	com.b	[w5++], [w1++]
	and	#0x07, w2
	bra	z, linv_end
linv_el:
	subr	w2, #8, w2
	mov	#0xff, w4
	lsr	w4, w2, w4
	xor.b	w4, [w1], [w1]
linv_end:
	return

/*
W0=x W1=y W2=w W3=h
*/
InvRect:
	cp	w0, #LCD_MAX_X
	bra	c, invr_ret
	cp	w1, #LCD_MAX_Y
	bra	c, invr_ret

	cp0	w2
	bra	le, invr_ret
	cp0	w3
	bra	le, invr_ret

	push.d	w4
	push	w6

	mov	#LCD_MAX_X, w5
	add	w0, w2, w4
	cpslt	w4, w5
	sub	w5, w0, w2

	mov	#LCD_MAX_Y, w5
	add	w1, w3, w4
	cpslt	w4, w5
	sub	w5, w1, w3

invr_next:
	push	w1
	push	w2
	mul.uu	w1, #LCD_LINE_BYTE, w4
	mov	#Video_buffer, w5
	add	w4, w5, w1
	lsr	w0, #3, w4
	add	w1, w4, w1

	rcall	Line_Inv

	pop	w2
	pop	w1

	inc	w1, w1
	dec	w3, w3
	bra	nz, invr_next

	pop	w6
	pop.d	w4
invr_ret:
	return
.else
/* ----------------------------------------------------------------------
W0=x W2=w W1=Video_buffer
------------------------------------------------------------------------- */
Line_Inv:
	mov	#0xffff, w4
	cp	w2, #15
	bra	gt, liv_b1
	sl	w4, w2, w4
	com	w4, w4
liv_b1:
	sl	w4, w0, w4
	xor	w4, [w1], [w1++]
	subr	w0, #16, w4
	sub	w2, w4, w2
	bra	le, liv_end

	cp	w2, #16
	bra	le, liv_el

	lsr	w2, #4, w4
	dec	w4, w4
	mov	w1, w5		/* [w1], [w1++]  --> stall */
	repeat	w4
	com	[w5++], [w1++]
;	com	[w1], [w1++]
	and	#0x0F, w2
	bra	z, liv_end
liv_el:
	subr	w2, #16, w2
	mov	#0xFFFF, w4
	lsr	w4, w2, w4
	xor	w4, [w1], [w1]
liv_end:
	return

/* ----------------------------------------------------------------------
W0=x W1=y W2=w W3=h
------------------------------------------------------------------------- */
InvRect:
	cp	w0, #LCD_MAX_X
	bra	c, ivr_ret
	cp	w1, #LCD_MAX_Y
	bra	c, ivr_ret

	cp0	w2
	bra	le, ivr_ret

	cp0	w3
	bra	le, ivr_ret

	push.d	w4

	mov	#LCD_MAX_X, w5
	add	w0, w2, w4
	cpslt	w4, w5
	sub	w5, w0, w2

	mov	#LCD_MAX_Y, w5
	add	w1, w3, w4
	cpslt	w4, w5
	sub	w5, w1, w3

	mul.uu	w1, #LCD_LINE_BYTE, w4
	mov	#Video_buffer, w5
	add	w4, w5, w1
	lsr	w0, #4, w4
	add	w1, w4, w1
	add	w1, w4, w1
	and	w0, #0x0F, w0

	dec	w3, w3
	do	w3, ivr_do
	push	w1
	push	w2
	rcall	Line_Inv
	pop	w2
	pop	w1
ivr_do: add	#LCD_LINE_BYTE, w1
	pop.d	w4
ivr_ret:
	return
.endif

Char8B_OutK4:
/*
W0=x W1=y W7=Char DataPointer K4
ret
w0=x + len  w3=xx  w4=xx  w5=xx  w7=xx
*/
	push	w1

	mov.b	[w7++], w3
	swap	w3
	mov.b	[w7++], w3
	swap	w3

	btst	w3, #15
	bra	z, c8b_err
	and	w3, #0x1f, w4
	inc2	w4, w4
	add	w4, w0, [w15++]	/* new x + push*/

	lsr	w3, #5, w3
	and	w3, #0x1f, w5
	add	w1, w5, w1	/* y + offset */

	lsr	w3, #5, w3	/* h */
	and	w3, #0x1f, w3

/* Char Paint */
	mul.uu	w1, #LCD_LINE_BYTE, w4
	mov	#Video_buffer, w5
	add	w4, w5, w1
	lsr	w0, #3, w5
	add	w1, w5, w1

	mul.uu	w3, #LCD_LINE_BYTE, w4
	add	w1, w4, w4
	mov	#Video_buffer + LCD_MAX_MEM, w5

	and	w0, #0x07, w0	/* Test x is byte align */
	bra	z,c8b_ial
/* --- x not byte align --- */
	inc	w4, w4
	cp	w5, w4		/* Test Video_buffer Overflow */
	bra	le, c8bu_ov

	sub	w1, #LCD_LINE_BYTE - 1, w1
	do	w3, c8bu
	add	w1, #LCD_LINE_BYTE - 1, w1
	ze	[w7++], w3
	sl	w3, w0, w3
	ior.b	w3, [w1], [w1++]
	swap	w3
c8bu:	ior.b	w3, [w1], [w1]
	
	pop	w0		/* pop new x*/
	pop	w1
	return

c8bu_ov:
	do	w3, c8buov
	cp	w5, w1		/* Test Video_buffer + LCD_MAX_MEM */
	bra	le, c8b_break

	ze	[w7++], w3
	sl	w3, w0, w3
	ior.b	w3, [w1], [w1++]
	swap	w3
	cpseq	w1, w5		/* Test Video_buffer + LCD_MAX_MEM */
	ior.b	w3, [w1], [w1]
c8buov:	add	w1, #LCD_LINE_BYTE - 1, w1
	pop	w0		/* pop new x*/
c8b_err:
	pop	w1
	return

/* --- x byte align --- */
c8b_ial:
	cp	w5, w4		/* Test Video_buffer Overflow */
	bra	le, c8ba_ov

	sub	w1, #LCD_LINE_BYTE, w1
	do	w3, c8ba
	add	w1, #LCD_LINE_BYTE, w1
	mov.b	[w7++], w3
c8ba:	ior.b	w3, [w1], [w1]
	
	pop	w0		/* pop new x*/
	pop	w1
	return

c8ba_ov:
	do	w3, c8baov
	cp	w5, w1		/* Test Video_buffer + LCD_MAX_MEM */
	bra	le, c8b_break
	mov.b	[w7++], w3
	ior.b	w3, [w1], [w1]
c8baov:	add	w1, #LCD_LINE_BYTE, w1
	pop	w0		/* pop new x*/
	pop	w1
	return
/* --- DO LOOP break --- */
c8b_break:
	bset	CORCON, #EDT	/* clr DO LOOP hardware */
	pop	w0		/* pop new x*/
	pop	w1
	return

/*
W0=x W1=y W2=StrPointer W14=FontPointer
*/
StrOut:
	push.d	w4
	push.d	w6
	mov	#STR_MAX_BUF, w6
so_next_c:
	ze	[w2++],w3	/* Test end Char 0x00 */	
	bra	z, so_end
so_next_ct:
	dec	w6, w6		/* Test max Char */
	bra	z, so_end

	btst	w3, #7		/* Test Char >= 0x80 */
	bra	z, so_paint_c
	lsr	w3, #4, w5	/* Test Char 0x8n */
	cp	w5, #0x08
	bra	nz, so_next_c
	and	w3, #0x0f, w5
	cp	w5, #0x0e
	bra	z, so_x_m2
	cp	w5, #0x0f
	bra	z, so_x_m1
	inc	w5, w5
	add	w0, w5, w0
	bra	so_next_c
so_x_m2:
	dec	w0, w0
so_x_m1:
	dec	w0, w0
	bra	so_next_c

so_chk_bl:
	bra	lt, so_next_c
	mov	[w14], w5
	cp0	w5
	bra	z, so_next_c

	mov.b	[w5+1],w3
	btst	w3, #7
	bra	z, so_next_c
	mov.b	[w5],w3
	and	w3, #0x1f, w3
	inc2	w3, w3
	add	w0, w3, w0	/* new x */
	bra	so_next_c

so_paint_c:
	sub	#0x20, w3
	bra	le, so_chk_bl
;	cp	w3, #0x60
;	bra	ge, so_next_c

	sl	w3, #1, w3
	add	w3, w14, w5
	mov	[w5], w7
	cp0	w7
	bra	z, so_next_c
	rcall	Char8B_OutK4
	ze	[w2++],w3	/* Test end Char 0x00 */	
	bra	nz, so_next_ct
so_end:
	pop.d	w6
	pop.d	w4
	return

/*
W0=x W1=y W2=StrPointer
*/
StrOut8x13:
	mov	w14, [w15++]
	mov	#C8x13, w14
	rcall	StrOut
	mov	[--w15], w14
	return

/*
W0=x W1=len W2=StrPointer
return
w0=x w1=x+1Char
*/
StrLen8x13:
	mov	w14, [w15++]
	mov	#C8x13, w14
	rcall	StrLen
	mov	[--w15], w14
	return

StrLen:
	mov.d	w4, [w15++]
	inc	w1, w1
	mov	#STR_MAX_BUF-1, w4
	cpsgt	w1, w4
	mov	w1, w4
	mov	w0, w1
10:
	cp	w4, #0
	bra	le, 99f
	dec	w4, w4
	mov	w1, w0

	ze	[w2++],w3
	cp0	w3		/* Test end Char 0x00 */
	bra	z, 99f

	btst	w3, #7		/* Test Char >= 0x80 */
	bra	z,20f
	lsr	w3, #4, w5	/* Test Char 0x8n */
	cp	w5, #0x08
	bra	nz,20f
	and	w3, #0x0f, w5
	inc	w5, w5
	add	w1, w5, w1
	bra	10b
20:
	mov	#0x20, w5	/* Test Char 0x20 .. 0x7f */
	sub	w3, w5, w3
	bra	lt, 10b
	mov	#0x60, w5
	cp	w3, w5
	bra	ge, 10b

	sl	w3, #1, w3
;	mov	#C8x13, w5
	add	w3, w14, w5

	nop			; stall
	mov	[w5], w5
	cp0	w5
	bra	z, 10b

	mov.b	[w5+1],w3
	swap	w3
	mov.b	[w5],w3

	btst	w3, #15
	bra	z, 10b
	and	w3, #0x1f, w3
	add	w3, #2, w3
	add	w1, w3, w1	/* new x */
	bra	10b
99:
	mov.d	[--w15], w4
	return

/*
W0=x W1=y W2=StrPointer
*/
StrOut_cs:
	mov	w14, [w15++]
	mov	#Font_cs, w14
	rcall	StrOut
	mov	[--w15], w14
	return

/* ----------------------------------------------------------------------
W0=x W1=len W2=StrPointer W3=dvf
result: w0=x w1=x+1Char
------------------------------------------------------------------------- */
StrLen_dvf:
	mov	w14, [w15++]
	mov	#C8x13, w14
	btsc	w3, #bDVF_FONT
	mov	#Font_cs, w14
	rcall	StrLen
	mov	[--w15], w14
	return

/* ----------------------------------------------------------------------
W0=x W1=y W2=StrPointer
------------------------------------------------------------------------- */
StrOut_dvf:
	mov	w14, [w15++]
	mov	#C8x13, w14
	btsc	w3, #bDVF_FONT
	mov	#Font_cs, w14
	rcall	StrOut
	mov	[--w15], w14
	return

/* ______________________________________________________________________ */
.if 0
/*
W0=x W1=y W2=w W3=h w4=Data
*/
BixOut:
	cp	w0, #LCD_MAX_X
	bra	c, bixo_ret
	cp	w1, #LCD_MAX_Y
	bra	c, bixo_ret

	push	w5
	push	w6

	add	w0, w2, w5
	cp	w5, #LCD_MAX_X
	bra	gtu, bixo_end

	mov	#LCD_MAX_Y, w6
	add	w1, w3, w5
	cpslt	w5, w6
	sub	w6, w1, w3

	mul.uu	w1, #LCD_LINE_BYTE, w6
	mov	#Video_buffer, w5
	add	w6, w5, w1
	lsr	w0, #3, w5
	add	w1, w5, w1

	add	w2, #7, w5
	lsr	w5, #3, w5		/* bix byte */
	subr	w5, #LCD_LINE_BYTE, w6	/* line off add */

	dec	w5, w5
bixo_next:
	repeat	w5
	mov.b	[w4++], [w1++]
	add	w1, w6, w1
	dec	w3, w3
	bra	nz, bixo_next

bixo_end:
	pop	w6
	pop	w5
bixo_ret:
	return

BixFan:
	mov	#0,w0
	mov	#28,w1
	mov	#16,w2
	mov	#16,w3
	mov	#Fan16x16, w4
	bra	BixOut

BixVConst:
	mov	#0,w0
	mov	#11,w1
	mov	#16,w2
	mov	#16,w3
	mov	#VConst16x16, w4
	bra	BixOut
.endif

/* ______________________________________________________________________ */
.section .const, psv
ct_x20:
 .byte 0x06,0x80,0x00
ct_0:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x1c,0x36,0x63,0x63,0x63,0x63,0x63,0x63,0x36,0x1c
ct_1:  /* W=8 H=10 Yo=0 */
 .byte 0x06,0xa4,0x18,0x1c,0x1e,0x1b,0x18,0x18,0x18,0x18,0x18,0xff
ct_2:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x1c,0x36,0x63,0x60,0x60,0x30,0x18,0x0c,0x06,0x7f
ct_3:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x7f,0x60,0x30,0x18,0x1c,0x30,0x60,0x60,0x33,0x1e
ct_4:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x30,0x38,0x3c,0x36,0x33,0x33,0x7f,0x30,0x30,0x30
ct_5:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x7f,0x03,0x03,0x03,0x1f,0x30,0x60,0x60,0x33,0x1e
ct_6:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x3c,0x06,0x03,0x03,0x1f,0x33,0x63,0x63,0x36,0x1c
ct_7:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x7f,0x60,0x30,0x30,0x18,0x18,0x0c,0x0c,0x0c,0x0c
ct_8:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x1c,0x36,0x63,0x36,0x1c,0x36,0x63,0x63,0x36,0x1c
ct_9:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x1c,0x36,0x63,0x63,0x76,0x7c,0x60,0x60,0x30,0x1e
ct_percent:  /* W=8 H=10 Yo=0 */
 .byte 0x07,0xa4,0xce,0x7b,0x7b,0x36,0x18,0x18,0x6c,0xde,0xde,0x73
ct_pleft:  /* W=4 H=11 Yo=0 */
 .byte 0x03,0xa8,0x06,0x06,0x03,0x03,0x03,0x03,0x03,0x03,0x06,0x06,0x0c
ct_pright:  /* W=4 H=11 Yo=0 */
 .byte 0x03,0xa8,0x06,0x06,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x06,0x06,0x03
ct_dot:  /* W=3 H=2 Yo=8 */
 .byte 0x02,0x85,0x07,0x07
ct_plus:  /* W=8 H=7 Yo=2 */
 .byte 0x47,0x98,0x18,0x18,0x18,0xff,0x18,0x18,0x18
ct_minus:  /* W=7 H=1 Yo=5 */
 .byte 0xa6,0x80,0x7f
ct_slash:  /* W=8 H=10 Yo=0 */
 .byte 0x07,0xa4,0xc0,0x60,0x60,0x30,0x18,0x18,0x0c,0x06,0x06,0x03
ct_gt:  /* W=6 H=9 Yo=1 */
 .byte 0x25,0xa0,0x03,0x06,0x0c,0x18,0x30,0x18,0x0c,0x06,0x03
ct_colon:  /* W=3 H=7 Yo=3 */
 .byte 0x62,0x98,0x07,0x07,0x00,0x00,0x00,0x07,0x07
ct_equal:  /* W=7 H=4 Yo=4 */
 .byte 0x86,0x8c,0x7f,0x00,0x00,0x7f
ct_A:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x08,0x1c,0x1c,0x1c,0x36,0x3e,0x36,0x63,0x63,0x63
ct_B:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x3f,0x63,0x63,0x63,0x3f,0x63,0x63,0x63,0x63,0x3f
ct_C:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x3c,0x66,0x03,0x03,0x03,0x03,0x03,0x03,0x66,0x3c
ct_D:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x1f,0x33,0x63,0x63,0x63,0x63,0x63,0x63,0x33,0x1f
ct_E:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x7f,0x03,0x03,0x03,0x1f,0x03,0x03,0x03,0x03,0x7f
ct_F:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x7f,0x03,0x03,0x03,0x1f,0x03,0x03,0x03,0x03,0x03
ct_G:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x1c,0x36,0x63,0x03,0x03,0x73,0x63,0x63,0x36,0x1c
ct_H:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x63,0x63,0x63,0x63,0x7f,0x63,0x63,0x63,0x63,0x63
ct_I:  /* W=6 H=10 Yo=0 */
 .byte 0x05,0xa4,0x3f,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x3f
ct_J:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x78,0x30,0x30,0x30,0x30,0x30,0x30,0x33,0x33,0x1e
ct_K:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x63,0x33,0x1b,0x0f,0x07,0x0f,0x1b,0x33,0x63,0x63
ct_L:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x03,0x7f
ct_M:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x63,0x77,0x7f,0x6b,0x63,0x63,0x63,0x63,0x63,0x63
ct_N:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x63,0x63,0x67,0x6f,0x7b,0x73,0x63,0x63,0x63,0x63
ct_O:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x3e,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x3e
ct_P:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x1f,0x33,0x63,0x63,0x33,0x1f,0x03,0x03,0x03,0x03
ct_R:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x3f,0x63,0x63,0x63,0x63,0x3f,0x1b,0x33,0x63,0x63
ct_S:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x3e,0x63,0x03,0x03,0x3e,0x60,0x60,0x60,0x63,0x3e
ct_T:  /* W=6 H=10 Yo=0 */
 .byte 0x05,0xa4,0x3f,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c
ct_U:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x36,0x1c
ct_V:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x63,0x63,0x63,0x36,0x36,0x36,0x1c,0x1c,0x1c,0x08
ct_W:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x63,0x63,0x63,0x63,0x6b,0x6b,0x6b,0x7f,0x77,0x22
ct_Y:  /* W=8 H=10 Yo=0 */
 .byte 0x07,0xa4,0xc3,0xc3,0x66,0x3c,0x18,0x18,0x18,0x18,0x18,0x18
ct_Z:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x7f,0x60,0x60,0x30,0x18,0x0c,0x06,0x03,0x03,0x7f
ct_a:  /* W=7 H=7 Yo=3 */
 .byte 0x66,0x98,0x3e,0x60,0x60,0x7e,0x63,0x63,0x7e
ct_c:  /* W=7 H=7 Yo=3 */
 .byte 0x66,0x98,0x3e,0x63,0x03,0x03,0x03,0x63,0x3e
ct_d:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x60,0x60,0x60,0x7e,0x63,0x63,0x63,0x63,0x63,0x7e
ct_e:  /* W=7 H=7 Yo=3 */
 .byte 0x66,0x98,0x3e,0x63,0x63,0x7f,0x03,0x63,0x3e
ct_f:  /* W=6 H=10 Yo=0 */
 .byte 0x05,0xa4,0x1c,0x36,0x36,0x06,0x06,0x0f,0x06,0x06,0x06,0x06
ct_g:  /* W=7 H=10 Yo=2 */
 .byte 0x46,0xa4,0x7e,0x33,0x33,0x33,0x1e,0x03,0x3e,0x63,0x63,0x3e
ct_h:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x03,0x03,0x03,0x3f,0x63,0x63,0x63,0x63,0x63,0x63
ct_i:  /* W=6 H=10 Yo=0 */
 .byte 0x05,0xa4,0x0c,0x0c,0x00,0x0f,0x0c,0x0c,0x0c,0x0c,0x0c,0x3f
ct_k:  /* W=7 H=10 Yo=0 */
 .byte 0x06,0xa4,0x03,0x03,0x03,0x33,0x1b,0x0f,0x0f,0x1b,0x33,0x63
ct_l:  /* W=6 H=10 Yo=0 */
 .byte 0x05,0xa4,0x0f,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x0c,0x3f
ct_m:  /* W=8 H=7 Yo=3 */
 .byte 0x67,0x98,0x7f,0xdb,0xdb,0xdb,0xdb,0xdb,0xc3
ct_n:  /* W=7 H=7 Yo=3 */
 .byte 0x66,0x98,0x3b,0x67,0x63,0x63,0x63,0x63,0x63
ct_o:  /* W=7 H=7 Yo=3 */
 .byte 0x66,0x98,0x1c,0x36,0x63,0x63,0x63,0x36,0x1c
ct_p:  /* W=7 H=10 Yo=3 */
 .byte 0x46,0xa4,0x1f,0x33,0x63,0x63,0x63,0x33,0x1f,0x03,0x03,0x03
ct_q:  /* W=7 H=10 Yo=2 */
 .byte 0x46,0xa4,0x7c,0x66,0x63,0x63,0x63,0x66,0x7c,0x60,0x60,0x70
ct_r:  /* W=7 H=7 Yo=3 */
 .byte 0x66,0x98,0x3b,0x6e,0x06,0x06,0x06,0x06,0x06
ct_s:  /* W=7 H=7 Yo=3 */
 .byte 0x66,0x98,0x3e,0x63,0x03,0x3e,0x60,0x63,0x3e
ct_t:  /* W=7 H=9 Yo=1 */
 .byte 0x26,0xa0,0x0c,0x0c,0x3f,0x0c,0x0c,0x0c,0x0c,0x6c,0x38
ct_u:  /* W=7 H=7 Yo=3 */
 .byte 0x66,0x98,0x63,0x63,0x63,0x63,0x63,0x63,0x7e
ct_v:  /* W=7 H=7 Yo=3 */
 .byte 0x66,0x98,0x63,0x63,0x36,0x36,0x1c,0x1c,0x08
ct_w:  /* W=8 H=7 Yo=3 */
 .byte 0x67,0x98,0xc3,0xc3,0xdb,0xdb,0xdb,0xff,0x66
ct_x:  /* W=7 H=7 Yo=3 */
 .byte 0x66,0x98,0x63,0x36,0x1c,0x08,0x1c,0x36,0x63
ct_y:  /* W=7 H=10 Yo=3 */
 .byte 0x66,0xa4,0x63,0x63,0x36,0x36,0x36,0x1c,0x1c,0x0c,0x0d,0x06
ct_z:  /* W=6 H=7 Yo=3 */
 .byte 0x65,0x98,0x3f,0x30,0x18,0x0c,0x06,0x03,0x3f
ct_grad:  /* W=5 H=4 Yo=0 */
 .byte 0x04,0x8c,0x0e,0x1b,0x1b,0x0e

 .align 2
C8x13:
 .word	ct_x20,	0,	0,	0,	0,	ct_percent,	0,	0
 .word	ct_pleft,ct_pright,0,	ct_plus,0,	ct_minus,ct_dot,ct_slash

 .word	ct_0,	ct_1,	ct_2,	ct_3,	ct_4,	ct_5,	ct_6,	ct_7
 .word	ct_8,	ct_9,	ct_colon,0,	0,	ct_equal,ct_gt,	0

 .word	0,	ct_A,	ct_B,	ct_C,	ct_D,	ct_E,	ct_F,	ct_G
 .word	ct_H,	ct_I,	ct_J,	ct_K,	ct_L,	ct_M,	ct_N,	ct_O

 .word	ct_P,	0,	ct_R,	ct_S,	ct_T,	ct_U,	ct_V,	ct_W
 .word	0,	ct_Y,	ct_Z,	0,	0,	0,	0,	0

 .word	0,	ct_a,	0,	ct_c,	ct_d,	ct_e,	ct_f,	ct_g
 .word	ct_h,	ct_i,	0,	ct_k,	ct_l,	ct_m,	ct_n,	ct_o

 .word	ct_p,	ct_q,	ct_r,	ct_s,	ct_t,	ct_u,	ct_v,	ct_w
 .word	ct_x,	ct_y,	ct_z,	0,	0,	0,	0,	ct_grad

cs_x20:
 .byte 0x03,0x80,0x00
cs_0:  /* W=3 H=6 Yo=0 */
 .byte 0x03,0x94,0x02,0x05,0x05,0x05,0x05,0x02
cs_1:  /* W=3 H=6 Yo=0 */
 .byte 0x03,0x94,0x02,0x03,0x02,0x02,0x02,0x07
cs_2:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x06,0x09,0x08,0x04,0x02,0x0f
cs_3:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x0f,0x08,0x06,0x08,0x09,0x06
cs_4:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x04,0x06,0x05,0x0f,0x04,0x04
cs_5:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x0f,0x01,0x07,0x08,0x09,0x06
cs_6:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x06,0x01,0x07,0x09,0x09,0x06
cs_7:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x0f,0x08,0x04,0x04,0x02,0x02
cs_8:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x06,0x09,0x06,0x09,0x09,0x06
cs_9:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x06,0x09,0x09,0x0e,0x08,0x06
cs_dot:  /* W=2 H=2 Yo=4 */
 .byte 0x81,0x84,0x03,0x03
cs_plus:  /* W=5 H=5 Yo=1 */
 .byte 0x24,0x90,0x04,0x04,0x1f,0x04,0x04
cs_minus:  /* W=4 H=1 Yo=3 */
 .byte 0x63,0x80,0x0f
cs_slash:  /* W=4 H=4 Yo=1 */
 .byte 0x23,0x8c,0x08,0x04,0x02,0x01
cs_colon:  /* W=2 H=5 Yo=1 */
 .byte 0x21,0x90,0x03,0x03,0x00,0x03,0x03
cs_equal:  /* W=4 H=3 Yo=2 */
 .byte 0x43,0x88,0x0f,0x00,0x0f
cs_A:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x06,0x09,0x09,0x0f,0x09,0x09
cs_B:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x07,0x09,0x07,0x09,0x09,0x07
cs_C:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x06,0x09,0x01,0x01,0x09,0x06
cs_D:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x07,0x09,0x09,0x09,0x09,0x07
cs_E:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x0f,0x01,0x07,0x01,0x01,0x0f
cs_F:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x0f,0x01,0x07,0x01,0x01,0x01
cs_G:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x06,0x09,0x01,0x0d,0x09,0x0e
cs_H:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x09,0x09,0x0f,0x09,0x09,0x09
cs_I:  /* W=3 H=6 Yo=0 */
 .byte 0x02,0x94,0x07,0x02,0x02,0x02,0x02,0x07
cs_L:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x01,0x01,0x01,0x01,0x01,0x0f
cs_M:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x09,0x0f,0x0f,0x09,0x09,0x09
cs_N:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x09,0x0b,0x0b,0x0d,0x0d,0x09
cs_O:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x06,0x09,0x09,0x09,0x09,0x06
cs_P:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x07,0x09,0x09,0x07,0x01,0x01
cs_Q:  /* W=4 H=7 Yo=0 */
 .byte 0x03,0x94,0x06,0x09,0x09,0x0b,0x05,0x0A
cs_R:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x07,0x09,0x09,0x07,0x05,0x09
cs_S:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x06,0x09,0x02,0x04,0x09,0x06
cs_T:  /* W=3 H=6 Yo=0 */
 .byte 0x02,0x94,0x07,0x02,0x02,0x02,0x02,0x02
cs_V:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x09,0x09,0x09,0x09,0x06,0x06
cs_W:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x09,0x09,0x09,0x0f,0x0f,0x09
cs_Y:  /* W=3 H=6 Yo=0 */
 .byte 0x02,0x94,0x05,0x05,0x05,0x02,0x02,0x02
cs_Z:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x0f,0x08,0x04,0x02,0x01,0x0f
cs_a:  /* W=4 H=4 Yo=2 */
 .byte 0x43,0x8c,0x0e,0x09,0x0d,0x0a
cs_c:  /* W=3 H=4 Yo=2 */
 .byte 0x42,0x8c,0x06,0x01,0x01,0x06
cs_d:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x08,0x08,0x0e,0x09,0x09,0x0e
cs_e:  /* W=4 H=4 Yo=2 */
 .byte 0x43,0x8c,0x06,0x0d,0x03,0x06
cs_g:  /* W=4 H=5 Yo=2 */
 .byte 0x43,0x90,0x0e,0x09,0x0e,0x08,0x06
cs_h:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x01,0x01,0x07,0x09,0x09,0x09
cs_i:  /* W=3 H=6 Yo=0 */
 .byte 0x02,0x94,0x02,0x00,0x03,0x02,0x02,0x07
cs_k:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x01,0x01,0x05,0x03,0x05,0x09
cs_l:  /* W=3 H=6 Yo=0 */
 .byte 0x02,0x94,0x03,0x02,0x02,0x02,0x02,0x07
cs_m:  /* W=4 H=4 Yo=2 */
 .byte 0x43,0x8c,0x05,0x0f,0x09,0x09
cs_n:  /* W=4 H=4 Yo=2 */
 .byte 0x43,0x8c,0x07,0x09,0x09,0x09
cs_o:  /* W=4 H=4 Yo=2 */
 .byte 0x43,0x8c,0x06,0x09,0x09,0x06
cs_p:  /* W=4 H=5 Yo=2 */
 .byte 0x43,0x90,0x07,0x09,0x09,0x07,0x01
cs_q:  /* W=4 H=5 Yo=2 */
 .byte 0x43,0x90,0x0e,0x09,0x09,0x0e,0x08
cs_s:  /* W=4 H=4 Yo=2 */
 .byte 0x43,0x8c,0x0e,0x03,0x0c,0x07
cs_t:  /* W=4 H=6 Yo=0 */
 .byte 0x03,0x94,0x02,0x02,0x07,0x02,0x02,0x0c
cs_u:  /* W=4 H=4 Yo=2 */
 .byte 0x43,0x8c,0x09,0x09,0x09,0x0e
cs_v:  /* W=3 H=4 Yo=2 */
 .byte 0x42,0x8c,0x05,0x05,0x05,0x02
cs_w:  /* W=4 H=4 Yo=2 */
 .byte 0x43,0x8c,0x09,0x09,0x0f,0x0f
cs_x:  /* W=4 H=4 Yo=2 */
 .byte 0x43,0x8c,0x09,0x06,0x06,0x09
cs_z:  /* W=4 H=4 Yo=2 */
 .byte 0x43,0x8c,0x0f,0x04,0x02,0x0f
cs_plusmin:  /* W=3 H=6 Yo=0 */
 .byte 0x02,0x94,0x02,0x07,0x02,0x00,0x07,0x00
cs_grad:  /* W=3 H=3 Yo=0 */
 .byte 0x02,0x88,0x02,0x05,0x02

 .align 2
Font_cs:
 .word	cs_x20,	0,	0,	0,	0,	0,	0,	0
 .word	0,	0,	0,	cs_plus,0,	cs_minus,cs_dot,cs_slash

 .word	cs_0,	cs_1,	cs_2,	cs_3,	cs_4,	cs_5,	cs_6,	cs_7
 .word	cs_8,	cs_9,	cs_colon,0,	0,	cs_equal,0,	0

 .word	0,	cs_A,	cs_B,	cs_C,	cs_D,	cs_E,	cs_F,	cs_G
 .word	cs_H,	cs_I,	0,	0,	cs_L,	cs_M,	cs_N,	cs_O

 .word	cs_P,	cs_Q,	cs_R,	cs_S,	cs_T,	0,	cs_V,	cs_W
 .word	0,	cs_Y,	cs_Z,	0,	0,	0,	0,	0

 .word	0,	cs_a,	0,	cs_c,	cs_d,	cs_e,	0,	cs_g
 .word	cs_h,	cs_i,	0,	cs_k,	cs_l,	cs_m,	cs_n,	cs_o

 .word	cs_p,	cs_q,	0,	cs_s,	cs_t,	cs_u,	cs_v,	cs_w
 .word	cs_x,	0,	cs_z,	0,	0,	0,	cs_plusmin,cs_grad



.if 0
Fan16x16:
 .word	0x0000
 .word	0x7006
 .word	0x7c0e
 .word	0x3e1e
 .word	0x1e1c
 .word	0x073c
 .word	0x0378
 .word	0x01e0
 .word	0x0780
 .word	0x1ec0
 .word	0x3ce0
 .word	0x3878
 .word	0x787c
 .word	0x703e
 .word	0x600e
 .word	0x0000

VConst16x16:
 .word	0x0000
 .word	0x1808
 .word	0x0c08
 .word	0x063e
 .word	0x0308
 .word	0x0188
 .word	0x3cc0
 .word	0x3760
 .word	0x19e0
 .word	0x0c00
 .word	0x0660
 .word	0x03e0
 .word	0x03e0
 .word	0x07e0
 .word	0x07e0
 .word	0x0000
.endif
