/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "Analog.inc"
.include "Param.inc"
.include "../math/libm.inc"
.include "../main.inc"

.if USE_DEBUG_UART_USER
.include "uart_user.inc"
.endif

	USE_RDY		= 1

.section .nbss, bss, near
.if USE_DEBUG_UART_USER
MCP3421_ReadCount:		.space 2
.endif
MCP3421_ms_count:		.space 2
InitFlag:			.space 2
AdcOrgVout:			.space 4
AdcPa_1616:			.space 2
AdcPa:				.space 2

LastTemp_PA:			.space 2

fAdcPA_mV:		.space F48_SIZE
fAdcPA_GlobalMul:	.space F48_SIZE

fAdc_Temp_TC:		.space F48_SIZE

fAdcH_Offs:		.space F48_SIZE		; idx 0
fAdcH_Korr_p:		.space F48_SIZE		; idx 1
fAdcH_Korr_n:		.space F48_SIZE		; idx 2

fAdcH_Offs_0:		.space F48_SIZE
fAdcH_Korr_0_p:		.space F48_SIZE
fAdcH_Korr_0_n:		.space F48_SIZE
fAdcH_Temp_KTC:		.space F48_SIZE

.if ADC_PA_MCP3421_AUTORANGE
fAdcL_Offs:		.space F48_SIZE		; idx 0
fAdcL_Korr_p:		.space F48_SIZE		; idx 1
fAdcL_Korr_n:		.space F48_SIZE		; idx 2

fAdcL_Offs_0:		.space F48_SIZE
fAdcL_Korr_0_p:		.space F48_SIZE
fAdcL_Korr_0_n:		.space F48_SIZE
fAdcL_Temp_KTC:		.space F48_SIZE
.endif

.if USE_DEBUG_UART_USER
.global MCP3421_ReadCount
.endif

.global MCP3421_ms_count
.global AdcOrgVout
.global AdcPa_1616
.global AdcPa

.global fAdcPA_mV
.global fAdcPA_GlobalMul
.global fAdcH_Offs
.global fAdcL_Offs

.section .text
;.global MCP3421_INIT
.global MCP3421_INIT_18B
.global MCP3421_Handle
.global MCP3421_i2c_run
.global MCP3421_TempKorr
.global MCP3421_SetParam
.global PaAdcSetGlobMul

;MCP3421_INIT:
;	mov	#1, w0
;	mov	w0, InitFlag
;	return

MCP3421_INIT_18B:
	mov	#3, w0
	mov	w0, InitFlag

	mov	#0x0000, w0
	mov	w0, fAdcPA_GlobalMul
	mov	#0x8000, w0
	mov	w0, fAdcPA_GlobalMul+2
	mov	#0x3fff, w0
	mov	w0, fAdcPA_GlobalMul+4	; 1.0

	mov	#FC48_MCP3421_H_MUL, w2
	mov	#fAdcH_Korr_0_p, w4
	rcall	f48_mov
	mov	#fAdcH_Korr_0_n, w4
	rcall	f48_mov

.if ADC_PA_MCP3421_AUTORANGE
	mov	#FC48_MCP3421_L_MUL, w2
	mov	#fAdcL_Korr_0_p, w4
	rcall	f48_mov
	mov	#fAdcL_Korr_0_n, w4
	rcall	f48_mov
.endif
	bra	clr_LastTemp_PA_force

MCP3421_Handle:
	cp0	i2c2_Lock
	bra	nz, lend

	btst	InitFlag, #0
	bra	nz, SetConfig
.if USE_RDY
	mov	#4, w0
	btss	InitFlag, #1
	mov	#3, w0
.else
	mov	#3, w0
	btss	InitFlag, #1
	mov	#2, w0
.endif
	mov	w0, i2c2_DataSize
	mov	#I2C_READ, w0
Send:
	btst	I2CErrorFlag, #I2C_MCP3421
	bra	nz, SetTimeDefect

	mov	w0, i2c2_cmd
	mov	#0xd0, w0
	mov	w0, i2c2_id

	clr	i2c2_Flag
	bset	i2c2_Flag, #I2C_bNoADR

	mov	#I2C_MCP3421, w0
	mov	w0, i2c2_Lock

.if USE_RDY
	mov	#250, w0
.else
	mov	#333, w0
.endif
	mov	w0, MCP3421_ms_count
	return

SetConfig:
;	mov	#0x9F, w0	/* 3 SPS (18 bits) |  PGA=8x */
;	mov	#0x9C, w0	/* 3 SPS (18 bits) |  PGA=1x */
;	mov	#0x9D, w0	/* 3 SPS (18 bits) |  PGA=2x */
	mov	#0x1E, w0	/* 3 SPS (18 bits) |  PGA=4x */
.if ADC_PA_MCP3421_AUTORANGE
	btss	InitFlag, #1
	mov	#0x1F, w0	/* 3 SPS (18 bits) |  PGA=8x */
.endif
	mov	w0, i2c2_Buf
	mov	#1, w0
	mov	w0, i2c2_DataSize
	mov	#I2C_WRITE, w0
	bra	Send

SetTimeDefect:
	mov	#10000, w0
	mov	w0, MCP3421_ms_count
	clr	InitFlag
lend:
	return

MCP3421_i2c_run:
	mov	i2c2_cmd, w0
	cp0	w0
	bra	nz, chk_err

	btst	InitFlag, #0
	bra	nz, Conf

	mov	#i2c2_Buf, w3
	nop
	se	[w3++], w1
	mov.b	[w3++], w0
	swap	w0
	mov.b	[w3++], w0
.if USE_RDY
	mov.b	[w3], w2
	btst	w2, #7
	bra	nz, wait_rdy
.endif

.if ADC_PA_MCP3421_AUTORANGE
	lsr	w1, w3
	rrc	w0, w2
	lsr	w3, w3
	rrc	w2, w2


	btst	TestFlags, #TF_PA_ADC_H_ON
	bra	z, no_test
	btst	InitFlag, #1
	bra	nz, ar_end
	bra	switch_range_H

no_test:
	btst	InitFlag, #1
	bra	nz, is_H

	mov	#0x7A00, w3
	cp	w2, w3
	bra	gt, switch_range_H
	mov	#0x8600, w3
	cp	w2, w3
	bra	lt, switch_range_H
	bra	ar_end
is_H:
	mov	#0x4700, w3
	cp	w2, w3
	bra	gt, ar_end
	mov	#0xB900, w3
	cp	w2, w3
	bra	lt, ar_end
	bra	switch_range_L
ar_end:
.endif

.if USE_DEBUG_UART_USER
	inc	MCP3421_ReadCount
.endif

	mov	w0, AdcOrgVout
	mov	w1, AdcOrgVout+2

	mov	#fAdcH_Offs, w8
.if ADC_PA_MCP3421_AUTORANGE
	btss	InitFlag, #1
	mov	#fAdcL_Offs, w8
.endif
	add	w8, #F48_SIZE, w2	; _Korr_p
	btsc	w1, #15
	add	w2, #F48_SIZE, w2	; _Korr_n

	lnk	#F48_SIZE

	mov	w14, w3
	rcall	i32_f48_w3

	mov	w14, w4
	rcall	f48_mul

	mov	w8, w2			; add _Offs
	mov	w14, w3
	rcall	f48_add

	mov	#fAdcPA_GlobalMul, w2
	mov	#fAdcPA_mV, w4
	rcall	f48_mul

	rcall	f48_i1616

	ulnk

	mov	w0, AdcPa_1616
	mov	w1, AdcPa


	clr	i2c2_Lock
	btst	AutoCalDAC, #bAC_MCP3421
	bra	nz, AutoCal
	return

.if USE_RDY
wait_rdy:
; inc TestVal0
	mov	#25, w0
	mov	w0, MCP3421_ms_count
	clr	i2c2_Lock
	return
.endif

.if ADC_PA_MCP3421_AUTORANGE
switch_range_L:
	mov	#1, w0
	bra	switch_range
switch_range_H:
	mov	#3, w0
switch_range:
	mov	w0, InitFlag
	mov	#2, w0
	mov	w0, MCP3421_ms_count
	clr	i2c2_Lock
	return
.endif

Conf:
	bclr	InitFlag, #0
	clr	i2c2_Lock
	return

chk_err:
	cp	w0, #I2C_ERROR
	bra	z, is_err
	return		/* Wait */

is_err:
	bset	I2CErrorFlag, #I2C_MCP3421
	clr	i2c2_Lock
	return

/* w0 = 0.1C */
MCP3421_TempKorr:
.if USE_DEBUG_UART_USER
	btst	uuCAL_DAC_Mod, #bMCP3421_KORR_OFF
	bra	nz, clr_LastTemp_PA
.endif
	mov	Temp_OP1_0025, w0	/* 0.025C */

	mov	#1200, w1
	sub	w0, w1, w0		; Temp - 30.0C
	bra	z, clr_LastTemp_PA

	mov	LastTemp_PA, w1
	sub	w1, w0, w1
	btsc	SR, #Z
	return
	btsc	w1, #15
	neg	w1, w1
	dec	w1, w1
	btsc	SR, #Z
	return
	mov	w0, LastTemp_PA

	lnk	#F48_SIZE*2

	add	w14, #F48_SIZE, w3
	rcall	i16_f48_w3

	mov	#fAdc_Temp_TC, w2
	mov	w14, w4
	rcall	f48_mul

	mov	#fAdcH_Offs_0, w2
	mov	w14, w3
	mov	#fAdcH_Offs, w4
	rcall	f48_add
.if ADC_PA_MCP3421_AUTORANGE
	mov	#fAdcL_Offs_0, w2
	mov	w14, w3
	mov	#fAdcL_Offs, w4
	rcall	f48_add
.endif

	mov	#fAdcH_Temp_KTC, w2
	add	w14, #F48_SIZE, w3
	mov	w14, w4
	rcall	f48_mul

	mov	w14, w2
	mov	#FC48_1, w3
	rcall	f48_add

	mov	#fAdcH_Korr_0_p, w3
	mov	#fAdcH_Korr_p, w4
	rcall	f48_mul


	mov	#fAdcH_Korr_0_n, w3
	mov	#fAdcH_Korr_n, w4
	rcall	f48_mul

.if ADC_PA_MCP3421_AUTORANGE
	mov	#fAdcL_Temp_KTC, w2
	add	w14, #F48_SIZE, w3
	mov	w14, w4
	rcall	f48_mul

	mov	w14, w2
	mov	#FC48_1, w3
	rcall	f48_add

	mov	#fAdcL_Korr_0_p, w3
	mov	#fAdcL_Korr_p, w4
	rcall	f48_mul

	mov	#fAdcL_Korr_0_n, w3
	mov	#fAdcL_Korr_n, w4
	rcall	f48_mul
.endif
	ulnk
	return

MCP3421_SetParam:
	lnk	#F48_SIZE
	mov.b	EEData0+oFR_ADC_PA_TC, WREG		; 0.01mV/C
	se	w0, w0
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_MCP3421_TC, w2
	mov	#fAdc_Temp_TC, w4
	rcall	f48_mul

	mov	EEData0+oFR_ADCH_PA_OFFS, w0		; 0.01mV
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_001, w2
	mov	#fAdcH_Offs_0, w4
	rcall	f48_mul

	mov	EEData0+oFR_ADCH_PA_KORR_P, w0		; 0.001%
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_000001, w2
	mov	#fAdcH_Korr_0_p, w4
	rcall	f48_mul

	mov.b	EEData0+oFR_ADCH_PA_KORR_N, WREG	; 0.001%
	se	w0, w0
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_000001, w2
	mov	w14, w4
	rcall	f48_mul
	mov	#fAdcH_Korr_0_p, w2
	mov	w14, w3
	mov	#fAdcH_Korr_0_n, w4
	rcall	f48_add

	mov	#fAdcH_Korr_0_p, w2
	mov	#FC48_1, w3
	mov	w2, w4
	rcall	f48_add
	mov	#FC48_MCP3421_H_MUL, w3
	rcall	f48_mul

	mov	#fAdcH_Korr_0_n, w2
	mov	#FC48_1, w3
	mov	w2, w4
	rcall	f48_add
	mov	#FC48_MCP3421_H_MUL, w3
	rcall	f48_mul


	mov.b	EEData0+oFR_ADCH_PA_K_TC, WREG		; 0.0001 %/C
	se	w0, w0
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_MCP3421_KTC, w2
	mov	#fAdcH_Temp_KTC, w4
	rcall	f48_mul

.if ADC_PA_MCP3421_AUTORANGE
	mov	EEData0+oFR_ADCL_PA_OFFS, w0		; 0.01mV
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_001, w2
	mov	#fAdcL_Offs_0, w4
	rcall	f48_mul

	mov	EEData0+oFR_ADCL_PA_KORR_P, w0		; 0.001%
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_000001, w2
	mov	#fAdcL_Korr_0_p, w4
	rcall	f48_mul

	mov.b	EEData0+oFR_ADCL_PA_KORR_N, WREG	; 0.001%
	se	w0, w0
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_000001, w2
	mov	w14, w4
	rcall	f48_mul
	mov	#fAdcL_Korr_0_p, w2
	mov	w14, w3
	mov	#fAdcL_Korr_0_n, w4
	rcall	f48_add

	mov	#fAdcL_Korr_0_p, w2
	mov	#FC48_1, w3
	mov	w2, w4
	rcall	f48_add
	mov	#FC48_MCP3421_L_MUL, w3
	rcall	f48_mul

	mov	#fAdcL_Korr_0_n, w2
	mov	#FC48_1, w3
	mov	w2, w4
	rcall	f48_add
	mov	#FC48_MCP3421_L_MUL, w3
	rcall	f48_mul

	mov.b	EEData0+oFR_ADCL_PA_K_TC, WREG		 ; 0.0001 %/C
	se	w0, w0
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_MCP3421_KTC, w2
	mov	#fAdcL_Temp_KTC, w4
	rcall	f48_mul
.endif
	ulnk
	bra	clr_LastTemp_PA_force

clr_LastTemp_PA:
	cp0	LastTemp_PA
	btsc	SR, #Z
	return
clr_LastTemp_PA_force:
	mov	#fAdcH_Korr_0_p, w2
	mov	#fAdcH_Korr_p, w4
	rcall	f48_mov
	mov	#fAdcH_Korr_0_n, w2
	mov	#fAdcH_Korr_n, w4
	rcall	f48_mov

	mov	#fAdcH_Offs_0, w2
	mov	#fAdcH_Offs, w4
	rcall	f48_mov

.if ADC_PA_MCP3421_AUTORANGE
	mov	#fAdcL_Korr_0_p, w2
	mov	#fAdcL_Korr_p, w4
	rcall	f48_mov
	mov	#fAdcL_Korr_0_n, w2
	mov	#fAdcL_Korr_n, w4
	rcall	f48_mov

	mov	#fAdcH_Offs_0, w2
	mov	#fAdcH_Offs, w4
	rcall	f48_mov
.endif
	clr	LastTemp_PA
	return

PaAdcSetGlobMul:
	mov	#fImpDmmMul, w2
	mov	#fAdcPA_GlobalMul, w4
	rcall	f48_mov
	return

.section .const, psv
.align 2
/*
R1=470k+330k+33k=833k   
R2=150k || 91k = 56.6390k
MCP3421 RG4 = 562.5k
R2g4 = 56.639 || 562.5 / 2 = 47.1448k
G4=833 / 47.1448 + 1 = 18.6690

MCP3421 RG8 = 281.25k
R2g8 = 56.639 || 281.25 / 2 = 40.37665k
G8=833 / 40.37665 + 1 = 21.6307

2048mV / 4 = 512 / 131072 = 0.00390625  * 18.6690 = 0.0729257 mV/digit
2048mV / 8 = 256 / 131072 = 0.001953125 * 21.6307 = 0.0422475 mV/digit
*/

FC48_MCP3421_L_MUL:
	.word	0xb6ed, 0xad0b, 0x3ffa		; 0.0422475 mV/digit

FC48_MCP3421_H_MUL:
	.word	0x11c4, 0x955a, 0x3ffb		; 0.0729257 mV/digit


;0.01mV/C * 0.025C = 0.00025
FC48_MCP3421_TC:
	.word	0x6e98, 0x8312, 0x3ff3		; 0.00025

;0.0001 %/C / 100 * 0.025C = 0.025E-6

FC48_MCP3421_KTC:
	.word	0x94d6, 0xd6bf, 0x3fe5		; 0.025E-6
