/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
;.include "Analog.inc"
;.include "Param.inc"

/*  Use MCP47FEB22 A0+A1 */


.section .nbss, bss, near
MCP47FEB_OutFlag:		.space 2
MCP47FEB_IntFlag:		.space 2
.equ bMK_INI_REF_A0,		8
.equ bMK_INI_REF_A1,		9

dac_channel:			.space 2
;InitFlag:			.space 2
MCP47FEB_Data:			.space 8

.global MCP47FEB_OutFlag

.section .text
.global MCP47FEB_INIT
.global MCP47FEB_Handle
.global MCP47FEB_i2c_run
.global MCP47FEB_SetValue
.global MCP47FEB_GetValue

MCP47FEB_INIT:
;	bset	MCP47FEB_OutFlag, #bMK_INI_REF_A0
;	bset	MCP47FEB_OutFlag, #bMK_INI_REF_A1
	mov	#0x7FF0, w0
	mov	w0, MCP47FEB_Data
	mov	w0, MCP47FEB_Data+2
	mov	w0, MCP47FEB_Data+4
	mov	w0, MCP47FEB_Data+6
;	mov	#0x03, w0	/* Default = #0x7FF0 */
;	mov	w0, MCP47FEB_OutFlag
	mov	#0x05, w0
	mov	w0, MCP47FEB_IntFlag
	return

MCP47FEB_Handle:
	cp0	i2c2_Lock
	bra	nz, lend

	btst	I2CErrorFlag, #I2C_MCP47FEB
	bra	nz, lerr

;	btst	MCP47FEB_OutFlag, #bMK_INI_REF_A0
;	bra	nz, IniRef

	cp0	MCP47FEB_IntFlag
	bra	nz, IniRef

	mov	MCP47FEB_OutFlag, w0
	ff1r	w0, w4
	cp0	w4
	bra	z, lend
	cp	w4, #4
	bra	gtu, lerr

	dec	w4, w4			/* channel */
	mov	w4, dac_channel
	mov	#1, w0
	sl	w0, w4, w0
	com	w0, w0
	and	MCP47FEB_OutFlag	/* clr bit */

	sl	w4, w2
	mov	#MCP47FEB_Data, w3
	add	w3, w2, w3

	and	w4, #0x01, w0
	sl	w0, #3, w0
	mov	[w3], w1
	lsr	w1, #4, w1

Send_cd:
	mov	#i2c2_Buf, w2
	swap	w1
	mov.b	w0, [w2++]		/* Command + Reg */
	mov.b	w1, [w2++]		/* Data */
	swap	w1
	mov.b	w1, [w2++]

	mov	#3, w0
	mov	w0, i2c2_DataSize

	mov	#0xc0, w0
	mov	dac_channel, w1
	and	#0x02, w1
	add	w0, w1, w0
	mov	w0, i2c2_id

	mov	#I2C_WRITE, w0
	mov	w0, i2c2_cmd

	clr	i2c2_Flag
	bset	i2c2_Flag, #I2C_bNoADR

	mov	#I2C_MCP47FEB, w0
	mov	w0, i2c2_Lock

lend:
	return

lerr:
	clr	MCP47FEB_OutFlag
	return

IniRef:
	mov	MCP47FEB_IntFlag, w0
	ff1r	w0, w4
	dec	w4, w4			/* channel */
	mov	w4, dac_channel
	mov	#1, w0
	sl	w0, w4, w0
	com	w0, w0
	and	MCP47FEB_IntFlag	/* clr bit */


;	bclr	MCP47FEB_OutFlag, #bMK_INI_REF_A0
	mov	#0x40, w0	/* Command + Reg */
	mov	#0x0005, w1	/* use Ref. Internal 1.22V */
	bra	Send_cd


MCP47FEB_i2c_run:
	mov	i2c2_cmd, w0
	cp0	w0
	bra	nz, chk_err
	clr	i2c2_Lock
	return

chk_err:
	cp	w0, #I2C_ERROR
	bra	z, is_err
	return		/* Wait */

is_err:
	bset	I2CErrorFlag, #I2C_MCP47FEB
	clr	i2c2_Lock
	return

/* w0=Value w2=DAC */
MCP47FEB_SetValue:
.if 1
	mov	#MCP47FEB_Data, w3
	mov	#0x03, w1
	rcall	DAC_KorrSetValue
	btss	SR, #Z
	ior	MCP47FEB_OutFlag
	return
.else
	mov	#MCP47FEB_Data, w3

	and	#0x01, w2
	add	w3, w2, w3
	add	w3, w2, w3
	mov	0xfff0, w1
	and	w0, w1, w0
	cp	w0, [w3]
	btsc	SR, #Z
	return

	mov	w0, [w3]

	mov	#1, w0
	sl	w0, w2, w0
	ior	MCP47FEB_OutFlag
	return
.endif

/* w0=Value w2=DAC */
MCP47FEB_GetValue:
	mov	#MCP47FEB_Data, w3
	mov	#0x03, w1
	bra	DAC_KorrGetValue
