/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "Param.inc"
.include "eingVx.inc"

.if XR_DATA_SEG_SIZE
.section .nbss, bss, near
XRamData:	.space XR_DATA_SEG_SIZE * FR_MAX_DATA_SEG

.endif


.section .text

.global GetDVF_ParaSize_w3
.global GetDVF_ConvertValue
.global EingVx_LoadValue
.global EingVx_SaveValue
.global EingVx_ValueToMem

GetDVF_ParaSize_w3:
	btst	w0, #bDVF_IS_1616
	bra	nz, elv_rd_1616
	btst	w0, #bDVF_IS_BYTE
	bra	nz, elv_rd_8

	mov	#1, w3
	btsc	w0, #bDVF_DP_X10		/* Test eing8 off16 on */
	mov	#0, w3
	return

elv_rd_8:
	mov	#0, w3
	return

elv_rd_1616:
	mov	#3, w3
	btsc	w0, #bDVF_DP_X10		/* Test 10x mod on */
	mov	#1, w3
	return


get_fr_size_w3:
.if 1
	push	w0
	mov	[w5+oEVX_DISP_FLAG], w0
	rcall	GetDVF_ParaSize_w3
	pop	w0
	return
.else
	push	w8
	mov	[w5+oEVX_DISP_FLAG], w8
	btst	w8, #bDVF_IS_1616
	bra	nz, elv_rd_1616
	btst	w8, #bDVF_IS_BYTE
	bra	nz, elv_rd_8

	mov	#1, w3
	btsc	w8, #bDVF_DP_X10		/* Test eing8 off16 on */
	mov	#0, w3
	pop	w8
	return

elv_rd_8:
	mov	#0, w3
	pop	w8
	return

elv_rd_1616:
	mov	#3, w3
	btsc	w8, #bDVF_DP_X10		/* Test 10x mod on */
	mov	#1, w3
	pop	w8
	return
.endif

.if XR_DATA_SEG_SIZE
/* w3=size w5=Vx.pointer */
get_mem_data:
	mov	[w5+oEVX_RAM_DATA_ADR], w2
	cp	w3, #3
	bra	gtu, gmd_ov

	btst	[w5], #bEVSTAT_REL_RAM
	bra	nz, gmd_x
	cp	w2, #0x0001
	bra	le, gmd_ov
	bra	FRam_GetValue
gmd_x:
	mov	#(XR_DATA_SEG_SIZE * FR_MAX_DATA_SEG), w0
	add	w2, w3, w1
	cp	w1, w0
	bra	geu, gmd_ov

	mov	#XRamData, w0
	add	w2, w0, w2
	bra	Ram_GetValue
gmd_ov:
	bset	SR, #OV
	return
.endif

/* w0=Value  w2=DVF  w3=size 0,1,3 1,2,4 byte  w6=FramOrigVal.Ponter */
GetDVF_ConvertValue:
	cp	w3, #3
	bra	z, lvd_sok
	btst	w2, #bDVF_IS_UNSIGN
	bra	nz, lvd_sok

	cp	w3, #0
	btsc	SR, #Z
	se	w0, w0
	asr	w0, #15, w1
lvd_sok:
	cp0	w6
	bra	z, lvd_w6_0
	mov	w0, [w6]
	mov	w1, [w6+2]
lvd_w6_0:
	btst	w2, #bDVF_IS_1616
	bra	z, lvd_no_div10
	btst	w2, #bDVF_DP_X10		/* Test 10x mod on */
	bra	z, lvd_no_div10
	mov	#10, w2
	rcall	div_s32_u16_1616
lvd_no_div10:
	return

/* w5=Vx.pointer  w6=FramOrigVal.Ponter */
EingVx_LoadValue:
.if XR_DATA_SEG_SIZE
	rcall	get_fr_size_w3
	rcall	get_mem_data
.else
	mov	[w5+oEVX_RAM_DATA_ADR], w2
	cp	w2, #0x0001
	bra	le, elv_ov

	rcall	get_fr_size_w3
	rcall	FRam_GetValue
.endif
	bra	ov, elv_ov

	mov	[w5+oEVX_DISP_FLAG], w2
.if 1
	rcall	GetDVF_ConvertValue
.else
	cp	w3, #3
	bra	z, elv_sok
	btst	w2, #bDVF_IS_UNSIGN
	bra	nz, elv_sok

	cp	w3, #0
	btsc	SR, #Z
	se	w0, w0
	asr	w0, #15, w1
elv_sok:
	mov	w0, [w6]
	mov	w1, [w6+2]

	btst	w2, #bDVF_IS_1616
	bra	z, no_div10
	btst	w2, #bDVF_DP_X10		/* Test 10x mod on */
	bra	z, no_div10
	mov	#10, w2
	rcall	div_s32_u16_1616
no_div10:
.endif
	bclr	SR, #OV
	return
elv_ov:
	clr	w0
	clr	w1
	bset	[w5], #bEVSTAT_ERROR
	bset	SR, #OV
	return

korr_vx_val_w0w1:
	mov	[w5+oEVX_DISP_FLAG], w2
	btst	w2, #bDVF_IS_1616
	bra	nz, ch_x10
	mov	w1, w0
	clr	w1
	return
ch_x10:
	btss	w2, #bDVF_DP_X10		/* Test 10x mod on */
	return
	push	w3
	mov	#10, w2				/* v1616 * 10 --> v16 */
	mov	w1, w3
	mul.uu	w0, w2, w0
	sl	w0, w0
	addc	w1, #0, w1	; round
	mul.us	w2, w3, w2
	add	w1, w2, w1
	pop	w3
	mov	w1, w0
	return

/* w0:w1=Value w5=Vx.pointer  w6=owner w7=FramOrigVal.Ponter */
EingVx_SaveValue:
	btsc	[w5], #bEVSTAT_ERROR
	return

	rcall	get_fr_size_w3
	rcall	korr_vx_val_w0w1
	cp0	w7
	bra	z, elvs_save

	mov	[w7], w2
	cp	w0, w2
	bra	nz, elvs_save
	cp	w3, #1
	bra	leu, elvs_ret
	mov	[w7+2], w2
	cp	w1, w2
	bra	z, elvs_ret
elvs_save:
	mov	[w5+oEVX_RAM_DATA_ADR], w2
	cp	w2, #0x0002
	bra	geu, FRam_SaveValue
elvs_ret:
	return

.if XR_DATA_SEG_SIZE
/* w0:w1=Value w3=size 0..3 (1-4Byte) w5=Vx.pointer */
set_mem_data:
	mov	[w5+oEVX_RAM_DATA_ADR], w2
	cp	w3, #3
	bra	gtu, smd_ov

	btst	[w5], #bEVSTAT_REL_RAM
	bra	nz, smd_x
	cp	w2, #0x0002
	bra	ltu, smd_ov
	bra	FRam_SetValue
smd_x:
	push	w0
	mov	#(XR_DATA_SEG_SIZE * FR_MAX_DATA_SEG), w0
	add	w2, w3, w4
	cp	w4, w0
	pop	w0
	bra	geu, smd_ov

	mov	#XRamData, w4
	add	w2, w4, w2
	bra	Ram_SetValue
smd_ov:
	bset	SR, #OV
	return
.endif

/* w0:w1=Value w5=Vx.pointer  w6=owner */
EingVx_ValueToMem:
	btsc	[w5], #bEVSTAT_ERROR
	return
	rcall	get_fr_size_w3
	rcall	korr_vx_val_w0w1
.if XR_DATA_SEG_SIZE
	bra	set_mem_data
.else
	mov	[w5+oEVX_RAM_DATA_ADR], w2
	cp	w2, #0x0002
	bra	ltu, elvm_ret
	rcall	FRam_SetValue
elvm_ret:
	return
.endif
