/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "eingVx.inc"

	bEVX_DEC	= 0
	bEVX_INC	= 1

.section .nbss, bss, near

.if ETSR_SIZE != 8
.ERROR  "ETSR_SIZE != 8"
.endif

tmp_etsr:		.space ETSR_SIZE
tmp_cursX:		.space 2
/* ---------------------------------------------------------------------- */
.section .text

.global etsr_InitTabStep
.global etsr_SetTabStepRef
.global etsr_SetupVx
.global etsr_Key

/* -------------------------------------------------------
w0:w1	Start Value
w2	Start Step idx
w5	Vx.pointer
w6	etsr.pointer
result: 
  C	ERROR
--------------------------------------------------------- */
etsr_InitTabStep:
	cp0	w5
	bra	z, its_err
	cp0	w6
	bra	z, its_err

	mov	[w6+oETSR_P_IDX], w3
	mov	w6, [w5+oEVX_OWNER]
	mov	w2, [w3]

	clr	w2
	rcall	etsr_SetTabStepRef
	bra	nc, etsr_SetupVx

	bclr	SR, #C
	return
its_err:
	bset	SR, #C
	return

/* -------------------------------------------------------
  w2	0 or bEVX_INC or bEVX_DEC
  w5	Vx.pointer
  w6	etsr.pointer
result: 
  w8	tab Pointer
  C	ERROR (W6 == 0)
  NZ	Error and FqHz_Step set 
--------------------------------------------------------- */
etsr_SetTabStepRef:
 
	clr	tmp_etsr
	push.d	w0
	cp0	w6
	bra	z, gw8_err

	mov	[w6+oETSR_P_ETS], w8	; tab_sel Pointer
	mov	[w6+oETSR_P_IDX], w9	; Pointer Step idx
	mov	[w6+oETSR_X], w0	; Displ X
	mov	[w6+oETSR_Y], w1	; Displ Y
	mov	w8, tmp_etsr+oETSR_P_ETS
	mov	w9, tmp_etsr+oETSR_P_IDX
	mov	w0, tmp_etsr+oETSR_X
	mov	w1, tmp_etsr+oETSR_Y

	clr	w3		/* Error Mk */
	mov	[w8++], w1	/* Tab items */
	cp0	w1
	bra	z, gw8_err

	mov	[w9], w0
	btst	w0, #15
	bra	nz, gw8_err_cx_min

	cp	w0, w1
	bra	ge, gw8_err_cx_max

	btst	w2, #bEVX_DEC
	bra	z, gw8_nodec
	cp0	w0
	bra	z, gw8_set_e
	dec	w0, w0
	mov	w0, [w9]
	bra	gw8_set
gw8_nodec:
	btst	w2, #bEVX_INC
	bra	z, gw8_set
	dec	w1, w1
	cp	w0, w1
	bra	ge, gw8_set_e
	inc	w0, w0
	mov	w0, [w9]
	bra	gw8_set
gw8_set_e:
	inc	w3, w3
gw8_set:
	mulw.uu	w0, #ETS_SIZE, w0
	add	w8, w0, w8
	cp0	w3
	pop.d	w0
	bclr	SR, #C
	return

gw8_err_cx_min:
	clr	[w9]
	clr	w0
	inc	w3, w3
	bra	gw8_set

gw8_err_cx_max:
	dec	w1, w0
	btsc	w0, #15
	clr	w0
	mov	w0, [w9]
	inc	w3, w3
	bra	gw8_set
gw8_err:
	pop.d	w0
	bclr	SR, #Z
	bset	SR, #C
	return


/* -------------------- etsr_SetupVx -----------------------
w0:w1	Value
w5	Vx.pointer
w8	ets.pointer
result: 
  C	ERROR
--------------------------------------------------------- */
etsr_SetupVx:
	mov	[w8+oETS_MIN], w2		/* min. */
	mov	[w8+oETS_MAX], w3		/* max. */
	mov	[w8+oETS_DISP_FLAG], w4
	rcall	EingVx_SetValueMinMaxDflag	/* w0:w1 Value */

	mov	[w8], w2		/* txt */
	and	#0x0f, w2
	mov	tmp_etsr+oETSR_X, w0
	mov	tmp_etsr+oETSR_Y, w1
	rcall	EingVx_SetDispXYTxt
	bclr	SR, #C
	return

/* -------------------- etsr_Key -----------------------
w5	Vx.pointer
result: 
  NZ	Data Change
  C	ERROR

--------------------------------------------------------- */
etsr_Key:
	cp0	w6
	bra	z, key_err

	btst	[w5], #bEVSTAT_VAL_HANDLE
	bra	z, key_0

	btst	[w5], #bEVSTAT_VAL_CHANGE
	bra	nz, key_change

	btst	[w5], #bEVSTAT_UV_MIN
	bra	nz, key_uv
	btst	[w5], #bEVSTAT_OV_MAX
	bra	nz, key_ov

	bra	Key_RetOk

/* ------------------- key UV ----------------------------- */
key_uv:
	mov	#(1<<bEVX_DEC), w2
	rcall	etsr_SetTabStepRef
	bra	nz, key_0

	mov	[w8+oETS_DISP_FLAG], w1
	lsr	w1, #DVF_NUM_SIFT, w1
	and	#DVF_NUM_MASK, w1

	mov	[w5+oEVX_CURS], w3
	and	#EVCURS_POS_MASK, w3

	cp	w1, #5
	bra	z, kuv_n3
	cp	w1, #4
	bra	z, kuv_n2
	cp	w1, #3
	bra	z, kuv_n1
	bra	kuv_n0

kuv_n3:
	mov	#1000, w2
	cp	w3, #3
	bra	gt, kuv_nx
	bra	z, kuv_n
kuv_n2:
	mov	#100, w2
	cp	w3, #2
	bra	gt, kuv_nx
	bra	z, kuv_n
kuv_n1:
	mov	#10, w2
	cp	w3, #1
	bra	gt, kuv_nx
	bra	z, kuv_n
kuv_n0:
	mov	#1, w2
	cp	w3, #0
	bra	gt, kuv_nx
kuv_n:
	btss	[w8], #bETS_FLAG_DEC_NO_MUL
	mulw.uu	w2, #10, w2
	mov	[w5+oEVX_VALUE], w0
	btss	[w8], #bETS_FLAG_DEC_NO_MUL
	mul.uu	w0, #10, w0
	sub	w0, w2, w0
	bra	kuv_n_ok

kuv_nx:
	btsc	[w8], #bETS_FLAG_DEC_NO_MUL
	mulw.uu	w2, #10, w2
	mov	[w5+oEVX_VALUE], w0
	clr	w1
	repeat	#17
	div.u	w0, w2
	sub	w0, #9, w0
	mul.uu	w0, w2, w0
	mov	[w5+oEVX_VALUE], w1
	sub	w1, w0, w0
	btss	[w8], #bETS_FLAG_DEC_NO_MUL
	mulw.uu	w0, #10, w0
kuv_n_ok:
	rcall	etsr_SetupVx
	btst	[w5], #bEVSTAT_XCURS_XMAX
	bra	nz, k_ToCursX_max
	btss	[w8], #bETS_FLAG_DEC_NO_MUL
	rcall	EingVx_IncXCursor
	bra	key_change
/* ------------------- key OV ----------------------------- */
key_ov:
	mov	#(1<<bEVX_INC), w2
	rcall	etsr_SetTabStepRef
	bra	nz, Key_RetOk

	mov	[w5+oEVX_VALUE], w0
	btst	[w8], #bETS_FLAG_INC_NO_DIV
	bra	nz, kov_no_div_0

	clr	w1
	mov	#10, w2
	repeat	#17
	div.u	w0, w2
/* Round */
	lsr	w2, w2
	sub	w1, w2, w1
	addc	w0, #0, w0
kov_no_div_0:
	mov	[w5+oEVX_CURS], w3
	and	#EVCURS_POS_MASK, w3
	mov	w3, tmp_cursX
	btsc	[w8], #bETS_FLAG_INC_NO_DIV
	inc	w3, w3

	mov	#1000, w1
	cp	w3, #4
	bra	z, kov_s_ov
	mov	#100, w1
	cp	w3, #3
	bra	z, kov_s_ov
	mov	#10, w1
	cp	w3, #2
	bra	z, kov_s_ov
	mov	#1, w1
	cp	w3, #1
	bra	z, kov_s_ov
	mov	#0, w1
kov_s_ov:
	add	w0, w1, w0
	rcall	etsr_SetupVx
	btst	[w8], #bETS_FLAG_INC_NO_DIV
	bra	nz, k_chk_CursX_max
	btst	[w5], #bEVSTAT_XCURS_XMAX
	bra	nz, k_ToCursX_max

	mov	tmp_cursX, w0	; DecXCursor
	dec	w0, w0
	bra	k_SetCursX_w0

/* ------------------- set CursX max ---------------------- */
k_chk_CursX_max:
	btst	[w5], #bEVSTAT_XCURS_XMAX
	bra	z, key_change
	btst	[w8], #bETS_FLAG_NO_CHK_CMAX
	bra	nz, key_change

k_ToCursX_max:
	mov	#15, w0
k_SetCursX_w0:
	rcall	EingVx_SetXCursor

/* ------------------- result ----------------------------- */
key_change:
	bclr	SR, #Z
	bclr	SR, #C
	return

key_0:
	bset	SR, #Z
	bclr	SR, #C
	return

key_err:
	bset	SR, #Z
	bset	SR, #C
	return


