/*
 Copyright (C) 2014-2020 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "rs232ald.inc"


.section .nbss, bss, near
RxD_Step:	.space 2
TxD_0:		.space 2
RxD_Err:	.space 2
RxD_tgl:	.space 2
RxD_t_ms:	.space 2
RxD_Bef:	.space 2
RxD_len:	.space 2
RxD_ctl:	.space 2
RxD_sum:	.space 2
RxD_idx:	.space 2


.global RxD_Bef
.global RxD_len

.if USE_UART_I2C_BRIDGE
uart_i2c2_cmd:		.space 2
uart_i2c2_Flag:		.space 2
uart_i2c2_id:		.space 2
uart_i2c2_adr:		.space 2
uart_i2c2_DataSize:	.space 2
uart_i2c2_Buf:		.space I2C_MAX_BUF

.global uart_i2c2_cmd
.endif



.section .text
.global rs232ald_Init
.global rs232ald_RxD
.global rs232ald_TxD

.global SendErr_READ_ONLY
.global SendErr_LEN_OVERFLOW
.global SendErr_ADDR_FAULT
.global SendErr_BEF_SIZE
.global SendErr_UNKNOWN_BEF
.global SendErr_BUFLEN
.global SendErr
.global SendErr2

.global px02_Ok

.if USE_UART_I2C_BRIDGE
.global UART_I2C_Bridge_Handle
.global I2C2_UART_run
.endif

rs232ald_Init:
	clr	RxD_Step
	clr	TxD_0
	clr	RxD_Err
	mov	#CTL_TOGGLE, w0
	mov	w0, RxD_tgl
	mov	t_ms, w0
	mov	w0, RxD_t_ms
	return

/*
unsigned int GetCtlCRC(unsigned int dlen, unsigned int bef, unsigned int ctl)
w0=dlen w1=bef w2=ctl
*/
/*
GetCtlCRC:
; return ((bef + dlen) ^ ~((bef + dlen + (ctl & 0xf0)) >> 4)) & 0x0f;
	add	w1, w0, w1
	and	#0xf0, w2
	add	w1, w2, w2
	lsr	w2, #0x4, w2
	com	w2, w2
	xor	w1, w2, w1
	and	w1, #0xf, w0
	return
*/

chk_send_err:
/* Test Send Error + 25ms */
	sub	w2, #25, w2
	bra	ge, SendErr
	bra	RxDLoop

rs232ald_Timeout:
	mov	w1, RxD_t_ms
; TEST_PIN1_TGL
	bra	ResetRxD

rs232ald_RxD:
/* Test Timeout 300ms */
	mov	t_ms, w1
	mov	RxD_t_ms, w0
	sub	w1, w0, w2
	mov	#300, w0
	cp	w2, w0
	bra	ge, rs232ald_Timeout
;NoTineout:
	cp0	RxD_Err
	bra	nz, chk_send_err

RxDLoop:
	rcall	GetByte_UART_dbg_RxD	/* w0 = 0=false 1=true   w1=byte */
	cp0	w0
	bra	z, RetRxD

	mov	t_ms, w0
	mov	w0, RxD_t_ms

	mov	RxD_Step, w0
	cp0	w0
	bra	z, Step0
	dec	w0, w0
	bra	z, Step1
	dec	w0, w0
	bra	z, Step2
	dec	w0, w0
	bra	z, Step3

	cp0	RxD_Err		/* Test Error */
	bra	nz, RxDLoop

	mov	RxD_idx, w0
	cp	RxD_len
	bra	z, StepSum
/* Add n Byte */
	mov	#RxD_dbgbuf, w2
	inc	RxD_idx
	mov.b	w1, [w0+w2]
	mov	w1, w0
	add	RxD_sum
	bra	RxDLoop

Step0:			/* read start byte 0xff */
/* test start byte 0xff */
	inc.b	w1, w1
	bra	nz, RxDLoop
	inc	RxD_Step
	bra	RxDLoop
Step1:			/* read Bef */
	mov	w1, RxD_Bef
	add	#0x39, w1
	mov	w1, RxD_sum
	inc	RxD_Step
	bra	RxDLoop
Step2:			/* read len */
	mov	#MAX_RS_BUFFER - 4, w0
	cp	w1, w0
	bra	gtu, ResetRxD
	mov	w1, RxD_len
	mov	w1, w0
	add	RxD_sum
	inc	RxD_Step
	bra	RxDLoop
Step3:			/* read ctl */
	clr	RxD_idx
	mov	w1, RxD_ctl
.if 1
; return ((bef + dlen) ^ ~((bef + dlen + (ctl & 0xf0)) >> 4)) & 0x0f;
	mov	RxD_len, w0
	add	RxD_Bef, WREG
	mov	w1, w2
	and	#0xf0, w2
	add	w0, w2, w2
	lsr	w2, #0x4, w2
	com	w2, w2
	xor	w0, w2, w0
	and	w0, #0x0f, w0
; if (GetCtlCRC == (ctl & 0x0f))
	and	w1, #0x0f, w2
	cp.b	w0, w2
	bra	nz, ResetRxD

	btst	w1, #bCTL_4x
	bra	z, s3_nx
	mov	RxD_len, w0
	sl	w0, #2, w0
	mov	w0, RxD_len
	mov	#MAX_RS_BUFFER, w2
	sub	w0, w2, w2
	bra	leu, s3_nx
	mov	#RSERR_BUFLEN, w0	/* RxD_Err = RSERR_BUFLEN; */
	mov	w0, RxD_Err
s3_nx:
	mov	w1, w0
.else
	mov	w1, w3
	mov	w1, w2
	mov	RxD_len, w0
	mov	RxD_Bef, w1
	rcall	GetCtlCRC
	and	w3, #0x0f, w1
	cp.b	w0, w1
	bra	nz, ResetRxD
	btst	w3, #bCTL_4x
	bra	z, s3_nx
	mov	RxD_len, w0
	sl	w0, #2, w0
	mov	w0, RxD_len
	mov	#MAX_RS_BUFFER, w1
	sub	w0, w1, w1
	bra	leu, s3_nx
	mov	#RSERR_BUFLEN, w0	/* RxD_Err = RSERR_BUFLEN; */
	mov	w0, RxD_Err
s3_nx:
	mov	w3, w0
.endif
	add	RxD_sum
	inc	RxD_Step
	bra	RxDLoop
StepSync:
	rcall	px02_Ok
	clr	TxD_0		;Clr  1.TxD
	return
StepSum:
	mov	RxD_sum, w0
	cp.b	w0, w1		;Chk CRC
	bra	nz, SendErr_CRC

	mov	RxD_Bef, w0
	cp.b	w0, #RSBEF_SYNC
	bra	z, StepSync

	mov	RxD_ctl, w0	;Test tgl Ok
	and	w0, #CTL_TOGGLE, w0
	cp0	TxD_0		;Test 1.TxD
	bra	z, 20f
	cp	RxD_tgl
	bra	nz, 20f
	rcall	ReStart_UART_dbg_TxD
	bra	ResetRxD
20:
	mov	w0, RxD_tgl
	mov	RxD_Bef, w0
	cp	w0, #31
.if USE_DEBUG_UART_USER
	bra	leu, 40f
	rcall	__rs232ald_user
	bra	ResetRxD
40:
.else
	bra	gtu, SendErr_UNKNOWN_BEF
.endif
	add	w0, w0, w0
	mov	#RxDproc, w1
	mov	[w0+w1], w0
	cp0	w0
	bra	z, SendErr_UNKNOWN_BEF
	call	w0

ResetRxD:
	clr	RxD_Step
	clr	RxD_Err
RetRxD:
	return


.if USE_DEBUG_UART_BRIDGE
ChkBridgeMod:
	mov.d	w0, [w15++]
	mov	BridgeMod, w0
	mov	#BRIDGE_RUN, w1
	cp	w0, w1
	bra	z, 90f

	mov	#BRIDGE_OPEN, w1
	cp	w0, w1
	bra	nz, 10f
	mov	#BRIDGE_RUN, w1
	mov	w1, BridgeMod
	mov.d	[--w15], w0
	rcall	noBridgeMod
	clr	TxD_0		;Clr  1.TxD
	rcall	OpenUART_dbg_bridge
	retlw	#1, w0

10:  /* BRIDGE_CL0SE */
	clr	BridgeMod
	rcall	CloseUART_dbg_bridge
	mov.d	[--w15], w0
	bra	noBridgeMod
90:
	mov.d	[--w15], w0
	retlw	#0, w0
.endif

/* W0=Bef W1=len */
rs232ald_TxD:
.if USE_DEBUG_UART_BRIDGE
	cp0	BridgeMod
	bra	nz, ChkBridgeMod
noBridgeMod:
.endif
	clr	RxD_Step
	clr	RxD_Err
	mov	#MAX_RS_BUFFER - 4, w2
	cp	w1, w2			/* Test max. len */
	bra	gtu, TxD_ret_false

	setm	TxD_0

	mov	#TxD_dbgbuf, w3
	mov.b	w0, [w3++]		/* Bef */
	mov.b	w1, [w3++]		/* len */

	mov	RxD_tgl, w2
	mov.b	w2, [w3]
.if 0
	rcall	GetCtlCRC
.else
; return ((bef + dlen) ^ ~((bef + dlen + (ctl & 0xf0)) >> 4)) & 0x0f;
	add	w1, w0, w1
	and	#0xf0, w2
	add	w1, w2, w2
	lsr	w2, #0x4, w2
	com	w2, w2
	xor	w1, w2, w1
	and	w1, #0xf, w0
.endif
	ior.b	w0, [w3], [w3--]	/* ctl */
	mov	#2, w0
	add.b	w0, [w3--], w0
	mov	#0x39, w1

	repeat	w0
	add.b	w1, [w3++], w1
	mov.b	w1, [w3++]
	inc	w0, w0
	rcall	Start_UART_dbg_TxD
	retlw	#1, w0

TxD_ret_false:
	retlw	#0, w0

/* ---------------------- ERROR ----------------------------------------- */
Send_BRIDGE_MODE:
.if USE_DEBUG_UART_BRIDGE
	bra	px02_Ok
.else
	mov	#RSERR_NO_BRIDGE_MODE, w0
	bra	SendErr
.endif

SendErr_READ_ONLY:
	mov	#RSERR_READ_ONLY, w0
	bra	SendErr
SendErr_LEN_OVERFLOW:
	mov	#RSERR_LEN_OVERFLOW, w0
	bra	SendErr
SendErr_ADDR_FAULT:
	mov	#RSERR_ADDR_FAULT, w0
	bra	SendErr
SendErr_BEF_SIZE:
	mov	RxD_len, w0
	swap	w0
	mov.b	#RSERR_BEF_SIZE, w0
	bra	SendErr2
SendErr_UNKNOWN_BEF:
	mov	#RSERR_UNKNOWN_BEF, w0
	bra	SendErr
SendErr_BUFLEN:
	mov	#RSERR_BUFLEN, w0
	bra	SendErr
SendErr_CRC:
	mov	#RSERR_CRC, w0

/* w0=Error Nr */
SendErr:
	mov	#RS232ALD_TxD_Data, w1
	mov.b	w0, [w1]
	mov	#RSBEF_ERROR, w0
	mov	#1, w1
	bra	rs232ald_TxD

/* w0=Error Nr */
SendErr2:
	mov	#RS232ALD_TxD_Data, w1
	mov.b	w0, [w1++]
	swap	w0
	mov.b	w0, [w1]

	mov	#RSBEF_ERROR, w0
	mov	#2, w1
	bra	rs232ald_TxD

/* ---------------------- Send Data ------------------------------------- */
px02_Ok:
	mov	#RSBEF_OK, w0
	clr	w1
	bra	rs232ald_TxD

px04_Info:
	mov	RxD_len, w0
	cp	w0, #1
	bra	nz, SendErr_UNKNOWN_BEF
	cp0.b	RxD_dbgbuf
	bra	nz, SendErr_UNKNOWN_BEF
	mov	#RS232ALD_TxD_Data, w3
	mov	#Info0, w0
.if USE_UART_I2C_BRIDGE
	mov	#16, w2
.else
	mov	#14, w2
.endif
	rcall	pnmove
	rcall	GetDevID
	mov	#RS232ALD_TxD_Data + 2, w2
	mov.b	w0,[w2++]
	swap	w0
	mov.b	w0,[w2++]
	mov.b	w1,[w2++]
	swap	w1
	mov.b	w1,[w2]
	mov	#RSBEF_GET_INFO, w0
.if USE_UART_I2C_BRIDGE
	mov	#16, w1
.else
	mov	#14, w1
.endif
	bra	rs232ald_TxD

px08_RdData:
	mov	RxD_dbgbuf + 2, w2
	mov	RxD_dbgbuf + 4, w3
	ze	w3, w3

	mov	RxD_len, w0
	cp	w0, #5		/* addr = 4Byte */
	bra	z, 10f
	cp	w0, #4		/* addr = 3Byte */
	bra	nz, SendErr_BEF_SIZE
	ze	w2, w2
	mov.b	RxD_dbgbuf + 3, WREG
	mov	w0, w3
10:
	cp0	w2		/* TODO addr > 0xffff */
	bra	nz, SendErr_ADDR_FAULT
	mov	#MAX_RS_BUFFER - 4, w0
	cp	w3, w0
	bra	gtu, SendErr_LEN_OVERFLOW

	mov	RxD_dbgbuf, w0
	mov	w3, w2
	push	w3
	mov	#RS232ALD_TxD_Data, w3
	rcall	pnmove
	pop	w1
	mov	#RSBEF_READ_DATA, w0
	bra	rs232ald_TxD

px0A_RdData:
.if HAS_VIDEO_BUFFER
	mov	#RxD_dbgbuf, w3
	ze	[w3++], w0
	cp	w0, #0x07		/* crt ram Grafik */
	bra	nz, SendErr_UNKNOWN_BEF

	mov.b	[w3++], w0
	swap	w0
	mov.b	[w3++], w0
	swap	w0

	ze	[w3++], w1
	mov	RxD_len, w2
	cp	w2, #5		/* addr = 3Byte */
	bra	z, px0A_rl
	cp	w2, #6		/* addr = 4Byte */
	bra	nz, SendErr_BEF_SIZE
	swap	w1
	mov.b	[w3++], w1
	swap	w1
px0A_rl:
	mov	#Video_buffer, w2
	add	w0, w2, w0
	addc	w1, #0, w1

	cp0	w1
	bra	nz, SendErr_ADDR_FAULT
	ze	[w3++], w2
	mov	#MAX_RS_BUFFER - 2, w3
	cp	w2, w3
	bra	gtu, SendErr_LEN_OVERFLOW
	push	w2
	mov	#RS232ALD_TxD_Data, w3
	rcall	pnmove
	pop	w1
	mov	#RSBEF_TRD_DATA, w0
	bra	rs232ald_TxD
.else
	bra	SendErr_UNKNOWN_BEF
.endif

px0C_WrData:
	mov	RxD_len, w0
	cp	w0, #4		/* addr = 4Byte */
	bra	ltu, SendErr_BEF_SIZE
	bra	z, px02_Ok
	mov	RxD_dbgbuf, w0
	mov	RxD_dbgbuf + 2, w3
	cp0	w3		/* TODO addr > 0xffff */
	bra	nz, SendErr_ADDR_FAULT
	mov	#0x45, w1
	sub	w0, w1, w1
	bra	leu, SendErr_READ_ONLY
	mov	#MAX_RAM, w1
	sub	w0, w1, w1
	bra	geu, SendErr_READ_ONLY
	mov	w0, w3
	mov	#RxD_dbgbuf + 4, w0
	mov	RxD_len, w2
	sub	w2, #4, w2
	rcall	pnmove
	bra	px02_Ok

px0D_Bridge:
.if USE_DEBUG_UART_BRIDGE
	mov	RxD_len, w0
	cp	w0, #2		/* 0xf200 */
	bra	nz, 10f

	mov	RxD_dbgbuf, w0
	mov	#BRIDGE_OPEN, w1
	cp	w0, w1
	bra	nz, 10f
	mov	w1, BridgeMod
	bra	Send_BRIDGE_MODE
10:
	cp0	BridgeMod
	bra	z, 20f
	mov	#BRIDGE_CL0SE, w1
	mov	w1, BridgeMod
20:
.endif
	bra	Send_BRIDGE_MODE


.if USE_UART_I2C_BRIDGE
I2C_BRIDGE:
	mov	RxD_len, w0
	cp	w0, #6
	bra	lt, SendErr_BEF_SIZE

	mov	#RxD_dbgbuf, w2
	ze	[w2++], w3
;	mov	w1, uart_i2c2_cmd
	ze	[w2++], w1
	mov	w1, uart_i2c2_Flag
	ze	[w2++], w1
	mov	w1, uart_i2c2_id
	mov.b	[w2++], w1
	swap	w1
	mov.b	[w2++], w1
	swap	w1
	mov	w1, uart_i2c2_adr
	ze	[w2++], w1
	mov	w1, uart_i2c2_DataSize

	cp	w3, #I2C_WRITE
	bra	z, Bridge_I2C_WR
	cp	w3, #I2C_READ
	bra	z, Bridge_I2C_RD
	bra	SendErr_UNKNOWN_BEF

Bridge_I2C_WR:
	cp	w0, #I2C_MAX_BUF + 6
	bra	gt, SendErr_BEF_SIZE
	mov	uart_i2c2_DataSize, w1
	cp0	w1
	bra	z, SendErr_BEF_SIZE
	cp	w1, #I2C_MAX_BUF
	bra	gt, SendErr_BEF_SIZE
	add	w1, #6, w1
	cp	w0, w1
	bra	nz, SendErr_BEF_SIZE
; bra	px02_Ok
	mov	w3, uart_i2c2_cmd
	mov	#uart_i2c2_Buf, w3
	mov	w2, w0
	mov	uart_i2c2_DataSize, w2
	rcall	pnmove
	return

Bridge_I2C_RD:
	cp	w0, #6
	bra	nz, SendErr_BEF_SIZE
	mov	w3, uart_i2c2_cmd
	return

UART_I2C_Bridge_Handle:
	cp0	i2c2_Lock
	bra	nz, lend

	mov	#I2C2_UART, w0
	mov	w0, i2c2_Lock

	mov	uart_i2c2_Flag, w0
	mov	w0, i2c2_Flag
	mov	uart_i2c2_id, w0
	mov	w0, i2c2_id
	mov	uart_i2c2_adr, w0
	mov	w0, i2c2_adr
	mov	uart_i2c2_DataSize, w2
	mov	w2, i2c2_DataSize

	mov	uart_i2c2_cmd, w0
	mov	w0, i2c2_cmd
	cp	w0, #I2C_WRITE
	bra	nz, lend
	mov	#i2c2_Buf, w3
	mov	#uart_i2c2_Buf, w0
	rcall	pnmove
lend:
	return

I2C2_UART_run:
	mov	i2c2_cmd, w0
	cp0	w0
	bra	nz, chk_err

	mov	uart_i2c2_cmd, w0
	cp	w0, #I2C_WRITE
	bra	z, ret_send_cmd

	clr	i2c2_Lock

	mov	#RS232ALD_TxD_Data, w0
	mov	uart_i2c2_cmd, w1
	clr	uart_i2c2_cmd

	mov.b	w1, [w0++]
	mov	uart_i2c2_id, w1
	mov.b	w1, [w0++]

	mov	w0, w3
	mov	#i2c2_Buf, w0
	mov	i2c2_DataSize, w2
	rcall	pnmove
	mov	#0x40, w0
	mov	i2c2_DataSize, w1
	inc2	w1, w1
	bra	rs232ald_TxD

chk_err:
	cp	w0, #I2C_ERROR
	bra	z, ret_send_cmd
	return			/* Wait */

ret_send_cmd:
	clr	i2c2_Lock
	clr	uart_i2c2_cmd
	mov	#RS232ALD_TxD_Data, w0
	mov	i2c2_cmd, w1
	mov.b	w1, [w0++]
	mov	uart_i2c2_id, w1
	mov.b	w1, [w0]
	mov	#0x40, w0
	mov	#2, w1
	bra	rs232ald_TxD

.else
  I2C_BRIDGE	= 0 
.endif

.section .const, psv
RxDproc:
	.dc	0,		0,		px02_Ok,	0
	.dc	px04_Info,	0,		0,		0
	.dc	px08_RdData,	0,		px0A_RdData,	0
	.dc	px0C_WrData,	px0D_Bridge,	0,		0
	.dc	I2C_BRIDGE,	0,		0,		0
	.dc	0,		0,		0,		0
	.dc	0,		0,		0,		0
	.dc	0,		0,		0,		0
Info0:
 .word	(CPU_FACTORY << 8), 	0,	0,	INFO_DataArt
 .word	0x0108,		128,		64
 .word	INFO0_EX
