/*
 Copyright (C) 2014-2020 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "libm.inc"

.section .text
.global f48_add
.global f48_sub

/* -------------------- f48_add ---------------------------
Input:  w2=Fx Pointer  +  w3=Fx Pointer  ==>  w4=Fx Pointer
Output: 				Error SR set #OV
used; w0:w1:w5:w6:w7			94 Clk  (MAC 82 Clk)
------------------------------------------------------------ */
f48_add:
	mov	[w3+4], w5
f48_addsub:
/* ------------------------------------------------------------
op1 w0:w1  exp=w10
op2 w8:w9  exp=w5
------------------------------------------------------------ */
	push.d	w8
	push.d	w10
	mov.d	[w3], w8


	mov.d	[w2], w0
	mov	[w2+4], w10


	btst	w9, #15			; te op2 0
	bra	z, f48res_op1

	btst	w1, #15			; te op1 0
	bra	z, f48res_op2

	mov	#0x7FFF, w6
	and	w10, w6, w11		; w11 exp1
	and	w5, w6, w7		; w7  exp2
.if F48_CHK_OVERFLOW
	cp	w11, w6			; te op1 error
	bra	z, f48res_op1
	cp	w7, w6			; te op2 error
	bra	z, f48res_op2
.endif

	sub	w11, w7, w6		; exp1 - exp2
	bra	z, f48add_tn
	bra	nn, f48add_sh_op2
	mov	w7, w11			; save exp op2
	subr	w6, #0, w6		; exp2 - exp1
	cp	w6, #31
	bra	gt, f48res_op2
	cp	w6, #16
	bra	ge, f48add_op1_sh16

	subr	w6, #16, w7
	sl	w1, w7, w7
	lsr	w0, w6, w0
	lsr	w1, w6, w1
	ior	w0, w7, w0
	bra	f48add_tn

f48add_op1_sh16:
	mov	w1, w0
	mov	#0, w1
	and	w6, #0x0f, w6
;	bra	z, f48add_tn
	lsr	w0, w6, w0
	bra	f48add_tn

f48add_sh_op2:
	cp	w6, #31
	bra	gt, f48res_op1
	cp	w6, #16
	bra	lt, f48add_op2_sh
	mov	w9, w8			; lsr 16
	mov	#0, w9
	and	w6, #0x0f, w6
	lsr	w8, w6, w8
	bra	f48add_tn

f48add_op2_sh:
	subr	w6, #16, w7
	sl	w9, w7, w7
	lsr	w8, w6, w8
	lsr	w9, w6, w9
	ior	w8, w7, w8
f48add_tn:
	mul.uu	w6, #0, w6		; clr w6:w7
	btst	w10, #15		; te op1 pos
	bra	z, f48add_op1_pos
	subr	w0, #0, w0		; -op1 w0:w1:w6
	subbr	w1, #0, w1
	subbr.b	w6, #0, w6
f48add_op1_pos:
	btst	w5, #15			; te op2 pos
	bra	z, f48add_op2_pos
	subr	w8, #0, w8		; -op2 w8:w9:w7
	subbr	w9, #0, w9
	subbr.b	w7, #0, w7
f48add_op2_pos:
	add	w0, w8, w0		; op1 + op2 -> op1 w0:w1:w7
	addc	w1, w9, w1
	addc.b	w6, w7, w7		; w7L=ov+sig w7H=0

	mov	w11, w5
	pop.d	w10

	btst	w7, #7
	bra	z, f48add_res_pos
	subr	w0, #0, w0
	subbr	w1, #0, w1
	subbr	w7, #0, w7		; w7L=ov w7H=sig
f48add_res_pos:
; printf("w7=0x%04x w0=0x%04x%04x w5=0x%04x\n", w7, w1, w0, w5);
	cp0.b	w7
	bra	nz, f48add_ov
	cp0	w1
.if 1
	bra	z, f48add_sh_w0
	ff1l    w1, w6
; printf("w0=0x%04x%04x w6=0x%04x w5=0x%04x\n", w1, w0, w6, w5);
	dec	w6, w6
	bra	z, f48add_ok		; jp shift 0
	sub	w5, w6, w5		; exp - shift
	subr	w6, #16, w8
	lsr	w0, w8, w8
	sl	w0, w6, w0
	sl	w1, w6, w1
	ior	w1, w8, w1
f48add_ok:
	pop.d	w8
.if F48_CHK_OVERFLOW
	mov	#0x7FFF, w6
	cp	w5, w6
	bra	geu, f48res_INF_pw4
.endif
	mov.d	w0, [w4]
	btsc	w7, #15
	bset	w5, #15
	mov	w5, [w4+4]
	return

f48add_sh_w0:
	cp0	w0
	bra	z, f48add_0		; test res 0
	sub	#16, w5			; exp - 16
	exch	w0, w1

	ff1l    w1, w6
	dec	w6, w6
	sub	w5, w6, w5		; exp - shift
	sl	w1, w6, w1

	pop.d	w8
.if F48_CHK_OVERFLOW
	mov	#0x7FFF, w6
	cp	w5, w6
	bra	geu, f48res_INF_pw4
.endif
	mov.d	w0, [w4]
	btsc	w7, #15
	bset	w5, #15
	mov	w5, [w4+4]
	return


.else
	bra	nz, f48add_sh_w1
	cp0	w0
	bra	z, f48add_0		; test res 0
	sub	#16, w5			; exp - 16
	exch	w0, w1
f48add_sh_w1:
	ff1l    w1, w6
; printf("w0=0x%04x%04x w6=0x%04x w5=0x%04x\n", w1, w0, w6, w5);
	dec	w6, w6
	bra	z, f48add_ok		; jp shift 0
	sub	w5, w6, w5		; exp - shift
	subr	w6, #16, w8
	lsr	w0, w8, w8
	sl	w0, w6, w0
	sl	w1, w6, w1
	ior	w1, w8, w1
f48add_ok:
	pop.d	w8
.if F48_CHK_OVERFLOW
	mov	#0x7FFF, w6
	cp	w5, w6
	bra	geu, f48res_INF_pw4
.endif
	mov.d	w0, [w4]
	btsc	w7, #15
	bset	w5, #15
	mov	w5, [w4+4]
	return
.endif
f48add_ov:
	lsr.b	w7, w7			; w7L=ov
	rrc	w1, w1
	rrc	w0, w0
	inc	w5, w5
; printf("w7=0x%04x w0=0x%04x%04x w5=0x%04x\n", w7, w1, w0, w5);
.if 1
	pop.d	w8
.if F48_CHK_OVERFLOW
	mov	#0x7FFF, w6
	cp	w5, w6
	bra	geu, f48res_INF_pw4
.endif
	mov.d	w0, [w4]
	btsc	w7, #15
	bset	w5, #15
	mov	w5, [w4+4]
	return
.else
	bra	f48add_ok
.endif
f48add_0:
	pop.d	w8
	bra	f48res_0_pw4

f48res_op1:
	mov.d	w0, [w4]
	mov	w10, [w4+4]
	pop.d	w10
	pop.d	w8
	bra	f48res_ChkExp_OV_pw4

f48res_op2:
	mov.d	w8, [w4]
	mov	w5, [w4+4]
	pop.d	w10
	pop.d	w8
	bra	f48res_ChkExp_OV_pw4

/* -------------------- f48_sub ---------------------------
Input:  w2=Fx Pointer  +  w3=Fx Pointer  ==>  w4=Fx Pointer
Output: 				Error SR set #OV
used; w0:w1:w5:w6:w7			99 Clk  (MAC 87 Clk)
------------------------------------------------------------ */
f48_sub:
	mov	[w3+4], w5
	btg	w5, #15
	bra	f48_addsub
