/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "libm.inc"

.section .text
.global f48_div

/* -------------------- f48_div -------------------------------
Input:  w2=Fx Pointer  /  w3=Fx Pointer  ==>  w4=Fx Pointer
Output: 			Error SR set #OV
used:   w0:w1:w5:w6:w7		ca. 460 Clk
------------------------------------------------------------ */
f48_div:
	push.d	w8
	push.d	w2
	mov	#0x7FFF, w5
	mov	[w2+4], w0
	mov	[w3+4], w1
	xor	w0, w1, w9		; set sig
	push	w9

	and	w0, w5, w0		; w0 exp1
	and	w1, w5, w1		; w1 exp2
.if F48_CHK_OVERFLOW
	cp	w0, w5			; te op1 error
	bra	z, f48div_NAN
	cp	w1, w5			; te op2 error
	bra	z, f48div_NAN
.endif
	sub	w0, w1, w5
	mov	#0x3FFF, w0
	add	w5, w0, w5

	mov.d	[w3], w8
	ior	w8, w9, w6		; te div x/0
	bra	z, f48div_NAN

	mov.d	[w2], w6
	btst	w7, #15			; te div 0/x
	bra	z, f48div_0

	mov	#0x0000,w1	; w8:w1   res 0
	mov	#0x0003,w0

	sub	w6, w8, w2	; w2:w3  op1 - op2 -> tmp
	subb	w7, w9, w3
	bra	n, f48div_op2_gt
	lsr	w9, w9		; op2 >> 1
	rrc	w8, w8
; addc	w8, #0, w8
; addc	w9, #0, w9

	lsr	w7, w7		; op1 >> 1
	rrc	w6, w6
; addc	w6, #0, w6
; addc	w7, #0, w7
	sub	w6, w8, w6	; op1 - op2 -> op1
	subb	w7, w9, w7
	bra	f48div_loop

f48div_op2_gt:
	lsr	w9, w9		; op2 >> 1
	rrc	w8, w8
; addc w8, #0, w8
; addc w9, #0, w9
	dec	w5, w5

	sub	w6, w8, w6	; op1 - op2 -> op1
	subb	w7, w9, w7
	bra	f48div_loop

f48div_next:
	sl	w0, w0		; w0:w1   res << 1
	rlc	w1, w1
	bra     c, f48div_end
f48div_loop:
	sl	w6, w6		; w6:w7 op1 << 1
	rlc	w7, w7
;f48div_entry:
	sub	w6, w8, w2	; w2:w3  op1 - op2 -> tmp
	subb	w7, w9, w3
	bra	nc, f48div_next
	mov.d	w2, w6		; tmp -> op1
	rlc	w0, w0		; w0:w1   (res << 1) + C
	rlc	w1, w1
	bra	nc, f48div_loop
f48div_end:
.if 0    /* round */
	add	w0, #1, w2
	addc	w1, #0, w3
	ior	w2, w3, w2
	bra	z, f48div_nr
	sl	w6, w6		; w6:w7 op1 << 1
	rlc	w7, w7
	sub	w6, w8, w2	; w2:w3  op1 - op2 -> tmp
	subb	w7, w9, w3
	bra	nc, f48div_nr
	addc	w0, #0, w0
	addc	w1, #0, w1
f48div_nr:
.endif
	pop	w7
	pop.d	w2

	pop.d	w8

.if F48_CHK_OVERFLOW
	mov	#0x7FFF, w6
	cp	w5, w6
	bra	geu, f48res_INF_pw4
.endif
	mov.d	w0, [w4]
	btsc	w7, #15
	bset	w5, #15
	mov	w5, [w4+4]
	return

f48div_NAN:
.if 1
	pop	w7
	pop.d	w2
	pop.d	w8
	bra	f48res_NAN_pw4
.else
	mul.uu	w0, #0, w0
	mov	#0x7FFF, w5
	mov.d	w0, [w4]
	mov	w5, [w4+4]
	bset	SR, #OV

	pop	w7
	pop.d	w2
	pop.d	w8
	return
.endif
f48div_0:
	pop	w7
	pop.d	w2
	pop.d	w8
	bra	f48res_0_pw4
