/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "libm.inc"

.section .text
.global f48_div3216

/* -------------------- f48_div3216 ---------------------------
Input:  w2=Fx Pointer  /  w3=Fx Pointer  ==>  w4=Fx Pointer
Output: 			Error SR set #OV
used:   w0:w1:w5:w6:w7		ca. 87 Clk
------------------------------------------------------------ */

f48_div3216:
	mov	[w2+4], w0
	mov	[w3+4], w1
	xor	w0, w1, [w15++]		; set sig

	mov	#0x7FFF, w5
	and	w0, w5, w0		; w0 exp1
	and	w1, w5, w1		; w1 exp2
.if F48_CHK_OVERFLOW
	cp	w0, w5			; te op1 error
	bra	z, f48div3216_NAN
	cp	w1, w5			; te op2 error
	bra	z, f48div3216_NAN
.endif
	sub	w0, w1, w5
	mov	#0x3FFF, w0
	add	w5, w0, w5

	mov.d	[w2], w0
	btst	w1, #15			; te div 0/x
	bra	z, f48div3216_0
	lsr	w1, w1
	rrc	w0, w0

	mov	[w3+2], w7
	btst	w7, #15			; te div x/0
	bra	z, f48div3216_NAN
.if 1
	sl	[w3], w6	/* round */
	addc	w7, #0, w7
	bra	c, f48div3216_Round_OV
.endif
f48div3216_nr:
	clr	w6
	repeat	#17			; div 16/16
	div.ud	w0, w7
	exch	w0, w6
	repeat	#17
	div.ud	w0, w7

	pop	w7

	btst	w6, #15
	bra	nz, f48div3216_nsh
	dec	w5, w5
	sl	w0, w0
	rlc	w6, w6
f48div3216_nsh:
.if F48_CHK_OVERFLOW
	mov	#0x7FFF, w1
	cp	w5, w1
	bra	geu, f48res_INF_pw4
.endif
	mov	w0, [w4]
	mov	w6, [w4+2]
	btsc	w7, #15
	bset	w5, #15
	mov	w5, [w4+4]
	return

f48div3216_Round_OV:
	mov	#0x8000, w7
	dec	w5, w5
	bra	f48div3216_nr

f48div3216_NAN:
	pop	w7
	bra	f48res_NAN_pw4

f48div3216_0:
	pop	w0
	bra	f48res_0_pw4

