/*
 Copyright (C) 2014-2020 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "libm.inc"

.section .text
.global f48_log2_17
.global f48_log2_17_base

/* -------------------- f48_log2_17   only 17Bit --------------
Input:  w2=Fx Pointer  log2(pw2)  ==>  w4=Fx Pointer
Output: 			Error SR set #OV
used:   w0:w1:w5:w6:w7		ca. 305 Clk
------------------------------------------------------------ */

f48_log2_17_base:
	mov	#0x7FFF, w3
	mov	[w2+4], w5
	btst	w5, #15
	bra	nz, f48log2_NAN
	and	w5, w3, w5		; w0 exp1
.if F48_CHK_OVERFLOW
	cp	w3, w5			; te op1 error
	bra	z, f48log2_NAN
.endif
	mov.d	[w2], w6
	ior	w6, w7, w3
	bra	z, f48log2_INF_neg

	mov	#0x3FFF, w3
	sub	w5, w3, w5

  /* log2 1.17 */
	clr	w0
	sl	w5, w1
	mov	#0x8000, w5

	mul.uu	w6, w7, w2	/* valL * valH */
	mul.uu	w7, w7, w6	/* valH * valH */
	add	w3, w6, w6
	addc	w7, #0, w7
	add	w3, w6, w6
	addc	w7, #0, w7

	bra	n, ul_b17
	sl	w6, w6
	rlc	w7, w7
	bra	ul_st
ul_b17:
	bset	w1, #0
	bra	ul_st

ul_c:
	ior	w0, w5, w0
	lsr	w5, w5
	bra	c, ul_e
ul_st:
	mul.uu	w6, w7, w2	/* valL * valH */
	mul.uu	w7, w7, w6	/* valH * valH */
	add	w3, w6, w6
	addc	w7, #0, w7
	add	w3, w6, w6
	addc	w7, #0, w7

	bra	n, ul_c
	sl	w6, w6
	rlc	w7, w7
	lsr	w5, w5
	bra	nc, ul_st
ul_e:
	mov	#0x400d, w5		; set exp
.if 1
  /* round */
	mul.uu	w7, w7, w6	/* val * val */
	sl	w7, w3
	addc	w0, #0, w0
	addc	w1, #0, w1
.endif
	return



f48log2_NAN:
	bra	f48res_NAN_pw4

f48log2_INF_neg:
	mov	#0xFFFF, w5
	mov	#0x0000, w0
	mov	#0x8000, w1
	mov.d	w0, [w4]
	mov	w5, [w4+4]
	bset	SR, #OV
	return


f48_log2_17:
	push.d	w8
	push.d	w2
	rcall	f48_log2_17_base
	bra	ov, f48log2_end

	ior	w0, w1, w3
	bra	z, f48log2_0

	btst	w1, #15			; test negative
	bra	z, f48_log2_noneg
	bset    w5, #15			; set negative
	subr    w0, #0, w0
	subbr   w1, #0, w1
f48_log2_noneg:
	ff1l    w1, w3
	bra     nc, f48_log2_sh		; jp w0 > 0
	sub	#16, w5			; exp - 16
	exch	w0, w1
	ff1l    w1, w3
f48_log2_sh:
	dec	w3, w3
	bra	z, f48_log2_no_sh	; jp shift 0
	sub	w5, w3, w5		; exp - shift
	subr	w3, #16, w2
	lsr	w0, w2, w2
	sl	w0, w3, w0
	sl	w1, w3, w1
	ior	w1, w2, w1

f48_log2_no_sh:
	mov.d	w0, [w4];
	mov	w5, [w4+4];

f48log2_end:
	pop.d	w2
	pop.d	w8
	return

f48log2_0:
	mul.uu	w0, #0, w0
	mov.d	w0, [w4]
	mov	w0, [w4+4]
	pop.d	w2
	pop.d	w8
	return
