/*
 Copyright (C) 2014-2020 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "libm.inc"

.section .text
.global f48_mul

/* -------------------- f48_mul -------------------------------
Input:  w2=Fx Pointer  -  w3=Fx Pointer  ==>  w4=Fx Pointer
Output: 				Error SR set #OV
used:   w0:w1:w5:w6:w7			60 Clk
------------------------------------------------------------ */

f48_mul:
;        w2l*w3l
;    w2h*w3l
;    w2l*w3h
;w2h*w3h
	push.d	w2

	clr	w5
	mov	[w2], w0
	mul.uu	w0, [w3], w0		;w2l*w3l

	mov	[w2+2], w6
	mul.uu	w6, [w3++], w6		;w2h*w3l
	sl	w0, w0
	addc	w1, w6, w6
	addc	w7, #0, w7
;	addc	w5, #0, w5

	mov	[w3], w0
	mul.uu	w0, [w2++], w0		;w2l*w3h
	add	w0, w6, [w15++]		;push low
	addc	w1, w7, w7
	addc	w5, #0, w5

	mov	[w2++], w0
	mul.uu	w0, [w3++], w0		;w2h*w3h
	add	w7, w0, w0
	addc	w5, w1, w1

	mov	#0x7FFF, w6
	and	w6, [w2], w5		; exp1
	and	w6, [w3], w7		; exp2
.if F48_CHK_OVERFLOW
	cp	w5, w6			; te op1 error
	bra	z, f48mul_NAN
	cp	w7, w6			; te op2 error
	bra	z, f48mul_NAN
.endif
	ior	w0, w1, w6	;test 0
	bra	z, f48mul_0

	add	w5, w7, w5
	mov	#0x3FFE, w7
	sub	w5, w7, w5

	mov	[w2], w7
	xor	w7, [w3], w7		;set sig

	pop	w6			;pop low
	pop.d	w2

	btst	w1, #15
	bra	nz, f48mul_nsh
	sl	w6, w6
	rlc	w0, w0
	rlc	w1, w1
	dec	w5, w5
f48mul_nsh:
.if F48_CHK_OVERFLOW
	mov	#0x7FFF, w6
	cp	w5, w6
	bra	geu, f48res_INF_pw4
.endif
	mov.d	w0, [w4]
	btsc	w7, #15
	bset	w5, #15
	mov	w5, [w4+4]
	return

f48mul_0:
	mul.uu	w0, #0, w0
	mov.d	w0, [w4]
	mov	w0, [w4+4]
	pop	w6
	pop.d	w2
	return

.if F48_CHK_OVERFLOW
f48mul_NAN:
	pop	w6
	pop.d	w2
	bra	f48res_NAN_pw4
.endif
