/*
 Copyright (C) 2014-2020 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "libm.inc"

.section .text
.global f48_sq

/* -------------------- f48_sq -------------------------------
Input:  w2=Fx Pointer  *  w2=Fx Pointer  ==>  w4=Fx Pointer
Output: 				Error SR set #OV
used:   w0:w1:w5:w6:w7			45 Clk
------------------------------------------------------------ */

f48_sq:
;        w2l*w3l
;    w2h*w3l
;    w2l*w3h
;w2h*w3h

	mov	[w2], w0
	mul.uu	w0, w0, w0		;w2l*w2l

	clr	w5
	mov	[w2+2], w0
	mul.uu	w0, [w2], w6		;w2h*w2l
	sl	w6, w6
	rlc	w7, w7
	rlc	w5, w5			;w2h*w2l * 2

	add	w6, w1, w6
	addc	w7, #0, w7
	addc	w5, #0, w5
; printf("w5=0x%04x w6=0x%04x%04x\n", w5, w7, w6);

	mul.uu	w0, w0, w0		;w2h*w2h
	add	w7, w0, w0
	addc	w5, w1, w1

	mov	[w2+4], w5
	mov	#0x7FFF, w7
	and	w5, w7, w5		;exp
#if F48_CHK_OVERFLOW
	cp	w5, w7			;te op1 error
	bra	z, f48mul_NAN
#endif
	lsr	w1, #14, w7		;test 0
	bra	z, f48res_0_pw4

	add	w5, w5, w5
	mov	#0x3FFE, w7
	sub	w5, w7, w5
#if F48_CHK_OVERFLOW
	mov	#0x7FFF, w7
	cp	w5, w7
	bra	geu, f48res_INF_pw4
#endif

	btst	w1, #15
	bra	nz, f48mul_nsh
	sl	w6, w6
	rlc	w0, [w4]
	rlc	w1, w1
	dec	w5, w5
	mov	w1, [w4+2]
	mov	w5, [w4+4]
	return
f48mul_nsh:
	mov.d	w0, [w4]
	mov	w5, [w4+4]
	return

#if F48_CHK_OVERFLOW
f48mul_NAN:
	mov.d	[w2], w0
	mov	[w2+4], w5
	mov.d	w0, [w4]
	mov	w5, [w4+4]
	bset	SR, #OV
	return
#endif

