/*
 Copyright (C) 2014-2020 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "libm.inc"

.section .text
.global f48_sqrt

/* -------------------- f48_sqrt ------------------------------
Input:  w2=Fx Pointer  sqrt  ==>  w4=Fx Pointer
Output: 			Error SR set #OV
used:   w0:w1:w5:w6:w7		ca. 500 Clk
------------------------------------------------------------ */

f48_sqrt:
	mov	[w2+4], w5
	btst	w5, #15			; test neg jp NAN
	bra	nz, f48res_NAN_pw4
.if F48_CHK_OVERFLOW
	mov	#0x7FFF, w6
	cp	w5, w6			; te op error
	bra	z, f48res_NAN_pw4
.endif

	mov.d	[w2], w0
	btst	w1, #15			; te op 0
	bra	z, f48res_0_pw4

	push.d	w8
	push.d	w10

	mov	#0x2000, w7	/* 30 bits */
	mul.uu	w8,#0,w8 	/* w8:w9 result */
	lsr	w1, w1
	rrc	w0, w0

	btst	w5, #0
	bra	z, f48_sqrt_entry
	lsr	w1, w1
	rrc	w0, w0
	inc	w5, w5
	bra	f48_sqrt_entry

f48_sqrt_qnext:
	lsr	w7, w7			; w6:w7  bit >>= 1
	bra	c, f48_sqrt_end		; C = 1 = bits 0
f48_sqrt_loop:
	sl	w0, w0			; w0:w1 r * 2
	rlc	w1, w1
f48_sqrt_entry:
	sl	w9, w11			; w10:w11 2 * q
	add	w7, w11, w11		; w10:w11 2q + bit
	sub	w1, w11, w11		; w10:w11 r - (2q + bit)
	bra     n, f48_sqrt_qnext

	mov	w11, w1			; w0:w1   t -> r
	ior	w9, w7, w9		; w8:w9   q | bit
	lsr	w7, w7			; w6:w7  bit >>= 1
	bra	nc, f48_sqrt_loop	; C = 1 = bits 0
f48_sqrt_end:

	mov	#0x8000, w7	; +16 bits
	bra	f48_sqrt_loop32

f48_sqrt_qnext32:
	lsr	w7, w7			; w6:w7  bit >>= 1
	bra	c, f48_sqrt_end32	; C = 1 = bits 0
f48_sqrt_loop32:
	sl	w0, w0			; w0:w1 r * 2
	rlc	w1, w1
	sl	w8, w10			; w10:w11 2 * q
	rlc	w9, w11

	add	w10, w7, w10		; w10:w11 2q + bit
	addc	w11, #0, w11
	sub	w0, w10, w10		; w10:w11 r - (2q + bit)
	subb	w1, w11, w11
	bra	n, f48_sqrt_qnext32

	mov.d	w10, w0			; w0:w1   t -> r
	ior	w8, w7, w8		; w8:w9   q | bit
	lsr     w7, w7			; w6:w7  bit >>= 1
	bra     nc, f48_sqrt_loop32	; C = 1 = bits 0
f48_sqrt_end32:

	sl	w8, w8
	rlc	w9, w9
	sl	w8, w8
	rlc	w9, w9
; printf("w9=0x%04x SR=0x%04x\n", w9, MEMW(SR));

	mov	#0x3fff, w0
	sub	w5, w0, w5
	asr	w5, w5
	add	w5, w0, w5

.if 0
	btst	w9, #15
	bra	nz, f48_sqrt_nsl
	sl	w8, w8
	rlc	w9, w9
	dec	w5, w5
f48_sqrt_nsl:
.endif
	mov.d	w8, [w4];
	mov	w5, [w4+4];

	pop.d	w10
	pop.d	w8
	return
