/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/eingVx.inc"
.include "lib/Analog.inc"
.include "lib/Param.inc"
.include "math/libm.inc"
.if USE_DEBUG_UART_USER
.include "lib/uart_user.inc"
.endif


.section .nbss, bss, near
fOffs:			.space F48_SIZE		; -9000.0 ... 0 ... +9000.0

fPaOffs_mV:		.space F48_SIZE		; FR_OFF_PA
fPaOffs_mVeDC:		.space F48_SIZE		; FR_OFF_PA + FR_OFF_PAEDC
fPaOffs_mV_pC:		.space F48_SIZE		/* 0.01 mV / C */
fPaOffs_mV_TC:		.space F48_SIZE
fPaDacOffs_mV_TC:	.space F48_SIZE
fDigPaOffs_mV_mul:	.space F48_SIZE

fDigPaPow_mV:		.space F48_SIZE
fDigPaDC_mV:		.space F48_SIZE
fDigPaPow_mV_mul:	.space F48_SIZE
fDigPaDC_mV_mul:	.space F48_SIZE


fDigPaPow_TC_k:		.space F48_SIZE		/* 0.1 digit / C */
fDigPaDC_TC_k:		.space F48_SIZE		/* 0.1 digit / C */
fDigError_mV:		.space F48_SIZE
fTemp_OP1:		.space F48_SIZE

;fOffsNomPa:		.space F48_SIZE
;fOffsMaxPa:		.space F48_SIZE
fPaDC_Offs:		.space F48_SIZE
fPaPow_Offs:		.space F48_SIZE

iOffs:			.space 2		; -9000 ... 0 ... +9000
uOffs:			.space 2		; 0 ... +9000

PaDC_Offs:		.space 2

PaOffs_TeFreqTime:		.space 2
PaOffs_TeFreqTc:		.space 2
PaOffs_TeFreqFlag:		.space 2

PaPow_Offs:		.space 2
DcLastTemp:		.space 2
PaLastTemp:		.space 2
PaPowTC_Offs:		.space 2

.global fOffs

.global fDigPaPow_TC_k
.global fDigPaPow_TC_mul

.global fDigPaPow_mV
.global fPaPow_Offs
.global fDigPaDC_mV
.global fPaDC_Offs

.global PaOffs_TeFreqTime
.global iOffs
.global uOffs

/* ______________________________________________________________________ */
.section .text

.global MathPaOffs
.global SetDacPaDC_Offset
.global SetDacPaPow_Offset
.global SetDacPaFine_Offset
.global SetUpOffs
.global SetOffsTemp
.global SetParamOffsTemp
.global SetPaOffsTemp
.global SetParamPa

.global SetOffsDcFq
.global SetOffsDcFqInit
.global VxOcPaOffs

.if USE_DEBUG_UART_USER
.global dctk_dac_set
.endif

MathPaOffs:
	bclr	OV_FlagEg, #bOVF_OFFSET
	lnk	#F48_SIZE*4

	mov	EingCp_Offs+oEVX_VALUE_1616, w0
	mov	EingCp_Offs+oEVX_VALUE, w1
	mov	w14, w3
	rcall	i1616_f48_w3

	btst	TestFlags, #TF_DC_FREQ_ON
	bra	z, spao_nte
	add	w14, #4, w0
	btst	PaOffs_TeFreqFlag, #0
	bra	z, spao_nte
	btg	[w0], #15	; inv sign
spao_nte:

	mov	#fImpGenMul, w2
	mov	#fOffs, w4	; -9000.0 ... 0 ... +9000.0
	rcall	f48_mul

	rcall	f48_i16r
	mov	w0, iOffs	; -9000 ... 0 ... +9000

	mov	#fOffs, w2
	mov	w14, w4
	rcall	f48_abs_mov

	rcall	f48_i16r
	mov	w0, uOffs	; 0 ... +9000

	add	w14, #F48_SIZE*3, w3
	mov	#0x0000, w0
	mov	w0, [w3]
	mov	#0x81b0, w0
	mov	w0, [w3+2]
	mov	#0x400b, w0
	mov	w0, [w3+4]		; 4150.0

	btst	GV_PORT, #GV_BIT
	bra	z, spao_mod_dc

.if 1
/* Chk OverFlow  (Ampl/2 + (abs)Offs)  > 8300mV */

	mov	#-1, w0			; Ampl / 2
	mov	#fAmpl, w2
	add	w14, #F48_SIZE, w4
	rcall	f48_shift

	mov	w14, w2
	mov	w4, w3
	add	w14, #F48_SIZE*2, w4
	rcall	f48_add

	mov	w4, w2
	mov	#FC48_PA_V_8300, w3
	rcall	f48_cp
	bra	le, spao_norm

/* Chk OverFlow  (Ampl/2 + (abs)Offs)  > 9000mV */
	mov	#FC48_PA_MAX_V, w3
	rcall	f48_cp
	bra	le, spao_norm_ok

	mov	#FC48_PA_MAX_V, w2
	add	w14, #F48_SIZE*2, w4
	rcall	f48_mov
spao_norm_ok:
/* Split-V = (Ampl/2 + (abs)Offs) / 2 */
	mov	#-1, w0			; Max / 2
	add	w14, #F48_SIZE*2, w2
	add	w14, #F48_SIZE*3, w4
	rcall	f48_shift
spao_norm:

.else
	mov	#-1, w0
	mov	#fAmpl, w2
	add	w14, #F48_SIZE, w4
	rcall	f48_shift
	mov	w4, w2
	rcall	f48_cp
	bra	le, spao_norm

	add	w14, #F48_SIZE*3, w4
	rcall	f48_mov
spao_norm:
	mov	#-1, w0
	mov	w14, w2
	add	w14, #F48_SIZE, w4
	rcall	f48_shift
	mov	w4, w2
	rcall	f48_cp
	bra	le, spao_norm_off

	add	w14, #F48_SIZE*3, w4
	rcall	f48_mov
spao_norm_off:
.endif
	mov	w14, w2
	add	w14, #F48_SIZE*3, w3
	rcall	f48_cp
	bra	gt, spao_sq_2x

	mov	#fOffs, w2
	mov	#fPaPow_Offs, w4
	rcall	f48_mov

	clr	fPaDC_Offs
	clr	fPaDC_Offs+2
	clr	fPaDC_Offs+4
	bra	spo_end

spao_sq_2x:
	add	w14, #F48_SIZE*3, w2
	mov	#fPaPow_Offs, w4
	rcall	f48_mov		; -> PaPow_Offs

	mov	#fOffs, w2
	rcall	f48_copy_sign


	mov	#fOffs, w2
	mov	#fPaPow_Offs, w3
	mov	#fPaDC_Offs, w4
	rcall	f48_sub		; ffs - PaPow_Offs  -> PaDC_Offs

/* Chk OverFlow  9000 - Ampl/2 - (abs)PaPow_Offs = max PaDC_Offs */
	mov	#fPaPow_Offs, w2
	add	w14, #F48_SIZE*2, w4
	rcall	f48_abs_mov

	mov	#-1, w0
	mov	#fAmpl, w2
	add	w14, #F48_SIZE, w4
	rcall	f48_shift

	mov	#FC48_PA_MAX_V, w2
	mov	w4, w3
	rcall	f48_sub

	mov	w4, w2
	add	w14, #F48_SIZE*2, w3
	rcall	f48_sub
	add	w14, #F48_SIZE*2, w4
	rcall	f48_neg_mov

	mov	w4, w3
	mov	#fPaDC_Offs, w4
	rcall	f48_lim
	bra	z, spo_end
	bset	OV_FlagEg, #bOVF_OFFSET
	bra	spo_end

spao_mod_dc:
	add	w14, #F48_SIZE*3, w2		; 4150.0 * 2
	mov	#1, w0
	add	w14, #F48_SIZE, w4
	rcall	f48_shift

	mov	w4, w3
	mov	w14, w2
	rcall	f48_cp
	bra	le, spao_norm_dc
	mov	#-1, w0				; Offset / 2
	mov	w14, w2
	add	w14, #F48_SIZE*3, w4
	rcall	f48_shift
spao_norm_dc:
	mov	w14, w2
	add	w14, #F48_SIZE*3, w3
	rcall	f48_cp
	bra	gt, spao_dc_2x

	mov	#fOffs, w2
	mov	#fPaDC_Offs, w4
	rcall	f48_mov

	clr	fPaPow_Offs
	clr	fPaPow_Offs+2
	clr	fPaPow_Offs+4
	bra	spo_end

spao_dc_2x:
	add	w14, #F48_SIZE*3, w2
	mov	#fPaDC_Offs, w4
	rcall	f48_mov
	mov	#fOffs, w2
	rcall	f48_copy_sign

	mov	#fOffs, w2
	mov	#fPaDC_Offs, w3
	mov	#fPaPow_Offs, w4
	rcall	f48_sub
spo_end:
	ulnk
	rcall	pao_f2i

	return


pao_f2i:
	lnk	#F48_SIZE*3
/* --- PA DAC Pow --- */
	mov	#fImpGenMulKorr, w2
	mov	#fDigPaPow_mV_mul, w3
	mov	w14, w4
	rcall	f48_mul		; digit mul

	mov	w4, w2
	mov	#fPaPow_Offs, w3
	add	w14, #F48_SIZE, w4
	rcall	f48_mul		; Pow_mV * digit mul = PowDac digit
	rcall	f48_i16r	; -> int PowDac digit
	mov	w0, PaPow_Offs

	add	w14, #F48_SIZE*2, w3
	rcall	i16_f48_w3

	mov	w4, w2
	rcall	f48_sub		; PowDac digit - int PowDac digit = offs digit
	
	mov	w14, w3		; offs digit / digit mul = Pow Error_mV
	mov	#fDigError_mV, w4
	rcall	f48_div3216

/* --- PA DAC DC --- */

	mov	#fImpGenMulKorr, w2
	mov	#fDigPaDC_mV_mul, w3
	mov	w14, w4
	rcall	f48_mul		; digit mul

	mov	w4, w2
	mov	#fPaDC_Offs, w3
	add	w14, #F48_SIZE, w4
	rcall	f48_mul		; DC_mV * digit mul = DC_Dac digit
	rcall	f48_i16r	; -> int DC_Dac digit
	mov	w0, PaDC_Offs

	add	w14, #F48_SIZE*2, w3
	rcall	i16_f48_w3

	mov	w4, w2
	rcall	f48_sub		; DC_Dac digit - int DC_Dac digit = offs digit
	
	mov	w14, w3		; offs digit / digit mul = DC Error_mV
	rcall	f48_div3216

	mov	#fDigError_mV, w3
	mov	w3, w4
	rcall	f48_add

	rcall	MathOffsTemp
	ulnk
	return


SetDacPaDC_Offset:
	mov	PaDC_Offs, w0
	mov	EEData0+oFR_OFF_PA_DC, w1
	rcall	add16s_sat
	mov	#PA_DC, w2
	bra	DacSetValue_LinKorr

SetDacPaPow_Offset:
	mov	EEData0+oFR_OFF_PA_POW, w1
.if USE_DEBUG_UART_USER
	btst	uuCAL_DAC_Mod, #bDAC_PA_OFFSET_KORR_OFF
	bra	nz, sdPav_nk
	mov	PaPowTC_Offs, w0
	add	w1, w0, w1
sdPav_nk:
.else
	mov	PaPowTC_Offs, w0
	add	w1, w0, w1
.endif
	mov	PaPow_Offs, w0
	rcall	add16s_sat
	mov	#PA_POW, w2
	bra	DacSetValue_LinKorr

SetDacPaFine_Offset:	/* max +-8.0 mV */
	lnk	#F48_SIZE
	mov	#fPaOffs_mV, w2
	btss	GV_PORT, #GV_BIT
	mov	#fPaOffs_mVeDC, w2
	mov	#fPaOffs_mV_TC, w3
	mov	w14, w4
	rcall	f48_add
	mov	w14, w2
.if USE_DEBUG_UART_USER
	btst	uuCAL_DAC_Mod, #bDAC_PA_OFFSET_KORR_OFF
	bra	nz, sdPf_nk
.endif
	mov	#fDigError_mV, w3
	rcall	f48_add

	mov	#fPaDacOffs_mV_TC, w3
	rcall	f48_add
.if USE_DEBUG_UART_USER
sdPf_nk:
.endif
	mov	#fDigPaOffs_mV_mul, w3
	rcall	f48_mul
	rcall	f48_i16r

	mov	#PAVO, w2
	rcall	DacSetValue

	ulnk
	return

SetUpOffs:
	rcall	SetParamOffs
	rcall	SetParamPa
	rcall	MathPaOffs
	rcall	SetDacPaDC_Offset
	rcall	SetDacPaPow_Offset
	rcall	SetDacPaFine_Offset
	return

/* lnk min F48_SIZE*2 */
MathOffsTemp:
	mov	#fDigPaPow_TC_k, w2
	mov	#fTemp_OP1, w3
	mov	w14, w4
	rcall	f48_mul
	mov	#fPaPow_Offs, w2
	mov	w4, w3
	rcall	f48_mul		; C * k/C * PaPow_mV = k1_mV

	mov	#fDigPaDC_TC_k, w2
	mov	#fTemp_OP1, w3
	add	w14, #F48_SIZE, w4
	rcall	f48_mul
	mov	#fPaDC_Offs, w2
	mov	w4, w3
	rcall	f48_mul		; C * k/C * PaDC_mV = k2_mV

	mov	w14, w2
	mov	w14, w4
	rcall	f48_add		; k1_mV + k2_mV = k_mV

	mov	PaPowTC_Offs, w0
	add	w14, #F48_SIZE, w3
	rcall	i16_f48_w3
	mov	#fDigPaPow_mV, w2
	mov	w3, w4
	rcall	f48_mul		; iOffs * 0.146mV/dig = Akt_Offs_mV

	mov	w14, w2
	rcall	f48_sub
	rcall	f48_abs		; abs(k_mV - Akt_Offs_mV) = Diff_mV

	mov	w4, w2
	mov	#fDigPaPow_mV, w3
	rcall	f48_cp		; if Diff_mV < 0.146mV/dig goto
	bra	lt, sot_powv_ok
	mov	w14, w2
	mov	#fDigPaPow_mV_mul, w3
	rcall	f48_mul		; k_mV / 0.146mV/dig = fTC_Offs
	rcall	f48_i16r	; fTC_Offs -> TC_Offs
	mov	w0, PaPowTC_Offs
sot_powv_ok:
	mov	PaPowTC_Offs, w0
	add	w14, #F48_SIZE, w3
	rcall	i16_f48_w3
	mov	#fDigPaPow_mV, w2
	mov	w3, w4
	rcall	f48_mul		; iOffs * 0.146mV/dig = Akt_Offs_mV
	mov	w14, w2
	mov	#fPaDacOffs_mV_TC, w4
	rcall	f48_sub		; k_mV - Akt_Offs_mV = fPaDacOffs_mV_TC
	return


SetOffsTemp:
.if USE_DEBUG_UART_USER
	btst	uuCAL_DAC_Mod, #bDAC_PA_OFFSET_KORR_OFF
	bra	nz, dctk_clr
.endif

	mov	Temp_OP1_0025, w0

	mov	#1200, w1
	sub	w0, w1, w0		; Temp - 30.0C
	bra	z, dctk_clr

	mov	DcLastTemp, w1
	sub	w1, w0, w1
	btsc	SR, #Z
	return
	btsc	w1, #15
	neg	w1, w1
	dec	w1, w1
	btsc	SR, #Z
	return
	mov	w0, DcLastTemp

	lnk	#F48_SIZE*2

	mov	w14, w3
	rcall	i16_f48_w3

	mov	#FC48_0025, w2
	mov	#fTemp_OP1, w4
	rcall	f48_mul

	rcall	MathOffsTemp

	ulnk
	bra	dctk_dac_set

dctk_clr:
	cp0	DcLastTemp
	btsc	SR, #Z
	return

	clr	DcLastTemp
	clr	PaPowTC_Offs
	clr	fTemp_OP1
	clr	fTemp_OP1+2
	clr	fTemp_OP1+4
	clr	fPaDacOffs_mV_TC
	clr	fPaDacOffs_mV_TC+2
	clr	fPaDacOffs_mV_TC+4
dctk_dac_set:
	rcall	SetDacPaFine_Offset
	rcall	SetDacPaPow_Offset
	return

SetParamOffsTemp:
	rcall	SetParamOffs
	clr	DcLastTemp
	return

SetParamOffs:

	lnk	#F48_SIZE*3

	mov	EEData0+oFR_PA_POW_V_KORR, w0
	asr	w0, #15, w1
	mov	#10, w2
	rcall	div_s32_u16_1616
	mov	#DAC_PA_POW_BASE, w2
	add	w1, w2, w1
	mov	w14, w3
	rcall	i1616_f48_w3
	mov	#FC48_001, w2
	mov	w14, w4
	rcall	f48_mul
	mov	#FC48_0001, w2
	mov	#fDigPaPow_mV, w4
	rcall	f48_mul

	mov	#FC48_1, w2		; 1 / fDigPaPow_mV
	mov	#fDigPaPow_mV, w3
	mov	#fDigPaPow_mV_mul, w4
	rcall	f48_div

	mov	EEData0+oFR_PA_DC_V_KORR, w0
	asr	w0, #15, w1
	mov	#10, w2
	rcall	div_s32_u16_1616
	mov	#DAC_PA_POW_BASE, w2
	add	w1, w2, w1
	mov	w14, w3
	rcall	i1616_f48_w3
	mov	#FC48_001, w2
	mov	w14, w4
	rcall	f48_mul
	mov	#FC48_0001, w2
	mov	#fDigPaDC_mV, w4
	rcall	f48_mul

	mov	#FC48_1, w2		; 1 / fDigPaDC_mV
	mov	#fDigPaDC_mV, w3
	mov	#fDigPaDC_mV_mul, w4
	rcall	f48_div

	mov.b	EEData0+oFR_DAC_POW_TC, WREG
	se	w0, w0
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_0000001, w2
	mov	#fDigPaPow_TC_k, w4
	rcall	f48_mul

	mov.b	EEData0+oFR_DAC_DC_TC, WREG
	se	w0, w0
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_0000001, w2
	mov	#fDigPaDC_TC_k, w4
	rcall	f48_mul
/*
	mov	#0x0000, w0
	mov	w0, fOffsMaxPa
	mov	#0x8ca0, w0
	mov	w0, fOffsMaxPa+2
	mov	#0x400b, w0
	mov	w0, fOffsMaxPa+4		; 4500mV
*/
	ulnk
	clr	DcLastTemp
	return

SetPaOffsTemp:
	mov	Temp_PA_0025, w0

	mov	#1200, w1
	sub	w0, w1, w0		; Temp - 30.0C
	bra	z, patk_clr

	mov	PaLastTemp, w1
	sub	w1, w0, w1
	btsc	SR, #Z
	return
	btsc	w1, #15
	neg	w1, w1
	dec	w1, w1
	btsc	SR, #Z
	return
	mov	w0, PaLastTemp

	lnk	#F48_SIZE*2
	mov	w14, w3
	rcall	i16_f48_w3

	mov	#FC48_0025, w2
	mov	w14, w4
	rcall	f48_mul

	mov	#fPaOffs_mV_pC, w2
	mov	#fPaOffs_mV_TC, w4
	rcall	f48_mul

	ulnk
	bra	SetDacPaFine_Offset

patk_clr:
	clr	PaLastTemp
	clr	fPaOffs_mV_TC
	clr	fPaOffs_mV_TC+2
	clr	fPaOffs_mV_TC+4
	bra	SetDacPaFine_Offset
	return

SetParamPa:

	lnk	#F48_SIZE

	mov	EEData0+oFR_PA_OFFS_V_KORR, w0
	mov	#DAC_PA_OFFSET_BASE, w2
	add	w0, w2, w0
	mov	w14, w3
	rcall	u16_f48_w3
	mov	#FC48_0000001, w2
	mov	w14, w4
	rcall	f48_mul
	mov	#FC48_1, w2		; 1 / fDigPaOffs_mV
	mov	w4, w3
	mov	#fDigPaOffs_mV_mul, w4
	rcall	f48_div
	mov	#4, w0			; shift 4  12bit to 16bit
	rcall	f48_add_exp_w4

	mov.b	EEData0+oFR_TC_PA_OUT, WREG
	se	w0, w0
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_001, w2
	mov	#fPaOffs_mV_pC, w4
	rcall	f48_mul

	mov.b	EEData0+oFR_OFF_PA, WREG
	se	w0, w0
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_001, w2
	mov	#fPaOffs_mV, w4
	rcall	f48_mul

	mov.b	EEData0+oFR_OFF_PA, WREG
	se	w0, w1
	mov.b	EEData0+oFR_OFF_PAEDC, WREG
	se	w0, w0
	add	w0, w1, w0
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#FC48_001, w2
	mov	#fPaOffs_mVeDC, w4
	rcall	f48_mul

	ulnk
	clr	PaLastTemp
	return



/* -------------------- PaOffs Dc Test Freq ----------------------------- */
SetOffsDcFq:
	cp0	PaOffs_TeFreqTc
	bra	z, sosfq_tgl
	dec	PaOffs_TeFreqTc
	cp0	PaOffs_TeFreqTc
	bra	z, MathPaOffs
	return
sosfq_tgl:
	rcall	SetDacPaDC_Offset
	rcall	SetDacPaPow_Offset

	btg	PaOffs_TeFreqFlag, #0
	mov	PaOffs_TeFreqTime, w0
	mov	w0, PaOffs_TeFreqTc
	return

SetOffsDcFqInit:
	cp0	PaOffs_TeFreqTime
	bra	nz, sosfi
	mov	#500, w0
	mov	w0, PaOffs_TeFreqTime
sosfi:
	clr	PaOffs_TeFreqFlag
	mov	PaOffs_TeFreqTime, w0
	mov	w0, PaOffs_TeFreqTc
	return

/* -------------------- VxOcPaOffs --------------------------------
OnCange(w0:w1=Value, w3=stat, w6=Owner)
------------------------------------------------------------------- */
VxOcPaOffs:

	cp	w3, #ON_CHANGE_VALUE
	bra	z, ocpa_change
	cp	w3, #ON_CHANGE_OPEN
	bra	z, ocpa_init
	return

ocpa_change:
	push	w6
	mov	#fPaDC_Offs, w3
	btsc	w6, #0
	mov	#fPaPow_Offs, w3
	rcall	i1616_f48_w3
	rcall	pao_f2i

	rcall	SetDacPaDC_Offset
	rcall	SetDacPaPow_Offset
	bra	ocpa_rd_dac

ocpa_init:
	push	w6
	push	w5

	mov	#fPaDC_Offs, w4
	btsc	w6, #0
	mov	#fPaPow_Offs, w4
	rcall	f48_i1616		; read value
	pop	w5

	mov	#-4500, w2
	mov	#4500, w3
	rcall	EingVx_SetValueMinMax
	bset	[w5], #bEVSTAT_OV_SET_MAX

ocpa_rd_dac:
	pop	w6
	mov	#PA_DC, w2
	btsc	w6, #0
	mov	#PA_POW, w2
	rcall	DacGetValue
	mov	w0, LcdTeVal1
	return

.section .const, psv

FC48_PA_MAX_V:
	.word	0x0000, 0x8ca0, 0x400c		; 9000.0
FC48_PA_V_8300:
	.word	0x0000, 0x81b0, 0x400c		; 8300.0
