/*
 Copyright (C) 2014-2022 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "main.inc"
.include "lib/eingVx.inc"
.include "lib/Analog.inc"
.include "lib/Param.inc"
.include "../math/libm.inc"

	SIZE_TAB_PULS		= 12
	oPULS_MIN_T		= 0
	oPULS_MAX_T		= 2
	oPULS_DAC_I		= 4
	oPULS_DAC_VAL_MIN	= 6
	oPULS_DAC_VAL_MAX	= 8
	oPULS_OPT		= 10

/* ---------------------------------------------------------------------- */
.section .text

.global SetPuls

SetPuls:
	mov	FqTime_Step, w0
	and	#0x07, w0
	sl	w0, w0
	mov	#tabPulsSeg, w6
	add	w6, w0, w6
	mov	EingCp_Time+oEVX_VALUE, w0
	mov	[w6], w6
	cp0	w6
	btsc	SR, #Z
	return
spv_loop:
	mov	[w6+oPULS_MIN_T], w2
	btst	w2, #15
	bra	nz, spv_cpu
	cp0	w2
	bra	z, spv_0
	mov	[w6+oPULS_MAX_T], w1

	cp	w0, w1
	bra	leu, spv_math
	add	#SIZE_TAB_PULS, w6
	bra	spv_loop
spv_0:
;	C1_CLR
;	C2_CLR
;	C3_CLR
	bclr	C1_PORT, #C1_BIT
	bclr	C2_PORT, #C2_BIT
	bclr	C3_PORT, #C3_BIT
	return

spv_math:
	btst	w2, #15
	bra	nz, spv_0	; CPU TIME   TODO !!!!

	mov	[w6+oPULS_DAC_I], w0
	sl	w0, #4, w0
	mov	#AIMP, w2
	rcall	DacSetValue

	mov	[w6+oPULS_OPT], w0
	lsr	w0, #8, w0
	mov	#EEData0+oFR_PULS_2NS, w1
	add	w0, w1, w0
	mov	[w6+oPULS_DAC_VAL_MIN], w4
	mov	[w6+oPULS_DAC_VAL_MAX], w5
	add	w4, [w0++], w4
	add	w5, [w0++], w5
	sub	w5, w4, w1


	mov	[w6+oPULS_MIN_T], w3
	mov	[w6+oPULS_MAX_T], w2
	sub	w2, w3, w2
	
	mov	EingCp_Time+oEVX_VALUE, w0
	sub	w0, w3, w0
	mul.ss	w0, w1, w0
	repeat	#17
	div.sd	w0, w2
	add	w0, w4, w0

	mov	#VIMP, w2
	sl	w0, #4, w0
	rcall	DacSetValue

	mov	[w6+oPULS_OPT], w0
	bclr	C1_PORT, #C1_BIT
	btsc	w0, #0
	bset	C1_PORT, #C1_BIT

	bclr	C2_PORT, #C2_BIT
	btsc	w0, #1
	bset	C2_PORT, #C2_BIT

	bclr	C3_PORT, #C3_BIT
	btsc	w0, #2
	bset	C3_PORT, #C3_BIT

	clr	CpuPulsTime
	clr	CpuPulsTime+2

	bset	CpuFreqStat, #bCPU_FQ_PULS_ANL
	bra	spv_set_omod

spv_cpu:
	bclr	CpuFreqStat, #bCPU_FQ_PULS_ANL
;	C1_CLR
;	C2_CLR
;	C3_CLR
	bclr	C1_PORT, #C1_BIT
	bclr	C2_PORT, #C2_BIT
	bclr	C3_PORT, #C3_BIT

	lnk	#F48_SIZE
	bclr	w2, #15
	push	w2

	mov	EingCp_Time+oEVX_VALUE, w0
	mov	w14, w3
	rcall	u16_f48_w3

	mov	#FC48_00001, w2
	mov	w3, w4
	rcall	f48_mul

	pop	w0
	cp0	w0
	bra	z, no_div
	mov	#FC48_01, w2
	cp	w0, #1
	bra	z, x_div
	mov	#FC48_001, w2
	cp	w0, #2
	bra	z, x_div
	mov	#FC48_0001, w2
x_div:
	rcall	f48_mul
no_div:
	mov	#FC48_CPU_FREQ, w2
	rcall	f48_mul

	rcall	f48_u32r
	mov	w0, CpuPulsTime
	mov	w1, CpuPulsTime+2
	ulnk

spv_set_omod:
	btsc	PA_OutFlag, #bPAOF_DC
	return
	btst	PA_OutFlag, #bPAOF_SQ_P
	bra	nz, MathFreq
	return

/* ______________________________________________________________________ */
.section .const, psv

/* ----------------------------------------------------------------------
  t_start, t_end, Aimp, k1, k2, Opt
  Opt  ->  b0..b2=C   b8..b15=FRam-idx
-----------------------------------------------------------------------*/
tabPulse_vs0:
/* 2.0 - 7.0ns (StepDig=270) */
 .word	20,	70,	128,	620,	2580,	0x0000
/* 7.1 - 19.9ns (StepDig=112) */
 .word	71,	199,	2500,	600,	2700,	0x0400
 .word	0

tabPulse_vs1:
/* 20 - 100ns (StepDig=231) (100/500Hz  min/max -0/-40Dig) */
 .word	20,	99,	3660,	200,	3700,	0x0800
 .word	0

tabPulse_vs2:
/* 101 - 320ns (StepDig=96) (100/500Hz  min/max -30/-110Dig) */
 .word	100,	319,	2970,	830,	3700,	0x0C01
/* 321 - 999ns (StepDig=29) (100/500Hz  min/max -10/-25Dig) */
 .word	320,	999,	2840,	860,	3700,	0x1002
 .word	0

tabPulse_vs3:
/* 1.00 - 4.00us (StepDig=75) (100/500Hz  min/max -15/-60Dig) */
 .word	100,	399,	3690,	500,	3700,	0x1403
/* 4.01 - 9.99us (dig=30) (100/500Hz  min/max -15/-40Dig) */
 .word	400,	999,	3390,	1170,	3700,	0x1804
 .word	0

tabPulse_vs4:
/* 10.0 - 62.5us (dig=49) (100/500Hz  min/max -20/-90Dig) */
 .word	100,	625,	3943,	256,	3840,	0x1C07
/* 62.6us -99.9us use cpu timer */
 .word	0x8003  /* value / 1000 */

tabPulse_vs5:
/* 100us - 999us use cpu timer */
 .word	0x8002  /* value / 100 */

tabPulse_vs6:
/* 1.00ms - 9.99ms use cpu timer */
 .word	0x8001  /* value / 10 */

tabPulse_vs7:
/* 10.0ms - 99.9ms use cpu timer */
 .word	0x8000

tabPulsSeg: 
	.word tabPulse_vs0, tabPulse_vs1, tabPulse_vs2, tabPulse_vs3
	.word tabPulse_vs4, tabPulse_vs5, tabPulse_vs6, tabPulse_vs7
