/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"
.include "lib/Param.inc"
.include "lib/Analog.inc"
.include "lib/CDCE913.inc"
.if USE_DEBUG_UART_USER
.include "lib/uart_user.inc"
.endif

.section .text
.global SetGenSym
.global SetGenPllSym

SetGenSym:
	mov	EEData0+oFR_APSYM, w0
	mov	#APSYM, w2
	rcall	DacSetValue_R_Align

	mov	EEData0+oFR_AGTI0, w0
	mov	#AGTI0, w2
	rcall	DacSetValue_R_Align

	mov	EEData0+oFR_AGTI1, w0
	mov	#AGTI1, w2
	rcall	DacSetValue_R_Align

	return

SetGenPllSym:
	btst	pll_stat, #bPLLST_POW_ON
	bra	nz, pll_pow_on
	mov	#-2048, w0
	mov	#OSYM, w2
	rcall	DacSetValue_R_Align
	bra	psy_le_1M

pll_pow_on:
	mov	CDCE913_DIV1, w0
	cp	w0, #1
	bra	z, pll_115
	mov	EEData0+oFR_OSYM, w0
	bra	pll_OSYM
pll_115:
	mov	EEData0+oFR_OSYM_115MHZ, w0
pll_OSYM:
	mov	#OSYM, w2
	rcall	DacSetValue_R_Align

.if USE_DEBUG_UART_USER
	btst	uuCAL_DAC_Mod, #bGEN_SYM_OFF
	bra	nz, psy_le_1M
.endif


	mov	CDCE913_fout, w0
	mov	CDCE913_fout+2, w1
	mov	#10000, w2
	repeat	#17
	div.ud	w0, w2

	mov	#tabCalSym, w4
	mov	#tabSymOffs, w5
	cp	w0, [w4]
	bra	leu, psy_le_1M

psy_loop:
	mov	[w4+2], w3
	cp0	w3
	bra	z, psy_i0
	cp	w0, w3
	bra	ltu, psy_find
	inc2	w4, w4
	inc2	w5, w5
	bra	psy_loop

psy_find:
	sub	w3, w0, w1
	sub	w3, [w4], w2
	cp	w1, w2
	bra	z, psy_i0

	clr	w0
	repeat	#17
	div.ud	w0, w2

	mov.b	[w5+2], w1	; APSYM offset
	ze	w1, w1
	sub.b	w1, [w5++], w2
	se	w2, w2
	mul.us	w0, w2, w2
	sl	w2, w2
	addc	w3, #0, w3

	sub	w1, w3, w2
	push	w2

	mov.b	[w5+2], w1	; GTI0 offset
	se	w1, w1
	mov.b	[w5], w2
	se	w2, w2
	sub	w1, w2, w2
	mul.us	w0, w2, w2
	sl	w2, w2
	addc	w3, #0, w3
	sub	w1, w3, w1
	pop	w0
	bra	psy_add

psy_i0:
	ze	[w5++], w0
	se	[w5], w1
	bra	psy_add

psy_le_1M:
	clr	w0
	clr	w1
psy_add:
	sl	w0, #2, w0
	mov	EEData0+oFR_APSYM, w2
	add	w0, w2, w0
	mov	EEData0+oFR_AGTI0, w2
	add	w1, w2, w1
	push	w1
	mov	#APSYM, w2
	rcall	DacSetValue_R_Align
	pop	w0
	mov	#AGTI0, w2
	rcall	DacSetValue_R_Align
	return

/* ---------------------------------------------------------------------- */
.section .const, psv

.if 1
.if 1
tabCalSym:
/*  Freq 10kHz */
 .word	50
 .word	200
 .word	500
 .word	1000
 .word	2500
 .word	5000
 .word	7500
 .word	8750
 .word	10000
 .word	12500
 .word	14000
 .word	16000
 .word	17000
 .word	19000
 .word	20500
 .word	23000
 .word	0

tabSymOffs:
/* APSYM / 4   GTI0 */
 .byte	0, 0			; 500k
 .byte	30, -30			; 2M
 .byte	65, -45			; 5M
 .byte	84, -45			; 10M
 .byte	105, -40		; 25M
 .byte	66, -32			; 50M
 .byte	63, -15			; 75M
 .byte	91, -20			; 87.5M
 .byte	51, -15			; 100M
 .byte	89, 28			; 125M
 .byte	105, 15			; 140M
 .byte	219, -35		; 160M
 .byte	218, -65		; 170M
 .byte	181, -110		; 190M
 .byte	193, -115		; 205M
 .byte	179, -60		; 230M
 .byte	0, 0

.else
tabCalSym:
/*  Freq 10kHz */
 .word	100
 .word	200
 .word	500
 .word	1000
 .word	2000

 .word	3000
 .word	4000
 .word	5000
 .word	6000
 .word	7000

 .word	8000
 .word	9000
 .word	10000
 .word	11000
 .word	12000

 .word	13000
 .word	14000
 .word	15000
 .word	16000
 .word	17000

 .word	18000
 .word	19000
 .word	20000
 .word	21000
 .word	22000

 .word	23000
 .word	0

tabSymOffs:
 .byte	0, 0			; 1M
 .byte	12, -20			; 2M
 .byte	48, -20			; 5M
 .byte	68, -45			; 10M
 .byte	92, -46			; 20M

 .byte	83, -43			; 30M
 .byte	66, -41			; 40M
 .byte	59, -37			; 50M
 .byte	56, -35			; 60M
 .byte	60, -28			; 70M

 .byte	64, -25			; 80M
 .byte	75, -29			; 90M
 .byte	46, -24			; 100M
 .byte	69, -9			; 110M
 .byte	63, 1			; 120M

 .byte	86, 3			; 130M
 .byte	103, -17		; 140M
 .byte	153, -42		; 150M
 .byte	216, -56		; 160M
 .byte	229, -80		; 170M

 .byte	223, -104		; 180M
 .byte	203, -121		; 190M
 .byte	203, -121		; 200M
 .byte	201, -114		; 210M
 .byte	185, -106		; 220M

 .byte	168, -101		; 230M
 .byte	0, 0
.endif
.else

tabCalSym:
/*  Freq 10kHz */
 .word	100
 .word	0

tabSymOffs:
 .byte	0, 0			; 1M
 .byte	0, 0
.endif
