/*
 Copyright (C) 2014-2023 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

.include "sys.inc"

.if USE_DEBUG_UART_USER

.include "main.inc"
.include "lib/Analog.inc"
.include "lib/eingVx.inc"
.include "math/libm.inc"
.include "lib/rs232ald.inc"
.include "lib/uart_user.inc"
.include "lib/LCD_128x64.inc"

.section .nbss, bss, near

SOURce:			.space 2
uuCAL_On_Off:		.space 2
uuCAL_DAC_CHx:		.space 2
uuCAL_DAC_Mod:		.space 2
uuCAL_DAC_Extr:		.space 2

rs232ald_wait:		.space 2
;rs232ald_wait_bef:	.space 2

MCP3421_LastRead:	.space 4
ADS112c04_LastRead:	.space 10

	BEF_SYS		= 0
	BEF_RANGE	= 1
	BEF_SOUR	= 2
	BEF_MEAS	= 3
	BEF_CAL		= 4
	BEF_PARAM	= 5

	SOUR_OUT_V		= 0
	SOUR_OP1_V		= 1
	SOUR_PA_TEMP		= 2
	SOUR_OP1_TEMP		= 3
	SOUR_GEN_TEMP		= 4
	SOUR_GOV		= 5
	SOUR_GRV		= 6

	SOUR_OUT_V_DIGIT	= 16
	SOUR_OP1_V_DIGIT	= 17

/* BEF_RANGE */
	RANGE_FREQ		= 0
	RANGE_ATT		= 1
	RANGE_AMPL		= 3
	RANGE_OFFS		= 4
	RANGE_MOD		= 5

/* BEF_READ */
	MEAS_READ		= 2

/* BEF_CAL */
	CAL_ON_OFF		= 0
	CAL_ATT			= 1
	CAL_DAC			= 5
	CAL_DAC_VSET		= 6
	CAL_DAC_MOD		= 7
	CAL_DAC_EXTR		= 8

/* BEF_PARAM */
	PARAM_MENU0		= 0
	PARAM_MENU		= 1
	PARAM_NEXT		= 2
	PARAM_OPEN		= 3
	PARAM_CLOSE		= 4

.global uuCAL_DAC_Mod
.global uuCAL_DAC_Extr
.global rs232ald_wait

/* ---------------------------------------------------------------------- */
.section .text
.global rs232ald_user_init
.global __rs232ald_user
.global rs232ald_user_Handle

rs232ald_user_init:
	return

rs232ald_user_Handle:
;	mov	rs232ald_wait_bef, w0
	mov	rs232ald_wait, w1
	cp	w1, #BEF_MEAS
	bra	z, LrMEAS_READ_Wait
	clr	rs232ald_wait
	return


__rs232ald_user:
	clr	rs232ald_wait
	mov	RxD_len, w1
	cp	w1, #2
	bra	lt, SendErr_BEF_SIZE
	mov	#RxD_dbgbuf, w3
	mov	#RS232ALD_TxD_Data, w4
	dec2	w1, w1
	mov.b	[w3++], w2
	mov.b	w2, [w4]
	mov.b	[w3++], w4

	sub	#0x40, w0
	cp	w0, #BEF_SYS
	bra	z, vxSYS
	cp	w0, #BEF_RANGE
	bra	z, vxRANGE
	cp	w0, #BEF_SOUR
	bra	z, vxSOUR
	cp	w0, #BEF_MEAS
	bra	z, vxMEAS
	cp	w0, #BEF_CAL
	bra	z, vxCAL
.if USE_UART_LIST_PARA
	cp	w0, #BEF_PARAM
	bra	z, vxPARAM
.endif
	bra	SendErr_UNKNOWN_BEF

uuGetVal16:
	cp	w1, #2
	bra	nz, uuGetValX_err
	mov.b	[w3++], w0
	swap	w0
	mov.b	[w3++], w0
	swap	w0
	cp0	w0
	bclr	SR, #C
	return

uuGetVal32:
	cp	w1, #4
	bra	nz, uuGetValX_err
rd_4byte:
	mov.b	[w3++], w0
	swap	w0
	mov.b	[w3++], w0
	swap	w0
	mov.b	[w3++], w1
	swap	w1
	mov.b	[w3++], w1
	swap	w1
	bclr	SR, #C
	return

uuGetValX_err:
	bset	SR, #C
	return

TxD_AV_int16:
	mov	#RS232ALD_TxD_Data+1, w3
	mov	#0, w4
	mov.b	w4, [w3++]
	mov.b	w0, [w3++]
	swap	w0
	mov.b	w0, [w3++]
	mov	RxD_Bef, w0
	mov	#4, w1
	bra	rs232ald_TxD

TxD_AV_AX_16:
	btss	w2, #bEINH_UINT
	asr	w0, #15, w1
	btsc	w2, #bEINH_UINT
	clr	w1
TxD_AV_AX:
	mov	#RS232ALD_TxD_Data+1, w3
	mov	#RD_FLAG_AV_AX, w4
	mov.b	w4, [w3++]
	mov.b	w0, [w3++]
	swap	w0
	mov.b	w0, [w3++]
	mov.b	w1, [w3++]
	swap	w1
	mov.b	w1, [w3++]
	mov.b	w2, [w3++]
	swap	w2
	mov.b	w2, [w3]
	mov	RxD_Bef, w0
	mov	#8, w1
	bra	rs232ald_TxD

TxD_Float_ulnk_w14:
	mov	w14, w4
	rcall	f48_f32
	ulnk
;TxD_Float:
	mov	#RS232ALD_TxD_Data+1, w3
	mov	#FLAG_FLOAT32, w2
	mov.b	w2, [w3++]
	mov.b	w0, [w3++]
	swap	w0
	mov.b	w0, [w3++]
	mov.b	w1, [w3++]
	swap	w1
	mov.b	w1, [w3++]
	mov	RxD_Bef, w0
	mov	#6, w1
	bra	rs232ald_TxD

/* ----------------- SYS -------------------- */
vxSYS:
	btst	w4, #bFLAG_WR
	bra	nz, SendErr_UNKNOWN_BEF

	cp0	w2
	bra	z, ReadIDN
	bra	SendErr_UNKNOWN_BEF

ReadIDN:
	mov	#RS232ALD_TxD_Data, w3
	mov.b	w2, [w3++]
	clr.b	[w3++]

	mov	#txtInfo1, w0
	call	ppc_move
	mov	#' ', w0
	mov.b	w0, [w3++]

	mov	#txtInfo2, w0
	call	ppc_move
vxSendData:
	mov	#RS232ALD_TxD_Data, w1
	sub	w3, w1, w1
	inc	w1, w1
	mov	RxD_Bef, w0
	bra	rs232ald_TxD

/* ----------------- RANGE -------------------- */
vxRANGE:
	btst	w4, #bFLAG_WR
	bra	nz, vxRANGE_WR

	cp	w2, #RANGE_FREQ
	bra	z, LrRANGE_FREQ
	cp	w2, #RANGE_ATT
	bra	z, LrRANGE_ATT
	cp	w2, #RANGE_AMPL
	bra	z, LrRANGE_AMPL
	cp	w2, #RANGE_OFFS
	bra	z, LrRANGE_OFFS
	cp	w2, #RANGE_MOD
	bra	z, LrRANGE_MOD
	bra	SendErr_UNKNOWN_BEF

/* ----------------- RANGE RD -------------------- */
LrRANGE_FREQ:
	rcall	GetEingCp_Freq	; w0:w1=freq  w2=fq / div
	lnk	#F48_SIZE*2
	push	w2
	mov	w14, w3
	rcall	u32_f48_w3
	pop	w0
	cp	w0, #1
	bra	le, TxD_Float_ulnk_w14

	add	w14, #F48_SIZE, w3
	rcall	u16_f48_w3
	mov	w14, w2
	mov	w14, w4
	rcall	f48_div3216
	bra	TxD_Float_ulnk_w14

LrRANGE_ATT:
	clr	w0
	btsc	Impedanz_Flag, #bIMP_HIGHZ_ON
	mov	#1, w0
	bra	TxD_AV_int16

LrRANGE_AMPL:
	lnk	#F48_SIZE
	mov	EingCp_Ampl+oEVX_VALUE, w0
	mov	w14, w3
	rcall	u16_f48_w3

	mov	#FC48_0001, w2
	cp0	FqAmpl_Step
	btsc	SR, #Z
	mov	#FC48_00001, w2
	mov	w3, w4
	rcall	f48_mul
	bra	TxD_Float_ulnk_w14

LrRANGE_OFFS:
	lnk	#F48_SIZE
	mov	EingCp_Offs+oEVX_VALUE_1616, w0
	mov	EingCp_Offs+oEVX_VALUE, w1
	mov	w14, w3
	rcall	i1616_f48_w3
	mov	#FC48_0001, w2
	mov	w14, w4
	rcall	f48_mul
	bra	TxD_Float_ulnk_w14

LrRANGE_MOD:
	mov	PA_OutFlag, w0
	and	#PAOF_SOPDC_MASK, w0
	bra	TxD_AV_int16

/* ----------------- RANGE WR -------------------- */
vxRANGE_WR:
	cp	w2, #RANGE_FREQ
	bra	z, LwRANGE_FREQ
	cp	w2, #RANGE_ATT
	bra	z, LwRANGE_ATT
	cp	w2, #RANGE_AMPL
	bra	z, LwRANGE_AMPL
	cp	w2, #RANGE_OFFS
	bra	z, LwRANGE_OFFS
	cp	w2, #RANGE_MOD
	bra	z, LwRANGE_MOD
	bra	SendErr_UNKNOWN_BEF

LwRANGE_FREQ:
;	cp	w1, #4
;	bra	nz, SendErr_BEF_SIZE

	rcall	uuGetVal32
	bra	c, SendErr_BEF_SIZE

	rcall	Set_uuFreq
	bra	c, SendErr_BEF_SIZE
	bra	px02_Ok

LwRANGE_ATT:
	rcall	uuGetVal16
	bra	c, SendErr_BEF_SIZE
	rcall	SetImpedanz_HZ
	bra	px02_Ok

LwRANGE_AMPL:
;	cp	w1, #4
;	bra	nz, SendErr_BEF_SIZE
	btst	w4, #bFLAG_FLOAT32
	bra	z, SendErr_UNKNOWN_BEF

	rcall	uuGetVal32
	bra	c, SendErr_BEF_SIZE
	rcall	Set_uuAmpl
	bra	c, SendErr_BEF_SIZE
	bra	px02_Ok

LwRANGE_OFFS:
;	cp	w1, #4
;	bra	nz, SendErr_BEF_SIZE
	btst	w4, #bFLAG_FLOAT32
	bra	z, SendErr_UNKNOWN_BEF

	rcall	uuGetVal32
	bra	c, SendErr_BEF_SIZE
	rcall	Set_uuOffs
	bra	c, SendErr_BEF_SIZE
	bra	px02_Ok

LwRANGE_MOD:
	rcall	uuGetVal16
	bra	c, SendErr_BEF_SIZE

	btsc	w0, #2
	bset	INV_PORT, #INV_BIT
	btss	w0, #2
	bclr	INV_PORT, #INV_BIT

	mov	PA_OutFlag, w2
	mov	#PAOF_SOPDC_MASK, w3
	and	w0, w3, w0
	and	w2, w3, w2
	cp	w0, w2
	bra	z, px02_Ok
	cp0	uuCAL_On_Off
	bra	z, rm_mss
	btst	w0, #bPAOF_DC
	bra	z, rm_mss

	bclr	PA_OutFlag, #bPAOF_SQ_P
	bset	PA_OutFlag, #bPAOF_DC
	bra	dm_dc_on

rm_mss:
	rcall	ModSquareSet
	bra	px02_Ok
	

/* ----------------- SOUR --------------------- */
vxSOUR:
	btst	w4, #0
	bra	nz, vxSOUR_WR
	cp0	w2
	bra	z, LrSOUR_MEAS
	bra	SendErr_UNKNOWN_BEF

/* ----------------- SOUR RD --------------------- */
LrSOUR_MEAS:
	mov	SOURce, w0
	bra	TxD_AV_int16

/* ----------------- SOUR WR --------------------- */
vxSOUR_WR:
	cp0	w2
	bra	z, LwSOUR_MEAS
	bra	SendErr_UNKNOWN_BEF

LwSOUR_MEAS:
	rcall	uuGetVal16
	bra	c, SendErr_BEF_SIZE
	mov	w0, SOURce
	rcall	chk_sour
	bra	c, SendErr_ADDR_FAULT
	bra	px02_Ok

chk_sour:
	mov	SOURce, w0
	ze	w0, w0
	cp	w0, #SOUR_OUT_V
	bra	z, chk_sour_ok
	cp	w0, #SOUR_OP1_V
	bra	z, chk_sour_ok
	cp	w0, #SOUR_PA_TEMP
	bra	z, chk_sour_ok
	cp	w0, #SOUR_OP1_TEMP
	bra	z, chk_sour_ok
	cp	w0, #SOUR_GEN_TEMP
	bra	z, chk_sour_ok

	cp	w0, #SOUR_GOV
	bra	z, chk_sour_ok
	cp	w0, #SOUR_GRV
	bra	z, chk_sour_ok

	cp	w0, #SOUR_OUT_V_DIGIT
	bra	z, chk_sour_ok
	cp	w0, #SOUR_OP1_V_DIGIT
	bra	z, chk_sour_ok

	bset	SR, #C
	return

chk_sour_ok:
	bclr	SR, #C
	return

/* ----------------- MEAS --------------------- */
vxMEAS:
	btst	w4, #bFLAG_WR
	bra	nz, SendErr_UNKNOWN_BEF
	cp0	w1
	bra	nz, SendErr_BEF_SIZE

	rcall	chk_sour
	bra	c, SendErr_ADDR_FAULT

	mov	#RS232ALD_TxD_Data, w3
	mov.b	w2, [w3]

	cp	w2, #MEAS_READ
	bra	z, LrMEAS_READ
	bra	SendErr_UNKNOWN_BEF

LrMEAS_READ_Wait:
	rcall	chk_sour
	bra	c, SendErr_ADDR_FAULT
LrMEAS_READ:
	cp	w0, #SOUR_OUT_V
	bra	z, LrSOUR_OUT_V
	cp	w0, #SOUR_OP1_V
	bra	z, LrSOUR_OP1_V
	cp	w0, #SOUR_PA_TEMP
	bra	z, LrSOUR_PA_TEMP
	cp	w0, #SOUR_OP1_TEMP
	bra	z, LrSOUR_OP1_TEMP
	cp	w0, #SOUR_GEN_TEMP
	bra	z, LrSOUR_GEN_TEMP

	cp	w0, #SOUR_GOV
	bra	z, LrSOUR_GOV
	cp	w0, #SOUR_GRV
	bra	z, LrSOUR_GRV

	cp	w0, #SOUR_OUT_V_DIGIT
	bra	z, LrSOUR_OUT_V_DIGIT
	cp	w0, #SOUR_OP1_V_DIGIT
	bra	z, LrSOUR_OP1_V_DIGIT


	bra	SendErr_UNKNOWN_BEF


ChkRdMCP3421:
	mov	MCP3421_ReadCount, w4
	cp	w4, [w5]
	btsc	SR, #Z
	return
	mov	w4, [w5]
	clr	rs232ald_wait
	return

LrSOUR_OUT_V:
	mov	#MCP3421_LastRead, w5
	rcall	ChkRdMCP3421
	bra	z, LrSOUR_Wait

	mov	AdcPa_1616, w0
	mov	AdcPa, w1

	mov	#EINH_FORMAT_1616 | EINH_EMUL_0001, w2
	bra	TxD_AV_AX

LrSOUR_OUT_V_DIGIT:
	mov	#MCP3421_LastRead+2, w5
	rcall	ChkRdMCP3421
	bra	z, LrSOUR_Wait

	mov	AdcOrgVout, w0
	mov	AdcOrgVout+2, w1

	mov	#EINH_INT, w2
	bra	TxD_AV_AX

ChkRdADS112c04:
	mov	ADS112c04_ReadCount, w4
	cp	w4, [w5]
	btsc	SR, #Z
	return
	mov	w4, [w5]
	clr	rs232ald_wait
	return

LrSOUR_Wait:
	mov	#BEF_MEAS, w0
	mov	w0, rs232ald_wait
	return

LrSOUR_OP1_V:
	mov	#ADS112c04_LastRead+4, w5
	rcall	ChkRdADS112c04
	bra	z, LrSOUR_Wait

	mov	OP1V_1616_001mV, w0
	mov	OP1V_1616_001mV+2, w1
	mov	#EINH_FORMAT_1616 | EINH_EMUL_00001, w2
	bra	TxD_AV_AX


LrSOUR_OP1_V_DIGIT:
	mov	#ADS112c04_LastRead+8, w5
	rcall	ChkRdADS112c04
	bra	z, LrSOUR_Wait

	mov	Anl_OP1V_orig, w0
	mov	#EINH_EMUL_1, w2
	bra	TxD_AV_AX_16

LrSOUR_PA_TEMP:
.if 1
	mov	Temp_PA_0025, w0
	mov	#EINH_MUL_25 | EINH_EMUL_0001, w2
.else
	mov	Temp_PA, w0
	mov	#EINH_EMUL_01, w2
.endif
	bra	TxD_AV_AX_16

LrSOUR_OP1_TEMP:
;	mov	Temp_OP1, w0
;	mov	#EINH_EMUL_01, w2
	mov	Temp_OP1_0025, w0
	mov	#EINH_MUL_25 | EINH_EMUL_0001, w2
	bra	TxD_AV_AX_16

LrSOUR_GEN_TEMP:
	mov	#ADS112c04_LastRead+6, w5
	rcall	ChkRdADS112c04
	bra	z, LrSOUR_Wait

	mov	Temp_GEN_0025, w0
	mov	#EINH_MUL_25 | EINH_EMUL_0001, w2
;	asr	w0, w0
;	asr	w0, w0
;	addc	w0, #0, w0
;	mov	#EINH_EMUL_01, w2
	bra	TxD_AV_AX_16

LrSOUR_GOV:
	mov	#ADS112c04_LastRead, w5
	rcall	ChkRdADS112c04
	bra	z, LrSOUR_Wait

	mov	GOV_01mV, w0
	mov	#EINH_EMUL_00001, w2
	bra	TxD_AV_AX_16

LrSOUR_GRV:
	mov	#ADS112c04_LastRead+2, w5
	rcall	ChkRdADS112c04
	bra	z, LrSOUR_Wait

	mov	GRV_01mV, w0
	mov	#EINH_EMUL_00001, w2
	bra	TxD_AV_AX_16

/* ----------------- CAL --------------------------------------------- */
vxCAL:
	btst	w4, #bFLAG_WR
	bra	nz, vxCAL_WR

	cp	w2, #CAL_ATT
	bra	z, LrCAL_ATT

	bra	SendErr_UNKNOWN_BEF

LrCAL_ATT:
	rcall	GetK_dbX
	bra	TxD_AV_int16

/* ----------------- CAL wr --------------------- */
vxCAL_WR:
	cp	w2, #CAL_ON_OFF
	bra	z, LwCAL_ON_OFF
	cp	w2, #CAL_ATT
	bra	z, LwCAL_ATT
	cp	w2, #CAL_DAC
	bra	z, LwCAL_DAC
	cp	w2, #CAL_DAC_VSET
	bra	z, LwCAL_DAC_VSET
	cp	w2, #CAL_DAC_MOD
	bra	z, LwCAL_DAC_MOD
	cp	w2, #CAL_DAC_EXTR
	bra	z, LwCAL_DAC_EXTR

	bra	SendErr_UNKNOWN_BEF

LwCAL_ON_OFF:
	rcall	uuGetVal16
	bra	c, SendErr_BEF_SIZE
	mov	w0, uuCAL_On_Off
	cp0	w0
	bra	nz, Set_CAL_ON

	clr	uuCAL_DAC_Mod
	clr	TestFlags
	clr	uuCAL_DAC_Extr
;	rcall	uuRecalcTemp


	mov	PA_OutFlag, w0
	rcall	ModSquareSet
	bset	MathFlags, #bMF_AMPL

	bra	px02_Ok

Set_CAL_ON:
	btsc	w0, #1
	bset	TestFlags, #bTF_DAC
	bra	px02_Ok

LwCAL_ATT:
	rcall	uuGetVal16
	bra	c, SendErr_BEF_SIZE
	rcall	SetAmplRelais
	bra	px02_Ok

LwCAL_DAC:
	rcall	uuGetVal16
	bra	c, SendErr_BEF_SIZE
	mov	w0, uuCAL_DAC_CHx
	bra	px02_Ok

LwCAL_DAC_MOD:
	rcall	uuGetVal16
	bra	c, SendErr_BEF_SIZE
	mov	w0, uuCAL_DAC_Mod

.if ADC_PA_MCP3421_AUTORANGE
	bclr	TestFlags, #TF_PA_ADC_H_ON
	btsc	w0, #bPA_ADC_H_ON
	bset	TestFlags, #TF_PA_ADC_H_ON
.endif
	btst	w0, #bGEN_DC_VCC_ON
	bra	z, dm_dcoff
dm_dc_on:
	btst	PA_OutFlag, #bPAOF_DC
	bra	z, SendErr_UNKNOWN_BEF
.if USE_PLL_CDCE913
	rcall	pll_off
	rcall	SetGenPllSym
.endif
	rcall	OC1_PinDisabel
	bset	TestFlags, #TF_DC_GEN_ON

	bset	GenLongTimeFlag, #bGLT_ON
	btsc	uuCAL_DAC_Mod, #bGEN_LONGTIME_OFF
	bclr	GenLongTimeFlag, #bGLT_ON

	bset	GV_PORT, #GV_BIT
	bset	SQUARE_PORT, #SQUARE_BIT
	bra	px02_Ok

dm_dcoff:
	bclr	TestFlags, #TF_DC_GEN_ON
	btst	PA_OutFlag, #bPAOF_DC
	bra	nz, dm_dcoff_DC
	btst	GV_PORT, #GV_BIT
	bra	nz, px02_Ok
	rcall	SetUpFreq
	bra	px02_Ok
dm_dcoff_DC:
	bclr	GV_PORT, #GV_BIT
	bra	px02_Ok

LwCAL_DAC_EXTR:
	rcall	uuGetVal16
	bra	c, SendErr_BEF_SIZE
	mov	w0, uuCAL_DAC_Extr
	mov	w0, w1
	bclr	w1, #0
	cp	w1, #0x02
	bra	z, cdex_gain
	cp	w1, #0x04
	bra	z, px02_Ok
	bra	SendErr_UNKNOWN_BEF
cdex_gain:
	rcall	uuExtr_Gain
	bra	px02_Ok

LwCAL_DAC_VSET:
	btst	uuCAL_DAC_Extr, #2
	bra	nz, padc_float

	rcall	uuGetVal16
	bra	c, SendErr_BEF_SIZE
;	cp0	uuCAL_On_Off
;	bra	z, SendErr_UNKNOWN_BEF
	cp0	uuCAL_DAC_CHx
	bra	z, SendErr_UNKNOWN_BEF

	mov	uuCAL_DAC_CHx, w2
	ze	w2, w1
	btst	uuCAL_DAC_Mod, #bDAC_PA_OFFSET_KORR_OFF
	bra	nz, padc_no_korr
	cp	w1, #PA_POW & 0x00FF
	bra	z, papow_korr
	cp	w1, #PA_DC & 0x00FF
	bra	z, padc_korr
padc_no_korr:
	rcall	DacSetValue_LinKorr
	bra	px02_Ok

papow_korr:
	lnk	#F48_SIZE
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#fDigPaPow_mV, w2
	mov	#fPaPow_Offs, w4
	bra	pax_korr_set

padc_korr:
	lnk	#F48_SIZE
	mov	w14, w3
	rcall	i16_f48_w3
	mov	#fDigPaDC_mV, w2
	mov	#fPaDC_Offs, w4
pax_korr_set:
	rcall	f48_mul
	ulnk
	rcall	dctk_dac_set
	bra	px02_Ok

padc_float:
	btst	w4, #bFLAG_FLOAT32
	bra	z, SendErr_UNKNOWN_BEF
	rcall	uuGetVal32
	bra	c, SendErr_BEF_SIZE

	mov	uuCAL_DAC_CHx, w2
	ze	w2, w2
	cp	w2, #AGVN & 0x00FF
	bra	z, padc_genv
	cp	w2, #AGVP & 0x00FF
	bra	z, padc_genv
	bra	SendErr_UNKNOWN_BEF

padc_genv:
	lnk	#F48_SIZE*3
	mov	w14, w2
	rcall	f32_f48

	mov	#F48_MaxGenV, w3
	rcall	f48_cp
	bra	gt, padc_genv_err

	mov	#F48_MinGenV, w3
	rcall	f48_cp
	bra	lt, padc_genv_err

	bclr	GainRelais, #0
	btsc	K01DB_PORT, #K01DB_BIT
	bset	GainRelais, #0

	mov	#FC48_1000, w3
	mov	w14, w4
	rcall	f48_mul

	rcall	SetGenV_w2
	ulnk
	bra	px02_Ok

padc_genv_err:
	ulnk
	bra	SendErr_UNKNOWN_BEF

/* ______________________________________________________________________ */
.if USE_UART_LIST_PARA
vxPARAM:
	btst	w4, #bFLAG_WR
	bra	nz, vxPARAM_WR

	cp	w2, #PARAM_MENU
	bra	z, LrPARAM_MENU

	bra	SendErr_UNKNOWN_BEF

LrPARAM_MENU:
	rcall	ucMenuData
;	bra	TxD_AV_Str
TxD_ParamStr:
	mov	#RS232ALD_TxD_Data+1, w3
	mov	#FLAG_STR, w2
	mov.b	w2, [w3++]

	mov	#Str_buffer, w4
	mov	#STR_MAX_BUF, w2
ctx_loop:
	ze	[w4++], w0
	cp0	w0
	bra	z, ctxe
	cp	w0, #0x7f
	bra	lt, ctx_next
	bra	z, ctx_deg
	mov	#0x20, w0
	bra	ctx_next

ctx_deg:
	mov	#0xb0, w0
ctx_next:
	mov.b	w0, [w3++]
	dec	w2, w2
	bra	nz, ctx_loop
ctxe:
	clr.b	[w3]
	mov	#Str_buffer-2, w1
	sub	w4, w1, w1
	mov	RxD_Bef, w0
	bra	rs232ald_TxD

/* ----------------- PARA wr --------------------- */
vxPARAM_WR:
	cp	w2, #PARAM_MENU0
	bra	z, LwPARAM_MENU0
	cp	w2, #PARAM_NEXT
	bra	z, LwPARAM_NEXT
	cp	w2, #PARAM_OPEN
	bra	z, LwPARAM_OPEN
	cp	w2, #PARAM_CLOSE
	bra	z, LwPARAM_CLOSE
	bra	SendErr_UNKNOWN_BEF

LwPARAM_MENU0:
	cp	w1, #0
	bra	nz, SendErr_BEF_SIZE
	rcall	ucMenu0
	bra	px02_Ok

LwPARAM_NEXT:
	cp	w1, #0
	bra	nz, SendErr_BEF_SIZE
	rcall	ucMenuNext
	bra	px02_Ok

LwPARAM_OPEN:
	cp	w1, #0
	bra	nz, SendErr_BEF_SIZE
	rcall	ucMenuOpen
	bra	px02_Ok

LwPARAM_CLOSE:
	cp	w1, #0
	bra	nz, SendErr_BEF_SIZE
	rcall	ucMenuClose
	bra	px02_Ok

.endif  /* USE_UART_LIST_PARA */

/* ______________________________________________________________________ */

.section .const, psv

F48_MaxGenV:
	.word 0xcccd, 0x8ccc, 0x3fff		; 1.1
F48_MinGenV:
	.word 0xcccd, 0xcccc, 0x3ffe		; 0.8



.endif  /* USE_DEBUG_UART */
